// EnergyPlus, Copyright (c) 1996-2019, The Board of Trustees of the University of Illinois,
// The Regents of the University of California, through Lawrence Berkeley National Laboratory
// (subject to receipt of any required approvals from the U.S. Dept. of Energy), Oak Ridge
// National Laboratory, managed by UT-Battelle, Alliance for Sustainable Energy, LLC, and other
// contributors. All rights reserved.
//
// NOTICE: This Software was developed under funding from the U.S. Department of Energy and the
// U.S. Government consequently retains certain rights. As such, the U.S. Government has been
// granted for itself and others acting on its behalf a paid-up, nonexclusive, irrevocable,
// worldwide license in the Software to reproduce, distribute copies to the public, prepare
// derivative works, and perform publicly and display publicly, and to permit others to do so.
//
// Redistribution and use in source and binary forms, with or without modification, are permitted
// provided that the following conditions are met:
//
// (1) Redistributions of source code must retain the above copyright notice, this list of
//     conditions and the following disclaimer.
//
// (2) Redistributions in binary form must reproduce the above copyright notice, this list of
//     conditions and the following disclaimer in the documentation and/or other materials
//     provided with the distribution.
//
// (3) Neither the name of the University of California, Lawrence Berkeley National Laboratory,
//     the University of Illinois, U.S. Dept. of Energy nor the names of its contributors may be
//     used to endorse or promote products derived from this software without specific prior
//     written permission.
//
// (4) Use of EnergyPlus(TM) Name. If Licensee (i) distributes the software in stand-alone form
//     without changes from the version obtained under this License, or (ii) Licensee makes a
//     reference solely to the software portion of its product, Licensee must refer to the
//     software as "EnergyPlus version X" software, where "X" is the version number Licensee
//     obtained under this License and may not use a different name for the software. Except as
//     specifically required in this Section (4), Licensee shall not use in a company name, a
//     product name, in advertising, publicity, or other promotional activities any name, trade
//     name, trademark, logo, or other designation of "EnergyPlus", "E+", "e+" or confusingly
//     similar designation, without the U.S. Department of Energy's prior written consent.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.

#ifndef DataConversions_hh_INCLUDED
#define DataConversions_hh_INCLUDED

// EnergyPlus Headers
#include <EnergyPlus.hh>

namespace EnergyPlus {

namespace DataConversions {

    // Data
    // -only module should be available to other modules and routines.
    // Thus, all variables in this module must be PUBLIC.

    // MODULE PARAMETER DEFINITIONS:
    // REAL(r64), PARAMETER:: CFC     =4.184D0            ! Specific Heat:  BTU/(LB*R) * CFC = KJ/(KG*K)
    extern Real64 const CFC; // Specific Heat:  BTU/(LB*R) * CFC = KJ/(KG*K)
    //  above is listed in July 1976 publication as "International Table"
    extern Real64 const CFL; // Length:         FT * CFL = M
    extern Real64 const CFM; // Mass:           LB * CFM = KG
    // REAL(r64), PARAMETER:: CFP     =249.082D0          ! Pressure:       IN-H2O * CFP = N/M**2
    // above is listed in July 1976 publication as in-water at 39.2 deg F
    extern Real64 const CFP; // Pressure:       IN-H2O * CFP = N/M**2
    //  above is listed in July 1976 publication as in-water at 60 deg F
    extern Real64 const DELTMP; // Temperature:    (F + DELTMP) * CFT = C
    extern Real64 const CFA;    // Area:           FT**2 * CFA = M**2
    extern Real64 const CFT;    // Temperature:    R * CFT = K
    extern Real64 const CFV;    // Volume:         FT**3 * CFV = M**3
    extern Real64 const CFE;    // Energy:         BTU * CFE = W-HR
    extern Real64 const CFD;    // Density:        LB/FT**3 * CFD = KG/M**3
    extern Real64 const CFH;    // Enthalpy:       BTU/LB * CFH = J/KG
    extern Real64 const CFK;    // Conductivity:   BTU/(HR*FT*R) * CFK = W/(M*K)
    extern Real64 const CFMF;   // Mass Flow:      LB/HR * CFMF = KG/SEC
    extern Real64 const CFQ;    // Power:          BTU/HR * CFQ = W
    extern Real64 const CFU;    // U-Value:        BTU/(HR*FT**2*R) * CFU = W/(M**2*K)
    // Note:  R-Value = 1/U-Value
    extern Real64 const CFS;   // Speed:          FT/MIN * CFS = M/SEC
    extern Real64 const CFVF;  // Volume Flow:    FT**3/MIN * CFVF = M**3/SEC
    extern Real64 const CFHF;  // Heat Flux:      BTU/(HR*FT**2) * CFHF = W/M**2
    extern Real64 const CFTMP; // Temperature:    Same as DELTMP

} // namespace DataConversions

} // namespace EnergyPlus

#endif
