// EnergyPlus, Copyright (c) 1996-2019, The Board of Trustees of the University of Illinois,
// The Regents of the University of California, through Lawrence Berkeley National Laboratory
// (subject to receipt of any required approvals from the U.S. Dept. of Energy), Oak Ridge
// National Laboratory, managed by UT-Battelle, Alliance for Sustainable Energy, LLC, and other
// contributors. All rights reserved.
//
// NOTICE: This Software was developed under funding from the U.S. Department of Energy and the
// U.S. Government consequently retains certain rights. As such, the U.S. Government has been
// granted for itself and others acting on its behalf a paid-up, nonexclusive, irrevocable,
// worldwide license in the Software to reproduce, distribute copies to the public, prepare
// derivative works, and perform publicly and display publicly, and to permit others to do so.
//
// Redistribution and use in source and binary forms, with or without modification, are permitted
// provided that the following conditions are met:
//
// (1) Redistributions of source code must retain the above copyright notice, this list of
//     conditions and the following disclaimer.
//
// (2) Redistributions in binary form must reproduce the above copyright notice, this list of
//     conditions and the following disclaimer in the documentation and/or other materials
//     provided with the distribution.
//
// (3) Neither the name of the University of California, Lawrence Berkeley National Laboratory,
//     the University of Illinois, U.S. Dept. of Energy nor the names of its contributors may be
//     used to endorse or promote products derived from this software without specific prior
//     written permission.
//
// (4) Use of EnergyPlus(TM) Name. If Licensee (i) distributes the software in stand-alone form
//     without changes from the version obtained under this License, or (ii) Licensee makes a
//     reference solely to the software portion of its product, Licensee must refer to the
//     software as "EnergyPlus version X" software, where "X" is the version number Licensee
//     obtained under this License and may not use a different name for the software. Except as
//     specifically required in this Section (4), Licensee shall not use in a company name, a
//     product name, in advertising, publicity, or other promotional activities any name, trade
//     name, trademark, logo, or other designation of "EnergyPlus", "E+", "e+" or confusingly
//     similar designation, without the U.S. Department of Energy's prior written consent.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.

// EnergyPlus Headers
#include <DataDefineEquip.hh>
#include <DataPrecisionGlobals.hh>

namespace EnergyPlus {

namespace DataDefineEquip {

    // MODULE INFORMATION
    //             AUTHOR:  Russ Taylor
    //       DATE WRITTEN:  Sept 1997

    // PURPOSE OF THIS MODULE:
    // This module  contains the essential coil information that is needed by water and air
    // loop managers as well as the coil simulations

    // METHODOLOGY EMPLOYED:
    // Needs description, as appropriate.

    // REFERENCES: none

    // OTHER NOTES: none

    // Using/Aliasing
    using namespace DataPrecisionGlobals;

    // Data
    // -only module should be available to other modules and routines.
    // Thus, all variables in this module must be PUBLIC.

    // MODULE PARAMETER DEFINITIONS
    int const MaxZoneAirComponents(1);
    // INTEGER, PARAMETER :: MaxZoneAirControls = 4
    // Equipment Types covered by ZoneAirLoopEquipment:
    int const DualDuctConstVolume(1);
    int const DualDuctVAV(2);
    int const SingleDuctVAVReheat(3);
    int const SingleDuctVAVNoReheat(4);
    int const SingleDuctConstVolReheat(5);
    int const SingleDuctConstVolNoReheat(6);
    int const SingleDuct_SeriesPIU_Reheat(7);
    int const SingleDuct_ParallelPIU_Reheat(8);
    int const SingleDuct_ConstVol_4PipeInduc(9);
    int const SingleDuctVAVReheatVSFan(10);
    int const SingleDuctCBVAVReheat(11);
    int const SingleDuctCBVAVNoReheat(12);
    int const SingleDuctConstVolCooledBeam(13);
    int const DualDuctVAVOutdoorAir(14);
    int const SingleDuctUserDefined(15);
    int const SingleDuctATMixer(16);
    int const SingleDuctConstVolFourPipeBeam(17);

    // DERIVED TYPE DEFINITIONS

    // MODULE VARIABLE DECLARATIONS:
    // components of air system
    int NumAirDistUnits(0);

    // Object Data
    Array1D<ZoneAirEquip> AirDistUnit; // Used to specify zone related

    // Clears the global data in DataDefineEquip.
    // Needed for unit tests, should not be normally called.
    void clear_state()
    {
        NumAirDistUnits = 0;
        AirDistUnit.deallocate();
    }

} // namespace DataDefineEquip

} // namespace EnergyPlus
