// EnergyPlus, Copyright (c) 1996-2019, The Board of Trustees of the University of Illinois,
// The Regents of the University of California, through Lawrence Berkeley National Laboratory
// (subject to receipt of any required approvals from the U.S. Dept. of Energy), Oak Ridge
// National Laboratory, managed by UT-Battelle, Alliance for Sustainable Energy, LLC, and other
// contributors. All rights reserved.
//
// NOTICE: This Software was developed under funding from the U.S. Department of Energy and the
// U.S. Government consequently retains certain rights. As such, the U.S. Government has been
// granted for itself and others acting on its behalf a paid-up, nonexclusive, irrevocable,
// worldwide license in the Software to reproduce, distribute copies to the public, prepare
// derivative works, and perform publicly and display publicly, and to permit others to do so.
//
// Redistribution and use in source and binary forms, with or without modification, are permitted
// provided that the following conditions are met:
//
// (1) Redistributions of source code must retain the above copyright notice, this list of
//     conditions and the following disclaimer.
//
// (2) Redistributions in binary form must reproduce the above copyright notice, this list of
//     conditions and the following disclaimer in the documentation and/or other materials
//     provided with the distribution.
//
// (3) Neither the name of the University of California, Lawrence Berkeley National Laboratory,
//     the University of Illinois, U.S. Dept. of Energy nor the names of its contributors may be
//     used to endorse or promote products derived from this software without specific prior
//     written permission.
//
// (4) Use of EnergyPlus(TM) Name. If Licensee (i) distributes the software in stand-alone form
//     without changes from the version obtained under this License, or (ii) Licensee makes a
//     reference solely to the software portion of its product, Licensee must refer to the
//     software as "EnergyPlus version X" software, where "X" is the version number Licensee
//     obtained under this License and may not use a different name for the software. Except as
//     specifically required in this Section (4), Licensee shall not use in a company name, a
//     product name, in advertising, publicity, or other promotional activities any name, trade
//     name, trademark, logo, or other designation of "EnergyPlus", "E+", "e+" or confusingly
//     similar designation, without the U.S. Department of Energy's prior written consent.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.

#ifndef DataHVACControllers_hh_INCLUDED
#define DataHVACControllers_hh_INCLUDED

// ObjexxFCL Headers
#include <ObjexxFCL/Array1D.hh>

// EnergyPlus Headers
#include <EnergyPlus.hh>

namespace EnergyPlus {

namespace DataHVACControllers {

    // Data
    // -only module should be available to other modules and routines.
    // Thus, all variables in this module must be PUBLIC.

    // MODULE PARAMETER DEFINITIONS:

    extern int const ControllerSimple_Type;
    extern Array1D_string const ControllerTypes;

    // Controller action used in modules HVACControllers and ZoneControllers
    extern int const iNoAction;
    extern int const iReverseAction;
    extern int const iNormalAction;
    extern Array1D_string const ActionTypes;

    // Controller mode used in modules HVACControllers and ZoneControllers
    extern int const iModeWrongAction; // Controller error. E.g., bad action
    extern int const iModeNone;        // Controller mode not yet determined
    extern int const iModeOff;         // Controller off (no air flow in loop)
    extern int const iModeInactive;    // Controller inactive (equip not available for current step)
    extern int const iModeActive;      // Controller active (schedule>0 and min<actuated<max)
    extern int const iModeMinActive;   // Controller active and min-constrained (equip available and actuated=min)
    extern int const iModeMaxActive;   // Controller active and max-constrained (equip available and actuated=max)

    extern int const iFirstMode; // First operating mode in range
    extern int const iLastMode;  // Last operating mode in range
    extern Array1D_string const ControllerModeTypes;

    // Controller operation used in module HVACControllers
    extern int const iControllerOpColdStart;   // Reset for cold start
    extern int const iControllerOpWarmRestart; // Reset for warm restart with previous solution
    extern int const iControllerOpIterate;     // Check convergence and estimate next guess if needed
    extern int const iControllerOpEnd;         // Check convergence only and trace

    // Controller restart flag used in module HVACControllers
    extern int const iControllerWarmRestartNone;    // Indicates that warm restart was not attempted
    extern int const iControllerWarmRestartFail;    // Indicates that warm restart failed
    extern int const iControllerWarmRestartSuccess; // Indicates that warm restart was successful

} // namespace DataHVACControllers

} // namespace EnergyPlus

#endif
