// EnergyPlus, Copyright (c) 1996-2019, The Board of Trustees of the University of Illinois,
// The Regents of the University of California, through Lawrence Berkeley National Laboratory
// (subject to receipt of any required approvals from the U.S. Dept. of Energy), Oak Ridge
// National Laboratory, managed by UT-Battelle, Alliance for Sustainable Energy, LLC, and other
// contributors. All rights reserved.
//
// NOTICE: This Software was developed under funding from the U.S. Department of Energy and the
// U.S. Government consequently retains certain rights. As such, the U.S. Government has been
// granted for itself and others acting on its behalf a paid-up, nonexclusive, irrevocable,
// worldwide license in the Software to reproduce, distribute copies to the public, prepare
// derivative works, and perform publicly and display publicly, and to permit others to do so.
//
// Redistribution and use in source and binary forms, with or without modification, are permitted
// provided that the following conditions are met:
//
// (1) Redistributions of source code must retain the above copyright notice, this list of
//     conditions and the following disclaimer.
//
// (2) Redistributions in binary form must reproduce the above copyright notice, this list of
//     conditions and the following disclaimer in the documentation and/or other materials
//     provided with the distribution.
//
// (3) Neither the name of the University of California, Lawrence Berkeley National Laboratory,
//     the University of Illinois, U.S. Dept. of Energy nor the names of its contributors may be
//     used to endorse or promote products derived from this software without specific prior
//     written permission.
//
// (4) Use of EnergyPlus(TM) Name. If Licensee (i) distributes the software in stand-alone form
//     without changes from the version obtained under this License, or (ii) Licensee makes a
//     reference solely to the software portion of its product, Licensee must refer to the
//     software as "EnergyPlus version X" software, where "X" is the version number Licensee
//     obtained under this License and may not use a different name for the software. Except as
//     specifically required in this Section (4), Licensee shall not use in a company name, a
//     product name, in advertising, publicity, or other promotional activities any name, trade
//     name, trademark, logo, or other designation of "EnergyPlus", "E+", "e+" or confusingly
//     similar designation, without the U.S. Department of Energy's prior written consent.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.

#ifndef EcoRoofManager_hh_INCLUDED
#define EcoRoofManager_hh_INCLUDED

// EnergyPlus Headers
#include <EnergyPlus.hh>

namespace EnergyPlus {

namespace EcoRoofManager {

    // Data
    // MODULE PARAMETER DEFINITIONS
    // na

    // DERIVED TYPE DEFINITIONS
    // na

    // MODULE VARIABLE DECLARATIONS:

    extern Real64 CumRunoff; // Cumulative runoff, updated each time step (m) mult by roof area to get volume
    extern Real64 CumET;     // Cumulative evapotranspiration from soil and plants (m)
    extern Real64 CumPrecip;
    extern Real64 CumIrrigation; // Cumulative irrigation, updated each time step (m) mult by roof area to get volume
    extern Real64 CurrentRunoff;
    extern Real64 CurrentET;
    extern Real64 CurrentPrecipitation; // units of (m) per timestep
    extern Real64 CurrentIrrigation;    // units of (m) per timestep

    extern Real64 Tfold; // leaf temperature from the previous time step
    extern Real64 Tgold; // ground temperature from the previous time step
    extern bool EcoRoofbeginFlag;

    // Functions

    void CalcEcoRoof(int const SurfNum, // Indicator of Surface Number for the current surface
                     int const ZoneNum, // Indicator for zone number where the current surface
                     int &ConstrNum,    // Indicator for contruction index for the current surface
                     Real64 &TempExt    // Exterior temperature boundary condidtion
    );

    void UpdateSoilProps(Real64 &Moisture,
                         Real64 &MeanRootMoisture,
                         Real64 const MoistureMax,
                         Real64 const MoistureResidual,
                         Real64 const SoilThickness,
                         Real64 const Vfluxf, // Water mass flux from vegetation [m/s]
                         Real64 const Vfluxg, // Water mass flux from soil surface [m/s]
                         int &ConstrNum,      // Indicator for contruction index for the current surface
                         Real64 &Alphag,
                         int const unit,    // unused1208
                         Real64 const Tg,   // unused1208
                         Real64 const Tf,   // unused1208
                         Real64 const Qsoil // unused1208
    );

} // namespace EcoRoofManager

} // namespace EnergyPlus

#endif
