// EnergyPlus, Copyright (c) 1996-2019, The Board of Trustees of the University of Illinois,
// The Regents of the University of California, through Lawrence Berkeley National Laboratory
// (subject to receipt of any required approvals from the U.S. Dept. of Energy), Oak Ridge
// National Laboratory, managed by UT-Battelle, Alliance for Sustainable Energy, LLC, and other
// contributors. All rights reserved.
//
// NOTICE: This Software was developed under funding from the U.S. Department of Energy and the
// U.S. Government consequently retains certain rights. As such, the U.S. Government has been
// granted for itself and others acting on its behalf a paid-up, nonexclusive, irrevocable,
// worldwide license in the Software to reproduce, distribute copies to the public, prepare
// derivative works, and perform publicly and display publicly, and to permit others to do so.
//
// Redistribution and use in source and binary forms, with or without modification, are permitted
// provided that the following conditions are met:
//
// (1) Redistributions of source code must retain the above copyright notice, this list of
//     conditions and the following disclaimer.
//
// (2) Redistributions in binary form must reproduce the above copyright notice, this list of
//     conditions and the following disclaimer in the documentation and/or other materials
//     provided with the distribution.
//
// (3) Neither the name of the University of California, Lawrence Berkeley National Laboratory,
//     the University of Illinois, U.S. Dept. of Energy nor the names of its contributors may be
//     used to endorse or promote products derived from this software without specific prior
//     written permission.
//
// (4) Use of EnergyPlus(TM) Name. If Licensee (i) distributes the software in stand-alone form
//     without changes from the version obtained under this License, or (ii) Licensee makes a
//     reference solely to the software portion of its product, Licensee must refer to the
//     software as "EnergyPlus version X" software, where "X" is the version number Licensee
//     obtained under this License and may not use a different name for the software. Except as
//     specifically required in this Section (4), Licensee shall not use in a company name, a
//     product name, in advertising, publicity, or other promotional activities any name, trade
//     name, trademark, logo, or other designation of "EnergyPlus", "E+", "e+" or confusingly
//     similar designation, without the U.S. Department of Energy's prior written consent.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.

#ifndef HeatBalanceIntRadExchange_hh_INCLUDED
#define HeatBalanceIntRadExchange_hh_INCLUDED

// ObjexxFCL Headers
#include <ObjexxFCL/Array1A.hh>
#include <ObjexxFCL/Array1S.hh>
#include <ObjexxFCL/Array2A.hh>
#include <ObjexxFCL/Array2S.hh>
#include <ObjexxFCL/Optional.hh>

// EnergyPlus Headers
#include <EnergyPlus.hh>

namespace EnergyPlus {

#define EP_HBIRE_SEQ

namespace HeatBalanceIntRadExchange {

    // Data
    // MODULE PARAMETER DEFINITIONS

    // DERIVED TYPE DEFINITIONS
    // na

    // MODULE VARIABLE DECLARATIONS:
    extern int MaxNumOfZoneSurfaces; // Max saved to get large enough space for user input view factors

    // SUBROUTINE SPECIFICATIONS FOR MODULE HeatBalanceIntRadExchange

    // Functions
    void clear_state();

    void CalcInteriorRadExchange(Array1S<Real64> const SurfaceTemp,       // Current surface temperatures
                                 int const SurfIterations,                // Number of iterations in calling subroutine
                                 Array1<Real64> &NetLWRadToSurf,          // Net long wavelength radiant exchange from other surfaces
                                 Optional_int_const ZoneToResimulate = _, // if passed in, then only calculate for this zone
                                 std::string const &CalledFrom = "");

    void UpdateMovableInsulationFlag(bool &MovableInsulationChange, // set to true if there is a change in the movable insulation state
                                     int const SurfNum              // surface number of surface being investigated
    );

    void InitInteriorRadExchange();

    void GetInputViewFactors(std::string const &ZoneName, // Needed to check for user input view factors.
                             int const N,                 // NUMBER OF SURFACES
                             Array2A<Real64> F,           // USER INPUT DIRECT VIEW FACTOR MATRIX (N X N)
                             Array1A_int const SPtr,      // pointer to actual surface number
                             bool &NoUserInputF,          // Flag signifying no input F's for this
                             bool &ErrorsFound            // True when errors are found in number of fields vs max args
    );

    void GetInputViewFactorsbyName(std::string const &ZoneName, // Needed to check for user input view factors.
                                   int const N,                 // NUMBER OF SURFACES
                                   Array2A<Real64> F,           // USER INPUT DIRECT VIEW FACTOR MATRIX (N X N)
                                   Array1A_int const SPtr,      // pointer to actual surface number
                                   bool &NoUserInputF,          // Flag signifying no input F's for this
                                   bool &ErrorsFound            // True when errors are found in number of fields vs max args
    );

    void CalcApproximateViewFactors(int const N,                   // NUMBER OF SURFACES
                                    Array1A<Real64> const A,       // AREA VECTOR- ASSUMED,BE N ELEMENTS LONG
                                    Array1A<Real64> const Azimuth, // Facing angle of the surface (in degrees)
                                    Array1A<Real64> const Tilt,    // Tilt angle of the surface (in degrees)
                                    Array2A<Real64> F,             // APPROXIMATE DIRECT VIEW FACTOR MATRIX (N X N)
                                    Array1A_int const SPtr         // pointer to REAL(r64) surface number (for error message)
    );

    void FixViewFactors(int const N,                // NUMBER OF SURFACES
                        Array1A<Real64> const A,    // AREA VECTOR- ASSUMED,BE N ELEMENTS LONG
                        Array2A<Real64> F,          // APPROXIMATE DIRECT VIEW FACTOR MATRIX (N X N)
                        int const ZoneNum,          // Zone number being fixe
                        Real64 &OriginalCheckValue, // check of SUM(F) - N
                        Real64 &FixedCheckValue,    // check after fixed of SUM(F) - N
                        Real64 &FinalCheckValue,    // the one to go with
                        int &NumIterations,         // number of iterations to fixed
                        Real64 &RowSum              // RowSum of Fixed
    );

    void CalcScriptF(int const N,             // Number of surfaces
                     Array1<Real64> const &A, // AREA VECTOR- ASSUMED,BE N ELEMENTS LONG
                     Array2<Real64> const &F, // DIRECT VIEW FACTOR MATRIX (N X N)
                     Array1<Real64> &EMISS,   // VECTOR OF SURFACE EMISSIVITIES
                     Array2<Real64> &ScriptF  // MATRIX OF SCRIPT F FACTORS (N X N) //Tuned Transposed
    );

    void CalcMatrixInverse(Array2<Real64> &A, // Matrix: Gets reduced to L\U form
                           Array2<Real64> &I  // Returned as inverse matrix
    );

} // namespace HeatBalanceIntRadExchange

} // namespace EnergyPlus

#endif
