#include <memory>
#include <gtest/gtest.h>

#include "WCESpectralAveraging.hpp"
#include "WCECommon.hpp"


using namespace SpectralAveraging;
using namespace FenestrationCommon;

class TestSampleNFRC_1042_PHOTOPIC : public testing::Test {

private:
	std::shared_ptr< CSpectralSample > m_Sample;

protected:
	std::shared_ptr< CSeries > getSolarRadiation() const {
		auto solarRadiation = std::make_shared< CSeries >();

		// Full CIE Illuminant D651 nm ssp table (used for PHOTOPIC properties)
		solarRadiation->addProperty( 0.3000, 0.0341 );
		solarRadiation->addProperty( 0.3010, 0.3601 );
		solarRadiation->addProperty( 0.3020, 0.6862 );
		solarRadiation->addProperty( 0.3030, 1.0122 );
		solarRadiation->addProperty( 0.3040, 1.3383 );
		solarRadiation->addProperty( 0.3050, 1.6643 );
		solarRadiation->addProperty( 0.3060, 1.9903 );
		solarRadiation->addProperty( 0.3070, 2.3164 );
		solarRadiation->addProperty( 0.3080, 2.6424 );
		solarRadiation->addProperty( 0.3090, 2.9685 );
		solarRadiation->addProperty( 0.3100, 3.2945 );
		solarRadiation->addProperty( 0.3110, 4.9887 );
		solarRadiation->addProperty( 0.3120, 6.6828 );
		solarRadiation->addProperty( 0.3130, 8.3770 );
		solarRadiation->addProperty( 0.3140, 10.0711 );
		solarRadiation->addProperty( 0.3150, 11.7652 );
		solarRadiation->addProperty( 0.3160, 13.4594 );
		solarRadiation->addProperty( 0.3170, 15.1535 );
		solarRadiation->addProperty( 0.3180, 16.8477 );
		solarRadiation->addProperty( 0.3190, 18.5418 );
		solarRadiation->addProperty( 0.3200, 20.2360 );
		solarRadiation->addProperty( 0.3210, 21.9177 );
		solarRadiation->addProperty( 0.3220, 23.5995 );
		solarRadiation->addProperty( 0.3230, 25.2812 );
		solarRadiation->addProperty( 0.3240, 26.9630 );
		solarRadiation->addProperty( 0.3250, 28.6447 );
		solarRadiation->addProperty( 0.3260, 30.3265 );
		solarRadiation->addProperty( 0.3270, 32.0082 );
		solarRadiation->addProperty( 0.3280, 33.6900 );
		solarRadiation->addProperty( 0.3290, 35.3717 );
		solarRadiation->addProperty( 0.3300, 37.0535 );
		solarRadiation->addProperty( 0.3310, 37.3430 );
		solarRadiation->addProperty( 0.3320, 37.6326 );
		solarRadiation->addProperty( 0.3330, 37.9221 );
		solarRadiation->addProperty( 0.3340, 38.2116 );
		solarRadiation->addProperty( 0.3350, 38.5011 );
		solarRadiation->addProperty( 0.3360, 38.7907 );
		solarRadiation->addProperty( 0.3370, 39.0802 );
		solarRadiation->addProperty( 0.3380, 39.3697 );
		solarRadiation->addProperty( 0.3390, 39.6593 );
		solarRadiation->addProperty( 0.3400, 39.9488 );
		solarRadiation->addProperty( 0.3410, 40.4451 );
		solarRadiation->addProperty( 0.3420, 40.9414 );
		solarRadiation->addProperty( 0.3430, 41.4377 );
		solarRadiation->addProperty( 0.3440, 41.9340 );
		solarRadiation->addProperty( 0.3450, 42.4302 );
		solarRadiation->addProperty( 0.3460, 42.9265 );
		solarRadiation->addProperty( 0.3470, 43.4228 );
		solarRadiation->addProperty( 0.3480, 43.9191 );
		solarRadiation->addProperty( 0.3490, 44.4154 );
		solarRadiation->addProperty( 0.3500, 44.9117 );
		solarRadiation->addProperty( 0.3510, 45.0844 );
		solarRadiation->addProperty( 0.3520, 45.2570 );
		solarRadiation->addProperty( 0.3530, 45.4297 );
		solarRadiation->addProperty( 0.3540, 45.6023 );
		solarRadiation->addProperty( 0.3550, 45.7750 );
		solarRadiation->addProperty( 0.3560, 45.9477 );
		solarRadiation->addProperty( 0.3570, 46.1203 );
		solarRadiation->addProperty( 0.3580, 46.2930 );
		solarRadiation->addProperty( 0.3590, 46.4656 );
		solarRadiation->addProperty( 0.3600, 46.6383 );
		solarRadiation->addProperty( 0.3610, 47.1834 );
		solarRadiation->addProperty( 0.3620, 47.7285 );
		solarRadiation->addProperty( 0.3630, 48.2735 );
		solarRadiation->addProperty( 0.3640, 48.8186 );
		solarRadiation->addProperty( 0.3650, 49.3637 );
		solarRadiation->addProperty( 0.3660, 49.9088 );
		solarRadiation->addProperty( 0.3670, 50.4539 );
		solarRadiation->addProperty( 0.3680, 50.9989 );
		solarRadiation->addProperty( 0.3690, 51.5440 );
		solarRadiation->addProperty( 0.3700, 52.0891 );
		solarRadiation->addProperty( 0.3710, 51.8777 );
		solarRadiation->addProperty( 0.3720, 51.6664 );
		solarRadiation->addProperty( 0.3730, 51.4550 );
		solarRadiation->addProperty( 0.3740, 51.2437 );
		solarRadiation->addProperty( 0.3750, 51.0323 );
		solarRadiation->addProperty( 0.3760, 50.8209 );
		solarRadiation->addProperty( 0.3770, 50.6096 );
		solarRadiation->addProperty( 0.3780, 50.3982 );
		solarRadiation->addProperty( 0.3790, 50.1869 );
		solarRadiation->addProperty( 0.3800, 49.9755 );
		solarRadiation->addProperty( 0.3810, 50.4428 );
		solarRadiation->addProperty( 0.3820, 50.9100 );
		solarRadiation->addProperty( 0.3830, 51.3773 );
		solarRadiation->addProperty( 0.3840, 51.8446 );
		solarRadiation->addProperty( 0.3850, 52.3118 );
		solarRadiation->addProperty( 0.3860, 52.7791 );
		solarRadiation->addProperty( 0.3870, 53.2464 );
		solarRadiation->addProperty( 0.3880, 53.7137 );
		solarRadiation->addProperty( 0.3890, 54.1809 );
		solarRadiation->addProperty( 0.3900, 54.6482 );
		solarRadiation->addProperty( 0.3910, 57.4589 );
		solarRadiation->addProperty( 0.3920, 60.2695 );
		solarRadiation->addProperty( 0.3930, 63.0802 );
		solarRadiation->addProperty( 0.3940, 65.8909 );
		solarRadiation->addProperty( 0.3950, 68.7015 );
		solarRadiation->addProperty( 0.3960, 71.5122 );
		solarRadiation->addProperty( 0.3970, 74.3229 );
		solarRadiation->addProperty( 0.3980, 77.1336 );
		solarRadiation->addProperty( 0.3990, 79.9442 );
		solarRadiation->addProperty( 0.4000, 82.7549 );
		solarRadiation->addProperty( 0.4010, 83.6280 );
		solarRadiation->addProperty( 0.4020, 84.5011 );
		solarRadiation->addProperty( 0.4030, 85.3742 );
		solarRadiation->addProperty( 0.4040, 86.2473 );
		solarRadiation->addProperty( 0.4050, 87.1204 );
		solarRadiation->addProperty( 0.4060, 87.9936 );
		solarRadiation->addProperty( 0.4070, 88.8667 );
		solarRadiation->addProperty( 0.4080, 89.7398 );
		solarRadiation->addProperty( 0.4090, 90.6129 );
		solarRadiation->addProperty( 0.4100, 91.4860 );
		solarRadiation->addProperty( 0.4110, 91.6806 );
		solarRadiation->addProperty( 0.4120, 91.8752 );
		solarRadiation->addProperty( 0.4130, 92.0697 );
		solarRadiation->addProperty( 0.4140, 92.2643 );
		solarRadiation->addProperty( 0.4150, 92.4589 );
		solarRadiation->addProperty( 0.4160, 92.6535 );
		solarRadiation->addProperty( 0.4170, 92.8481 );
		solarRadiation->addProperty( 0.4180, 93.0426 );
		solarRadiation->addProperty( 0.4190, 93.2372 );
		solarRadiation->addProperty( 0.4200, 93.4318 );
		solarRadiation->addProperty( 0.4210, 92.7568 );
		solarRadiation->addProperty( 0.4220, 92.0819 );
		solarRadiation->addProperty( 0.4230, 91.4069 );
		solarRadiation->addProperty( 0.4240, 90.7320 );
		solarRadiation->addProperty( 0.4250, 90.0570 );
		solarRadiation->addProperty( 0.4260, 89.3821 );
		solarRadiation->addProperty( 0.4270, 88.7071 );
		solarRadiation->addProperty( 0.4280, 88.0322 );
		solarRadiation->addProperty( 0.4290, 87.3572 );
		solarRadiation->addProperty( 0.4300, 86.6823 );
		solarRadiation->addProperty( 0.4310, 88.5006 );
		solarRadiation->addProperty( 0.4320, 90.3188 );
		solarRadiation->addProperty( 0.4330, 92.1371 );
		solarRadiation->addProperty( 0.4340, 93.9554 );
		solarRadiation->addProperty( 0.4350, 95.7736 );
		solarRadiation->addProperty( 0.4360, 97.5919 );
		solarRadiation->addProperty( 0.4370, 99.4102 );
		solarRadiation->addProperty( 0.4380, 101.2280 );
		solarRadiation->addProperty( 0.4390, 103.0470 );
		solarRadiation->addProperty( 0.4400, 104.8650 );
		solarRadiation->addProperty( 0.4410, 106.0790 );
		solarRadiation->addProperty( 0.4420, 107.2940 );
		solarRadiation->addProperty( 0.4430, 108.5080 );
		solarRadiation->addProperty( 0.4440, 109.7220 );
		solarRadiation->addProperty( 0.4450, 110.9360 );
		solarRadiation->addProperty( 0.4460, 112.1510 );
		solarRadiation->addProperty( 0.4470, 113.3650 );
		solarRadiation->addProperty( 0.4480, 114.5790 );
		solarRadiation->addProperty( 0.4490, 115.7940 );
		solarRadiation->addProperty( 0.4500, 117.0080 );
		solarRadiation->addProperty( 0.4510, 117.0880 );
		solarRadiation->addProperty( 0.4520, 117.1690 );
		solarRadiation->addProperty( 0.4530, 117.2490 );
		solarRadiation->addProperty( 0.4540, 117.3300 );
		solarRadiation->addProperty( 0.4550, 117.4100 );
		solarRadiation->addProperty( 0.4560, 117.4900 );
		solarRadiation->addProperty( 0.4570, 117.5710 );
		solarRadiation->addProperty( 0.4580, 117.6510 );
		solarRadiation->addProperty( 0.4590, 117.7320 );
		solarRadiation->addProperty( 0.4600, 117.8120 );
		solarRadiation->addProperty( 0.4610, 117.5170 );
		solarRadiation->addProperty( 0.4620, 117.2220 );
		solarRadiation->addProperty( 0.4630, 116.9270 );
		solarRadiation->addProperty( 0.4640, 116.6320 );
		solarRadiation->addProperty( 0.4650, 116.3360 );
		solarRadiation->addProperty( 0.4660, 116.0410 );
		solarRadiation->addProperty( 0.4670, 115.7460 );
		solarRadiation->addProperty( 0.4680, 115.4510 );
		solarRadiation->addProperty( 0.4690, 115.1560 );
		solarRadiation->addProperty( 0.4700, 114.8610 );
		solarRadiation->addProperty( 0.4710, 114.9670 );
		solarRadiation->addProperty( 0.4720, 115.0730 );
		solarRadiation->addProperty( 0.4730, 115.1800 );
		solarRadiation->addProperty( 0.4740, 115.2860 );
		solarRadiation->addProperty( 0.4750, 115.3920 );
		solarRadiation->addProperty( 0.4760, 115.4980 );
		solarRadiation->addProperty( 0.4770, 115.6040 );
		solarRadiation->addProperty( 0.4780, 115.7110 );
		solarRadiation->addProperty( 0.4790, 115.8170 );
		solarRadiation->addProperty( 0.4800, 115.9230 );
		solarRadiation->addProperty( 0.4810, 115.2120 );
		solarRadiation->addProperty( 0.4820, 114.5010 );
		solarRadiation->addProperty( 0.4830, 113.7890 );
		solarRadiation->addProperty( 0.4840, 113.0780 );
		solarRadiation->addProperty( 0.4850, 112.3670 );
		solarRadiation->addProperty( 0.4860, 111.6560 );
		solarRadiation->addProperty( 0.4870, 110.9450 );
		solarRadiation->addProperty( 0.4880, 110.2330 );
		solarRadiation->addProperty( 0.4890, 109.5220 );
		solarRadiation->addProperty( 0.4900, 108.8110 );
		solarRadiation->addProperty( 0.4910, 108.8650 );
		solarRadiation->addProperty( 0.4920, 108.9200 );
		solarRadiation->addProperty( 0.4930, 108.9740 );
		solarRadiation->addProperty( 0.4940, 109.0280 );
		solarRadiation->addProperty( 0.4950, 109.0820 );
		solarRadiation->addProperty( 0.4960, 109.1370 );
		solarRadiation->addProperty( 0.4970, 109.1910 );
		solarRadiation->addProperty( 0.4980, 109.2450 );
		solarRadiation->addProperty( 0.4990, 109.3000 );
		solarRadiation->addProperty( 0.5000, 109.3540 );
		solarRadiation->addProperty( 0.5010, 109.1990 );
		solarRadiation->addProperty( 0.5020, 109.0440 );
		solarRadiation->addProperty( 0.5030, 108.8880 );
		solarRadiation->addProperty( 0.5040, 108.7330 );
		solarRadiation->addProperty( 0.5050, 108.5780 );
		solarRadiation->addProperty( 0.5060, 108.4230 );
		solarRadiation->addProperty( 0.5070, 108.2680 );
		solarRadiation->addProperty( 0.5080, 108.1120 );
		solarRadiation->addProperty( 0.5090, 107.9570 );
		solarRadiation->addProperty( 0.5100, 107.8020 );
		solarRadiation->addProperty( 0.5110, 107.5010 );
		solarRadiation->addProperty( 0.5120, 107.2000 );
		solarRadiation->addProperty( 0.5130, 106.8980 );
		solarRadiation->addProperty( 0.5140, 106.5970 );
		solarRadiation->addProperty( 0.5150, 106.2960 );
		solarRadiation->addProperty( 0.5160, 105.9950 );
		solarRadiation->addProperty( 0.5170, 105.6940 );
		solarRadiation->addProperty( 0.5180, 105.3920 );
		solarRadiation->addProperty( 0.5190, 105.0910 );
		solarRadiation->addProperty( 0.5200, 104.7900 );
		solarRadiation->addProperty( 0.5210, 105.0800 );
		solarRadiation->addProperty( 0.5220, 105.3700 );
		solarRadiation->addProperty( 0.5230, 105.6600 );
		solarRadiation->addProperty( 0.5240, 105.9500 );
		solarRadiation->addProperty( 0.5250, 106.2390 );
		solarRadiation->addProperty( 0.5260, 106.5290 );
		solarRadiation->addProperty( 0.5270, 106.8190 );
		solarRadiation->addProperty( 0.5280, 107.1090 );
		solarRadiation->addProperty( 0.5290, 107.3990 );
		solarRadiation->addProperty( 0.5300, 107.6890 );
		solarRadiation->addProperty( 0.5310, 107.3610 );
		solarRadiation->addProperty( 0.5320, 107.0320 );
		solarRadiation->addProperty( 0.5330, 106.7040 );
		solarRadiation->addProperty( 0.5340, 106.3750 );
		solarRadiation->addProperty( 0.5350, 106.0470 );
		solarRadiation->addProperty( 0.5360, 105.7190 );
		solarRadiation->addProperty( 0.5370, 105.3900 );
		solarRadiation->addProperty( 0.5380, 105.0620 );
		solarRadiation->addProperty( 0.5390, 104.7330 );
		solarRadiation->addProperty( 0.5400, 104.4050 );
		solarRadiation->addProperty( 0.5410, 104.3690 );
		solarRadiation->addProperty( 0.5420, 104.3330 );
		solarRadiation->addProperty( 0.5430, 104.2970 );
		solarRadiation->addProperty( 0.5440, 104.2610 );
		solarRadiation->addProperty( 0.5450, 104.2250 );
		solarRadiation->addProperty( 0.5460, 104.1900 );
		solarRadiation->addProperty( 0.5470, 104.1540 );
		solarRadiation->addProperty( 0.5480, 104.1180 );
		solarRadiation->addProperty( 0.5490, 104.0820 );
		solarRadiation->addProperty( 0.5500, 104.0460 );
		solarRadiation->addProperty( 0.5510, 103.6410 );
		solarRadiation->addProperty( 0.5520, 103.2370 );
		solarRadiation->addProperty( 0.5530, 102.8320 );
		solarRadiation->addProperty( 0.5540, 102.4280 );
		solarRadiation->addProperty( 0.5550, 102.0230 );
		solarRadiation->addProperty( 0.5560, 101.6180 );
		solarRadiation->addProperty( 0.5570, 101.2140 );
		solarRadiation->addProperty( 0.5580, 100.8090 );
		solarRadiation->addProperty( 0.5590, 100.4050 );
		solarRadiation->addProperty( 0.5600, 100.0000 );
		solarRadiation->addProperty( 0.5610, 99.6334 );
		solarRadiation->addProperty( 0.5620, 99.2668 );
		solarRadiation->addProperty( 0.5630, 98.9003 );
		solarRadiation->addProperty( 0.5640, 98.5337 );
		solarRadiation->addProperty( 0.5650, 98.1671 );
		solarRadiation->addProperty( 0.5660, 97.8005 );
		solarRadiation->addProperty( 0.5670, 97.4339 );
		solarRadiation->addProperty( 0.5680, 97.0674 );
		solarRadiation->addProperty( 0.5690, 96.7008 );
		solarRadiation->addProperty( 0.5700, 96.3342 );
		solarRadiation->addProperty( 0.5710, 96.2796 );
		solarRadiation->addProperty( 0.5720, 96.2250 );
		solarRadiation->addProperty( 0.5730, 96.1703 );
		solarRadiation->addProperty( 0.5740, 96.1157 );
		solarRadiation->addProperty( 0.5750, 96.0611 );
		solarRadiation->addProperty( 0.5760, 96.0065 );
		solarRadiation->addProperty( 0.5770, 95.9519 );
		solarRadiation->addProperty( 0.5780, 95.8972 );
		solarRadiation->addProperty( 0.5790, 95.8426 );
		solarRadiation->addProperty( 0.5800, 95.7880 );
		solarRadiation->addProperty( 0.5810, 95.0778 );
		solarRadiation->addProperty( 0.5820, 94.3675 );
		solarRadiation->addProperty( 0.5830, 93.6573 );
		solarRadiation->addProperty( 0.5840, 92.9470 );
		solarRadiation->addProperty( 0.5850, 92.2368 );
		solarRadiation->addProperty( 0.5860, 91.5266 );
		solarRadiation->addProperty( 0.5870, 90.8163 );
		solarRadiation->addProperty( 0.5880, 90.1061 );
		solarRadiation->addProperty( 0.5890, 89.3958 );
		solarRadiation->addProperty( 0.5900, 88.6856 );
		solarRadiation->addProperty( 0.5910, 88.8177 );
		solarRadiation->addProperty( 0.5920, 88.9497 );
		solarRadiation->addProperty( 0.5930, 89.0818 );
		solarRadiation->addProperty( 0.5940, 89.2138 );
		solarRadiation->addProperty( 0.5950, 89.3459 );
		solarRadiation->addProperty( 0.5960, 89.4780 );
		solarRadiation->addProperty( 0.5970, 89.6100 );
		solarRadiation->addProperty( 0.5980, 89.7421 );
		solarRadiation->addProperty( 0.5990, 89.8741 );
		solarRadiation->addProperty( 0.6000, 90.0062 );
		solarRadiation->addProperty( 0.6010, 89.9655 );
		solarRadiation->addProperty( 0.6020, 89.9248 );
		solarRadiation->addProperty( 0.6030, 89.8841 );
		solarRadiation->addProperty( 0.6040, 89.8434 );
		solarRadiation->addProperty( 0.6050, 89.8026 );
		solarRadiation->addProperty( 0.6060, 89.7619 );
		solarRadiation->addProperty( 0.6070, 89.7212 );
		solarRadiation->addProperty( 0.6080, 89.6805 );
		solarRadiation->addProperty( 0.6090, 89.6398 );
		solarRadiation->addProperty( 0.6100, 89.5991 );
		solarRadiation->addProperty( 0.6110, 89.4091 );
		solarRadiation->addProperty( 0.6120, 89.2190 );
		solarRadiation->addProperty( 0.6130, 89.0290 );
		solarRadiation->addProperty( 0.6140, 88.8389 );
		solarRadiation->addProperty( 0.6150, 88.6489 );
		solarRadiation->addProperty( 0.6160, 88.4589 );
		solarRadiation->addProperty( 0.6170, 88.2688 );
		solarRadiation->addProperty( 0.6180, 88.0788 );
		solarRadiation->addProperty( 0.6190, 87.8887 );
		solarRadiation->addProperty( 0.6200, 87.6987 );
		solarRadiation->addProperty( 0.6210, 87.2577 );
		solarRadiation->addProperty( 0.6220, 86.8167 );
		solarRadiation->addProperty( 0.6230, 86.3757 );
		solarRadiation->addProperty( 0.6240, 85.9347 );
		solarRadiation->addProperty( 0.6250, 85.4936 );
		solarRadiation->addProperty( 0.6260, 85.0526 );
		solarRadiation->addProperty( 0.6270, 84.6116 );
		solarRadiation->addProperty( 0.6280, 84.1706 );
		solarRadiation->addProperty( 0.6290, 83.7296 );
		solarRadiation->addProperty( 0.6300, 83.2886 );
		solarRadiation->addProperty( 0.6310, 83.3297 );
		solarRadiation->addProperty( 0.6320, 83.3707 );
		solarRadiation->addProperty( 0.6330, 83.4118 );
		solarRadiation->addProperty( 0.6340, 83.4528 );
		solarRadiation->addProperty( 0.6350, 83.4939 );
		solarRadiation->addProperty( 0.6360, 83.5350 );
		solarRadiation->addProperty( 0.6370, 83.5760 );
		solarRadiation->addProperty( 0.6380, 83.6171 );
		solarRadiation->addProperty( 0.6390, 83.6581 );
		solarRadiation->addProperty( 0.6400, 83.6992 );
		solarRadiation->addProperty( 0.6410, 83.3320 );
		solarRadiation->addProperty( 0.6420, 82.9647 );
		solarRadiation->addProperty( 0.6430, 82.5975 );
		solarRadiation->addProperty( 0.6440, 82.2302 );
		solarRadiation->addProperty( 0.6450, 81.8630 );
		solarRadiation->addProperty( 0.6460, 81.4958 );
		solarRadiation->addProperty( 0.6470, 81.1285 );
		solarRadiation->addProperty( 0.6480, 80.7613 );
		solarRadiation->addProperty( 0.6490, 80.3940 );
		solarRadiation->addProperty( 0.6500, 80.0268 );
		solarRadiation->addProperty( 0.6510, 80.0456 );
		solarRadiation->addProperty( 0.6520, 80.0644 );
		solarRadiation->addProperty( 0.6530, 80.0831 );
		solarRadiation->addProperty( 0.6540, 80.1019 );
		solarRadiation->addProperty( 0.6550, 80.1207 );
		solarRadiation->addProperty( 0.6560, 80.1395 );
		solarRadiation->addProperty( 0.6570, 80.1583 );
		solarRadiation->addProperty( 0.6580, 80.1770 );
		solarRadiation->addProperty( 0.6590, 80.1958 );
		solarRadiation->addProperty( 0.6600, 80.2146 );
		solarRadiation->addProperty( 0.6610, 80.4209 );
		solarRadiation->addProperty( 0.6620, 80.6272 );
		solarRadiation->addProperty( 0.6630, 80.8336 );
		solarRadiation->addProperty( 0.6640, 81.0399 );
		solarRadiation->addProperty( 0.6650, 81.2462 );
		solarRadiation->addProperty( 0.6660, 81.4525 );
		solarRadiation->addProperty( 0.6670, 81.6588 );
		solarRadiation->addProperty( 0.6680, 81.8652 );
		solarRadiation->addProperty( 0.6690, 82.0715 );
		solarRadiation->addProperty( 0.6700, 82.2778 );
		solarRadiation->addProperty( 0.6710, 81.8784 );
		solarRadiation->addProperty( 0.6720, 81.4791 );
		solarRadiation->addProperty( 0.6730, 81.0797 );
		solarRadiation->addProperty( 0.6740, 80.6804 );
		solarRadiation->addProperty( 0.6750, 80.2810 );
		solarRadiation->addProperty( 0.6760, 79.8816 );
		solarRadiation->addProperty( 0.6770, 79.4823 );
		solarRadiation->addProperty( 0.6780, 79.0829 );
		solarRadiation->addProperty( 0.6790, 78.6836 );
		solarRadiation->addProperty( 0.6800, 78.2842 );
		solarRadiation->addProperty( 0.6810, 77.4279 );
		solarRadiation->addProperty( 0.6820, 76.5716 );
		solarRadiation->addProperty( 0.6830, 75.7153 );
		solarRadiation->addProperty( 0.6840, 74.8590 );
		solarRadiation->addProperty( 0.6850, 74.0027 );
		solarRadiation->addProperty( 0.6860, 73.1465 );
		solarRadiation->addProperty( 0.6870, 72.2902 );
		solarRadiation->addProperty( 0.6880, 71.4339 );
		solarRadiation->addProperty( 0.6890, 70.5776 );
		solarRadiation->addProperty( 0.6900, 69.7213 );
		solarRadiation->addProperty( 0.6910, 69.9101 );
		solarRadiation->addProperty( 0.6920, 70.0989 );
		solarRadiation->addProperty( 0.6930, 70.2876 );
		solarRadiation->addProperty( 0.6940, 70.4764 );
		solarRadiation->addProperty( 0.6950, 70.6652 );
		solarRadiation->addProperty( 0.6960, 70.8540 );
		solarRadiation->addProperty( 0.6970, 71.0428 );
		solarRadiation->addProperty( 0.6980, 71.2315 );
		solarRadiation->addProperty( 0.6990, 71.4203 );
		solarRadiation->addProperty( 0.7000, 71.6091 );
		solarRadiation->addProperty( 0.7010, 71.8831 );
		solarRadiation->addProperty( 0.7020, 72.1571 );
		solarRadiation->addProperty( 0.7030, 72.4311 );
		solarRadiation->addProperty( 0.7040, 72.7051 );
		solarRadiation->addProperty( 0.7050, 72.9790 );
		solarRadiation->addProperty( 0.7060, 73.2530 );
		solarRadiation->addProperty( 0.7070, 73.5270 );
		solarRadiation->addProperty( 0.7080, 73.8010 );
		solarRadiation->addProperty( 0.7090, 74.0750 );
		solarRadiation->addProperty( 0.7100, 74.3490 );
		solarRadiation->addProperty( 0.7110, 73.0745 );
		solarRadiation->addProperty( 0.7120, 71.8000 );
		solarRadiation->addProperty( 0.7130, 70.5255 );
		solarRadiation->addProperty( 0.7140, 69.2510 );
		solarRadiation->addProperty( 0.7150, 67.9765 );
		solarRadiation->addProperty( 0.7160, 66.7020 );
		solarRadiation->addProperty( 0.7170, 65.4275 );
		solarRadiation->addProperty( 0.7180, 64.1530 );
		solarRadiation->addProperty( 0.7190, 62.8785 );
		solarRadiation->addProperty( 0.7200, 61.6040 );
		solarRadiation->addProperty( 0.7210, 62.4322 );
		solarRadiation->addProperty( 0.7220, 63.2603 );
		solarRadiation->addProperty( 0.7230, 64.0885 );
		solarRadiation->addProperty( 0.7240, 64.9166 );
		solarRadiation->addProperty( 0.7250, 65.7448 );
		solarRadiation->addProperty( 0.7260, 66.5730 );
		solarRadiation->addProperty( 0.7270, 67.4011 );
		solarRadiation->addProperty( 0.7280, 68.2293 );
		solarRadiation->addProperty( 0.7290, 69.0574 );
		solarRadiation->addProperty( 0.7300, 69.8856 );
		solarRadiation->addProperty( 0.7310, 70.4057 );
		solarRadiation->addProperty( 0.7320, 70.9259 );
		solarRadiation->addProperty( 0.7330, 71.4460 );
		solarRadiation->addProperty( 0.7340, 71.9662 );
		solarRadiation->addProperty( 0.7350, 72.4863 );
		solarRadiation->addProperty( 0.7360, 73.0064 );
		solarRadiation->addProperty( 0.7370, 73.5266 );
		solarRadiation->addProperty( 0.7380, 74.0467 );
		solarRadiation->addProperty( 0.7390, 74.5669 );
		solarRadiation->addProperty( 0.7400, 75.0870 );
		solarRadiation->addProperty( 0.7410, 73.9376 );
		solarRadiation->addProperty( 0.7420, 72.7881 );
		solarRadiation->addProperty( 0.7430, 71.6387 );
		solarRadiation->addProperty( 0.7440, 70.4893 );
		solarRadiation->addProperty( 0.7450, 69.3398 );
		solarRadiation->addProperty( 0.7460, 68.1904 );
		solarRadiation->addProperty( 0.7470, 67.0410 );
		solarRadiation->addProperty( 0.7480, 65.8916 );
		solarRadiation->addProperty( 0.7490, 64.7421 );
		solarRadiation->addProperty( 0.7500, 63.5927 );
		solarRadiation->addProperty( 0.7510, 61.8752 );
		solarRadiation->addProperty( 0.7520, 60.1578 );
		solarRadiation->addProperty( 0.7530, 58.4403 );
		solarRadiation->addProperty( 0.7540, 56.7229 );
		solarRadiation->addProperty( 0.7550, 55.0054 );
		solarRadiation->addProperty( 0.7560, 53.2880 );
		solarRadiation->addProperty( 0.7570, 51.5705 );
		solarRadiation->addProperty( 0.7580, 49.8531 );
		solarRadiation->addProperty( 0.7590, 48.1356 );
		solarRadiation->addProperty( 0.7600, 46.4182 );
		solarRadiation->addProperty( 0.7610, 48.4569 );
		solarRadiation->addProperty( 0.7620, 50.4956 );
		solarRadiation->addProperty( 0.7630, 52.5344 );
		solarRadiation->addProperty( 0.7640, 54.5731 );
		solarRadiation->addProperty( 0.7650, 56.6118 );
		solarRadiation->addProperty( 0.7660, 58.6505 );
		solarRadiation->addProperty( 0.7670, 60.6892 );
		solarRadiation->addProperty( 0.7680, 62.7280 );
		solarRadiation->addProperty( 0.7690, 64.7667 );
		solarRadiation->addProperty( 0.7700, 66.8054 );
		solarRadiation->addProperty( 0.7710, 66.4631 );
		solarRadiation->addProperty( 0.7720, 66.1209 );
		solarRadiation->addProperty( 0.7730, 65.7786 );
		solarRadiation->addProperty( 0.7740, 65.4364 );
		solarRadiation->addProperty( 0.7750, 65.0941 );
		solarRadiation->addProperty( 0.7760, 64.7518 );
		solarRadiation->addProperty( 0.7770, 64.4096 );
		solarRadiation->addProperty( 0.7780, 64.0673 );
		solarRadiation->addProperty( 0.7790, 63.7251 );
		solarRadiation->addProperty( 0.7800, 63.3828 );
		solarRadiation->addProperty( 0.7810, 63.4749 );
		solarRadiation->addProperty( 0.7820, 63.5670 );
		solarRadiation->addProperty( 0.7830, 63.6592 );
		solarRadiation->addProperty( 0.7840, 63.7513 );
		solarRadiation->addProperty( 0.7850, 63.8434 );
		solarRadiation->addProperty( 0.7860, 63.9355 );
		solarRadiation->addProperty( 0.7870, 64.0276 );
		solarRadiation->addProperty( 0.7880, 64.1198 );
		solarRadiation->addProperty( 0.7890, 64.2119 );
		solarRadiation->addProperty( 0.7900, 64.3040 );
		solarRadiation->addProperty( 0.7910, 63.8188 );
		solarRadiation->addProperty( 0.7920, 63.3336 );
		solarRadiation->addProperty( 0.7930, 62.8484 );
		solarRadiation->addProperty( 0.7940, 62.3632 );
		solarRadiation->addProperty( 0.7950, 61.8779 );
		solarRadiation->addProperty( 0.7960, 61.3927 );
		solarRadiation->addProperty( 0.7970, 60.9075 );
		solarRadiation->addProperty( 0.7980, 60.4223 );
		solarRadiation->addProperty( 0.7990, 59.9371 );
		solarRadiation->addProperty( 0.8000, 59.4519 );
		solarRadiation->addProperty( 0.8010, 58.7026 );
		solarRadiation->addProperty( 0.8020, 57.9533 );
		solarRadiation->addProperty( 0.8030, 57.2040 );
		solarRadiation->addProperty( 0.8040, 56.4547 );
		solarRadiation->addProperty( 0.8050, 55.7054 );
		solarRadiation->addProperty( 0.8060, 54.9562 );
		solarRadiation->addProperty( 0.8070, 54.2069 );
		solarRadiation->addProperty( 0.8080, 53.4576 );
		solarRadiation->addProperty( 0.8090, 52.7083 );
		solarRadiation->addProperty( 0.8100, 51.9590 );
		solarRadiation->addProperty( 0.8110, 52.5072 );
		solarRadiation->addProperty( 0.8120, 53.0553 );
		solarRadiation->addProperty( 0.8130, 53.6035 );
		solarRadiation->addProperty( 0.8140, 54.1516 );
		solarRadiation->addProperty( 0.8150, 54.6998 );
		solarRadiation->addProperty( 0.8160, 55.2480 );
		solarRadiation->addProperty( 0.8170, 55.7961 );
		solarRadiation->addProperty( 0.8180, 56.3443 );
		solarRadiation->addProperty( 0.8190, 56.8924 );
		solarRadiation->addProperty( 0.8200, 57.4406 );
		solarRadiation->addProperty( 0.8210, 57.7278 );
		solarRadiation->addProperty( 0.8220, 58.0150 );
		solarRadiation->addProperty( 0.8230, 58.3022 );
		solarRadiation->addProperty( 0.8240, 58.5894 );
		solarRadiation->addProperty( 0.8250, 58.8765 );
		solarRadiation->addProperty( 0.8260, 59.1637 );
		solarRadiation->addProperty( 0.8270, 59.4509 );
		solarRadiation->addProperty( 0.8280, 59.7381 );
		solarRadiation->addProperty( 0.8290, 60.0253 );
		solarRadiation->addProperty( 0.8300, 60.3125 );

		return solarRadiation;
	}

	std::shared_ptr< std::vector< double > > getWavelengths() const {
		auto aWavelengths = std::make_shared< std::vector< double > >();

		aWavelengths->push_back( 0.380 );
		aWavelengths->push_back( 0.385 );
		aWavelengths->push_back( 0.390 );
		aWavelengths->push_back( 0.395 );
		aWavelengths->push_back( 0.400 );
		aWavelengths->push_back( 0.405 );
		aWavelengths->push_back( 0.410 );
		aWavelengths->push_back( 0.415 );
		aWavelengths->push_back( 0.420 );
		aWavelengths->push_back( 0.425 );
		aWavelengths->push_back( 0.430 );
		aWavelengths->push_back( 0.435 );
		aWavelengths->push_back( 0.440 );
		aWavelengths->push_back( 0.445 );
		aWavelengths->push_back( 0.450 );
		aWavelengths->push_back( 0.455 );
		aWavelengths->push_back( 0.460 );
		aWavelengths->push_back( 0.465 );
		aWavelengths->push_back( 0.470 );
		aWavelengths->push_back( 0.475 );
		aWavelengths->push_back( 0.480 );
		aWavelengths->push_back( 0.485 );
		aWavelengths->push_back( 0.490 );
		aWavelengths->push_back( 0.495 );
		aWavelengths->push_back( 0.500 );
		aWavelengths->push_back( 0.505 );
		aWavelengths->push_back( 0.510 );
		aWavelengths->push_back( 0.515 );
		aWavelengths->push_back( 0.520 );
		aWavelengths->push_back( 0.525 );
		aWavelengths->push_back( 0.530 );
		aWavelengths->push_back( 0.535 );
		aWavelengths->push_back( 0.540 );
		aWavelengths->push_back( 0.545 );
		aWavelengths->push_back( 0.550 );
		aWavelengths->push_back( 0.555 );
		aWavelengths->push_back( 0.560 );
		aWavelengths->push_back( 0.565 );
		aWavelengths->push_back( 0.570 );
		aWavelengths->push_back( 0.575 );
		aWavelengths->push_back( 0.580 );
		aWavelengths->push_back( 0.585 );
		aWavelengths->push_back( 0.590 );
		aWavelengths->push_back( 0.595 );
		aWavelengths->push_back( 0.600 );
		aWavelengths->push_back( 0.605 );
		aWavelengths->push_back( 0.610 );
		aWavelengths->push_back( 0.615 );
		aWavelengths->push_back( 0.620 );
		aWavelengths->push_back( 0.625 );
		aWavelengths->push_back( 0.630 );
		aWavelengths->push_back( 0.635 );
		aWavelengths->push_back( 0.640 );
		aWavelengths->push_back( 0.645 );
		aWavelengths->push_back( 0.650 );
		aWavelengths->push_back( 0.655 );
		aWavelengths->push_back( 0.660 );
		aWavelengths->push_back( 0.665 );
		aWavelengths->push_back( 0.670 );
		aWavelengths->push_back( 0.675 );
		aWavelengths->push_back( 0.680 );
		aWavelengths->push_back( 0.685 );
		aWavelengths->push_back( 0.690 );
		aWavelengths->push_back( 0.695 );
		aWavelengths->push_back( 0.700 );
		aWavelengths->push_back( 0.705 );
		aWavelengths->push_back( 0.710 );
		aWavelengths->push_back( 0.715 );
		aWavelengths->push_back( 0.720 );
		aWavelengths->push_back( 0.725 );
		aWavelengths->push_back( 0.730 );
		aWavelengths->push_back( 0.735 );
		aWavelengths->push_back( 0.740 );
		aWavelengths->push_back( 0.745 );
		aWavelengths->push_back( 0.750 );
		aWavelengths->push_back( 0.755 );
		aWavelengths->push_back( 0.760 );
		aWavelengths->push_back( 0.765 );
		aWavelengths->push_back( 0.770 );
		aWavelengths->push_back( 0.775 );
		aWavelengths->push_back( 0.780 );

		return aWavelengths;
	}

	std::shared_ptr< CSeries > getDetectorData() const {
		auto detectorData = std::make_shared< CSeries >();

		detectorData->addProperty( 0.380, 0.0000 );
		detectorData->addProperty( 0.385, 0.0001 );
		detectorData->addProperty( 0.390, 0.0001 );
		detectorData->addProperty( 0.395, 0.0002 );
		detectorData->addProperty( 0.400, 0.0004 );
		detectorData->addProperty( 0.405, 0.0006 );
		detectorData->addProperty( 0.410, 0.0012 );
		detectorData->addProperty( 0.415, 0.0022 );
		detectorData->addProperty( 0.420, 0.0040 );
		detectorData->addProperty( 0.425, 0.0073 );
		detectorData->addProperty( 0.430, 0.0116 );
		detectorData->addProperty( 0.435, 0.0168 );
		detectorData->addProperty( 0.440, 0.0230 );
		detectorData->addProperty( 0.445, 0.0298 );
		detectorData->addProperty( 0.450, 0.0380 );
		detectorData->addProperty( 0.455, 0.0480 );
		detectorData->addProperty( 0.460, 0.0600 );
		detectorData->addProperty( 0.465, 0.0739 );
		detectorData->addProperty( 0.470, 0.0910 );
		detectorData->addProperty( 0.475, 0.1126 );
		detectorData->addProperty( 0.480, 0.1390 );
		detectorData->addProperty( 0.485, 0.1693 );
		detectorData->addProperty( 0.490, 0.2080 );
		detectorData->addProperty( 0.495, 0.2586 );
		detectorData->addProperty( 0.500, 0.3230 );
		detectorData->addProperty( 0.505, 0.4073 );
		detectorData->addProperty( 0.510, 0.5030 );
		detectorData->addProperty( 0.515, 0.6082 );
		detectorData->addProperty( 0.520, 0.7100 );
		detectorData->addProperty( 0.525, 0.7932 );
		detectorData->addProperty( 0.530, 0.8620 );
		detectorData->addProperty( 0.535, 0.9149 );
		detectorData->addProperty( 0.540, 0.9540 );
		detectorData->addProperty( 0.545, 0.9803 );
		detectorData->addProperty( 0.550, 0.9950 );
		detectorData->addProperty( 0.555, 1.0000 );
		detectorData->addProperty( 0.560, 0.9950 );
		detectorData->addProperty( 0.565, 0.9786 );
		detectorData->addProperty( 0.570, 0.9520 );
		detectorData->addProperty( 0.575, 0.9154 );
		detectorData->addProperty( 0.580, 0.8700 );
		detectorData->addProperty( 0.585, 0.8163 );
		detectorData->addProperty( 0.590, 0.7570 );
		detectorData->addProperty( 0.595, 0.6949 );
		detectorData->addProperty( 0.600, 0.6310 );
		detectorData->addProperty( 0.605, 0.5668 );
		detectorData->addProperty( 0.610, 0.5030 );
		detectorData->addProperty( 0.615, 0.4412 );
		detectorData->addProperty( 0.620, 0.3810 );
		detectorData->addProperty( 0.625, 0.3210 );
		detectorData->addProperty( 0.630, 0.2650 );
		detectorData->addProperty( 0.635, 0.2170 );
		detectorData->addProperty( 0.640, 0.1750 );
		detectorData->addProperty( 0.645, 0.1382 );
		detectorData->addProperty( 0.650, 0.1070 );
		detectorData->addProperty( 0.655, 0.0816 );
		detectorData->addProperty( 0.660, 0.0610 );
		detectorData->addProperty( 0.665, 0.0446 );
		detectorData->addProperty( 0.670, 0.0320 );
		detectorData->addProperty( 0.675, 0.0232 );
		detectorData->addProperty( 0.680, 0.0170 );
		detectorData->addProperty( 0.685, 0.0119 );
		detectorData->addProperty( 0.690, 0.0082 );
		detectorData->addProperty( 0.695, 0.0057 );
		detectorData->addProperty( 0.700, 0.0041 );
		detectorData->addProperty( 0.705, 0.0029 );
		detectorData->addProperty( 0.710, 0.0021 );
		detectorData->addProperty( 0.715, 0.0015 );
		detectorData->addProperty( 0.720, 0.0010 );
		detectorData->addProperty( 0.725, 0.0007 );
		detectorData->addProperty( 0.730, 0.0005 );
		detectorData->addProperty( 0.735, 0.0004 );
		detectorData->addProperty( 0.740, 0.0002 );
		detectorData->addProperty( 0.745, 0.0002 );
		detectorData->addProperty( 0.750, 0.0001 );
		detectorData->addProperty( 0.755, 0.0001 );
		detectorData->addProperty( 0.760, 0.0001 );
		detectorData->addProperty( 0.765, 0.0000 );
		detectorData->addProperty( 0.770, 0.0000 );
		detectorData->addProperty( 0.775, 0.0000 );
		detectorData->addProperty( 0.780, 0.0000 );

		return detectorData;
	}

	std::shared_ptr< CSpectralSampleData > getMeasurements() const {
		auto sampleMeasurements = std::make_shared< CSpectralSampleData >();

		sampleMeasurements->addRecord( 0.300, 0.0006, 0.0518, 0.2713 );
		sampleMeasurements->addRecord( 0.305, 0.0006, 0.0509, 0.2624 );
		sampleMeasurements->addRecord( 0.310, 0.0017, 0.0508, 0.2576 );
		sampleMeasurements->addRecord( 0.315, 0.0072, 0.0508, 0.2569 );
		sampleMeasurements->addRecord( 0.320, 0.0219, 0.0511, 0.2579 );
		sampleMeasurements->addRecord( 0.325, 0.0486, 0.0525, 0.2607 );
		sampleMeasurements->addRecord( 0.330, 0.0857, 0.0560, 0.2646 );
		sampleMeasurements->addRecord( 0.335, 0.1280, 0.0623, 0.2664 );
		sampleMeasurements->addRecord( 0.340, 0.1707, 0.0719, 0.2668 );
		sampleMeasurements->addRecord( 0.345, 0.2125, 0.0840, 0.2680 );
		sampleMeasurements->addRecord( 0.350, 0.2536, 0.0990, 0.2706 );
		sampleMeasurements->addRecord( 0.355, 0.2953, 0.1165, 0.2735 );
		sampleMeasurements->addRecord( 0.360, 0.3370, 0.1365, 0.2773 );
		sampleMeasurements->addRecord( 0.365, 0.3774, 0.1579, 0.2809 );
		sampleMeasurements->addRecord( 0.370, 0.4125, 0.1773, 0.2829 );
		sampleMeasurements->addRecord( 0.375, 0.4414, 0.1931, 0.2836 );
		sampleMeasurements->addRecord( 0.380, 0.4671, 0.2074, 0.2827 );
		sampleMeasurements->addRecord( 0.385, 0.4953, 0.2244, 0.2814 );
		sampleMeasurements->addRecord( 0.390, 0.5229, 0.2415, 0.2801 );
		sampleMeasurements->addRecord( 0.395, 0.5455, 0.2553, 0.2781 );
		sampleMeasurements->addRecord( 0.400, 0.5630, 0.2651, 0.2757 );
		sampleMeasurements->addRecord( 0.405, 0.5764, 0.2718, 0.2728 );
		sampleMeasurements->addRecord( 0.410, 0.5870, 0.2759, 0.2695 );
		sampleMeasurements->addRecord( 0.415, 0.5955, 0.2783, 0.2657 );
		sampleMeasurements->addRecord( 0.420, 0.6025, 0.2798, 0.2623 );
		sampleMeasurements->addRecord( 0.425, 0.6098, 0.2809, 0.2584 );
		sampleMeasurements->addRecord( 0.430, 0.6166, 0.2812, 0.2544 );
		sampleMeasurements->addRecord( 0.435, 0.6226, 0.2805, 0.2504 );
		sampleMeasurements->addRecord( 0.440, 0.6286, 0.2794, 0.2462 );
		sampleMeasurements->addRecord( 0.445, 0.6346, 0.2780, 0.2419 );
		sampleMeasurements->addRecord( 0.450, 0.6412, 0.2766, 0.2374 );
		sampleMeasurements->addRecord( 0.455, 0.6481, 0.2751, 0.2327 );
		sampleMeasurements->addRecord( 0.460, 0.6542, 0.2730, 0.2281 );
		sampleMeasurements->addRecord( 0.465, 0.6595, 0.2702, 0.2235 );
		sampleMeasurements->addRecord( 0.470, 0.6643, 0.2672, 0.2187 );
		sampleMeasurements->addRecord( 0.475, 0.6689, 0.2640, 0.2141 );
		sampleMeasurements->addRecord( 0.480, 0.6734, 0.2607, 0.2095 );
		sampleMeasurements->addRecord( 0.485, 0.6784, 0.2571, 0.2046 );
		sampleMeasurements->addRecord( 0.490, 0.6837, 0.2532, 0.1996 );
		sampleMeasurements->addRecord( 0.495, 0.6882, 0.2492, 0.1944 );
		sampleMeasurements->addRecord( 0.500, 0.6928, 0.2449, 0.1892 );
		sampleMeasurements->addRecord( 0.505, 0.6968, 0.2406, 0.1840 );
		sampleMeasurements->addRecord( 0.510, 0.7004, 0.2365, 0.1792 );
		sampleMeasurements->addRecord( 0.515, 0.7036, 0.2324, 0.1744 );
		sampleMeasurements->addRecord( 0.520, 0.7067, 0.2284, 0.1697 );
		sampleMeasurements->addRecord( 0.525, 0.7099, 0.2244, 0.1650 );
		sampleMeasurements->addRecord( 0.530, 0.7127, 0.2206, 0.1606 );
		sampleMeasurements->addRecord( 0.535, 0.7156, 0.2167, 0.1563 );
		sampleMeasurements->addRecord( 0.540, 0.7179, 0.2130, 0.1521 );
		sampleMeasurements->addRecord( 0.545, 0.7200, 0.2094, 0.1481 );
		sampleMeasurements->addRecord( 0.550, 0.7224, 0.2061, 0.1444 );
		sampleMeasurements->addRecord( 0.555, 0.7244, 0.2029, 0.1409 );
		sampleMeasurements->addRecord( 0.560, 0.7259, 0.1997, 0.1375 );
		sampleMeasurements->addRecord( 0.565, 0.7265, 0.1967, 0.1344 );
		sampleMeasurements->addRecord( 0.570, 0.7267, 0.1940, 0.1317 );
		sampleMeasurements->addRecord( 0.575, 0.7268, 0.1914, 0.1292 );
		sampleMeasurements->addRecord( 0.580, 0.7267, 0.1889, 0.1270 );
		sampleMeasurements->addRecord( 0.585, 0.7257, 0.1868, 0.1252 );
		sampleMeasurements->addRecord( 0.590, 0.7249, 0.1849, 0.1237 );
		sampleMeasurements->addRecord( 0.595, 0.7237, 0.1835, 0.1224 );
		sampleMeasurements->addRecord( 0.600, 0.7228, 0.1825, 0.1217 );
		sampleMeasurements->addRecord( 0.605, 0.7211, 0.1815, 0.1211 );
		sampleMeasurements->addRecord( 0.610, 0.7187, 0.1806, 0.1207 );
		sampleMeasurements->addRecord( 0.615, 0.7164, 0.1802, 0.1209 );
		sampleMeasurements->addRecord( 0.620, 0.7138, 0.1801, 0.1214 );
		sampleMeasurements->addRecord( 0.625, 0.7110, 0.1802, 0.1223 );
		sampleMeasurements->addRecord( 0.630, 0.7078, 0.1807, 0.1236 );
		sampleMeasurements->addRecord( 0.635, 0.7037, 0.1813, 0.1251 );
		sampleMeasurements->addRecord( 0.640, 0.6999, 0.1823, 0.1269 );
		sampleMeasurements->addRecord( 0.645, 0.6958, 0.1837, 0.1292 );
		sampleMeasurements->addRecord( 0.650, 0.6916, 0.1849, 0.1317 );
		sampleMeasurements->addRecord( 0.655, 0.6870, 0.1865, 0.1345 );
		sampleMeasurements->addRecord( 0.660, 0.6823, 0.1885, 0.1377 );
		sampleMeasurements->addRecord( 0.665, 0.6773, 0.1906, 0.1411 );
		sampleMeasurements->addRecord( 0.670, 0.6723, 0.1931, 0.1451 );
		sampleMeasurements->addRecord( 0.675, 0.6670, 0.1957, 0.1491 );
		sampleMeasurements->addRecord( 0.680, 0.6614, 0.1985, 0.1533 );
		sampleMeasurements->addRecord( 0.685, 0.6551, 0.2015, 0.1580 );
		sampleMeasurements->addRecord( 0.690, 0.6492, 0.2047, 0.1630 );
		sampleMeasurements->addRecord( 0.695, 0.6432, 0.2080, 0.1682 );
		sampleMeasurements->addRecord( 0.700, 0.6366, 0.2115, 0.1733 );
		sampleMeasurements->addRecord( 0.705, 0.6299, 0.2150, 0.1790 );
		sampleMeasurements->addRecord( 0.710, 0.6231, 0.2186, 0.1847 );
		sampleMeasurements->addRecord( 0.715, 0.6163, 0.2223, 0.1908 );
		sampleMeasurements->addRecord( 0.720, 0.6093, 0.2259, 0.1965 );
		sampleMeasurements->addRecord( 0.725, 0.6023, 0.2299, 0.2030 );
		sampleMeasurements->addRecord( 0.730, 0.5953, 0.2338, 0.2094 );
		sampleMeasurements->addRecord( 0.735, 0.5877, 0.2381, 0.2159 );
		sampleMeasurements->addRecord( 0.740, 0.5808, 0.2422, 0.2228 );
		sampleMeasurements->addRecord( 0.745, 0.5732, 0.2466, 0.2299 );
		sampleMeasurements->addRecord( 0.750, 0.5655, 0.2502, 0.2360 );
		sampleMeasurements->addRecord( 0.755, 0.5580, 0.2542, 0.2430 );
		sampleMeasurements->addRecord( 0.760, 0.5506, 0.2585, 0.2501 );
		sampleMeasurements->addRecord( 0.765, 0.5433, 0.2631, 0.2574 );
		sampleMeasurements->addRecord( 0.770, 0.5355, 0.2671, 0.2643 );
		sampleMeasurements->addRecord( 0.775, 0.5280, 0.2717, 0.2720 );
		sampleMeasurements->addRecord( 0.780, 0.5207, 0.2763, 0.2794 );
		sampleMeasurements->addRecord( 0.785, 0.5137, 0.2806, 0.2872 );
		sampleMeasurements->addRecord( 0.790, 0.5057, 0.2854, 0.2947 );
		sampleMeasurements->addRecord( 0.795, 0.4980, 0.2898, 0.3023 );
		sampleMeasurements->addRecord( 0.800, 0.4910, 0.2945, 0.3095 );
		sampleMeasurements->addRecord( 0.805, 0.4843, 0.2990, 0.3179 );
		sampleMeasurements->addRecord( 0.810, 0.4781, 0.3035, 0.3262 );
		sampleMeasurements->addRecord( 0.815, 0.4710, 0.3085, 0.3347 );
		sampleMeasurements->addRecord( 0.820, 0.4633, 0.3132, 0.3422 );
		sampleMeasurements->addRecord( 0.825, 0.4567, 0.3172, 0.3502 );
		sampleMeasurements->addRecord( 0.830, 0.4496, 0.3217, 0.3570 );
		sampleMeasurements->addRecord( 0.835, 0.4409, 0.3260, 0.3654 );
		sampleMeasurements->addRecord( 0.840, 0.4340, 0.3296, 0.3724 );
		sampleMeasurements->addRecord( 0.845, 0.4272, 0.3327, 0.3783 );
		sampleMeasurements->addRecord( 0.850, 0.4210, 0.3355, 0.3841 );
		sampleMeasurements->addRecord( 0.855, 0.4152, 0.3400, 0.3919 );
		sampleMeasurements->addRecord( 0.860, 0.4092, 0.3443, 0.3992 );
		sampleMeasurements->addRecord( 0.865, 0.4034, 0.3483, 0.4062 );
		sampleMeasurements->addRecord( 0.870, 0.3976, 0.3524, 0.4132 );
		sampleMeasurements->addRecord( 0.875, 0.3920, 0.3560, 0.4198 );
		sampleMeasurements->addRecord( 0.880, 0.3864, 0.3595, 0.4263 );
		sampleMeasurements->addRecord( 0.885, 0.3811, 0.3632, 0.4327 );
		sampleMeasurements->addRecord( 0.890, 0.3757, 0.3665, 0.4390 );
		sampleMeasurements->addRecord( 0.895, 0.3704, 0.3704, 0.4455 );
		sampleMeasurements->addRecord( 0.900, 0.3650, 0.3740, 0.4519 );
		sampleMeasurements->addRecord( 0.905, 0.3599, 0.3768, 0.4575 );
		sampleMeasurements->addRecord( 0.910, 0.3547, 0.3796, 0.4629 );
		sampleMeasurements->addRecord( 0.915, 0.3498, 0.3823, 0.4684 );
		sampleMeasurements->addRecord( 0.920, 0.3448, 0.3854, 0.4739 );
		sampleMeasurements->addRecord( 0.925, 0.3397, 0.3885, 0.4793 );
		sampleMeasurements->addRecord( 0.930, 0.3351, 0.3914, 0.4849 );
		sampleMeasurements->addRecord( 0.935, 0.3303, 0.3946, 0.4906 );
		sampleMeasurements->addRecord( 0.940, 0.3256, 0.3980, 0.4962 );
		sampleMeasurements->addRecord( 0.945, 0.3212, 0.4011, 0.5020 );
		sampleMeasurements->addRecord( 0.950, 0.3166, 0.4045, 0.5079 );
		sampleMeasurements->addRecord( 0.955, 0.3122, 0.4077, 0.5136 );
		sampleMeasurements->addRecord( 0.960, 0.3078, 0.4110, 0.5192 );
		sampleMeasurements->addRecord( 0.965, 0.3036, 0.4146, 0.5249 );
		sampleMeasurements->addRecord( 0.970, 0.2995, 0.4177, 0.5305 );
		sampleMeasurements->addRecord( 0.975, 0.2953, 0.4212, 0.5363 );
		sampleMeasurements->addRecord( 0.980, 0.2914, 0.4247, 0.5424 );
		sampleMeasurements->addRecord( 0.985, 0.2875, 0.4281, 0.5478 );
		sampleMeasurements->addRecord( 0.990, 0.2836, 0.4318, 0.5537 );
		sampleMeasurements->addRecord( 0.995, 0.2799, 0.4352, 0.5593 );
		sampleMeasurements->addRecord( 1.000, 0.2760, 0.4388, 0.5649 );
		sampleMeasurements->addRecord( 1.005, 0.2723, 0.4414, 0.5693 );
		sampleMeasurements->addRecord( 1.010, 0.2688, 0.4441, 0.5738 );
		sampleMeasurements->addRecord( 1.015, 0.2653, 0.4468, 0.5784 );
		sampleMeasurements->addRecord( 1.020, 0.2616, 0.4494, 0.5829 );
		sampleMeasurements->addRecord( 1.025, 0.2586, 0.4524, 0.5876 );
		sampleMeasurements->addRecord( 1.030, 0.2550, 0.4549, 0.5918 );
		sampleMeasurements->addRecord( 1.035, 0.2520, 0.4579, 0.5964 );
		sampleMeasurements->addRecord( 1.040, 0.2487, 0.4606, 0.6007 );
		sampleMeasurements->addRecord( 1.045, 0.2455, 0.4634, 0.6050 );
		sampleMeasurements->addRecord( 1.050, 0.2424, 0.4660, 0.6092 );
		sampleMeasurements->addRecord( 1.055, 0.2393, 0.4689, 0.6137 );
		sampleMeasurements->addRecord( 1.060, 0.2363, 0.4713, 0.6179 );
		sampleMeasurements->addRecord( 1.065, 0.2334, 0.4740, 0.6215 );
		sampleMeasurements->addRecord( 1.070, 0.2303, 0.4765, 0.6258 );
		sampleMeasurements->addRecord( 1.075, 0.2276, 0.4791, 0.6297 );
		sampleMeasurements->addRecord( 1.080, 0.2247, 0.4816, 0.6336 );
		sampleMeasurements->addRecord( 1.085, 0.2221, 0.4842, 0.6375 );
		sampleMeasurements->addRecord( 1.090, 0.2194, 0.4867, 0.6412 );
		sampleMeasurements->addRecord( 1.095, 0.2170, 0.4893, 0.6449 );
		sampleMeasurements->addRecord( 1.100, 0.2142, 0.4915, 0.6483 );
		sampleMeasurements->addRecord( 1.105, 0.2118, 0.4941, 0.6518 );
		sampleMeasurements->addRecord( 1.110, 0.2092, 0.4961, 0.6551 );
		sampleMeasurements->addRecord( 1.115, 0.2065, 0.4985, 0.6583 );
		sampleMeasurements->addRecord( 1.120, 0.2043, 0.5006, 0.6615 );
		sampleMeasurements->addRecord( 1.125, 0.2019, 0.5028, 0.6647 );
		sampleMeasurements->addRecord( 1.130, 0.1997, 0.5053, 0.6680 );
		sampleMeasurements->addRecord( 1.135, 0.1975, 0.5075, 0.6712 );
		sampleMeasurements->addRecord( 1.140, 0.1953, 0.5098, 0.6742 );
		sampleMeasurements->addRecord( 1.145, 0.1931, 0.5119, 0.6773 );
		sampleMeasurements->addRecord( 1.150, 0.1910, 0.5141, 0.6801 );
		sampleMeasurements->addRecord( 1.155, 0.1887, 0.5163, 0.6833 );
		sampleMeasurements->addRecord( 1.160, 0.1869, 0.5187, 0.6863 );
		sampleMeasurements->addRecord( 1.165, 0.1851, 0.5209, 0.6894 );
		sampleMeasurements->addRecord( 1.170, 0.1827, 0.5230, 0.6921 );
		sampleMeasurements->addRecord( 1.175, 0.1808, 0.5253, 0.6950 );
		sampleMeasurements->addRecord( 1.180, 0.1788, 0.5273, 0.6977 );
		sampleMeasurements->addRecord( 1.185, 0.1772, 0.5295, 0.7005 );
		sampleMeasurements->addRecord( 1.190, 0.1751, 0.5321, 0.7032 );
		sampleMeasurements->addRecord( 1.195, 0.1735, 0.5341, 0.7060 );
		sampleMeasurements->addRecord( 1.200, 0.1715, 0.5363, 0.7088 );
		sampleMeasurements->addRecord( 1.205, 0.1697, 0.5385, 0.7112 );
		sampleMeasurements->addRecord( 1.210, 0.1678, 0.5405, 0.7139 );
		sampleMeasurements->addRecord( 1.215, 0.1663, 0.5426, 0.7161 );
		sampleMeasurements->addRecord( 1.220, 0.1647, 0.5448, 0.7185 );
		sampleMeasurements->addRecord( 1.225, 0.1631, 0.5470, 0.7209 );
		sampleMeasurements->addRecord( 1.230, 0.1611, 0.5492, 0.7236 );
		sampleMeasurements->addRecord( 1.235, 0.1598, 0.5519, 0.7263 );
		sampleMeasurements->addRecord( 1.240, 0.1583, 0.5540, 0.7286 );
		sampleMeasurements->addRecord( 1.245, 0.1568, 0.5562, 0.7311 );
		sampleMeasurements->addRecord( 1.250, 0.1551, 0.5584, 0.7334 );
		sampleMeasurements->addRecord( 1.255, 0.1537, 0.5607, 0.7354 );
		sampleMeasurements->addRecord( 1.260, 0.1522, 0.5627, 0.7374 );
		sampleMeasurements->addRecord( 1.265, 0.1507, 0.5649, 0.7396 );
		sampleMeasurements->addRecord( 1.270, 0.1494, 0.5671, 0.7416 );
		sampleMeasurements->addRecord( 1.275, 0.1480, 0.5691, 0.7436 );
		sampleMeasurements->addRecord( 1.280, 0.1468, 0.5716, 0.7458 );
		sampleMeasurements->addRecord( 1.285, 0.1451, 0.5738, 0.7477 );
		sampleMeasurements->addRecord( 1.290, 0.1440, 0.5761, 0.7500 );
		sampleMeasurements->addRecord( 1.295, 0.1428, 0.5785, 0.7517 );
		sampleMeasurements->addRecord( 1.300, 0.1416, 0.5809, 0.7539 );
		sampleMeasurements->addRecord( 1.305, 0.1403, 0.5831, 0.7558 );
		sampleMeasurements->addRecord( 1.310, 0.1392, 0.5857, 0.7580 );
		sampleMeasurements->addRecord( 1.315, 0.1378, 0.5874, 0.7592 );
		sampleMeasurements->addRecord( 1.320, 0.1367, 0.5897, 0.7611 );
		sampleMeasurements->addRecord( 1.325, 0.1355, 0.5921, 0.7629 );
		sampleMeasurements->addRecord( 1.330, 0.1342, 0.5946, 0.7648 );
		sampleMeasurements->addRecord( 1.335, 0.1332, 0.5970, 0.7669 );
		sampleMeasurements->addRecord( 1.340, 0.1319, 0.5995, 0.7686 );
		sampleMeasurements->addRecord( 1.345, 0.1309, 0.6016, 0.7705 );
		sampleMeasurements->addRecord( 1.350, 0.1300, 0.6040, 0.7720 );
		sampleMeasurements->addRecord( 1.355, 0.1290, 0.6066, 0.7740 );
		sampleMeasurements->addRecord( 1.360, 0.1279, 0.6089, 0.7753 );
		sampleMeasurements->addRecord( 1.365, 0.1268, 0.6110, 0.7767 );
		sampleMeasurements->addRecord( 1.370, 0.1255, 0.6133, 0.7786 );
		sampleMeasurements->addRecord( 1.375, 0.1245, 0.6154, 0.7797 );
		sampleMeasurements->addRecord( 1.380, 0.1235, 0.6168, 0.7805 );
		sampleMeasurements->addRecord( 1.385, 0.1225, 0.6187, 0.7818 );
		sampleMeasurements->addRecord( 1.390, 0.1215, 0.6201, 0.7829 );
		sampleMeasurements->addRecord( 1.395, 0.1206, 0.6219, 0.7847 );
		sampleMeasurements->addRecord( 1.400, 0.1197, 0.6240, 0.7865 );
		sampleMeasurements->addRecord( 1.405, 0.1185, 0.6262, 0.7884 );
		sampleMeasurements->addRecord( 1.410, 0.1178, 0.6285, 0.7900 );
		sampleMeasurements->addRecord( 1.415, 0.1166, 0.6313, 0.7921 );
		sampleMeasurements->addRecord( 1.420, 0.1157, 0.6338, 0.7934 );
		sampleMeasurements->addRecord( 1.425, 0.1149, 0.6367, 0.7951 );
		sampleMeasurements->addRecord( 1.430, 0.1143, 0.6394, 0.7965 );
		sampleMeasurements->addRecord( 1.435, 0.1136, 0.6423, 0.7983 );
		sampleMeasurements->addRecord( 1.440, 0.1127, 0.6449, 0.7993 );
		sampleMeasurements->addRecord( 1.445, 0.1117, 0.6474, 0.8005 );
		sampleMeasurements->addRecord( 1.450, 0.1109, 0.6501, 0.8021 );
		sampleMeasurements->addRecord( 1.455, 0.1101, 0.6529, 0.8038 );
		sampleMeasurements->addRecord( 1.460, 0.1093, 0.6554, 0.8050 );
		sampleMeasurements->addRecord( 1.465, 0.1086, 0.6578, 0.8062 );
		sampleMeasurements->addRecord( 1.470, 0.1077, 0.6602, 0.8072 );
		sampleMeasurements->addRecord( 1.475, 0.1069, 0.6627, 0.8088 );
		sampleMeasurements->addRecord( 1.480, 0.1061, 0.6652, 0.8099 );
		sampleMeasurements->addRecord( 1.485, 0.1053, 0.6673, 0.8111 );
		sampleMeasurements->addRecord( 1.490, 0.1047, 0.6696, 0.8123 );
		sampleMeasurements->addRecord( 1.495, 0.1040, 0.6718, 0.8134 );
		sampleMeasurements->addRecord( 1.500, 0.1032, 0.6741, 0.8147 );
		sampleMeasurements->addRecord( 1.505, 0.1024, 0.6763, 0.8159 );
		sampleMeasurements->addRecord( 1.510, 0.1016, 0.6783, 0.8171 );
		sampleMeasurements->addRecord( 1.515, 0.1010, 0.6804, 0.8182 );
		sampleMeasurements->addRecord( 1.520, 0.1002, 0.6824, 0.8192 );
		sampleMeasurements->addRecord( 1.525, 0.0997, 0.6844, 0.8204 );
		sampleMeasurements->addRecord( 1.530, 0.0989, 0.6864, 0.8215 );
		sampleMeasurements->addRecord( 1.535, 0.0982, 0.6885, 0.8225 );
		sampleMeasurements->addRecord( 1.540, 0.0974, 0.6904, 0.8235 );
		sampleMeasurements->addRecord( 1.545, 0.0968, 0.6921, 0.8244 );
		sampleMeasurements->addRecord( 1.550, 0.0960, 0.6940, 0.8258 );
		sampleMeasurements->addRecord( 1.555, 0.0953, 0.6959, 0.8269 );
		sampleMeasurements->addRecord( 1.560, 0.0946, 0.6976, 0.8277 );
		sampleMeasurements->addRecord( 1.565, 0.0939, 0.6992, 0.8285 );
		sampleMeasurements->addRecord( 1.570, 0.0934, 0.7008, 0.8294 );
		sampleMeasurements->addRecord( 1.575, 0.0929, 0.7025, 0.8305 );
		sampleMeasurements->addRecord( 1.580, 0.0920, 0.7040, 0.8315 );
		sampleMeasurements->addRecord( 1.585, 0.0912, 0.7056, 0.8324 );
		sampleMeasurements->addRecord( 1.590, 0.0906, 0.7072, 0.8334 );
		sampleMeasurements->addRecord( 1.595, 0.0898, 0.7087, 0.8344 );
		sampleMeasurements->addRecord( 1.600, 0.0891, 0.7100, 0.8353 );
		sampleMeasurements->addRecord( 1.605, 0.0886, 0.7112, 0.8362 );
		sampleMeasurements->addRecord( 1.610, 0.0882, 0.7126, 0.8369 );
		sampleMeasurements->addRecord( 1.615, 0.0874, 0.7140, 0.8378 );
		sampleMeasurements->addRecord( 1.620, 0.0867, 0.7151, 0.8387 );
		sampleMeasurements->addRecord( 1.625, 0.0862, 0.7163, 0.8395 );
		sampleMeasurements->addRecord( 1.630, 0.0856, 0.7176, 0.8406 );
		sampleMeasurements->addRecord( 1.635, 0.0849, 0.7187, 0.8414 );
		sampleMeasurements->addRecord( 1.640, 0.0842, 0.7196, 0.8421 );
		sampleMeasurements->addRecord( 1.645, 0.0836, 0.7207, 0.8429 );
		sampleMeasurements->addRecord( 1.650, 0.0832, 0.7218, 0.8437 );
		sampleMeasurements->addRecord( 1.655, 0.0826, 0.7228, 0.8448 );
		sampleMeasurements->addRecord( 1.660, 0.0820, 0.7236, 0.8453 );
		sampleMeasurements->addRecord( 1.665, 0.0814, 0.7245, 0.8460 );
		sampleMeasurements->addRecord( 1.670, 0.0806, 0.7252, 0.8469 );
		sampleMeasurements->addRecord( 1.675, 0.0801, 0.7260, 0.8478 );
		sampleMeasurements->addRecord( 1.680, 0.0796, 0.7267, 0.8486 );
		sampleMeasurements->addRecord( 1.685, 0.0790, 0.7275, 0.8493 );
		sampleMeasurements->addRecord( 1.690, 0.0783, 0.7281, 0.8500 );
		sampleMeasurements->addRecord( 1.695, 0.0778, 0.7288, 0.8507 );
		sampleMeasurements->addRecord( 1.700, 0.0773, 0.7295, 0.8516 );
		sampleMeasurements->addRecord( 1.705, 0.0768, 0.7301, 0.8525 );
		sampleMeasurements->addRecord( 1.710, 0.0763, 0.7306, 0.8529 );
		sampleMeasurements->addRecord( 1.715, 0.0758, 0.7311, 0.8536 );
		sampleMeasurements->addRecord( 1.720, 0.0753, 0.7323, 0.8555 );
		sampleMeasurements->addRecord( 1.725, 0.0746, 0.7328, 0.8561 );
		sampleMeasurements->addRecord( 1.730, 0.0742, 0.7331, 0.8567 );
		sampleMeasurements->addRecord( 1.735, 0.0736, 0.7336, 0.8573 );
		sampleMeasurements->addRecord( 1.740, 0.0732, 0.7339, 0.8579 );
		sampleMeasurements->addRecord( 1.745, 0.0727, 0.7344, 0.8585 );
		sampleMeasurements->addRecord( 1.750, 0.0722, 0.7348, 0.8592 );
		sampleMeasurements->addRecord( 1.755, 0.0716, 0.7350, 0.8599 );
		sampleMeasurements->addRecord( 1.760, 0.0711, 0.7353, 0.8605 );
		sampleMeasurements->addRecord( 1.765, 0.0707, 0.7354, 0.8610 );
		sampleMeasurements->addRecord( 1.770, 0.0703, 0.7358, 0.8617 );
		sampleMeasurements->addRecord( 1.775, 0.0698, 0.7362, 0.8623 );
		sampleMeasurements->addRecord( 1.780, 0.0693, 0.7363, 0.8628 );
		sampleMeasurements->addRecord( 1.785, 0.0688, 0.7363, 0.8632 );
		sampleMeasurements->addRecord( 1.790, 0.0683, 0.7365, 0.8638 );
		sampleMeasurements->addRecord( 1.795, 0.0678, 0.7375, 0.8653 );
		sampleMeasurements->addRecord( 1.800, 0.0674, 0.7371, 0.8651 );
		sampleMeasurements->addRecord( 1.805, 0.0668, 0.7372, 0.8655 );
		sampleMeasurements->addRecord( 1.810, 0.0664, 0.7373, 0.8659 );
		sampleMeasurements->addRecord( 1.815, 0.0660, 0.7373, 0.8665 );
		sampleMeasurements->addRecord( 1.820, 0.0656, 0.7376, 0.8671 );
		sampleMeasurements->addRecord( 1.825, 0.0652, 0.7379, 0.8676 );
		sampleMeasurements->addRecord( 1.830, 0.0647, 0.7382, 0.8682 );
		sampleMeasurements->addRecord( 1.835, 0.0643, 0.7385, 0.8687 );
		sampleMeasurements->addRecord( 1.840, 0.0637, 0.7386, 0.8692 );
		sampleMeasurements->addRecord( 1.845, 0.0632, 0.7387, 0.8698 );
		sampleMeasurements->addRecord( 1.850, 0.0629, 0.7390, 0.8700 );
		sampleMeasurements->addRecord( 1.855, 0.0625, 0.7392, 0.8704 );
		sampleMeasurements->addRecord( 1.860, 0.0622, 0.7412, 0.8730 );
		sampleMeasurements->addRecord( 1.865, 0.0619, 0.7413, 0.8737 );
		sampleMeasurements->addRecord( 1.870, 0.0614, 0.7415, 0.8742 );
		sampleMeasurements->addRecord( 1.875, 0.0609, 0.7417, 0.8746 );
		sampleMeasurements->addRecord( 1.880, 0.0605, 0.7422, 0.8751 );
		sampleMeasurements->addRecord( 1.885, 0.0602, 0.7425, 0.8755 );
		sampleMeasurements->addRecord( 1.890, 0.0599, 0.7427, 0.8759 );
		sampleMeasurements->addRecord( 1.895, 0.0596, 0.7429, 0.8766 );
		sampleMeasurements->addRecord( 1.900, 0.0593, 0.7431, 0.8770 );
		sampleMeasurements->addRecord( 1.905, 0.0589, 0.7432, 0.8775 );
		sampleMeasurements->addRecord( 1.910, 0.0583, 0.7433, 0.8779 );
		sampleMeasurements->addRecord( 1.915, 0.0579, 0.7436, 0.8784 );
		sampleMeasurements->addRecord( 1.920, 0.0576, 0.7437, 0.8787 );
		sampleMeasurements->addRecord( 1.925, 0.0573, 0.7438, 0.8790 );
		sampleMeasurements->addRecord( 1.930, 0.0571, 0.7441, 0.8796 );
		sampleMeasurements->addRecord( 1.935, 0.0567, 0.7443, 0.8800 );
		sampleMeasurements->addRecord( 1.940, 0.0562, 0.7444, 0.8802 );
		sampleMeasurements->addRecord( 1.945, 0.0559, 0.7446, 0.8806 );
		sampleMeasurements->addRecord( 1.950, 0.0557, 0.7448, 0.8811 );
		sampleMeasurements->addRecord( 1.955, 0.0554, 0.7448, 0.8813 );
		sampleMeasurements->addRecord( 1.960, 0.0551, 0.7449, 0.8815 );
		sampleMeasurements->addRecord( 1.965, 0.0547, 0.7453, 0.8820 );
		sampleMeasurements->addRecord( 1.970, 0.0545, 0.7455, 0.8824 );
		sampleMeasurements->addRecord( 1.975, 0.0542, 0.7457, 0.8829 );
		sampleMeasurements->addRecord( 1.980, 0.0536, 0.7459, 0.8833 );
		sampleMeasurements->addRecord( 1.985, 0.0534, 0.7460, 0.8837 );
		sampleMeasurements->addRecord( 1.990, 0.0532, 0.7461, 0.8839 );
		sampleMeasurements->addRecord( 1.995, 0.0527, 0.7465, 0.8844 );
		sampleMeasurements->addRecord( 2.000, 0.0524, 0.7468, 0.8849 );
		sampleMeasurements->addRecord( 2.005, 0.0520, 0.7471, 0.8854 );
		sampleMeasurements->addRecord( 2.010, 0.0517, 0.7477, 0.8859 );
		sampleMeasurements->addRecord( 2.015, 0.0516, 0.7481, 0.8864 );
		sampleMeasurements->addRecord( 2.020, 0.0513, 0.7484, 0.8868 );
		sampleMeasurements->addRecord( 2.025, 0.0510, 0.7489, 0.8873 );
		sampleMeasurements->addRecord( 2.030, 0.0508, 0.7496, 0.8880 );
		sampleMeasurements->addRecord( 2.035, 0.0504, 0.7501, 0.8886 );
		sampleMeasurements->addRecord( 2.040, 0.0502, 0.7507, 0.8892 );
		sampleMeasurements->addRecord( 2.045, 0.0499, 0.7512, 0.8896 );
		sampleMeasurements->addRecord( 2.050, 0.0496, 0.7515, 0.8901 );
		sampleMeasurements->addRecord( 2.055, 0.0492, 0.7521, 0.8908 );
		sampleMeasurements->addRecord( 2.060, 0.0491, 0.7527, 0.8912 );
		sampleMeasurements->addRecord( 2.065, 0.0489, 0.7531, 0.8914 );
		sampleMeasurements->addRecord( 2.070, 0.0485, 0.7533, 0.8919 );
		sampleMeasurements->addRecord( 2.075, 0.0483, 0.7536, 0.8922 );
		sampleMeasurements->addRecord( 2.080, 0.0480, 0.7539, 0.8926 );
		sampleMeasurements->addRecord( 2.085, 0.0476, 0.7542, 0.8931 );
		sampleMeasurements->addRecord( 2.090, 0.0474, 0.7545, 0.8934 );
		sampleMeasurements->addRecord( 2.095, 0.0472, 0.7547, 0.8937 );
		sampleMeasurements->addRecord( 2.100, 0.0468, 0.7549, 0.8942 );
		sampleMeasurements->addRecord( 2.105, 0.0467, 0.7552, 0.8947 );
		sampleMeasurements->addRecord( 2.110, 0.0464, 0.7555, 0.8952 );
		sampleMeasurements->addRecord( 2.115, 0.0461, 0.7556, 0.8957 );
		sampleMeasurements->addRecord( 2.120, 0.0459, 0.7559, 0.8965 );
		sampleMeasurements->addRecord( 2.125, 0.0457, 0.7545, 0.8951 );
		sampleMeasurements->addRecord( 2.130, 0.0455, 0.7548, 0.8958 );
		sampleMeasurements->addRecord( 2.135, 0.0451, 0.7548, 0.8965 );
		sampleMeasurements->addRecord( 2.140, 0.0448, 0.7551, 0.8975 );
		sampleMeasurements->addRecord( 2.145, 0.0446, 0.7554, 0.8987 );
		sampleMeasurements->addRecord( 2.150, 0.0446, 0.7549, 0.8989 );
		sampleMeasurements->addRecord( 2.155, 0.0440, 0.7543, 0.8994 );
		sampleMeasurements->addRecord( 2.160, 0.0440, 0.7536, 0.8998 );
		sampleMeasurements->addRecord( 2.165, 0.0438, 0.7527, 0.9004 );
		sampleMeasurements->addRecord( 2.170, 0.0433, 0.7505, 0.9003 );
		sampleMeasurements->addRecord( 2.175, 0.0430, 0.7481, 0.9005 );
		sampleMeasurements->addRecord( 2.180, 0.0426, 0.7455, 0.9012 );
		sampleMeasurements->addRecord( 2.185, 0.0424, 0.7414, 0.9004 );
		sampleMeasurements->addRecord( 2.190, 0.0421, 0.7383, 0.9008 );
		sampleMeasurements->addRecord( 2.195, 0.0418, 0.7353, 0.9011 );
		sampleMeasurements->addRecord( 2.200, 0.0414, 0.7322, 0.9008 );
		sampleMeasurements->addRecord( 2.205, 0.0409, 0.7299, 0.9007 );
		sampleMeasurements->addRecord( 2.210, 0.0409, 0.7279, 0.9000 );
		sampleMeasurements->addRecord( 2.215, 0.0408, 0.7266, 0.8994 );
		sampleMeasurements->addRecord( 2.220, 0.0403, 0.7267, 0.9001 );
		sampleMeasurements->addRecord( 2.225, 0.0402, 0.7268, 0.9003 );
		sampleMeasurements->addRecord( 2.230, 0.0399, 0.7275, 0.9005 );
		sampleMeasurements->addRecord( 2.235, 0.0400, 0.7288, 0.9012 );
		sampleMeasurements->addRecord( 2.240, 0.0399, 0.7301, 0.9018 );
		sampleMeasurements->addRecord( 2.245, 0.0396, 0.7317, 0.9025 );
		sampleMeasurements->addRecord( 2.250, 0.0395, 0.7330, 0.9025 );
		sampleMeasurements->addRecord( 2.255, 0.0391, 0.7345, 0.9030 );
		sampleMeasurements->addRecord( 2.260, 0.0391, 0.7365, 0.9043 );
		sampleMeasurements->addRecord( 2.265, 0.0391, 0.7384, 0.9050 );
		sampleMeasurements->addRecord( 2.270, 0.0389, 0.7402, 0.9061 );
		sampleMeasurements->addRecord( 2.275, 0.0387, 0.7417, 0.9065 );
		sampleMeasurements->addRecord( 2.280, 0.0385, 0.7437, 0.9079 );
		sampleMeasurements->addRecord( 2.285, 0.0383, 0.7447, 0.9081 );
		sampleMeasurements->addRecord( 2.290, 0.0380, 0.7461, 0.9088 );
		sampleMeasurements->addRecord( 2.295, 0.0379, 0.7470, 0.9088 );
		sampleMeasurements->addRecord( 2.300, 0.0379, 0.7484, 0.9094 );
		sampleMeasurements->addRecord( 2.305, 0.0377, 0.7495, 0.9100 );
		sampleMeasurements->addRecord( 2.310, 0.0376, 0.7516, 0.9121 );
		sampleMeasurements->addRecord( 2.315, 0.0373, 0.7521, 0.9120 );
		sampleMeasurements->addRecord( 2.320, 0.0372, 0.7529, 0.9126 );
		sampleMeasurements->addRecord( 2.325, 0.0370, 0.7537, 0.9132 );
		sampleMeasurements->addRecord( 2.330, 0.0368, 0.7544, 0.9135 );
		sampleMeasurements->addRecord( 2.335, 0.0365, 0.7557, 0.9147 );
		sampleMeasurements->addRecord( 2.340, 0.0365, 0.7560, 0.9148 );
		sampleMeasurements->addRecord( 2.345, 0.0360, 0.7567, 0.9153 );
		sampleMeasurements->addRecord( 2.350, 0.0358, 0.7564, 0.9151 );
		sampleMeasurements->addRecord( 2.355, 0.0358, 0.7566, 0.9154 );
		sampleMeasurements->addRecord( 2.360, 0.0355, 0.7577, 0.9165 );
		sampleMeasurements->addRecord( 2.365, 0.0356, 0.7569, 0.9156 );
		sampleMeasurements->addRecord( 2.370, 0.0355, 0.7569, 0.9163 );
		sampleMeasurements->addRecord( 2.375, 0.0349, 0.7568, 0.9163 );
		sampleMeasurements->addRecord( 2.380, 0.0350, 0.7562, 0.9164 );
		sampleMeasurements->addRecord( 2.385, 0.0348, 0.7552, 0.9156 );
		sampleMeasurements->addRecord( 2.390, 0.0347, 0.7555, 0.9172 );
		sampleMeasurements->addRecord( 2.395, 0.0348, 0.7546, 0.9164 );
		sampleMeasurements->addRecord( 2.400, 0.0345, 0.7550, 0.9175 );
		sampleMeasurements->addRecord( 2.405, 0.0347, 0.7539, 0.9179 );
		sampleMeasurements->addRecord( 2.410, 0.0338, 0.7540, 0.9188 );
		sampleMeasurements->addRecord( 2.415, 0.0337, 0.7517, 0.9175 );
		sampleMeasurements->addRecord( 2.420, 0.0336, 0.7524, 0.9198 );
		sampleMeasurements->addRecord( 2.425, 0.0337, 0.7505, 0.9198 );
		sampleMeasurements->addRecord( 2.430, 0.0326, 0.7500, 0.9203 );
		sampleMeasurements->addRecord( 2.435, 0.0329, 0.7476, 0.9202 );
		sampleMeasurements->addRecord( 2.440, 0.0333, 0.7451, 0.9207 );
		sampleMeasurements->addRecord( 2.445, 0.0323, 0.7436, 0.9204 );
		sampleMeasurements->addRecord( 2.450, 0.0324, 0.7437, 0.9210 );
		sampleMeasurements->addRecord( 2.455, 0.0330, 0.7417, 0.9210 );
		sampleMeasurements->addRecord( 2.460, 0.0339, 0.7409, 0.9199 );
		sampleMeasurements->addRecord( 2.465, 0.0306, 0.7374, 0.9198 );
		sampleMeasurements->addRecord( 2.470, 0.0291, 0.7366, 0.9207 );
		sampleMeasurements->addRecord( 2.475, 0.0295, 0.7359, 0.9229 );
		sampleMeasurements->addRecord( 2.480, 0.0343, 0.7301, 0.9167 );
		sampleMeasurements->addRecord( 2.485, 0.0282, 0.7327, 0.9163 );
		sampleMeasurements->addRecord( 2.490, 0.0301, 0.7354, 0.9226 );
		sampleMeasurements->addRecord( 2.495, 0.0286, 0.7317, 0.9226 );
		sampleMeasurements->addRecord( 2.500, 0.0330, 0.7297, 0.9266 );
		sampleMeasurements->addRecord( 3.000, 0.0028, 0.0546, 0.9347 );
		sampleMeasurements->addRecord( 3.500, 0.0020, 0.0405, 0.9444 );
		sampleMeasurements->addRecord( 4.000, 0.0025, 0.0424, 0.9472 );
		sampleMeasurements->addRecord( 4.500, 0.0113, 0.0337, 0.9504 );
		sampleMeasurements->addRecord( 5.000, 0.0003, 0.0308, 0.9546 );
		sampleMeasurements->addRecord( 5.500, 0.0005, 0.0278, 0.9548 );
		sampleMeasurements->addRecord( 6.000, 0.0006, 0.0242, 0.9466 );
		sampleMeasurements->addRecord( 6.500, 0.0002, 0.0206, 0.9486 );
		sampleMeasurements->addRecord( 7.000, 0.0004, 0.0127, 0.9572 );
		sampleMeasurements->addRecord( 7.500, 0.0005, 0.0055, 0.9590 );
		sampleMeasurements->addRecord( 8.000, 0.0003, 0.0049, 0.9608 );
		sampleMeasurements->addRecord( 8.500, 0.0003, 0.1089, 0.9619 );
		sampleMeasurements->addRecord( 9.000, 0.0005, 0.2089, 0.9623 );
		sampleMeasurements->addRecord( 9.500, 0.0006, 0.2911, 0.9637 );
		sampleMeasurements->addRecord( 10.000, 0.0006, 0.2345, 0.9638 );
		sampleMeasurements->addRecord( 10.500, 0.0001, 0.1906, 0.9621 );
		sampleMeasurements->addRecord( 11.000, 0.0004, 0.1455, 0.9612 );
		sampleMeasurements->addRecord( 11.500, 0.0003, 0.1011, 0.9617 );
		sampleMeasurements->addRecord( 12.000, 0.0002, 0.0708, 0.9615 );
		sampleMeasurements->addRecord( 12.500, 0.0008, 0.0697, 0.9618 );
		sampleMeasurements->addRecord( 13.000, 0.0003, 0.0803, 0.9623 );
		sampleMeasurements->addRecord( 13.500, 0.0001, 0.0756, 0.9627 );
		sampleMeasurements->addRecord( 14.000, 0.0002, 0.0662, 0.9627 );
		sampleMeasurements->addRecord( 14.500, 0.0002, 0.0596, 0.9640 );
		sampleMeasurements->addRecord( 15.000, 0.0009, 0.0540, 0.9626 );
		sampleMeasurements->addRecord( 15.500, 0.0009, 0.0494, 0.9642 );
		sampleMeasurements->addRecord( 16.000, 0.0003, 0.0440, 0.9649 );
		sampleMeasurements->addRecord( 16.500, 0.0002, 0.0387, 0.9646 );
		sampleMeasurements->addRecord( 17.000, 0.0001, 0.0337, 0.9646 );
		sampleMeasurements->addRecord( 17.500, 0.0004, 0.0269, 0.9644 );
		sampleMeasurements->addRecord( 18.000, 0.0005, 0.0221, 0.9641 );
		sampleMeasurements->addRecord( 18.500, 0.0007, 0.0281, 0.9645 );
		sampleMeasurements->addRecord( 19.000, 0.0005, 0.0733, 0.9654 );
		sampleMeasurements->addRecord( 19.500, 0.0003, 0.1379, 0.9660 );
		sampleMeasurements->addRecord( 20.000, 0.0005, 0.1903, 0.9657 );
		sampleMeasurements->addRecord( 20.500, 0.0007, 0.2237, 0.9653 );
		sampleMeasurements->addRecord( 21.000, 0.0001, 0.2412, 0.9640 );
		sampleMeasurements->addRecord( 21.500, 0.0005, 0.2451, 0.9647 );
		sampleMeasurements->addRecord( 22.000, 0.0006, 0.2385, 0.9659 );
		sampleMeasurements->addRecord( 22.500, 0.0006, 0.2257, 0.9670 );
		sampleMeasurements->addRecord( 23.000, 0.0006, 0.2125, 0.9669 );
		sampleMeasurements->addRecord( 23.500, 0.0010, 0.1998, 0.9661 );
		sampleMeasurements->addRecord( 24.000, 0.0007, 0.1873, 0.9703 );
		sampleMeasurements->addRecord( 24.500, 0.0008, 0.1762, 0.9648 );
		sampleMeasurements->addRecord( 25.000, 0.0008, 0.1693, 0.9678 );
		sampleMeasurements->addRecord( 26.000, 0.0007, 0.1551, 0.9665 );
		sampleMeasurements->addRecord( 27.000, 0.0008, 0.1448, 0.9682 );
		sampleMeasurements->addRecord( 28.000, 0.0003, 0.1385, 0.9693 );
		sampleMeasurements->addRecord( 29.000, 0.0004, 0.1314, 0.9683 );
		sampleMeasurements->addRecord( 30.000, 0.0010, 0.1275, 0.9700 );
		sampleMeasurements->addRecord( 32.000, 0.0004, 0.1229, 0.9701 );
		sampleMeasurements->addRecord( 34.000, 0.0006, 0.1210, 0.9704 );
		sampleMeasurements->addRecord( 36.000, 0.0001, 0.1219, 0.9736 );
		sampleMeasurements->addRecord( 38.000, 0.0006, 0.1221, 0.9705 );
		sampleMeasurements->addRecord( 40.000, 0.0003, 0.1237, 0.9757 );

		return sampleMeasurements;
	}

	void SetUp() override {
		auto solarRadiation = getSolarRadiation();
		auto aWavelengths = getWavelengths();
		auto detectorData = getDetectorData();
		auto sampleMeasurements = getMeasurements();

		m_Sample = std::make_shared< CSpectralSample >( sampleMeasurements, solarRadiation );
		m_Sample->setDetectorData( detectorData );

		m_Sample->setWavelengths( WavelengthSet::Custom, aWavelengths );
	}

public:
	std::shared_ptr< CSpectralSample > getSample() const {
		return m_Sample;
	};

};

TEST_F( TestSampleNFRC_1042_PHOTOPIC, TestSampleProperties ) {
	auto lowLambda = 0.38;
	auto highLambda = 0.78;

	auto aSample = getSample();
	auto transmittance = aSample->getProperty( lowLambda, highLambda, Property::T, Side::Front );
	EXPECT_NEAR( 0.714001, transmittance, 1e-6 );

	auto reflectanceFront = aSample->getProperty( lowLambda, highLambda, Property::R, Side::Front );
	EXPECT_NEAR( 0.207224, reflectanceFront, 1e-6 );

	auto reflectanceBack = aSample->getProperty( lowLambda, highLambda, Property::R, Side::Back );
	EXPECT_NEAR( 0.147697, reflectanceBack, 1e-6 );

}
