// EnergyPlus, Copyright (c) 1996-2019, The Board of Trustees of the University of Illinois,
// The Regents of the University of California, through Lawrence Berkeley National Laboratory
// (subject to receipt of any required approvals from the U.S. Dept. of Energy), Oak Ridge
// National Laboratory, managed by UT-Battelle, Alliance for Sustainable Energy, LLC, and other
// contributors. All rights reserved.
//
// NOTICE: This Software was developed under funding from the U.S. Department of Energy and the
// U.S. Government consequently retains certain rights. As such, the U.S. Government has been
// granted for itself and others acting on its behalf a paid-up, nonexclusive, irrevocable,
// worldwide license in the Software to reproduce, distribute copies to the public, prepare
// derivative works, and perform publicly and display publicly, and to permit others to do so.
//
// Redistribution and use in source and binary forms, with or without modification, are permitted
// provided that the following conditions are met:
//
// (1) Redistributions of source code must retain the above copyright notice, this list of
//     conditions and the following disclaimer.
//
// (2) Redistributions in binary form must reproduce the above copyright notice, this list of
//     conditions and the following disclaimer in the documentation and/or other materials
//     provided with the distribution.
//
// (3) Neither the name of the University of California, Lawrence Berkeley National Laboratory,
//     the University of Illinois, U.S. Dept. of Energy nor the names of its contributors may be
//     used to endorse or promote products derived from this software without specific prior
//     written permission.
//
// (4) Use of EnergyPlus(TM) Name. If Licensee (i) distributes the software in stand-alone form
//     without changes from the version obtained under this License, or (ii) Licensee makes a
//     reference solely to the software portion of its product, Licensee must refer to the
//     software as "EnergyPlus version X" software, where "X" is the version number Licensee
//     obtained under this License and may not use a different name for the software. Except as
//     specifically required in this Section (4), Licensee shall not use in a company name, a
//     product name, in advertising, publicity, or other promotional activities any name, trade
//     name, trademark, logo, or other designation of "EnergyPlus", "E+", "e+" or confusingly
//     similar designation, without the U.S. Department of Energy's prior written consent.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.

// EnergyPlus::VariableSpeedCoils Unit Tests

// Google Test Headers
#include "Fixtures/EnergyPlusFixture.hh"
#include <EnergyPlus/Psychrometrics.hh>
#include <EnergyPlus/VariableSpeedCoils.hh>
#include <gtest/gtest.h>

namespace EnergyPlus {

TEST_F(EnergyPlusFixture, VariableSpeedCoils_mixedCoilTypesInput)
{
    std::string const idf_objects = delimited_string(
        {"  Coil:Cooling:DX:VariableSpeed,",
         "    PSZ-AC_1:5_CoolC Standard 4-compressor IPAK,  !- Name",
         "    PSZ-AC_1:5_OA-PSZ-AC_1:5_CoolCNode,  !- Air Inlet Node Name",
         "    PSZ-AC_1:5_CoolC-PSZ-AC_1:5_HeatCNode,  !- Air Outlet Node Name",
         "    5,                      !- Number of Speeds {dimensionless}",
         "    5,                       !- Nominal Speed Level {dimensionless}",
         "    autosize,                !- Rated Total Cooling Capacity At Selected Nominal Speed Level {w}",
         "    autosize,                !- Rated Volumetric Air Flow Rate At Selected Nominal Speed Level {m3/s}",
         "    0,                       !- Nominal Time for Condensate to Begin Leaving the Coil {s}",
         "    0,                       !- Initial Moisture Evaporation Rate Divided by Steady-State AC Latent Capacity {dimensionless}",
         "    PLF Curve Multi Compressor 30prcnt unloading,               !- Energy Part Load Fraction Curve Name",
         "    PSZ-AC_1:5 OA Node,           !- Condenser Air Inlet Node Name",
         "    AirCooled,               !- Condenser Type",
         "    ,                        !- Evaporative Condenser Pump Rated Power Consumption {W}",
         "    ,                        !- Crankcase Heater Capacity {W}",
         "    10,                      !- Maximum Outdoor Dry-Bulb Temperature for Crankcase Heater Operation {C}",
         "    ,                        !- Minimum Outdoor Dry-Bulb Temperature for Compressor Operation {C}",
         "    ,                        !- Supply Water Storage Tank Name",
         "    ,                        !- Condensate Collection Water Storage Tank Name",
         "    ,                        !- Basin Heater Capacity {W/K}",
         "    2,                       !- Basin Heater Setpoint Temperature {C}",
         "    ,                        !- Basin Heater Operating Schedule Name",
         "    33861.72,             !- Speed 1 Reference Unit Total Cooling Capacity At Rated Conditions {w}",
         "    0.70,               !- Speed 1 Reference Unit Sensible Heat Ratio At Rated Conditions {dimensionless}",
         "    4.328973,             !- Speed 1 Reference Unit COP At Rated Conditions {dimensionless}",
         "    1.396964,              !- Speed 1 Reference Unit Air Flow Rate At Rated Conditions {m3/s}",
         "    ,                        !- Speed 1 Reference Unit Condenser Flow Rate at Rated Conditions {m3/s}",
         "    ,                        !- Speed 1 Reference Unit Pad Effectiveness of Evap Precooling at Rated Conditions {dimensionless}",
         "    5CapacityCurveIpakCS,         !- Speed 1 Total Cooling Capacity Function of Temperature Curve Name",
         "    CAPFF Curve IpakCS,             !- Speed 1 Total Cooling Capacity Function of Air Flow Fraction Curve Name",
         "    5PowerCurveIpakCS,            !- Speed 1 Energy Input Ratio Function of Temperature Curve Name",
         "    EIRFF Curve IpakCS,             !- Speed 1 Energy Input Ratio Function of Air Flow Fraction Curve Name",
         "    35516.08,             !- Speed 2 Reference Unit Total Cooling Capacity At Rated Conditions {w}",
         "    0.78,              !- Speed 2 Reference Unit Sensible Heat Ratio At Rated Conditions {dimensionless}",
         "    4.540061,             !- Speed 2 Reference Unit COP At Rated Conditions {dimensionless}",
         "    1.88779,              !- Speed 2 Reference Unit Air Flow Rate At Rated Conditions {m3/s}",
         "    ,                        !- Speed 2 Reference Unit Condenser Flow Rate at Rated Conditions {m3/s}",
         "    ,                        !- Speed 2 Reference Unit Pad Effectiveness of Evap Precooling at Rated Conditions {dimensionless}",
         "    4CapacityCurve,          !- Speed 2 Total Cooling Capacity Function of Temperature Curve Name",
         "    CAPFF Curve IpakCS,             !- Speed 2 Total Cooling Capacity Function of Air Flow Fraction Curve Name",
         "    4PowerCurveIpakCS,             !- Speed 2 Energy Input Ratio Function of Temperature Curve Name",
         "    EIRFF Curve IpakCS,             !- Speed 2 Energy Input Ratio Function of Air Flow Fraction Curve Name",
         "    65133.17,             !- Speed 3 Reference Unit Total Cooling Capacity At Rated Conditions {w}",
         "    0.70,             !- Speed 3 Reference Unit Sensible Heat Ratio At Rated Conditions {dimensionless}",
         "    4.164418,             !- Speed 3 Reference Unit COP At Rated Conditions {dimensionless}",
         "    2.831685,              !- Speed 3 Reference Unit Air Flow Rate At Rated Conditions {m3/s}",
         "    ,                        !- Speed 3 Reference Unit Condenser Flow Rate at Rated Conditions {m3/s}",
         "    ,                        !- Speed 3 Reference Unit Pad Effectiveness of Evap Precooling at Rated Conditions {dimensionless}",
         "    3CapacityCurveIpakCS,          !- Speed 3 Total Cooling Capacity Function of Temperature Curve Name",
         "    CAPFF Curve IpakCS,             !- Speed 3 Total Cooling Capacity Function of Air Flow Fraction Curve Name",
         "    3PowerCurveIpakCS,             !- Speed 3 Energy Input Ratio Function of Temperature Curve Name",
         "    EIRFF Curve IpakCS,             !- Speed 3 Energy Input Ratio Function of Air Flow Fraction Curve Name",
         "    119583.3,             !- Speed 4 Reference Unit Total Cooling Capacity At Rated Conditions {w}",
         "    0.62,              !- Speed 4 Reference Unit Sensible Heat Ratio At Rated Conditions {dimensionless}",
         "    3.469661,             !- Speed 4 Reference Unit COP At Rated Conditions {dimensionless}",
         "    3.553764,              !- Speed 4 Reference Unit Air Flow Rate At Rated Conditions {m3/s}",
         "    ,                        !- Speed 4 Reference Unit Condenser Flow Rate at Rated Conditions {m3/s}",
         "    ,                        !- Speed 4 Reference Unit Pad Effectiveness of Evap Precooling at Rated Conditions {dimensionless}",
         "    2CapacityCurveIpakCS,          !- Speed 4 Total Cooling Capacity Function of Temperature Curve Name",
         "    CAPFF Curve IpakCS,             !- Speed 4 Total Cooling Capacity Function of Air Flow Fraction Curve Name",
         "    2PowerCurveIpakCS ,             !- Speed 4 Energy Input Ratio Function of Temperature Curve Name",
         "    EIRFF Curve IpakCS,             !- Speed 4 Energy Input Ratio Function of Air Flow Fraction Curve Name",
         "    132769.7,             !- Speed 5 Reference Unit Total Cooling Capacity At Rated Conditions {w}",
         "    0.69,             !- Speed 5 Reference Unit Sensible Heat Ratio At Rated Conditions {dimensionless}",
         "    3.822957,             !- Speed 5 Reference Unit COP At Rated Conditions {dimensionless}",
         "    5.66336932,              !- Speed 5 Reference Unit Air Flow Rate At Rated Conditions {m3/s}",
         "    ,                        !- Speed 5 Reference Unit Condenser Flow Rate at Rated Conditions {m3/s}",
         "    ,                        !- Speed 5 Reference Unit Pad Effectiveness of Evap Precooling at Rated Conditions {dimensionless}",
         "    1CapacityCurveIpakCS,          !- Speed 5 Total Cooling Capacity Function of Temperature Curve Name",
         "    CAPFF Curve IpakCS,             !- Speed 5 Total Cooling Capacity Function of Air Flow Fraction Curve Name",
         "    1PowerCurveIpakCS,             !- Speed 5 Energy Input Ratio Function of Temperature Curve Name",
         "    EIRFF Curve IpakCS;             !- Speed 5 Energy Input Ratio Function of Air Flow Fraction Curve Name",

         "  OutdoorAir:Node, PSZ-AC_1:5 OA Node;",

         "  ZoneHVAC:WaterToAirHeatPump,",
         "    Lobby_ZN_1_FLR_2 WSHP,              !- Name",
         "    HVACOperationSchd,    !- Availability Schedule Name",
         "    Lobby_ZN_1_FLR_2 WSHP Inlet Node,      !- Air Inlet Node Name",
         "    Lobby_ZN_1_FLR_2 WSHP Outlet Node,       !- Air Outlet Node Name",
         "    ,        !- Outdoor Air Mixer Object Type",
         "    ,            !- Outdoor Air Mixer Name",
         "    Autosize,                !- Supply Air Flow Rate During Cooling Operation {m3/s}",
         "    Autosize,                !- Supply Air Flow Rate During Heating Operation {m3/s}",
         "    ,                     !- Supply Air Flow Rate When No Cooling or Heating is Needed {m3/s}",
         "    0.0,                     !- Outdoor Air Flow Rate During Cooling Operation {m3/s}",
         "    0.0,                     !- Outdoor Air Flow Rate During Heating Operation {m3/s}",
         "    ,                        !- Outdoor Air Flow Rate When No Cooling or Heating is Needed {m3/s}",
         "    Fan:OnOff,               !- Supply Air Fan Object Type",
         "    Lobby_ZN_1_FLR_2 WSHP Fan,              !- Supply Air Fan Name",
         "    Coil:Heating:WaterToAirHeatPump:VariableSpeedEquationFit,  !- Heating Coil Object Type",
         "    Lobby_ZN_1_FLR_2 WSHP Heating Mode,  !- Heating Coil Name",
         "    Coil:Cooling:WaterToAirHeatPump:VariableSpeedEquationFit,  !- Cooling Coil Object Type",
         "    Lobby_ZN_1_FLR_2 WSHP Cooling Mode,  !- Cooling Coil Name",
         "    2.5,                     !- Maximum Cycling Rate {cycles/hr}",
         "    60.0,                    !- Heat Pump Time Constant {s}",
         "    0.01,                    !- Fraction of On-Cycle Power Use",
         "    60,                      !- Heat Pump Fan Delay Time {s}",
         "    Coil:Heating:Electric,        !- Supplemental Heating Coil Object Type",
         "    Lobby_ZN_1_FLR_2 WSHP Supp Heating Coil,  !- Supplemental Heating Coil Name",
         "    50.0,                    !- Maximum Supply Air Temperature from Supplemental Heater {C}",
         "    20.0,                    !- Maximum Outdoor Dry-Bulb Temperature for Supplemental Heater Operation {C}",
         "    Lobby_ZN_1_FLR_2 WSHP OA Node,  !- Outdoor Dry-Bulb Temperature Sensor Node Name",
         "    BlowThrough,             !- Fan Placement",
         "    CyclingFanSch,           !- Supply Air Fan Operating Mode Schedule Name",
         "    ,!- Heat Pump Coil Water Flow Mode",
         "    ;!- Design Specification ZoneHVAC Sizing Object Name",

         "  OutdoorAir:Node,",
         "    Lobby_ZN_1_FLR_2 WSHP OA Node;",

         "  Coil:Cooling:WaterToAirHeatPump:VariableSpeedEquationFit,",
         "    Lobby_ZN_1_FLR_2 WSHP Cooling Mode,  !- Name",
         "    Lobby_ZN_1_FLR_2 WSHP Cooling Source Side Inlet Node,  !- Water-to-Refrigerant HX Water Inlet Node Name",
         "    Lobby_ZN_1_FLR_2 WSHP Cooling Source Side Outlet Node,  !- Water-to-Refrigerant HX Water Outlet Node Name",
         "    Lobby_ZN_1_FLR_2 WSHP Cooling Coil Air Inlet Node,  !- Indoor Air Inlet Node Name",
         "    Lobby_ZN_1_FLR_2 WSHP Heating Coil Air Inlet Node,  !- Indoor Air Outlet Node Name",
         "    9,                    !- Number of Speeds {dimensionless}",
         "    9,                      !- Nominal Speed Level {dimensionless}",
         "    Autosize,                !- Gross Rated Total Cooling Capacity At Selected Nominal Speed Level {w}",
         "    Autosize,                !- Rated Air Flow Rate At Selected Nominal Speed Level {m3/s}",
         "    Autosize,                !- Rated Water Flow Rate At Selected Nominal Speed Level {m3/s}",
         "    0.0,                     !- Nominal Time for Condensate to Begin Leaving the Coil {s}",
         "    0.0,                     !- Initial Moisture Evaporation Rate Divided by Steady-State AC Latent Capacity {dimensionless}",
         "    0,                       !- Flag for Using Hot Gas Reheat, 0 or 1 {dimensionless}",
         "    PLF Curve,               !- Energy Part Load Fraction Curve Name",
         "    4682.3964854,            !- Speed 1 Reference Unit Gross Rated Total Cooling Capacity {w}",
         "    0.97,                    !- Speed 1 Reference Unit Gross Rated Sensible Heat Ratio {dimensionless}",
         "    8.031554863,             !- Speed 1 Reference Unit Gross Rated Cooling COP {dimensionless}",
         "    0.408706486,             !- Speed 1 Reference Unit Rated Air Flow Rate {m3/s}",
         "    0.0008201726 ,                  !- Speed 1 Reference Unit Rated Water Flow Rate {m3/s}",
         "    10CoolingCapacityCurve     ,  !- Speed 1 Total Cooling Capacity Function of Temperature Curve Name",
         "    10CoolingCAPFFCurve,      !- Speed 1 Total Cooling Capacity Function of Air Flow Fraction Curve Name",
         "    10CoolingCAPFWCurve,      !- Speed 1 Total Cooling Capacity Function of Water Flow Fraction Curve Name",
         "    10CoolingPowerCurve   ,   !- Speed 1 Energy Input Ratio Function of Temperature Curve Name",
         "    10CoolingEIRFFCurve,      !- Speed 1 Energy Input Ratio Function of Air Flow Fraction Curve Name",
         "    10CoolingEIRFWCurve,      !- Speed 1 Energy Input Ratio Function of Water Flow Fraction Curve Name",
         "    0.0,                        !- Speed 1 Reference Unit Waste Heat Fraction of Input Power At Rated Conditions {dimensionless}",
         "    wasteHeat VS Temp1 Test,                        !- Speed 1 Waste Heat Function of Temperature Curve Name",
         "    5733.6424135,            !- Speed 2 Reference Unit Gross Rated Total Cooling Capacity {w}",
         "    0.96,                  !- Speed 2 Reference Unit Gross Rated Sensible Heat Ratio {dimensionless}",
         "    8.132826118,             !- Speed 2 Reference Unit Gross Rated Cooling COP {dimensionless}",
         "    0.449293966,               !- Speed 2 Reference Unit Rated Air Flow Rate {m3/s}",
         "    0.0008201726 ,                  !- Speed 2 Reference Unit Rated Water Flow Rate {m3/s}",
         "    9CoolingCapacityCurve,   !- Speed 2 Total Cooling Capacity Function of Temperature Curve Name",
         "    9CoolingCAPFFCurve,      !- Speed 2 Total Cooling Capacity Function of Air Flow Fraction Curve Name",
         "    9CoolingCAPFWCurve,      !- Speed 2 Total Cooling Capacity Function of Water Flow Fraction Curve Name",
         "    9CoolingPowerCurve   ,   !- Speed 2 Energy Input Ratio Function of Temperature Curve Name",
         "    9CoolingEIRFFCurve,      !- Speed 2 Energy Input Ratio Function of Air Flow Fraction Curve Name",
         "    9CoolingEIRFWCurve,      !- Speed 2 Energy Input Ratio Function of Water Flow Fraction Curve Name",
         "    0.0,                        !- Speed 2 Reference Unit Waste Heat Fraction of Input Power At Rated Conditions {dimensionless}",
         "    wasteHeat VS Temp1 Test,                        !- Speed 2 Waste Heat Function of Temperature Curve Name",
         "    6783.7160573,             !- Speed 3 Reference Unit Gross Rated Total Cooling Capacity {w}",
         "    0.95,                   !- Speed 3 Reference Unit Gross Rated Sensible Heat Ratio {dimensionless}",
         "    8.133952107,             !- Speed 3 Reference Unit Gross Rated Cooling COP {dimensionless}",
         "    0.489881446,              !- Speed 3 Reference Unit Rated Air Flow Rate {m3/s}",
         "    0.0008201726 ,                  !- Speed 3 Reference Unit Rated Water Flow Rate {m3/s}",
         "    8CoolingCapacityCurve,   !- Speed 3 Total Cooling Capacity Function of Temperature Curve Name",
         "    8CoolingCAPFFCurve,      !- Speed 3 Total Cooling Capacity Function of Air Flow Fraction Curve Name",
         "    8CoolingCAPFWCurve  ,    !- Speed 3 Total Cooling Capacity Function of Water Flow Fraction Curve Name",
         "    8CoolingPowerCurve   ,   !- Speed 3 Energy Input Ratio Function of Temperature Curve Name",
         "    8CoolingEIRFFCurve ,     !- Speed 3 Energy Input Ratio Function of Air Flow Fraction Curve Name",
         "    8CoolingEIRFWCurve,      !- Speed 3 Energy Input Ratio Function of Water Flow Fraction Curve Name",
         "    0.0,                        !- Speed 3 Reference Unit Waste Heat Fraction of Input Power At Rated Conditions {dimensionless}",
         "    wasteHeat VS Temp1 Test,                        !- Speed 3 Waste Heat Function of Temperature Curve Name",
         "    7819.1361476,            !- Speed 4 Reference Unit Gross Rated Total Cooling Capacity {w}",
         "    0.91,                   !- Speed 4 Reference Unit Gross Rated Sensible Heat Ratio {dimensionless}",
         "    8.077619987,             !- Speed 4 Reference Unit Gross Rated Cooling COP {dimensionless}",
         "    0.530468926,             !- Speed 4 Reference Unit Rated Air Flow Rate {m3/s}",
         "    0.0008201726 ,                  !- Speed 4 Reference Unit Rated Water Flow Rate {m3/s}",
         "    7CoolingCapacityCurve,   !- Speed 4 Total Cooling Capacity Function of Temperature Curve Name",
         "    7CoolingCAPFFCurve,      !- Speed 4 Total Cooling Capacity Function of Air Flow Fraction Curve Name",
         "    7CoolingCAPFWCurve,      !- Speed 4 Total Cooling Capacity Function of Water Flow Fraction Curve Name",
         "    7CoolingPowerCurve   ,   !- Speed 4 Energy Input Ratio Function of Temperature Curve Name",
         "    7CoolingEIRFFCurve,      !- Speed 4 Energy Input Ratio Function of Air Flow Fraction Curve Name",
         "    7CoolingEIRFWCurve,      !- Speed 4 Energy Input Ratio Function of Water Flow Fraction Curve Name",
         "    0.0,                        !- Speed 4 Reference Unit Waste Heat Fraction of Input Power At Rated Conditions {dimensionless}",
         "    wasteHeat VS Temp1 Test,                        !- Speed 4 Waste Heat Function of Temperature Curve Name",
         "    8827.8867705,             !- Speed 5 Reference Unit Gross Rated Total Cooling Capacity {w}",
         "    0.871,                   !- Speed 5 Reference Unit Gross Rated Sensible Heat Ratio {dimensionless}",
         "    7.974604129,             !- Speed 5 Reference Unit Gross Rated Cooling COP {dimensionless}",
         "    0.571056406,             !- Speed 5 Reference Unit Rated Air Flow Rate {m3/s}",
         "    0.0008201726 ,                  !- Speed 5 Reference Unit Rated Water Flow Rate {m3/s}",
         "    6CoolingCapacityCurve,   !- Speed 5 Total Cooling Capacity Function of Temperature Curve Name",
         "    6CoolingCAPFFCurve ,     !- Speed 5 Total Cooling Capacity Function of Air Flow Fraction Curve Name",
         "    6CoolingCAPFWCurve  ,    !- Speed 5 Total Cooling Capacity Function of Water Flow Fraction Curve Name",
         "    6CoolingPowerCurve   ,   !- Speed 5 Energy Input Ratio Function of Temperature Curve Name",
         "    6CoolingEIRFFCurve,      !- Speed 5 Energy Input Ratio Function of Air Flow Fraction Curve Name",
         "    6CoolingEIRFWCurve,      !- Speed 5 Energy Input Ratio Function of Water Flow Fraction Curve Name",
         "    0.0,                        !- Speed 5 Reference Unit Waste Heat Fraction of Input Power At Rated Conditions {dimensionless}",
         "    wasteHeat VS Temp1 Test,                        !- Speed 5 Waste Heat Function of Temperature Curve Name",
         "    10734.02101,            !- Speed 6 Reference Unit Gross Rated Total Cooling Capacity {w}",
         "    0.816,                   !- Speed 6 Reference Unit Gross Rated Sensible Heat Ratio {dimensionless}",
         "    7.661685232,             !- Speed 6 Reference Unit Gross Rated Cooling COP {dimensionless}",
         "    0.652231367,             !- Speed 6 Reference Unit Rated Air Flow Rate {m3/s}",
         "    0.0008201726 ,                  !- Speed 6 Reference Unit Rated Water Flow Rate {m3/s}",
         "    5CoolingCapacityCurve,   !- Speed 6 Total Cooling Capacity Function of Temperature Curve Name",
         "    5CoolingCAPFFCurve,      !- Speed 6 Total Cooling Capacity Function of Air Flow Fraction Curve Name",
         "    5CoolingCAPFWCurve,      !- Speed 6 Total Cooling Capacity Function of Water Flow Fraction Curve Name",
         "    5CoolingPowerCurve   ,   !- Speed 6 Energy Input Ratio Function of Temperature Curve Name",
         "    5CoolingEIRFFCurve,      !- Speed 6 Energy Input Ratio Function of Air Flow Fraction Curve Name",
         "    5CoolingEIRFWCurve,      !- Speed 6 Energy Input Ratio Function of Water Flow Fraction Curve Name",
         "    0.0,                        !- Speed 6 Reference Unit Waste Heat Fraction of Input Power At Rated Conditions {dimensionless}",
         "    wasteHeat VS Temp1 Test ,                        !- Speed 6 Waste Heat Function of Temperature Curve Name",
         "    12454.348191,            !- Speed 7 Reference Unit Gross Rated Total Cooling Capacity {w}",
         "    0.784,                    !- Speed 7 Reference Unit Gross Rated Sensible Heat Ratio {dimensionless}",
         "    7.257778666,             !- Speed 7 Reference Unit Gross Rated Cooling COP {dimensionless}",
         "    0.732934379,             !- Speed 7 Reference Unit Rated Air Flow Rate {m3/s}",
         "    0.0008201726 ,                  !- Speed 7 Reference Unit Rated Water Flow Rate {m3/s}",
         "    4CoolingCapacityCurve,   !- Speed 7 Total Cooling Capacity Function of Temperature Curve Name",
         "    4CoolingCAPFFCurve,      !- Speed 7 Total Cooling Capacity Function of Air Flow Fraction Curve Name",
         "    4CoolingCAPFWCurve,      !- Speed 7 Total Cooling Capacity Function of Water Flow Fraction Curve Name",
         "    4CoolingPowerCurve   ,   !- Speed 7 Energy Input Ratio Function of Temperature Curve Name",
         "    4CoolingEIRFFCurve,      !- Speed 7 Energy Input Ratio Function of Air Flow Fraction Curve Name",
         "    4CoolingEIRFWCurve,      !- Speed 7 Energy Input Ratio Function of Water Flow Fraction Curve Name",
         "    0.0,                        !- Speed 7 Reference Unit Waste Heat Fraction of Input Power At Rated Conditions {dimensionless}",
         "    wasteHeat VS Temp1 Test,                        !- Speed 7 Waste Heat Function of Temperature Curve Name",
         "    13963.37113,            !- Speed 8 Reference Unit Gross Rated Total Cooling Capacity {w}",
         "    0.766,                   !- Speed 8 Reference Unit Gross Rated Sensible Heat Ratio {dimensionless}",
         "    6.804761759,             !- Speed 8 Reference Unit Gross Rated Cooling COP {dimensionless}",
         "    0.81410934,             !- Speed 8 Reference Unit Rated Air Flow Rate {m3/s}",
         "    0.0008201726 ,                  !- Speed 8 Reference Unit Rated Water Flow Rate {m3/s}",
         "    3CoolingCapacityCurve,   !- Speed 8 Total Cooling Capacity Function of Temperature Curve Name",
         "    3CoolingCAPFFCurve,      !- Speed 8 Total Cooling Capacity Function of Air Flow Fraction Curve Name",
         "    3CoolingCAPFWCurve,      !- Speed 8 Total Cooling Capacity Function of Water Flow Fraction Curve Name",
         "    3CoolingPowerCurve   ,   !- Speed 8 Energy Input Ratio Function of Temperature Curve Name",
         "    3CoolingEIRFFCurve,      !- Speed 8 Energy Input Ratio Function of Air Flow Fraction Curve Name",
         "    3CoolingEIRFWCurve,      !- Speed 8 Energy Input Ratio Function of Water Flow Fraction Curve Name",
         "    0.0,                        !- Speed 8 Reference Unit Waste Heat Fraction of Input Power At Rated Conditions {dimensionless}",
         "    wasteHeat VS Temp1 Test,                        !- Speed 8 Waste Heat Function of Temperature Curve Name",
         "    16092.825525,            !- Speed 9 Reference Unit Gross Rated Total Cooling Capacity {w}",
         "    0.739,                   !- Speed 9 Reference Unit Gross Rated Sensible Heat Ratio {dimensionless}",
         "    5.765971166,             !- Speed 9 Reference Unit Gross Rated Cooling COP {dimensionless}",
         "    0.891980668,             !- Speed 9 Reference Unit Rated Air Flow Rate {m3/s}",
         "    0.0008201726 ,                  !- Speed 9 Reference Unit Rated Water Flow Rate {m3/s}",
         "    1CoolingCapacityCurve,   !- Speed 9 Total Cooling Capacity Function of Temperature Curve Name",
         "    1CoolingCAPFFCurve,      !- Speed 9 Total Cooling Capacity Function of Air Flow Fraction Curve Name",
         "    1CoolingCAPFWCurve,      !- Speed 9 Total Cooling Capacity Function of Water Flow Fraction Curve Name",
         "    1CoolingPowerCurve   ,   !- Speed 9 Energy Input Ratio Function of Temperature Curve Name",
         "    1CoolingEIRFFCurve,      !- Speed 9 Energy Input Ratio Function of Air Flow Fraction Curve Name",
         "    1CoolingEIRFWCurve,      !- Speed 9 Energy Input Ratio Function of Water Flow Fraction Curve Name",
         "    0.0,                        !- Speed 9 Reference Unit Waste Heat Fraction of Input Power At Rated Conditions {dimensionless}",
         "    wasteHeat VS Temp1 Test;                        !- Speed 9 Waste Heat Function of Temperature Curve Name",

         "  Coil:Heating:WaterToAirHeatPump:VariableSpeedEquationFit,",
         "    Lobby_ZN_1_FLR_2 WSHP Heating Mode,  !- Name",
         "    Lobby_ZN_1_FLR_2 WSHP Heating Source Side Inlet Node,  !- Water-to-Refrigerant HX Water Inlet Node Name",
         "    Lobby_ZN_1_FLR_2 WSHP Heating Source Side Outlet Node,  !- Water-to-Refrigerant HX Water Outlet Node Name",
         "    Lobby_ZN_1_FLR_2 WSHP Heating Coil Air Inlet Node,  !- Indoor Air Inlet Node Name",
         "    Lobby_ZN_1_FLR_2 WSHP SuppHeating Coil Air Inlet Node,  !- Indoor Air Outlet Node Name",
         "    9,                      !- Number of Speeds {dimensionless}",
         "    9,                       !- Nominal Speed Level {dimensionless}",
         "    autosize,                !- Rated Heating Capacity At Selected Nominal Speed Level {w}",
         "    autosize,                !- Rated Air Flow Rate At Selected Nominal Speed Level {m3/s}",
         "    autosize,                !- Rated Water Flow Rate At Selected Nominal Speed Level {m3/s}",
         "    PLF Curve,               !- Energy Part Load Fraction Curve Name",
         "    6437.5991236,            !- Speed 1 Reference Unit Gross Rated Heating Capacity {w}",
         "    9.965323721,              !- Speed 1 Reference Unit Gross Rated Heating COP {dimensionless}",
         "    0.408706486,             !- Speed 1 Reference Unit Rated Air Flow {m3/s}",
         "    0.0008201726 ,                  !- Speed 1 Reference Unit Rated Water Flow Rate {m3/s}",
         "    10HeatingCapacityCurve   ,!- Speed 1 Heating Capacity Function of Temperature Curve Name",
         "    10HeatingCAPFFCurve,      !- Speed 1 Total  Heating Capacity Function of Air Flow Fraction Curve Name",
         "    10HeatingCAPFWCurve,      !- Speed 1 Heating Capacity Function of Water Flow Fraction Curve Name",
         "    10HeatingPowerCurve    ,  !- Speed 1 Energy Input Ratio Function of Temperature Curve Name",
         "    10HeatingEIRFFCurve,      !- Speed 1 Energy Input Ratio Function of Air Flow Fraction Curve Name",
         "    10HeatingEIRFWCurve,      !- Speed 1 Energy Input Ratio Function of Water Flow Fraction Curve Name",
         "    0.0,                        !- Speed 1 Reference Unit Waste Heat Fraction of Input Power At Rated Conditions {dimensionless}",
         "    wasteHeat VS Temp1 Test,                        !- Speed 1 Waste Heat Function of Temperature Curve Name",
         "    7521.3759405,            !- Speed 2 Reference Unit Gross Rated Heating Capacity {w}",
         "    9.3549452,             !- Speed 2 Reference Unit Gross Rated Heating COP {dimensionless}",
         "    0.449293966,               !- Speed 2 Reference Unit Rated Air Flow Rate {m3/s}",
         "    0.0008201726 ,                  !- Speed 2 Reference Unit Rated Water Flow Rate {m3/s}",
         "    9HeatingCapacityCurve   ,!- Speed 2 Heating Capacity Function of Temperature Curve Name",
         "    9HeatingCAPFFCurve,      !- Speed 2 Total  Heating Capacity Function of Air Flow Fraction Curve Name",
         "    9HeatingCAPFWCurve,      !- Speed 2 Heating Capacity Function of Water Flow Fraction Curve Name",
         "    9HeatingPowerCurve    ,  !- Speed 2 Energy Input Ratio Function of Temperature Curve Name",
         "    9HeatingEIRFFCurve,      !- Speed 2 Energy Input Ratio Function of Air Flow Fraction Curve Name",
         "    9HeatingEIRFWCurve,      !- Speed 2 Energy Input Ratio Function of Water Flow Fraction Curve Name",
         "    0.0,                        !- Speed 2 Reference Unit Waste Heat Fraction of Input Power At Rated Conditions {dimensionless}",
         "    wasteHeat VS Temp1 Test,                        !- Speed 2 Waste Heat Function of Temperature Curve Name",
         "    8601.0497624,              !- Speed 3 Reference Unit Gross Rated Heating Capacity {w}",
         "    8.857929724,             !- Speed 3 Reference Unit Gross Rated Heating COP {dimensionless}",
         "    0.489881446,              !- Speed 3 Reference Unit Rated Air Flow Rate {m3/s}",
         "    0.0008201726 ,                  !- Speed 3 Reference Unit Rated Water Flow Rate {m3/s}",
         "    8HeatingCapacityCurve   ,!- Speed 3 Heating Capacity Function of Temperature Curve Name",
         "    8HeatingCAPFFCurve,      !- Speed 3 Total  Heating Capacity Function of Air Flow Fraction Curve Name",
         "    8HeatingCAPFWCurve,      !- Speed 3 Heating Capacity Function of Water Flow Fraction Curve Name",
         "    8HeatingPowerCurve    ,  !- Speed 3 Energy Input Ratio Function of Temperature Curve Name",
         "    8HeatingEIRFFCurve,      !- Speed 3 Energy Input Ratio Function of Air Flow Fraction Curve Name",
         "    8HeatingEIRFWCurve,      !- Speed 3 Energy Input Ratio Function of Water Flow Fraction Curve Name",
         "    0.0,                        !- Speed 3 Reference Unit Waste Heat Fraction of Input Power At Rated Conditions {dimensionless}",
         "    wasteHeat VS Temp1 Test,                        !- Speed 3 Waste Heat Function of Temperature Curve Name",
         "    9675.1552339,            !- Speed 4 Reference Unit Gross Rated Heating Capacity {w}",
         "    8.442543834,             !- Speed 4 Reference Unit Gross Rated Heating COP {dimensionless}",
         "    0.530468926,             !- Speed 4 Reference Unit Rated Air Flow Rate {m3/s}",
         "    0.0008201726 ,                  !- Speed 4 Reference Unit Rated Water Flow Rate {m3/s}",
         "    7HeatingCapacityCurve   ,!- Speed 4 Heating Capacity Function of Temperature Curve Name",
         "    7HeatingCAPFFCurve,      !- Speed 4 Total  Heating Capacity Function of Air Flow Fraction Curve Name",
         "    7HeatingCAPFWCurve,      !- Speed 4 Heating Capacity Function of Water Flow Fraction Curve Name",
         "    7HeatingPowerCurve    ,  !- Speed 4 Energy Input Ratio Function of Temperature Curve Name",
         "    7HeatingEIRFFCurve,      !- Speed 4 Energy Input Ratio Function of Air Flow Fraction Curve Name",
         "    7HeatingEIRFWCurve,      !- Speed 4 Energy Input Ratio Function of Water Flow Fraction Curve Name",
         "    0.0,                        !- Speed 4 Reference Unit Waste Heat Fraction of Input Power At Rated Conditions {dimensionless}",
         "    wasteHeat VS Temp1 Test,                        !- Speed 4 Waste Heat Function of Temperature Curve Name",
         "    10743.692355,            !- Speed 5 Reference Unit Gross Rated Heating Capacity {w}",
         "    8.090129785,             !- Speed 5 Reference Unit Gross Rated Heating COP {dimensionless}",
         "    0.571056406,             !- Speed 5 Reference Unit Rated Air Flow Rate {m3/s}",
         "    0.0008201726 ,                  !- Speed 5 Reference Unit Rated Water Flow Rate {m3/s}",
         "    6HeatingCapacityCurve   ,!- Speed 5 Heating Capacity Function of Temperature Curve Name",
         "    6HeatingCAPFFCurve,      !- Speed 5 Total  Heating Capacity Function of Air Flow Fraction Curve Name",
         "    6HeatingCAPFWCurve,      !- Speed 5 Heating Capacity Function of Water Flow Fraction Curve Name",
         "    6HeatingPowerCurve    ,  !- Speed 5 Energy Input Ratio Function of Temperature Curve Name",
         "    6HeatingEIRFFCurve,      !- Speed 5 Energy Input Ratio Function of Air Flow Fraction Curve Name",
         "    6HeatingEIRFWCurve,      !- Speed 5 Energy Input Ratio Function of Water Flow Fraction Curve Name",
         "    0.0,                        !- Speed 5 Reference Unit Waste Heat Fraction of Input Power At Rated Conditions {dimensionless}",
         "    wasteHeat VS Temp1 Test,                        !- Speed 5 Waste Heat Function of Temperature Curve Name",
         "    12861.716978,            !- Speed 6 Reference Unit Gross Rated Heating Capacity {w}",
         "    7.521471917,             !- Speed 6 Reference Unit Gross Rated Heating COP {dimensionless}",
         "    0.652231367,             !- Speed 6 Reference Unit Rated Air Flow Rate {m3/s}",
         "    0.0008201726 ,                  !- Speed 6 Reference Unit Rated Water Flow Rate {m3/s}",
         "    5HeatingCapacityCurve   ,!- Speed 6 Heating Capacity Function of Temperature Curve Name",
         "    5HeatingCAPFFCurve,      !- Speed 6 Total  Heating Capacity Function of Air Flow Fraction Curve Name",
         "    5HeatingCAPFWCurve,      !- Speed 6 Heating Capacity Function of Water Flow Fraction Curve Name",
         "    5HeatingPowerCurve    ,  !- Speed 6 Energy Input Ratio Function of Temperature Curve Name",
         "    5HeatingEIRFFCurve,      !- Speed 6 Energy Input Ratio Function of Air Flow Fraction Curve Name",
         "    5HeatingEIRFWCurve,      !- Speed 6 Energy Input Ratio Function of Water Flow Fraction Curve Name",
         "    0.0,                        !- Speed 6 Reference Unit Waste Heat Fraction of Input Power At Rated Conditions {dimensionless}",
         "    wasteHeat VS Temp1 Test,                        !- Speed 6 Waste Heat Function of Temperature Curve Name",
         "    14951.606778,            !- Speed 7 Reference Unit Gross Rated Heating Capacity {w}",
         "    7.072661674,             !- Speed 7 Reference Unit Gross Rated Heating COP {dimensionless}",
         "    0.732934379,             !- Speed 7 Reference Unit Rated Air Flow Rate {m3/s}",
         "    0.0008201726 ,                  !- Speed 7 Reference Unit Rated Water Flow Rate {m3/s}",
         "    4HeatingCapacityCurve   ,!- Speed 7 Heating Capacity Function of Temperature Curve Name",
         "    4HeatingCAPFFCurve,      !- Speed 7 Total  Heating Capacity Function of Air Flow Fraction Curve Name",
         "    4HeatingCAPFWCurve,      !- Speed 7 Heating Capacity Function of Water Flow Fraction Curve Name",
         "    4HeatingPowerCurve    ,  !- Speed 7 Energy Input Ratio Function of Temperature Curve Name",
         "    4HeatingEIRFFCurve,      !- Speed 7 Energy Input Ratio Function of Air Flow Fraction Curve Name",
         "    4HeatingEIRFWCurve,      !- Speed 7 Energy Input Ratio Function of Water Flow Fraction Curve Name",
         "    0.0,                        !- Speed 7 Reference Unit Waste Heat Fraction of Input Power At Rated Conditions {dimensionless}",
         "    wasteHeat VS Temp1 Test,                        !- Speed 7 Waste Heat Function of Temperature Curve Name",
         "    17011.8964,            !- Speed 8 Reference Unit Gross Rated Heating Capacity {w}",
         "    6.710807258,             !- Speed 8 Reference Unit Gross Rated Heating COP {dimensionless}",
         "    0.81410934,             !- Speed 8 Reference Unit Rated Air Flow Rate {m3/s}",
         "    0.0008201726 ,                  !- Speed 8 Reference Unit Rated Water Flow Rate {m3/s}",
         "    3HeatingCapacityCurve   ,!- Speed 8 Heating Capacity Function of Temperature Curve Name",
         "    3HeatingCAPFFCurve,      !- Speed 8 Total  Heating Capacity Function of Air Flow Fraction Curve Name",
         "    3HeatingCAPFWCurve,      !- Speed 8 Heating Capacity Function of Water Flow Fraction Curve Name",
         "    3HeatingPowerCurve    ,  !- Speed 8 Energy Input Ratio Function of Temperature Curve Name",
         "    3HeatingEIRFFCurve,      !- Speed 8 Energy Input Ratio Function of Air Flow Fraction Curve Name",
         "    3HeatingEIRFWCurve,      !- Speed 8 Energy Input Ratio Function of Water Flow Fraction Curve Name",
         "    0.0 ,                        !- Speed 8 Reference Unit Waste Heat Fraction of Input Power At Rated Conditions {dimensionless}",
         "    wasteHeat VS Temp1 Test,                        !- Speed 8 Waste Heat Function of Temperature Curve Name",
         "    20894.501936,            !- Speed 9 Reference Unit Gross Rated Heating Capacity {w}",
         "    5.89906887,               !- Speed 9 Reference Unit Gross Rated Heating COP {dimensionless}",
         "    0.891980668,             !- Speed 9 Reference Unit Rated Air Flow Rate {m3/s}",
         "    0.0008201726 ,                  !- Speed 9 Reference Unit Rated Water Flow Rate {m3/s}",
         "    1HeatingCapacityCurve   ,!- Speed 9 Heating Capacity Function of Temperature Curve Name",
         "    1HeatingCAPFFCurve,      !- Speed 9 Total  Heating Capacity Function of Air Flow Fraction Curve Name",
         "    1HeatingCAPFWCurve,      !- Speed 9 Heating Capacity Function of Water Flow Fraction Curve Name",
         "    1HeatingPowerCurve    ,  !- Speed 9 Energy Input Ratio Function of Temperature Curve Name",
         "    1HeatingEIRFFCurve,      !- Speed 9 Energy Input Ratio Function of Air Flow Fraction Curve Name",
         "    1HeatingEIRFWCurve,      !- Speed 9 Energy Input Ratio Function of Water Flow Fraction Curve Name",
         "    0.0,                        !- Speed 9 Reference Unit Waste Heat Fraction of Input Power At Rated Conditions {dimensionless}",
         "    wasteHeat VS Temp1 Test;                        !- Speed 9 Waste Heat Function of Temperature Curve Name",

         "  Fan:OnOff,",
         "    Lobby_ZN_1_FLR_2 WSHP Fan,              !- Name",
         "    ALWAYS_ON,    !- Availability Schedule Name",
         "    0.7,                     !- Fan Total Efficiency",
         "    113,                   !- Pressure Rise {Pa}",
         "    Autosize,                !- Maximum Flow Rate {m3/s}",
         "    0.9,                     !- Motor Efficiency",
         "    1.0,                     !- Motor In Airstream Fraction",
         "    Lobby_ZN_1_FLR_2 WSHP Inlet Node,    !- Air Inlet Node Name",
         "    Lobby_ZN_1_FLR_2 WSHP Cooling Coil Air Inlet Node,  !- Air Outlet Node Name",
         "    FanPowerCurve, !- Fan Efficiency Ratio Function of Speed Ratio Curve Name",
         "    ,",
         "    WSHP;",

         "  Curve:Exponent,",
         "    FanPowerCurve,      	     !- Name",
         "    0.254542407,              !- Coefficient1 Constant",
         "    0.837259009,              !- Coefficient2 x",
         "    3,                       !- Coefficient3 x**3",
         "    0.458,                 !- Minimum Value of x",
         "    1,                  !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "  Coil:Heating:Electric,",
         "    Lobby_ZN_1_FLR_2 WSHP Supp Heating Coil,  !- Name",
         "    ALWAYS_ON,    !- Availability Schedule Name",
         "    1.0,                     !- Gas Burner Efficiency",
         "    Autosize,                   !- Nominal Capacity {W}",
         "    Lobby_ZN_1_FLR_2 WSHP SuppHeating Coil Air Inlet Node,  !- Air Inlet Node Name",
         "    Lobby_ZN_1_FLR_2 WSHP Outlet Node;       !- Air Outlet Node Name",

         "Curve:Quadratic,",
         "    PLF Curve,               !- Name",
         "    1.0,                    !- Coefficient1 Constant",
         "    0.0,                    !- Coefficient2 x",
         "    0,                       !- Coefficient3 x**2",
         "    0,                       !- Minimum Value of x",
         "    1,                       !- Maximum Value of x",
         "    0.0,                        !- Minimum Curve Output",
         "    1.0,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    1CoolingCAPFFCurve,      !- Name",
         "    0.66189171,              !- Coefficient1 Constant",
         "    0.54317685,              !- Coefficient2 x",
         "    -.23204411,              !- Coefficient3 x**2",
         "    0.026975554,             !- Coefficient4 x**3",
         "    0.76402,                 !- Minimum Value of x",
         "    1.0947,                  !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    1CoolingEIRFFCurve,      !- Name",
         "    1.5,                     !- Coefficient1 Constant",
         "    -.974,                   !- Coefficient2 x",
         "    0.623,                   !- Coefficient3 x**2",
         "    -.149,                   !- Coefficient4 x**3",
         "    0.76402,                 !- Minimum Value of x",
         "    1.0947,                  !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    1CoolingCAPFWCurve,      !- Name",
         "    1.13,                    !- Coefficient1 Constant",
         "    -.449,                   !- Coefficient2 x",
         "    0.589,                   !- Coefficient3 x**2",
         "    -.268,                   !- Coefficient4 x**3",
         "    0.65385,                 !- Minimum Value of x",
         "    1.2,                     !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    1CoolingEIRFWCurve,      !- Name",
         "    1.48,                    !- Coefficient1 Constant",
         "    -.827,                   !- Coefficient2 x",
         "    0.497,                   !- Coefficient3 x**2",
         "    -.146,                   !- Coefficient4 x**3",
         "    0.65385,                 !- Minimum Value of x",
         "    1.2,                     !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    2CoolingCAPFFCurve,      !- Name",
         "    0.667,                   !- Coefficient1 Constant",
         "    0.533,                   !- Coefficient2 x",
         "    -.209,                   !- Coefficient3 x**2",
         "    0.00943,                 !- Coefficient4 x**3",
         "    0.69953,                 !- Minimum Value of x",
         "    1,                       !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    2CoolingEIRFFCurve,      !- Name",
         "    1.54,                    !- Coefficient1 Constant",
         "    -1.13,                   !- Coefficient2 x",
         "    0.801,                   !- Coefficient3 x**2",
         "    -.209,                   !- Coefficient4 x**3",
         "    0.69953,                 !- Minimum Value of x",
         "    1,                       !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    2CoolingCAPFWCurve,      !- Name",
         "    1.13,                    !- Coefficient1 Constant",
         "    -.449,                   !- Coefficient2 x",
         "    0.589,                   !- Coefficient3 x**2",
         "    -.268,                   !- Coefficient4 x**3",
         "    0.65385,                 !- Minimum Value of x",
         "    1.2,                     !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    2CoolingEIRFWCurve,      !- Name",
         "    1.48,                    !- Coefficient1 Constant",
         "    -.83,                    !- Coefficient2 x",
         "    0.496,                   !- Coefficient3 x**2",
         "    -.145,                   !- Coefficient4 x**3",
         "    0.65385,                 !- Minimum Value of x",
         "    1.2,                     !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    3CoolingCAPFFCurve,      !- Name",
         "    0.665,                   !- Coefficient1 Constant",
         "    0.538,                   !- Coefficient2 x",
         "    -.216,                   !- Coefficient3 x**2",
         "    0.0128,                  !- Coefficient4 x**3",
         "    0.70087,                 !- Minimum Value of x",
         "    1,                       !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    3CoolingEIRFFCurve ,     !- Name",
         "    1.57,                    !- Coefficient1 Constant",
         "    -1.22,                   !- Coefficient2 x",
         "    0.92,                    !- Coefficient3 x**2",
         "    -.261,                   !- Coefficient4 x**3",
         "    0.70087,                 !- Minimum Value of x",
         "    1,                       !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    3CoolingCAPFWCurve  ,    !- Name",
         "    1.13,                    !- Coefficient1 Constant",
         "    -.449,                   !- Coefficient2 x",
         "    0.589,                   !- Coefficient3 x**2",
         "    -.268,                   !- Coefficient4 x**3",
         "    0.65385,                 !- Minimum Value of x",
         "    1.2,                     !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    3CoolingEIRFWCurve,      !- Name",
         "    1.48,                    !- Coefficient1 Constant",
         "    -.829,                   !- Coefficient2 x",
         "    0.496,                   !- Coefficient3 x**2",
         "    -.145,                   !- Coefficient4 x**3",
         "    0.65385,                 !- Minimum Value of x",
         "    1.2,                     !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    4CoolingCAPFFCurve,      !- Name",
         "    0.644,                   !- Coefficient1 Constant",
         "    0.606,                   !- Coefficient2 x",
         "    -.291,                   !- Coefficient3 x**2",
         "    0.041,                   !- Coefficient4 x**3",
         "    0.70316,                 !- Minimum Value of x",
         "    1,                       !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    4CoolingEIRFFCurve,      !- Name",
         "    1.48,                    !- Coefficient1 Constant",
         "    -.907,                   !- Coefficient2 x",
         "    0.533,                   !- Coefficient3 x**2",
         "    -.106,                   !- Coefficient4 x**3",
         "    0.70316,                 !- Minimum Value of x",
         "    1,                       !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    4CoolingCAPFWCurve,      !- Name",
         "    1.13,                    !- Coefficient1 Constant",
         "    -.449,                   !- Coefficient2 x",
         "    0.589,                   !- Coefficient3 x**2",
         "    -.268,                   !- Coefficient4 x**3",
         "    0.65385,                 !- Minimum Value of x",
         "    1.2,                     !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    5CoolingCAPFFCurve ,     !- Name",
         "    0.662,                   !- Coefficient1 Constant",
         "    0.527,                   !- Coefficient2 x",
         "    -.185,                   !- Coefficient3 x**2",
         "    -.00437,                 !- Coefficient4 x**3",
         "    0.7055,                  !- Minimum Value of x",
         "    1,                       !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    5CoolingEIRFFCurve,      !- Name",
         "    1.58,                    !- Coefficient1 Constant",
         "    -1.23,                   !- Coefficient2 x",
         "    0.891,                   !- Coefficient3 x**2",
         "    -.24,                    !- Coefficient4 x**3",
         "    0.7055,                  !- Minimum Value of x",
         "    1,                       !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    5CoolingCAPFWCurve  ,    !- Name",
         "    1.13,                    !- Coefficient1 Constant",
         "    -.449,                   !- Coefficient2 x",
         "    0.589,                   !- Coefficient3 x**2",
         "    -.268,                   !- Coefficient4 x**3",
         "    0.65385,                 !- Minimum Value of x",
         "    1.2,                     !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    5CoolingEIRFWCurve,      !- Name",
         "    1.48,                    !- Coefficient1 Constant",
         "    -.832,                   !- Coefficient2 x",
         "    0.496,                   !- Coefficient3 x**2",
         "    -.144,                   !- Coefficient4 x**3",
         "    0.65385,                 !- Minimum Value of x",
         "    1.2,                     !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    6CoolingCAPFFCurve,      !- Name",
         "    0.625,                   !- Coefficient1 Constant",
         "    0.645,                   !- Coefficient2 x",
         "    -.313,                   !- Coefficient3 x**2",
         "    0.043,                   !- Coefficient4 x**3",
         "    0.70909,                 !- Minimum Value of x",
         "    1,                       !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    6CoolingEIRFFCurve,      !- Name",
         "    1.43,                    !- Coefficient1 Constant",
         "    -.682,                   !- Coefficient2 x",
         "    0.227,                   !- Coefficient3 x**2",
         "    0.0257,                  !- Coefficient4 x**3",
         "    0.70909,                 !- Minimum Value of x",
         "    1,                       !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    6CoolingCAPFWCurve,      !- Name",
         "    1.13,                    !- Coefficient1 Constant",
         "    -.449,                   !- Coefficient2 x",
         "    0.589,                   !- Coefficient3 x**2",
         "    -.268,                   !- Coefficient4 x**3",
         "    0.65385,                 !- Minimum Value of x",
         "    1.2,                     !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    6CoolingEIRFWCurve,      !- Name",
         "    1.48,                    !- Coefficient1 Constant",
         "    -.829,                   !- Coefficient2 x",
         "    0.496,                   !- Coefficient3 x**2",
         "    -.145,                   !- Coefficient4 x**3",
         "    0.65385,                 !- Minimum Value of x",
         "    1.2,                     !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    7CoolingCAPFFCurve,      !- Name",
         "    0.651,                   !- Coefficient1 Constant",
         "    0.552,                   !- Coefficient2 x",
         "    -.207,                   !- Coefficient3 x**2",
         "    0.00313,                 !- Coefficient4 x**3",
         "    0.71085,                 !- Minimum Value of x",
         "    1,                       !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    7CoolingEIRFFCurve,      !- Name",
         "    1.56,                    !- Coefficient1 Constant",
         "    -1.14,                   !- Coefficient2 x",
         "    0.763,                   !- Coefficient3 x**2",
         "    -.183,                   !- Coefficient4 x**3",
         "    0.71085,                 !- Minimum Value of x",
         "    1,                       !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    7CoolingCAPFWCurve,      !- Name",
         "    1.13,                    !- Coefficient1 Constant",
         "    -.449,                   !- Coefficient2 x",
         "    0.589,                   !- Coefficient3 x**2",
         "    -.268,                   !- Coefficient4 x**3",
         "    0.65385,                 !- Minimum Value of x",
         "    1.2,                     !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    7CoolingEIRFWCurve,      !- Name",
         "    1.48,                    !- Coefficient1 Constant",
         "    -.831,                   !- Coefficient2 x",
         "    0.496,                   !- Coefficient3 x**2",
         "    -.144,                   !- Coefficient4 x**3",
         "    0.65385,                 !- Minimum Value of x",
         "    1.2,                     !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    8CoolingCAPFFCurve,      !- Name",
         "    -7.41,                   !- Coefficient1 Constant",
         "    27.5,                    !- Coefficient2 x",
         "    -30.1,                   !- Coefficient3 x**2",
         "    11,                      !- Coefficient4 x**3",
         "    0.76397,                 !- Minimum Value of x",
         "    1,                       !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    8CoolingEIRFFCurve,      !- Name",
         "    10.9,                    !- Coefficient1 Constant",
         "    -32.2,                   !- Coefficient2 x",
         "    35.2,                    !- Coefficient3 x**2",
         "    -12.9,                   !- Coefficient4 x**3",
         "    0.76397,                 !- Minimum Value of x",
         "    1,                       !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    8CoolingCAPFWCurve,      !- Name",
         "    1.13,                    !- Coefficient1 Constant",
         "    -.449,                   !- Coefficient2 x",
         "    0.589,                   !- Coefficient3 x**2",
         "    -.268,                   !- Coefficient4 x**3",
         "    0.65385,                 !- Minimum Value of x",
         "    1.2,                     !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    8CoolingEIRFWCurve,      !- Name",
         "    1.48,                    !- Coefficient1 Constant",
         "    -.831,                   !- Coefficient2 x",
         "    0.496,                   !- Coefficient3 x**2",
         "    -.144,                   !- Coefficient4 x**3",
         "    0.65385,                 !- Minimum Value of x",
         "    1.2,                     !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    9CoolingCAPFFCurve,      !- Name",
         "    -.27,                    !- Coefficient1 Constant",
         "    1.28,                    !- Coefficient2 x",
         "    -.926,                   !- Coefficient3 x**2",
         "    0.913,                   !- Coefficient4 x**3",
         "    0.83298,                 !- Minimum Value of x",
         "    1,                       !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    9CoolingEIRFFCurve,      !- Name",
         "    -2.82,                   !- Coefficient1 Constant",
         "    5.09,                    !- Coefficient2 x",
         "    -.94,                    !- Coefficient3 x**2",
         "    -.331,                   !- Coefficient4 x**3",
         "    0.83298,                 !- Minimum Value of x",
         "    1,                       !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    9CoolingCAPFWCurve,      !- Name",
         "    1.13,                    !- Coefficient1 Constant",
         "    -.448,                   !- Coefficient2 x",
         "    0.589,                   !- Coefficient3 x**2",
         "    -.268,                   !- Coefficient4 x**3",
         "    0.65385,                 !- Minimum Value of ",
         "    1.2,                     !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    9CoolingEIRFWCurve,      !- Name",
         "    1.48,                    !- Coefficient1 Constant",
         "    -.831,                   !- Coefficient2 x",
         "    0.496,                   !- Coefficient3 x**2",
         "    -.144,                   !- Coefficient4 x**3",
         "    0.65385,                 !- Minimum Value of x",
         "    1.2,                     !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    10CoolingCAPFFCurve,     !- Name",
         "    -67.4,                   !- Coefficient1 Constant",
         "    74,                      !- Coefficient2 x",
         "    68.6,                    !- Coefficient3 x**2",
         "    -74.2,                   !- Coefficient4 x**3",
         "    0.9157,                  !- Minimum Value of x",
         "    1,                       !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    10CoolingEIRFFCurve,     !- Name",
         "    86.5,                    !- Coefficient1 Constant",
         "    -96.4,                   !- Coefficient2 x",
         "    -77.9,                   !- Coefficient3 x**2",
         "    88.8,                    !- Coefficient4 x**3",
         "    0.9157,                  !- Minimum Value of x",
         "    1,                       !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    10CoolingCAPFWCurve,     !- Name",
         "    1.13,                    !- Coefficient1 Constant",
         "    -.449,                   !- Coefficient2 x",
         "    0.589,                   !- Coefficient3 x**2",
         "    -.268,                   !- Coefficient4 x**3",
         "    0.65385,                 !- Minimum Value of x",
         "    1.2,                     !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    10CoolingEIRFWCurve,     !- Name",
         "    1.47,                    !- Coefficient1 Constant",
         "    -.816,                   !- Coefficient2 x",
         "    0.498,                   !- Coefficient3 x**2",
         "    -.151,                   !- Coefficient4 x**3",
         "    0.65385,                 !- Minimum Value of x",
         "    1.2,                     !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    1HeatingCAPFFCurve,      !- Name",
         "    0.721,                   !- Coefficient1 Constant",
         "    0.622,                   !- Coefficient2 x",
         "    -.483,                   !- Coefficient3 x**2",
         "    0.141,                   !- Coefficient4 x**3",
         "    0.76402,                 !- Minimum Value of x",
         "    1.0947,                  !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    1HeatingEIRFFCurve,      !- Name",
         "    2.82,                    !- Coefficient1 Constant",
         "    -3.86,                   !- Coefficient2 x",
         "    2.75,                    !- Coefficient3 x**2",
         "    -.711,                   !- Coefficient4 x**3",
         "    0.76402,                 !- Minimum Value of x",
         "    1.0947,                  !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    1HeatingCAPFWCurve,      !- Name",
         "    1.08,                    !- Coefficient1 Constant",
         "    -.337,                   !- Coefficient2 x",
         "    0.278,                   !- Coefficient3 x**2",
         "    -.026,                   !- Coefficient4 x**3",
         "    0.65385,                 !- Minimum Value of x",
         "    1.2,                     !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    1HeatingEIRFWCurve,      !- Name",
         "    1.38,                    !- Coefficient1 Constant",
         "    -.386,                   !- Coefficient2 x",
         "    -.257,                   !- Coefficient3 x**2",
         "    0.261,                   !- Coefficient4 x**3",
         "    0.65385,                 !- Minimum Value of x",
         "    1.2,                     !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    2HeatingCAPFFCurve,      !- Name",
         "    0.722,                   !- Coefficient1 Constant",
         "    0.653,                   !- Coefficient2 x",
         "    -.546,                   !- Coefficient3 x**2",
         "    0.172,                   !- Coefficient4 x**3",
         "    0.69953,                 !- Minimum Value of x",
         "    1,                       !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    2HeatingEIRFFCurve,      !- Name",
         "    2.89,                    !- Coefficient1 Constant",
         "    -4.24,                   !- Coefficient2 x",
         "    3.25,                    !- Coefficient3 x**2",
         "    -.901,                   !- Coefficient4 x**3",
         "    0.69953,                 !- Minimum Value of x",
         "    1,                       !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    2HeatingCAPFWCurve,      !- Name",
         "    0.983,                   !- Coefficient1 Constant",
         "    -.303,                   !- Coefficient2 x",
         "    0.629,                   !- Coefficient3 x**2",
         "    -.308,                   !- Coefficient4 x**3",
         "    0.65385,                 !- Minimum Value of x",
         "    1.2,                     !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    2HeatingEIRFWCurve,      !- Name",
         "    1.34,                    !- Coefficient1 Constant",
         "    -.677,                   !- Coefficient2 x",
         "    0.533,                   !- Coefficient3 x**2",
         "    -.196,                   !- Coefficient4 x**3",
         "    0.65385,                 !- Minimum Value of x",
         "    1.2,                     !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    3HeatingCAPFFCurve,      !- Name",
         "    0.719,                   !- Coefficient1 Constant",
         "    0.662,                   !- Coefficient2 x",
         "    -.557,                   !- Coefficient3 x**2",
         "    0.176,                   !- Coefficient4 x**3",
         "    0.70087,                 !- Minimum Value of x",
         "    1,                       !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    3HeatingEIRFFCurve,      !- Name",
         "    2.96,                    !- Coefficient1 Constant",
         "    -4.44,                   !- Coefficient2 x",
         "    3.47,                    !- Coefficient3 x**2",
         "    -.982,                   !- Coefficient4 x**3",
         "    0.70087,                 !- Minimum Value of x",
         "    1,                       !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    3HeatingCAPFWCurve,      !- Name",
         "    0.983,                   !- Coefficient1 Constant",
         "    -.303,                   !- Coefficient2 x",
         "    0.629,                   !- Coefficient3 x**2",
         "    -.308,                   !- Coefficient4 x**3",
         "    0.65385,                 !- Minimum Value of x",
         "    1.2,                     !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    3HeatingEIRFWCurve,      !- Name",
         "    1.34,                    !- Coefficient1 Constant",
         "    -.677,                   !- Coefficient2 x",
         "    0.533,                   !- Coefficient3 x**2",
         "    -.195,                   !- Coefficient4 x**3",
         "    0.65385,                 !- Minimum Value of x",
         "    1.2,                     !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    4HeatingCAPFFCurve,      !- Name",
         "    0.706,                   !- Coefficient1 Constant",
         "    0.703,                   !- Coefficient2 x",
         "    -.602,                   !- Coefficient3 x**2",
         "    0.193,                   !- Coefficient4 x**3",
         "    0.70316,                 !- Minimum Value of x",
         "    1,                       !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    4HeatingEIRFFCurve,      !- Name",
         "    3.09,                    !- Coefficient1 Constant",
         "    -4.9,                    !- Coefficient2 x",
         "    3.99,                    !- Coefficient3 x**2",
         "    -1.18,                   !- Coefficient4 x**3",
         "    0.70316,                 !- Minimum Value of x",
         "    1,                       !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    4HeatingEIRFWCurve,      !- Name",
         "    1.34,                    !- Coefficient1 Constant",
         "    -.675,                   !- Coefficient2 x",
         "    0.533,                   !- Coefficient3 x**2",
         "    -.197,                   !- Coefficient4 x**3",
         "    0.65385,                 !- Minimum Value of x",
         "    1.2,                     !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    4HeatingCAPFWCurve,      !- Name",
         "    0.983,                   !- Coefficient1 Constant",
         "    -.304,                   !- Coefficient2 x",
         "    0.629,                   !- Coefficient3 x**2",
         "    -.308,                   !- Coefficient4 x**3",
         "    0.65385,                 !- Minimum Value of x",
         "    1.2,                     !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    5HeatingCAPFFCurve,      !- Name",
         "    0.706,                   !- Coefficient1 Constant",
         "    0.694,                   !- Coefficient2 x",
         "    -.583,                   !- Coefficient3 x**2",
         "    0.183,                   !- Coefficient4 x**3",
         "    0.7055,                  !- Minimum Value of x",
         "    1,                       !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    5HeatingEIRFFCurve,      !- Name",
         "    3.1,                     !- Coefficient1 Constant",
         "    -4.87,                   !- Coefficient2 x",
         "    3.91,                    !- Coefficient3 x**2",
         "    -1.14,                   !- Coefficient4 x**3",
         "    0.7055,                  !- Minimum Value of x",
         "    1,                       !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    5HeatingCAPFWCurve,      !- Name",
         "    0.983,                   !- Coefficient1 Constant",
         "    -.303,                   !- Coefficient2 x",
         "    0.629,                   !- Coefficient3 x**2",
         "    -.309,                   !- Coefficient4 x**3",
         "    0.65385,                 !- Minimum Value of x",
         "    1.2,                     !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    5HeatingEIRFWCurve,      !- Name",
         "    1.34,                    !- Coefficient1 Constant",
         "    -.677,                   !- Coefficient2 x",
         "    0.533,                   !- Coefficient3 x**2",
         "    -.196,                   !- Coefficient4 x**3",
         "    0.65385,                 !- Minimum Value of x",
         "    1.2,                     !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    6HeatingCAPFFCurve,      !- Name",
         "    0.69,                    !- Coefficient1 Constant",
         "    0.743,                   !- Coefficient2 x",
         "    -.635,                   !- Coefficient3 x**2",
         "    0.201,                   !- Coefficient4 x**3",
         "    0.70909,                 !- Minimum Value of x",
         "    1,                       !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    6HeatingEIRFFCurve,      !- Name",
         "    3.14,                    !- Coefficient1 Constant",
         "    -4.92,                   !- Coefficient2 x",
         "    3.88,                    !- Coefficient3 x**2",
         "    -1.1,                    !- Coefficient4 x**3",
         "    0.70909,                 !- Minimum Value of x",
         "    1,                       !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    6HeatingCAPFWCurve,      !- Name",
         "    0.983,                   !- Coefficient1 Constant",
         "    -.303,                   !- Coefficient2 x",
         "    0.629,                   !- Coefficient3 x**2",
         "    -.308,                   !- Coefficient4 x**3",
         "    0.65385,                 !- Minimum Value of x",
         "    1.2,                     !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    6HeatingEIRFWCurve,      !- Name",
         "    1.34,                    !- Coefficient1 Constant",
         "    -.676,                   !- Coefficient2 x",
         "    0.533,                   !- Coefficient3 x**2",
         "    -.196,                   !- Coefficient4 x**3",
         "    0.65385,                 !- Minimum Value of x",
         "    1.2,                     !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    7HeatingCAPFFCurve,      !- Name",
         "    0.708,                   !- Coefficient1 Constant",
         "    0.678,                   !- Coefficient2 x",
         "    -.558,                   !- Coefficient3 x**2",
         "    0.172,                   !- Coefficient4 x**3",
         "    0.71085,                 !- Minimum Value of x",
         "    1,                       !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    7HeatingEIRFFCurve,      !- Name",
         "    3.01,                    !- Coefficient1 Constant",
         "    -4.47,                   !- Coefficient2 x",
         "    3.37,                    !- Coefficient3 x**2",
         "    -.91,                    !- Coefficient4 x**3",
         "    0.71085,                 !- Minimum Value of x",
         "    1.2,                     !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    7HeatingCAPFWCurve,      !- Name",
         "    0.983,                   !- Coefficient1 Constant",
         "    -.304,                   !- Coefficient2 x",
         "    0.629,                   !- Coefficient3 x**2",
         "    -.308,                   !- Coefficient4 x**3",
         "    0.65385,                 !- Minimum Value of x",
         "    1.2,                     !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    7HeatingEIRFWCurve,      !- Name",
         "    1.34,                    !- Coefficient1 Constant",
         "    -.674,                   !- Coefficient2 x",
         "    0.533,                   !- Coefficient3 x**2",
         "    -.197,                   !- Coefficient4 x**3",
         "    0.65385,                 !- Minimum Value of x",
         "    1.2,                     !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    8HeatingCAPFFCurve,      !- Name",
         "    -3.68,                   !- Coefficient1 Constant",
         "    15.3,                    !- Coefficient2 x",
         "    -16.8,                   !- Coefficient3 x**2",
         "    6.17,                    !- Coefficient4 x**3",
         "    0.76397,                 !- Minimum Value of x",
         "    1,                       !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    8HeatingEIRFFCurve,      !- Name",
         "    34.898344,               !- Coefficient1 Constant",
         "    -111.15261,              !- Coefficient2 x",
         "    121.80976,               !- Coefficient3 x**2",
         "    -44.555495,              !- Coefficient4 x**3",
         "    0.76397,                 !- Minimum Value of x",
         "    1,                       !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    8HeatingCAPFWCurve,      !- Name",
         "    0.983,                   !- Coefficient1 Constant",
         "    -.303,                   !- Coefficient2 x",
         "    0.629,                   !- Coefficient3 x**2",
         "    -.309,                   !- Coefficient4 x**3",
         "    0.65385,                 !- Minimum Value of x",
         "    1.2,                     !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    8HeatingEIRFWCurve,      !- Name",
         "    1.34,                    !- Coefficient1 Constant",
         "    -.678,                   !- Coefficient2 x",
         "    0.533,                   !- Coefficient3 x**2",
         "    -.195,                   !- Coefficient4 x**3",
         "    0.65385,                 !- Minimum Value of x",
         "    1.2,                     !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    9HeatingCAPFFCurve,      !- Name",
         "    -.872,                   !- Coefficient1 Constant",
         "    2.18,                    !- Coefficient2 x",
         "    -.93,                    !- Coefficient3 x**2",
         "    0.621,                   !- Coefficient4 x**3",
         "    0.83298,                 !- Minimum Value of x",
         "    1,                       !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    9HeatingEIRFFCurve,      !- Name",
         "    -5.4,                    !- Coefficient1 Constant",
         "    8.97,                    !- Coefficient2 x",
         "    -.946,                   !- Coefficient3 x**2",
         "    -1.62,                   !- Coefficient4 x**3",
         "    0.83298,                 !- Minimum Value of x",
         "    1,                       !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    9HeatingCAPFWCurve,      !- Name",
         "    0.983,                   !- Coefficient1 Constant",
         "    -.303,                   !- Coefficient2 x",
         "    0.629,                   !- Coefficient3 x**2",
         "    -.308,                   !- Coefficient4 x**3",
         "    0.65385,                 !- Minimum Value of x",
         "    1.2,                     !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    9HeatingEIRFWCurve,      !- Name",
         "    1.34,                    !- Coefficient1 Constant",
         "    -.683,                   !- Coefficient2 x",
         "    0.532,                   !- Coefficient3 x**2",
         "    -.193,                   !- Coefficient4 x**3",
         "    0.65385,                 !- Minimum Value of x",
         "    1.2,                     !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    10HeatingCAPFFCurve,     !- Name",
         "    -32.7,                   !- Coefficient1 Constant",
         "    35.6,                    !- Coefficient2 x",
         "    35.6,                    !- Coefficient3 x**2",
         "    -37.5,                   !- Coefficient4 x**3",
         "    0.9157,                  !- Minimum Value of x",
         "    1,                       !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    10HeatingEIRFFCurve,     !- Name",
         "    249.59347,               !- Coefficient1 Constant",
         "    -277.02158,              !- Coefficient2 x",
         "    -233.19952,              !- Coefficient3 x**2",
         "    261.62763,               !- Coefficient4 x**3",
         "    0.9157,                  !- Minimum Value of x",
         "    1,                       !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    10HeatingCAPFWCurve,     !- Name",
         "    0.983,                   !- Coefficient1 Constant",
         "    -.303,                   !- Coefficient2 x",
         "    0.629,                   !- Coefficient3 x**2",
         "    -.308,                   !- Coefficient4 x**3",
         "    0.65385,                 !- Minimum Value of x",
         "    1.2,                     !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    10HeatingEIRFWCurve,     !- Name",
         "    1.34,                    !- Coefficient1 Constant",
         "    -.684,                   !- Coefficient2 x",
         "    0.532,                   !- Coefficient3 x**2",
         "    -.192,                   !- Coefficient4 x**3",
         "    0.65385,                 !- Minimum Value of x",
         "    1.2,                     !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    4CoolingEIRFWCurve,      !- Name",
         "    1.48,                    !- Coefficient1 Constant",
         "    -.828,                   !- Coefficient2 x",
         "    0.496,                   !- Coefficient3 x**2",
         "    -.146,                   !- Coefficient4 x**3",
         "    0.65385,                 !- Minimum Value of x",
         "    1.2,                     !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Biquadratic,",
         "    1CoolingCapacityCurve     ,  !- Name",
         "    0.9562913,               !- Coefficient1 Constant",
         "    -.00328,                 !- Coefficient2 x",
         "    0.000787,                !- Coefficient3 x**2",
         "    -.00126,                 !- Coefficient4 y",
         "    -.0000706,               !- Coefficient5 y**2",
         "    -.00017,                 !- Coefficient6 x*y",
         "    10,                      !- Minimum Value of x",
         "    25.55556,                !- Maximum Value of x",
         "    7.2222,                  !- Minimum Value of y",
         "    48.88889,                !- Maximum Value of y",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Temperature,             !- Input Unit Type for X",
         "    Temperature,             !- Input Unit Type for Y",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Biquadratic,",
         "    2CoolingCapacityCurve,   !- Name",
         "    0.9562913,               !- Coefficient1 Constant",
         "    -.00328,                 !- Coefficient2 x",
         "    0.000787,                !- Coefficient3 x**2",
         "    -.00126,                 !- Coefficient4 y",
         "    -.0000706,               !- Coefficient5 y**2",
         "    -.00017,                 !- Coefficient6 x*y",
         "    10,                      !- Minimum Value of x",
         "    25.55556,                !- Maximum Value of x",
         "    7.2222,                  !- Minimum Value of y",
         "    48.88889,                !- Maximum Value of y",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Temperature,             !- Input Unit Type for X",
         "    Temperature,             !- Input Unit Type for Y",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Biquadratic,",
         "    3CoolingCapacityCurve,   !- Name",
         "    0.9562913,               !- Coefficient1 Constant",
         "    -.00328,                 !- Coefficient2 x",
         "    0.000787,                !- Coefficient3 x**2",
         "    -.00126,                 !- Coefficient4 y",
         "    -.0000706,               !- Coefficient5 y**2",
         "    -.00017,                 !- Coefficient6 x*y",
         "    10,                      !- Minimum Value of x",
         "    25.55556,                !- Maximum Value of x",
         "    7.2222,                  !- Minimum Value of y",
         "    48.88889,                !- Maximum Value of y",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Temperature,             !- Input Unit Type for X",
         "    Temperature,             !- Input Unit Type for Y",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Biquadratic,",
         "    4CoolingCapacityCurve,   !- Name",
         "    0.9562913,               !- Coefficient1 Constant",
         "    -.00328,                 !- Coefficient2 x",
         "    0.000787,                !- Coefficient3 x**2",
         "    -.00126,                 !- Coefficient4 y",
         "    -.0000706,               !- Coefficient5 y**2",
         "    -.00017,                 !- Coefficient6 x*y",
         "    10,                      !- Minimum Value of x",
         "    25.55556,                !- Maximum Value of x",
         "    7.2222,                  !- Minimum Value of y",
         "    48.88889,                !- Maximum Value of y",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Temperature,             !- Input Unit Type for X",
         "    Temperature,             !- Input Unit Type for Y",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Biquadratic,",
         "    5CoolingCapacityCurve,   !- Name",
         "    0.9562913,               !- Coefficient1 Constant",
         "    -.00328,                 !- Coefficient2 x",
         "    0.000787,                !- Coefficient3 x**2",
         "    -.00126,                 !- Coefficient4 y",
         "    -.0000706,               !- Coefficient5 y**2",
         "    -.00017,                 !- Coefficient6 x*y",
         "    10,                      !- Minimum Value of x",
         "    25.55556,                !- Maximum Value of x",
         "    7.2222,                  !- Minimum Value of y",
         "    48.88889,                !- Maximum Value of y",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Temperature,             !- Input Unit Type for X",
         "    Temperature,             !- Input Unit Type for Y",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Biquadratic,",
         "    6CoolingCapacityCurve,   !- Name",
         "    0.9562913,               !- Coefficient1 Constant",
         "    -.00328,                 !- Coefficient2 x",
         "    0.000787,                !- Coefficient3 x**2",
         "    -.00126,                 !- Coefficient4 y",
         "    -.0000706,               !- Coefficient5 y**2",
         "    -.00017,                 !- Coefficient6 x*y",
         "    10,                      !- Minimum Value of x",
         "    25.55556,                !- Maximum Value of x",
         "    7.2222,                  !- Minimum Value of y",
         "    48.88889,                !- Maximum Value of y",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Temperature,             !- Input Unit Type for X",
         "    Temperature,             !- Input Unit Type for Y",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Biquadratic,",
         "    7CoolingCapacityCurve,   !- Name",
         "    0.9562913,               !- Coefficient1 Constant",
         "    -.00328,                 !- Coefficient2 x",
         "    0.000787,                !- Coefficient3 x**2",
         "    -.00126,                 !- Coefficient4 y",
         "    -.0000706,               !- Coefficient5 y**2",
         "    -.00017,                 !- Coefficient6 x*y",
         "    10,                      !- Minimum Value of x",
         "    25.55556,                !- Maximum Value of x",
         "    7.2222,                  !- Minimum Value of y",
         "    48.88889,                !- Maximum Value of y",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Temperature,             !- Input Unit Type for X",
         "    Temperature,             !- Input Unit Type for Y",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Biquadratic,",
         "    8CoolingCapacityCurve,   !- Name",
         "    0.9562913,               !- Coefficient1 Constant",
         "    -.00328,                 !- Coefficient2 x",
         "    0.000787,                !- Coefficient3 x**2",
         "    -.00126,                 !- Coefficient4 y",
         "    -.0000706,               !- Coefficient5 y**2",
         "    -.00017,                 !- Coefficient6 x*y",
         "    10,                      !- Minimum Value of x",
         "    25.55556,                !- Maximum Value of x",
         "    7.2222,                  !- Minimum Value of y",
         "    48.88889,                !- Maximum Value of y",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Temperature,             !- Input Unit Type for X",
         "    Temperature,             !- Input Unit Type for Y",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Biquadratic,",
         "    9CoolingCapacityCurve,   !- Name",
         "    0.9562913,               !- Coefficient1 Constant",
         "    -.00328,                 !- Coefficient2 x",
         "    0.000787,                !- Coefficient3 x**2",
         "    -.00126,                 !- Coefficient4 y",
         "    -.0000706,               !- Coefficient5 y**2",
         "    -.00017,                 !- Coefficient6 x*y",
         "    10,                      !- Minimum Value of x",
         "    25.55556,                !- Maximum Value of x",
         "    7.2222,                  !- Minimum Value of y",
         "    48.88889,                !- Maximum Value of y",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Temperature,             !- Input Unit Type for X",
         "    Temperature,             !- Input Unit Type for Y",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Biquadratic,",
         "    10CoolingCapacityCurve,  !- Name",
         "    0.9801088,               !- Coefficient1 Constant",
         "    -.00216,                 !- Coefficient2 x",
         "    0.000774,                !- Coefficient3 x**2",
         "    -.00274,                 !- Coefficient4 y",
         "    -.000046,                !- Coefficient5 y**2",
         "    -.000182,                !- Coefficient6 x*y",
         "    10,                      !- Minimum Value of x",
         "    25.55556,                !- Maximum Value of x",
         "    12.77778,                !- Minimum Value of y",
         "    48.88889,                !- Maximum Value of y",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Temperature,             !- Input Unit Type for X",
         "    Temperature,             !- Input Unit Type for Y",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Biquadratic,",
         "    1CoolingPowerCurve   ,   !- Name",
         "    0.3166164,               !- Coefficient1 Constant",
         "    0.01101735,              !- Coefficient2 x",
         "    -.0003322144,            !- Coefficient3 x**2",
         "    0.01849558,              !- Coefficient4 y",
         "    0.0006097517,            !- Coefficient5 y**2",
         "    -.0008357081,            !- Coefficient6 x*y",
         "    10,                      !- Minimum Value of x",
         "    25.55556,                !- Maximum Value of x",
         "    7.2222,                  !- Minimum Value of y",
         "    48.88889,                !- Maximum Value of y",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Temperature,             !- Input Unit Type for X",
         "    Temperature,             !- Input Unit Type for Y",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Biquadratic,",
         "    2CoolingPowerCurve   ,   !- Name",
         "    0.3166164,               !- Coefficient1 Constant",
         "    0.01101735,              !- Coefficient2 x",
         "    -.0003322144,            !- Coefficient3 x**2",
         "    0.01849558,              !- Coefficient4 y",
         "    0.0006097517,            !- Coefficient5 y**2",
         "    -.0008357081,            !- Coefficient6 x*y",
         "    10,                      !- Minimum Value of x",
         "    25.55556,                !- Maximum Value of x",
         "    7.2222,                  !- Minimum Value of y",
         "    48.88889,                !- Maximum Value of y",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Temperature,             !- Input Unit Type for X",
         "    Temperature,             !- Input Unit Type for Y",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Biquadratic,",
         "    3CoolingPowerCurve   ,   !- Name",
         "    0.3166164,               !- Coefficient1 Constant",
         "    0.01101735,              !- Coefficient2 x",
         "    -.0003322144,            !- Coefficient3 x**2",
         "    0.01849558,              !- Coefficient4 y",
         "    0.0006097517,            !- Coefficient5 y**2",
         "    -.0008357081,            !- Coefficient6 x*y",
         "    10,                      !- Minimum Value of x",
         "    25.55556,                !- Maximum Value of x",
         "    7.2222,                  !- Minimum Value of y",
         "    48.88889,                !- Maximum Value of y",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Temperature,             !- Input Unit Type for X",
         "    Temperature,             !- Input Unit Type for Y",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Biquadratic,",
         "    4CoolingPowerCurve   ,   !- Name",
         "    0.3166164,               !- Coefficient1 Constant",
         "    0.01101735,              !- Coefficient2 x",
         "    -.0003322144,            !- Coefficient3 x**2",
         "    0.01849558,              !- Coefficient4 y",
         "    0.0006097517,            !- Coefficient5 y**2",
         "    -.0008357081,            !- Coefficient6 x*y",
         "    10,                      !- Minimum Value of x",
         "    25.55556,                !- Maximum Value of x",
         "    7.2222,                  !- Minimum Value of y",
         "    48.88889,                !- Maximum Value of y",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Temperature,             !- Input Unit Type for X",
         "    Temperature,             !- Input Unit Type for Y",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Biquadratic,",
         "    5CoolingPowerCurve   ,   !- Name",
         "    0.3166164,               !- Coefficient1 Constant",
         "    0.01101735,              !- Coefficient2 x",
         "    -.0003322144,            !- Coefficient3 x**2",
         "    0.01849558,              !- Coefficient4 y",
         "    0.0006097517,            !- Coefficient5 y**2",
         "    -.0008357081,            !- Coefficient6 x*y",
         "    10,                      !- Minimum Value of x",
         "    25.55556,                !- Maximum Value of x",
         "    7.2222,                  !- Minimum Value of y",
         "    48.88889,                !- Maximum Value of y",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Temperature,             !- Input Unit Type for X",
         "    Temperature,             !- Input Unit Type for Y",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Biquadratic,",
         "    6CoolingPowerCurve   ,   !- Name",
         "    0.3166164,               !- Coefficient1 Constant",
         "    0.01101735,              !- Coefficient2 x",
         "    -.0003322144,            !- Coefficient3 x**2",
         "    0.01849558,              !- Coefficient4 y",
         "    0.0006097517,            !- Coefficient5 y**2",
         "    -.0008357081,            !- Coefficient6 x*y",
         "    10,                      !- Minimum Value of x",
         "    25.55556,                !- Maximum Value of x",
         "    7.2222,                  !- Minimum Value of y",
         "    48.88889,                !- Maximum Value of y",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Temperature,             !- Input Unit Type for X",
         "    Temperature,             !- Input Unit Type for Y",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Biquadratic,",
         "    7CoolingPowerCurve   ,   !- Name",
         "    0.3166164,               !- Coefficient1 Constant",
         "    0.01101735,              !- Coefficient2 x",
         "    -.0003322144,            !- Coefficient3 x**2",
         "    0.01849558,              !- Coefficient4 y",
         "    0.0006097517,            !- Coefficient5 y**2",
         "    -.0008357081,            !- Coefficient6 x*y",
         "    10,                      !- Minimum Value of x",
         "    25.55556,                !- Maximum Value of x",
         "    7.2222,                  !- Minimum Value of y",
         "    48.88889,                !- Maximum Value of y",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Temperature,             !- Input Unit Type for X",
         "    Temperature,             !- Input Unit Type for Y",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Biquadratic,",
         "    8CoolingPowerCurve   ,   !- Name",
         "    0.3166164,               !- Coefficient1 Constant",
         "    0.01101735,              !- Coefficient2 x",
         "    -.0003322144,            !- Coefficient3 x**2",
         "    0.01849558,              !- Coefficient4 y",
         "    0.0006097517,            !- Coefficient5 y**2",
         "    -.0008357081,            !- Coefficient6 x*y",
         "    10,                      !- Minimum Value of x",
         "    25.55556,                !- Maximum Value of x",
         "    7.2222,                  !- Minimum Value of y",
         "    48.88889,                !- Maximum Value of y",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Temperature,             !- Input Unit Type for X",
         "    Temperature,             !- Input Unit Type for Y",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Biquadratic,",
         "    9CoolingPowerCurve   ,   !- Name",
         "    0.3166164,               !- Coefficient1 Constant",
         "    0.01101735,              !- Coefficient2 x",
         "    -.0003322144,            !- Coefficient3 x**2",
         "    0.01849558,              !- Coefficient4 y",
         "    0.0006097517,            !- Coefficient5 y**2",
         "    -.0008357081,            !- Coefficient6 x*y",
         "    10,                      !- Minimum Value of x",
         "    25.55556,                !- Maximum Value of x",
         "    7.2222,                  !- Minimum Value of y",
         "    48.88889,                !- Maximum Value of y",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Temperature,             !- Input Unit Type for X",
         "    Temperature,             !- Input Unit Type for Y",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Biquadratic,",
         "    10CoolingPowerCurve   ,  !- Name",
         "    0.30928,                 !- Coefficient1 Constant",
         "    0.0145872,               !- Coefficient2 x",
         "    -.00035964,              !- Coefficient3 x**2",
         "    0.0168408,               !- Coefficient4 y",
         "    0.00065448,              !- Coefficient5 y**2",
         "    -.0009104399,            !- Coefficient6 x*y",
         "    10,                      !- Minimum Value of x",
         "    25.55556,                !- Maximum Value of x",
         "    12.77778,                !- Minimum Value of y",
         "    48.88889,                !- Maximum Value of y",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Temperature,             !- Input Unit Type for X",
         "    Temperature,             !- Input Unit Type for Y",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Biquadratic,",
         "    1HeatingCapacityCurve   ,!- Name",
         "    0.625,                   !- Coefficient1 Constant",
         "    -.00189,                 !- Coefficient2 x",
         "    0.00000058,              !- Coefficient3 x**2",
         "    0.0191,                  !- Coefficient4 y",
         "    0.0000891,               !- Coefficient5 y**2",
         "    -.0000651,               !- Coefficient6 x*y",
         "    12.8,                    !- Minimum Value of x",
         "    26.66667,                !- Maximum Value of x",
         "    -3.88889,                !- Minimum Value of y",
         "    29.44444,                !- Maximum Value of y",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Temperature,             !- Input Unit Type for X",
         "    Temperature,             !- Input Unit Type for Y",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Biquadratic,",
         "    2HeatingCapacityCurve   ,!- Name",
         "    0.625,                   !- Coefficient1 Constant",
         "    -.00189,                 !- Coefficient2 x",
         "    0.00000058,              !- Coefficient3 x**2",
         "    0.0191,                  !- Coefficient4 y",
         "    0.0000891,               !- Coefficient5 y**2",
         "    -.0000651,               !- Coefficient6 x*y",
         "    12.8,                    !- Minimum Value of x",
         "    26.66667,                !- Maximum Value of x",
         "    -3.88889,                !- Minimum Value of y",
         "    29.44444,                !- Maximum Value of y",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Temperature,             !- Input Unit Type for X",
         "    Temperature,             !- Input Unit Type for Y",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Biquadratic,",
         "    3HeatingCapacityCurve   ,!- Name",
         "    0.625,                   !- Coefficient1 Constant",
         "    -.00189,                 !- Coefficient2 x",
         "    0.00000058,              !- Coefficient3 x**2",
         "    0.0191,                  !- Coefficient4 y",
         "    0.0000891,               !- Coefficient5 y**2",
         "    -.0000651,               !- Coefficient6 x*y",
         "    12.8,                    !- Minimum Value of x",
         "    26.66667,                !- Maximum Value of x",
         "    -3.88889,                !- Minimum Value of y",
         "    29.44444,                !- Maximum Value of y",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Temperature,             !- Input Unit Type for X",
         "    Temperature,             !- Input Unit Type for Y",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Biquadratic,",
         "    4HeatingCapacityCurve   ,!- Name",
         "    0.625,                   !- Coefficient1 Constant",
         "    -.00189,                 !- Coefficient2 x",
         "    0.00000058,              !- Coefficient3 x**2",
         "    0.0191,                  !- Coefficient4 y",
         "    0.0000891,               !- Coefficient5 y**2",
         "    -.0000651,               !- Coefficient6 x*y",
         "    12.8,                    !- Minimum Value of x",
         "    26.66667,                !- Maximum Value of x",
         "    -3.88889,                !- Minimum Value of y",
         "    29.44444,                !- Maximum Value of y",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Temperature,             !- Input Unit Type for X",
         "    Temperature,             !- Input Unit Type for Y",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Biquadratic,",
         "    5HeatingCapacityCurve   ,!- Name",
         "    0.625,                   !- Coefficient1 Constant",
         "    -.00189,                 !- Coefficient2 x",
         "    0.00000058,              !- Coefficient3 x**2",
         "    0.0191,                  !- Coefficient4 y",
         "    0.0000891,               !- Coefficient5 y**2",
         "    -.0000651,               !- Coefficient6 x*y",
         "    12.8,                    !- Minimum Value of x",
         "    26.66667,                !- Maximum Value of x",
         "    -3.88889,                !- Minimum Value of y",
         "    29.44444,                !- Maximum Value of y",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Temperature,             !- Input Unit Type for X",
         "    Temperature,             !- Input Unit Type for Y",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Biquadratic,",
         "    6HeatingCapacityCurve   ,!- Name",
         "    0.625,                   !- Coefficient1 Constant",
         "    -.00189,                 !- Coefficient2 x",
         "    0.00000058,              !- Coefficient3 x**2",
         "    0.0191,                  !- Coefficient4 y",
         "    0.0000891,               !- Coefficient5 y**2",
         "    -.0000651,               !- Coefficient6 x*y",
         "    12.8,                    !- Minimum Value of x",
         "    26.66667,                !- Maximum Value of x",
         "    -3.88889,                !- Minimum Value of y",
         "    29.44444,                !- Maximum Value of y",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Temperature,             !- Input Unit Type for X",
         "    Temperature,             !- Input Unit Type for Y",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Biquadratic,",
         "    7HeatingCapacityCurve   ,!- Name",
         "    0.625,                   !- Coefficient1 Constant",
         "    -.00189,                 !- Coefficient2 x",
         "    0.00000058,              !- Coefficient3 x**2",
         "    0.0191,                  !- Coefficient4 y",
         "    0.0000891,               !- Coefficient5 y**2",
         "    -.0000651,               !- Coefficient6 x*y",
         "    12.8,                    !- Minimum Value of x",
         "    26.66667,                !- Maximum Value of x",
         "    -3.88889,                !- Minimum Value of y",
         "    29.44444,                !- Maximum Value of y",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Temperature,             !- Input Unit Type for X",
         "    Temperature,             !- Input Unit Type for Y",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Biquadratic,",
         "    8HeatingCapacityCurve   ,!- Name",
         "    0.625,                   !- Coefficient1 Constant",
         "    -.00189,                 !- Coefficient2 x",
         "    0.00000058,              !- Coefficient3 x**2",
         "    0.0191,                  !- Coefficient4 y",
         "    0.0000891,               !- Coefficient5 y**2",
         "    -.0000651,               !- Coefficient6 x*y",
         "    12.8,                    !- Minimum Value of x",
         "    26.66667,                !- Maximum Value of x",
         "    -3.88889,                !- Minimum Value of y",
         "    29.44444,                !- Maximum Value of y",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Temperature,             !- Input Unit Type for X",
         "    Temperature,             !- Input Unit Type for Y",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Biquadratic,",
         "    9HeatingCapacityCurve   ,!- Name",
         "    0.625,                   !- Coefficient1 Constant",
         "    -.00189,                 !- Coefficient2 x",
         "    0.00000058,              !- Coefficient3 x**2",
         "    0.0191,                  !- Coefficient4 y",
         "    0.0000891,               !- Coefficient5 y**2",
         "    -.0000651,               !- Coefficient6 x*y",
         "    12.8,                    !- Minimum Value of x",
         "    26.66667,                !- Maximum Value of x",
         "    -3.88889,                !- Minimum Value of y",
         "    29.44444,                !- Maximum Value of y",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Temperature,             !- Input Unit Type for X",
         "    Temperature,             !- Input Unit Type for Y",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Biquadratic,",
         "    10HeatingCapacityCurve   ,  !- Name",
         "    0.629,                   !- Coefficient1 Constant",
         "    -.0018,                  !- Coefficient2 x",
         "    0.000000752,             !- Coefficient3 x**2",
         "    0.0186,                  !- Coefficient4 y",
         "    0.000106,                !- Coefficient5 y**2",
         "    -.00007,                 !- Coefficient6 x*y",
         "    12.8,                    !- Minimum Value of x",
         "    26.66667,                !- Maximum Value of x",
         "    -3.88889,                !- Minimum Value of y",
         "    29.44444,                !- Maximum Value of y",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Temperature,             !- Input Unit Type for X",
         "    Temperature,             !- Input Unit Type for Y",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Biquadratic,",
         "    1HeatingPowerCurve    ,  !- Name",
         "    0.748,                   !- Coefficient1 Constant",
         "    0.0268,                  !- Coefficient2 x",
         "    0.000261,                !- Coefficient3 x**2",
         "    -.0189,                  !- Coefficient4 y",
         "    0.000399,                !- Coefficient5 y**2",
         "    -.000483,                !- Coefficient6 x*y",
         "    12.8,                    !- Minimum Value of x",
         "    26.66667,                !- Maximum Value of x",
         "    -3.88889,                !- Minimum Value of y",
         "    29.44444,                !- Maximum Value of y",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Temperature,             !- Input Unit Type for X",
         "    Temperature,             !- Input Unit Type for Y",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Biquadratic,",
         "    2HeatingPowerCurve    ,  !- Name",
         "    0.748,                   !- Coefficient1 Constant",
         "    0.0268,                  !- Coefficient2 x",
         "    0.000261,                !- Coefficient3 x**2",
         "    -.0189,                  !- Coefficient4 y",
         "    0.000399,                !- Coefficient5 y**2",
         "    -.000483,                !- Coefficient6 x*y",
         "    12.8,                    !- Minimum Value of x",
         "    26.66667,                !- Maximum Value of x",
         "    -3.88889,                !- Minimum Value of y",
         "    29.44444,                !- Maximum Value of y",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Temperature,             !- Input Unit Type for X",
         "    Temperature,             !- Input Unit Type for Y",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Biquadratic,",
         "    3HeatingPowerCurve    ,  !- Name",
         "    0.748,                   !- Coefficient1 Constant",
         "    0.0268,                  !- Coefficient2 x",
         "    0.000261,                !- Coefficient3 x**2",
         "    -.0189,                  !- Coefficient4 y",
         "    0.000399,                !- Coefficient5 y**2",
         "    -.000483,                !- Coefficient6 x*y",
         "    12.8,                    !- Minimum Value of x",
         "    26.66667,                !- Maximum Value of x",
         "    -3.88889,                !- Minimum Value of y",
         "    29.44444,                !- Maximum Value of y",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Temperature,             !- Input Unit Type for X",
         "    Temperature,             !- Input Unit Type for Y",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Biquadratic,",
         "    4HeatingPowerCurve    ,  !- Name",
         "    0.748,                   !- Coefficient1 Constant",
         "    0.0268,                  !- Coefficient2 x",
         "    0.000261,                !- Coefficient3 x**2",
         "    -.0189,                  !- Coefficient4 y",
         "    0.000399,                !- Coefficient5 y**2",
         "    -.000483,                !- Coefficient6 x*y",
         "    12.8,                    !- Minimum Value of x",
         "    26.66667,                !- Maximum Value of x",
         "    -3.88889,                !- Minimum Value of y",
         "    29.44444,                !- Maximum Value of y",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Temperature,             !- Input Unit Type for X",
         "    Temperature,             !- Input Unit Type for Y",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Biquadratic,",
         "    5HeatingPowerCurve    ,  !- Name",
         "    0.748,                   !- Coefficient1 Constant",
         "    0.0268,                  !- Coefficient2 x",
         "    0.000261,                !- Coefficient3 x**2",
         "    -.0189,                  !- Coefficient4 y",
         "    0.000399,                !- Coefficient5 y**2",
         "    -.000483,                !- Coefficient6 x*y",
         "    12.8,                    !- Minimum Value of x",
         "    26.66667,                !- Maximum Value of x",
         "    -3.88889,                !- Minimum Value of y",
         "    29.44444,                !- Maximum Value of y",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Temperature,             !- Input Unit Type for X",
         "    Temperature,             !- Input Unit Type for Y",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Biquadratic,",
         "    6HeatingPowerCurve    ,  !- Name",
         "    0.748,                   !- Coefficient1 Constant",
         "    0.0268,                  !- Coefficient2 x",
         "    0.000261,                !- Coefficient3 x**2",
         "    -.0189,                  !- Coefficient4 y",
         "    0.000399,                !- Coefficient5 y**2",
         "    -.000483,                !- Coefficient6 x*y",
         "    12.8,                    !- Minimum Value of x",
         "    26.66667,                !- Maximum Value of x",
         "    -3.88889,                !- Minimum Value of y",
         "    29.44444,                !- Maximum Value of y",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Temperature,             !- Input Unit Type for X",
         "    Temperature,             !- Input Unit Type for Y",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Biquadratic,",
         "    7HeatingPowerCurve    ,  !- Name",
         "    0.748,                   !- Coefficient1 Constant",
         "    0.0268,                  !- Coefficient2 ",
         "    0.000261,                !- Coefficient3 x**2",
         "    -.0189,                  !- Coefficient4 y",
         "    0.000399,                !- Coefficient5 y**2",
         "    -.000483,                !- Coefficient6 x*y",
         "    12.8,                    !- Minimum Value of x",
         "    26.66667,                !- Maximum Value of x",
         "    -3.88889,                !- Minimum Value of y",
         "    29.44444,                !- Maximum Value of y",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Temperature,             !- Input Unit Type for X",
         "    Temperature,             !- Input Unit Type for Y",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Biquadratic,",
         "    8HeatingPowerCurve    ,  !- Name",
         "    0.748,                   !- Coefficient1 Constant",
         "    0.0268,                  !- Coefficient2 x",
         "    0.000261,                !- Coefficient3 x**2",
         "    -.0189,                  !- Coefficient4 y",
         "    0.000399,                !- Coefficient5 y**2",
         "    -.000483,                !- Coefficient6 x*y",
         "    12.8,                    !- Minimum Value of x",
         "    26.66667,                !- Maximum Value of x",
         "    -3.88889,                !- Minimum Value of y",
         "    29.44444,                !- Maximum Value of y",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve ",
         "    Temperature,             !- Input Unit Type for X",
         "    Temperature,             !- Input Unit Type for Y",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Biquadratic,",
         "    9HeatingPowerCurve    ,  !- Name",
         "    0.748,                   !- Coefficient1 Constant",
         "    0.0268,                  !- Coefficient2 x",
         "    0.000261,                !- Coefficient3 x**2",
         "    -.0189,                  !- Coefficient4 y",
         "    0.000399,                !- Coefficient5 y**2",
         "    -.000483,                !- Coefficient6 x*y",
         "    12.8,                    !- Minimum Value of x",
         "    26.66667,                !- Maximum Value of x",
         "    -3.88889,                !- Minimum Value of y",
         "    29.44444,                !- Maximum Value of y",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Temperature,             !- Input Unit Type for X",
         "    Temperature,             !- Input Unit Type for Y",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Biquadratic,",
         "    10HeatingPowerCurve    , !- Name",
         "    0.7207072,               !- Coefficient1 Constant",
         "    0.02447856,              !- Coefficient2 x",
         "    0.000232632,             !- Coefficient3 x**2",
         "    -.012888,                !- Coefficient4 y",
         "    0.00017496,              !- Coefficient5 y**2",
         "    -.00033372,              !- Coefficient6 x*y",
         "    12.8,                    !- Minimum Value of x",
         "    26.66667,                !- Maximum Value of x",
         "    -3.88889,                !- Minimum Value of y",
         "    29.44444,                !- Maximum Value of y",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Temperature,             !- Input Unit Type for X",
         "    Temperature,             !- Input Unit Type for Y",
         "    Dimensionless;           !- Output Unit Type",

         "  Curve:Biquadratic,",
         "    wasteHeat VS Temp1 Test, !- Name",
         "    1.0,                     !- Coefficient1 Constant",
         "    0,                       !- Coefficient2 x",
         "    0,                       !- Coefficient3 x**2",
         "    0,                       !- Coefficient4 y",
         "    0,                       !- Coefficient5 y**2",
         "    0,                       !- Coefficient6 x*y",
         "    10.0,                    !- Minimum Value of x",
         "    25.6,                    !- Maximum Value of x",
         "    7.2,                     !- Minimum Value of y",
         "    48.9;                    !- Maximum Value of y",

         "  Curve:Biquadratic,",
         "    Heating wasteHeat VS Temp1 Test,  !- Name",
         "    1.0,                     !- Coefficient1 Constant",
         "    0,                       !- Coefficient2 x",
         "    0,                       !- Coefficient3 x**2",
         "    0,                       !- Coefficient4 y",
         "    0,                       !- Coefficient5 y**2",
         "    0,                       !- Coefficient6 x*y",
         "    7.0,                     !- Minimum Value of x",
         "    27.0,                    !- Maximum Value of x",
         "    10.0,                    !- Minimum Value of y",
         "    30.0;                    !- Maximum Value of y",

         "Curve:Quadratic,",
         "    PLF Curve Multi Compressor 30prcnt unloading,               !- Name",
         "    0.85,                    !- Coefficient1 Constant",
         "    0.8333,                    !- Coefficient2 x",
         "    0.0,                       !- Coefficient3 x**2",
         "    0.0,                       !- Minimum Value of x",
         "    0.3,                       !- Maximum Value of x",
         "    0.85,                        !- Minimum Curve Output",
         "    1.0,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Biquadratic,",
         "    4CapacityCurve,          !- Name",
         "    1.16,                    !- Coefficient1 Constant",
         "    -.0155,                  !- Coefficient2 x",
         "    0.00128,                 !- Coefficient3 x**2",
         "    -.00673,                 !- Coefficient4 y",
         "    0.0000797,               !- Coefficient5 y**2",
         "    -.000305,                !- Coefficient6 x*y",
         "    10,                      !- Minimum Value of x",
         "    23.88889,                !- Maximum Value of x",
         "    23.88889,                !- Minimum Value of y",
         "    40.55556,                !- Maximum Value of y",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless,           !- Input Unit Type for Y",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Cubic,",
         "    CAPFF Curve IpakCS,             !- Name",
         "    1,                       !- Coefficient1 Constant",
         "    0,                       !- Coefficient2 x",
         "    0,                       !- Coefficient3 x**2",
         "    0,                       !- Coefficient4 x**3",
         "    0,                       !- Minimum Value of x",
         "    1,                       !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",
         "Curve:Cubic,",
         "    EIRFF Curve IpakCS,             !- Name",
         "    1,                       !- Coefficient1 Constant",
         "    0,                       !- Coefficient2 x",
         "    0,                       !- Coefficient3 x**",
         "    0,                       !- Coefficient4 x**3",
         "    0,                       !- Minimum Value of x",
         "    1,                       !- Maximum Value of x",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless;           !- Output Unit Type",

         "Curve:Biquadratic,",
         "    1CapacityCurveIpakCS,          !- Name",
         "    0.483,                   !- Coefficient1 Constant",
         "    0.0305,                   !- Coefficient2 x",
         "    0.0000458,                 !- Coefficient3 x**2",
         "    0.00511,                  !- Coefficient4 y",
         "    -1.50E-04,                !- Coefficient5 y**2",
         "    -1.28E-04,                 !- Coefficient6 x*y",
         "    8.88889,                 !- Minimum Value of x",
         "    21.6667,                 !- Maximum Value of x",
         "    12.7778,                 !- Minimum Value of y",
         "    51.6667,                 !- Maximum Value of y",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Temperature,             !- Input Unit Type for X",
         "    Temperature,             !- Input Unit Type for Y",
         "    Dimensionless;           !- Output Unit Type",
         "Curve:Biquadratic,",
         "    1PowerCurveIpakCS,             !- Name",
         "    1.33E+00,                    !- Coefficient1 Constant",
         "    -3.40E-02,                  !- Coefficient2 x",
         "    0.000939,                 !- Coefficient3 x**2",
         "    -0.00858,                  !- Coefficient4 y",
         "    0.000769,                !- Coefficient5 y**2",
         "    -0.000972,                !- Coefficient6 x*y",
         "    8.88889,                 !- Minimum Value of x",
         "    21.6667,                 !- Maximum Value of x",
         "    12.7778,                 !- Minimum Value of y",
         "    51.6667,                 !- Maximum Value of y",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Temperature,             !- Input Unit Type for X",
         "    Temperature,             !- Input Unit Type for Y",
         "    Dimensionless;           !- Output Unit Type",
         "Curve:Biquadratic,",
         "    2CapacityCurveIpakCS,          !- Name",
         "    0.515,                   !- Coefficient1 Constant",
         "    0.026,                  !- Coefficient2 x",
         "    0.000275,                 !- Coefficient3 x**2",
         "    0.00377,                  !- Coefficient4 y",
         "    -0.000115,                !- Coefficient5 y**2",
         "    -0.00017,                !- Coefficient6 x*y",
         "    8.88889,                 !- Minimum Value of x",
         "    21.6667,                 !- Maximum Value of x",
         "    12.7778,                 !- Minimum Value of y",
         "    51.6667,                 !- Maximum Value of y",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless,           !- Input Unit Type for Y",
         "    Dimensionless;           !- Output Unit Type",
         "Curve:Biquadratic,",
         "    2PowerCurveIpakCS,             !- Name",
         "    1.41,                    !- Coefficient1 Constant",
         "    -0.0457,                  !- Coefficient2 x",
         "    0.00116,                 !- Coefficient3 x**2",
         "    -0.00625,                  !- Coefficient4 y",
         "    0.000707,                 !- Coefficient5 y**2",
         "    -0.000889,                !- Coefficient6 x*y",
         "    8.88889,                 !- Minimum Value of x",
         "    21.6667,                 !- Maximum Value of x",
         "    12.7778,                 !- Minimum Value of y",
         "    51.6667,                 !- Maximum Value of y",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless,           !- Input Unit Type for Y",
         "    Dimensionless;           !- Output Unit Type",
         "Curve:Biquadratic,",
         "    3CapacityCurveIpakCS,          !- Name",
         "    0.5,                   !- Coefficient1 Constant",
         "    0.0316,                  !- Coefficient2 x",
         "    0.00013,                 !- Coefficient3 x**2",
         "    0.00213,                  !- Coefficient4 y",
         "    -0.000107,                !- Coefficient5 y**2",
         "    -0.000157,                !- Coefficient6 x*y",
         "    8.88889,                 !- Minimum Value of x",
         "    21.6667,                 !- Maximum Value of x",
         "    12.7778,                 !- Minimum Value of y",
         "    51.6667,                 !- Maximum Value of y",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless,           !- Input Unit Type for Y",
         "    Dimensionless;           !- Output Unit Type",
         "Curve:Biquadratic,",
         "    3PowerCurveIpakCS,             !- Name",
         "    0.99,                    !- Coefficient1 Constant",
         "    -0.039,                  !- Coefficient2 x",
         "    0.00111,                 !- Coefficient3 x**2",
         "    0.0144,                  !- Coefficient4 y",
         "    0.000489,                !- Coefficient5 y**2",
         "    -0.00111,                !- Coefficient6 x*y",
         "    8.88889,                 !- Minimum Value of x",
         "    21.6667,                 !- Maximum Value of x",
         "    12.7778,                 !- Minimum Value of y",
         "    51.6667,                 !- Maximum Value of y",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless,           !- Input Unit Type for Y",
         "    Dimensionless;           !- Output Unit Type",
         "Curve:Biquadratic,",
         "    4CapacityCurveIpakCS,          !- Name",
         "    0.528,                   !- Coefficient1 Constant",
         "    0.0324,                  !- Coefficient2 x",
         "    -0.0000156,                 !- Coefficient3 x**2",
         "    0.00265,                  !- Coefficient4 y",
         "    -0.000125,                !- Coefficient5 y**2",
         "    -0.000134,                !- Coefficient6 x*y",
         "    8.88889,                 !- Minimum Value of x",
         "    21.6667,                 !- Maximum Value of x",
         "    12.7778,                 !- Minimum Value of y",
         "    51.6667,                 !- Maximum Value of y",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless,           !- Input Unit Type for Y",
         "    Dimensionless;           !- Output Unit Type",
         "Curve:Biquadratic,",
         "    4PowerCurveIpakCS,             !- Name",
         "    0.972,                     !- Coefficient1 Constant",
         "    -0.0357,                   !- Coefficient2 x",
         "    0.00105,                  !- Coefficient3 x**2",
         "    0.0109,                  !- Coefficient4 y",
         "    0.000517,                !- Coefficient5 y**2",
         "    -0.00101,                !- Coefficient6 x*y",
         "    8.88889,                 !- Minimum Value of x",
         "    21.6667,                 !- Maximum Value of x",
         "    12.7778,                 !- Minimum Value of y",
         "    51.6667,                 !- Maximum Value of y",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless,           !- Input Unit Type for Y",
         "    Dimensionless;           !- Output Unit Type",
         "Curve:Biquadratic,",
         "    5CapacityCurveIpakCS,          !- Name",
         "    0.551,                   !- Coefficient1 Constant",
         "    0.0291,                  !- Coefficient2 x",
         "    0.000124,                 !- Coefficient3 x**2",
         "    0.00196,                  !- Coefficient4 y",
         "    -0.00011,                !- Coefficient5 y**",
         "    -0.000144,                !- Coefficient6 x*y",
         "    8.88889,                 !- Minimum Value of x",
         "    21.6667,                 !- Maximum Value of x",
         "    12.7778,                 !- Minimum Value of y",
         "    51.6667,                 !- Maximum Value of y",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless,           !- Input Unit Type for Y",
         "    Dimensionless;           !- Output Unit Type",
         "Curve:Biquadratic,",
         "    5PowerCurveIpakCS,             !- Name",
         "    0.916,                     !- Coefficient1 Constant",
         "    -0.0306,                   !- Coefficient2 x",
         "    0.000885,                  !- Coefficient3 x**2",
         "    0.0126,                  !- Coefficient4 y",
         "    0.000495,                !- Coefficient5 y**2",
         "    -0.00103,                !- Coefficient6 x*y",
         "    8.88889,                 !- Minimum Value of x",
         "    21.6667,                 !- Maximum Value of x",
         "    12.7778,                 !- Minimum Value of y",
         "    51.6667,                 !- Maximum Value of y",
         "    ,                        !- Minimum Curve Output",
         "    ,                        !- Maximum Curve Output",
         "    Dimensionless,           !- Input Unit Type for X",
         "    Dimensionless,           !- Input Unit Type for Y",
         "    Dimensionless;           !- Output Unit Type    "

        });

    ASSERT_TRUE(process_idf(idf_objects));

    VariableSpeedCoils::GetVarSpeedCoilInput();

    EXPECT_EQ(VariableSpeedCoils::VarSpeedCoil(1).Name, "LOBBY_ZN_1_FLR_2 WSHP COOLING MODE");

    EXPECT_EQ(VariableSpeedCoils::VarSpeedCoil(2).Name, "PSZ-AC_1:5_COOLC STANDARD 4-COMPRESSOR IPAK");
}

TEST_F(EnergyPlusFixture, CoilHeatingDXVariableSpeed_MinOADBTempCompOperLimit)
{

    // tests minimum limits of Minimum Outdoor Drybulb Temperature for Compressor Operation

    std::string const idf_objects = delimited_string({

        "  Version,9.1;",

        "  Curve:Biquadratic,",
        "    HPACHeatCapFT,           !- Name",
        "    0.8529681407,            !- Coefficient1 Constant",
        "    -0.0004847169,           !- Coefficient2 x",
        "    -0.0000010693,           !- Coefficient3 x**2",
        "    0.0185542164,            !- Coefficient4 y",
        "    0.0000872425,            !- Coefficient5 y**2",
        "    -0.0000166868,           !- Coefficient6 x*y",
        "    17.78,                   !- Minimum Value of x",
        "    23.33,                   !- Maximum Value of x",
        "    -28.89,                  !- Minimum Value of y",
        "    17.22,                   !- Maximum Value of y",
        "    0.3799,                  !- Minimum Curve Output",
        "    1.1896,                  !- Maximum Curve Output",
        "    Temperature,             !- Input Unit Type for X",
        "    Temperature,             !- Input Unit Type for Y",
        "    Dimensionless;           !- Output Unit Type",

        "  Curve:Cubic,",
        "    HPACHeatCapFFF,          !- Name",
        "    0.84,                    !- Coefficient1 Constant",
        "    0.16,                    !- Coefficient2 x",
        "    0.0,                     !- Coefficient3 x**2",
        "    0.0,                     !- Coefficient4 x**3",
        "    0.5,                     !- Minimum Value of x",
        "    1.5;                     !- Maximum Value of x",

        "  Curve:Biquadratic,",
        "    HPACHeatEIRFT,           !- Name",
        "    0.7077081462,            !- Coefficient1 Constant",
        "    0.0148163478,            !- Coefficient2 x",
        "    0.0002622589,            !- Coefficient3 x**2",
        "    -0.0113239622,           !- Coefficient4 y",
        "    0.0002939277,            !- Coefficient5 y**2",
        "    -0.0003605284,           !- Coefficient6 x*y",
        "    17.78,                   !- Minimum Value of x",
        "    23.33,                   !- Maximum Value of x",
        "    -28.89,                  !- Minimum Value of y",
        "    17.22,                   !- Maximum Value of y",
        "    0.8266,                  !- Minimum Curve Output",
        "    2.0277,                  !- Maximum Curve Output",
        "    Temperature,             !- Input Unit Type for X",
        "    Temperature,             !- Input Unit Type for Y",
        "    Dimensionless;           !- Output Unit Type",

        "  Curve:Quadratic,",
        "    HPACHeatEIRFFF,          !- Name",
        "    1.3824,                  !- Coefficient1 Constant",
        "    -0.4336,                 !- Coefficient2 x",
        "    0.0512,                  !- Coefficient3 x**2",
        "    0.0,                     !- Minimum Value of x",
        "    1.0;                     !- Maximum Value of x",

        "  Curve:Quadratic,",
        "    HPACHeatPLFFPLR,         !- Name",
        "    0.75,                    !- Coefficient1 Constant",
        "    0.25,                    !- Coefficient2 x",
        "    0.0,                     !- Coefficient3 x**2",
        "    0.0,                     !- Minimum Value of x",
        "    1.0;                     !- Maximum Value of x",

        "  Coil:Heating:DX:VariableSpeed,",
        "    Heating Coil VariableSpeed,     !- Name",
        "    Zone1PTHPDXCoolCoilOutletNode,  !- Indoor Air Inlet Node Name",
        "    Zone1PTHPDXHeatCoilOutletNode,  !- Indoor Air Outlet Node Name",
        "    10.0,                    !- Number of Speeds {dimensionless}",
        "    10.0,                    !- Nominal Speed Level {dimensionless}",
        "    7200.0,                  !- Rated Heating Capacity At Selected Nominal Speed Level {w}",
        "    0.4,                     !- Rated Air Flow Rate At Selected Nominal Speed Level {m3/s}",
        "    HPACHeatPLFFPLR,         !- Energy Part Load Fraction Curve Name",
        "    ,                        !- Defrost Energy Input Ratio Function of Temperature Curve Name",
        "   -60.0,                    !- Minimum Outdoor Dry-Bulb Temperature for Compressor Operation {C}",
        "    ,                        !- Outdoor Dry-Bulb Temperature to Turn On Compressor {C}",
        "    5.0,                     !- Maximum Outdoor Dry-Bulb Temperature for Defrost Operation {C}",
        "    200.0,                   !- Crankcase Heater Capacity {W}",
        "    10.0,                    !- Maximum Outdoor Dry-Bulb Temperature for Crankcase Heater Operation {C}",
        "    Resistive,               !- Defrost Strategy",
        "    TIMED,                   !- Defrost Control",
        "    0.166667,                !- Defrost Time Period Fraction",
        "    7200,                    !- Resistive Defrost Heater Capacity {W}",
        "    1838.7,                  !- Speed 1 Reference Unit Gross Rated Heating Capacity {w}",
        "    5.0,                     !- Speed 1 Reference Unit Gross Rated Heating COP {dimensionless}",
        "    0.1661088,               !- Speed 1 Reference Unit Rated Air Flow Rate {m3/s}",
        "    HPACHeatCapFT,           !- Speed 1 Heating Capacity Function of Temperature Curve Name",
        "    HPACHeatCapFFF,          !- Speed 1 Total  Heating Capacity Function of Air Flow Fraction Curve Name",
        "    HPACHeatEIRFT,           !- Speed 1 Energy Input Ratio Function of Temperature Curve Name",
        "    HPACHeatEIRFFF,          !- Speed 1 Energy Input Ratio Function of Air Flow Fraction Curve Name",
        "    2295.5,                  !- Speed 2 Reference Unit Gross Rated Heating Capacity {w}",
        "    5.0,                     !- Speed 2 Reference Unit Gross Rated Heating COP {dimensionless}",
        "    0.179322,                !- Speed 2 Reference Unit Rated Air Flow Rate {m3/s}",
        "    HPACHeatCapFT,           !- Speed 2 Heating Capacity Function of Temperature Curve Name",
        "    HPACHeatCapFFF,          !- Speed 2 Total  Heating Capacity Function of Air Flow Fraction Curve Name",
        "    HPACHeatEIRFT,           !- Speed 2 Energy Input Ratio Function of Temperature Curve Name",
        "    HPACHeatEIRFFF,          !- Speed 2 Energy Input Ratio Function of Air Flow Fraction Curve Name",
        "    2751.3,                  !- Speed 3 Reference Unit Gross Rated Heating Capacity {w}",
        "    5.0,                     !- Speed 3 Reference Unit Gross Rated Heating COP {dimensionless}",
        "    0.1925352,               !- Speed 3 Reference Unit Rated Air Flow Rate {m3/s}",
        "    HPACHeatCapFT,           !- Speed 3 Heating Capacity Function of Temperature Curve Name",
        "    HPACHeatCapFFF,          !- Speed 3 Total  Heating Capacity Function of Air Flow Fraction Curve Name",
        "    HPACHeatEIRFT,           !- Speed 3 Energy Input Ratio Function of Temperature Curve Name",
        "    HPACHeatEIRFFF,          !- Speed 3 Energy Input Ratio Function of Air Flow Fraction Curve Name",
        "    3659.6,                  !- Speed 4 Reference Unit Gross Rated Heating Capacity {w}",
        "    5.0,                     !- Speed 4 Reference Unit Gross Rated Heating COP {dimensionless}",
        "    0.2189616,               !- Speed 4 Reference Unit Rated Air Flow Rate {m3/s}",
        "    HPACHeatCapFT,           !- Speed 4 Heating Capacity Function of Temperature Curve Name",
        "    HPACHeatCapFFF,          !- Speed 4 Heating Capacity Function of Air Flow Fraction Curve Name",
        "    HPACHeatEIRFT,           !- Speed 4 Energy Input Ratio Function of Temperature Curve Name",
        "    HPACHeatEIRFFF,          !- Speed 4 Energy Input Ratio Function of Air Flow Fraction Curve Name",
        "    4563.7,                  !- Speed 5 Reference Unit Gross Rated Heating Capacity {w}",
        "    5.0,                     !- Speed 5 Reference Unit Gross Rated Heating COP {dimensionless}",
        "    0.245388,                !- Speed 5 Reference Unit Rated Air Flow Rate {m3/s}",
        "    HPACHeatCapFT,           !- Speed 5 Heating Capacity Function of Temperature Curve Name",
        "    HPACHeatCapFFF,          !- Speed 5 Heating Capacity Function of Air Flow Fraction Curve Name",
        "    HPACHeatEIRFT,           !- Speed 5 Energy Input Ratio Function of Temperature Curve Name",
        "    HPACHeatEIRFFF,          !- Speed 5 Energy Input Ratio Function of Air Flow Fraction Curve Name",
        "    5463.3,                  !- Speed 6 Reference Unit Gross Rated Heating Capacity {w}",
        "    5.0,                     !- Speed 6 Reference Unit Gross Rated Heating COP {dimensionless}",
        "    0.2718144,               !- Speed 6 Reference Unit Rated Air Flow Rate {m3/s}",
        "    HPACHeatCapFT,           !- Speed 6 Heating Capacity Function of Temperature Curve Name",
        "    HPACHeatCapFFF,          !- Speed 6 Heating Capacity Function of Air Flow Fraction Curve Name",
        "    HPACHeatEIRFT,           !- Speed 6 Energy Input Ratio Function of Temperature Curve Name",
        "    HPACHeatEIRFFF,          !- Speed 6 Energy Input Ratio Function of Air Flow Fraction Curve Name",
        "    6358.4,                  !- Speed 7 Reference Unit Gross Rated Heating Capacity {w}",
        "    5.0,                     !- Speed 7 Reference Unit Gross Rated Heating COP {dimensionless}",
        "    0.2982408,               !- Speed 7 Reference Unit Rated Air Flow Rate {m3/s}",
        "    HPACHeatCapFT,           !- Speed 7 Heating Capacity Function of Temperature Curve Name",
        "    HPACHeatCapFFF,          !- Speed 7 Heating Capacity Function of Air Flow Fraction Curve Name",
        "    HPACHeatEIRFT,           !- Speed 7 Energy Input Ratio Function of Temperature Curve Name",
        "    HPACHeatEIRFFF,          !- Speed 7 Energy Input Ratio Function of Air Flow Fraction Curve Name",
        "    7248.5,                  !- Speed 8 Reference Unit Gross Rated Heating Capacity {w}",
        "    5.0,                     !- Speed 8 Reference Unit Gross Rated Heating COP {dimensionless}",
        "    0.3246672,               !- Speed 8 Reference Unit Rated Air Flow Rate {m3/s}",
        "    HPACHeatCapFT,           !- Speed 8 Heating Capacity Function of Temperature Curve Name",
        "    HPACHeatCapFFF,          !- Speed 8 Heating Capacity Function of Air Flow Fraction Curve Name",
        "    HPACHeatEIRFT,           !- Speed 8 Energy Input Ratio Function of Temperature Curve Name",
        "    HPACHeatEIRFFF,          !- Speed 8 Energy Input Ratio Function of Air Flow Fraction Curve Name",
        "    8133.6,                  !- Speed 9 Reference Unit Gross Rated Heating Capacity {w}",
        "    5.0,                     !- Speed 9 Reference Unit Gross Rated Heating COP {dimensionless}",
        "    0.3510936,               !- Speed 9 Reference Unit Rated Air Flow Rate {m3/s}",
        "    HPACHeatCapFT,           !- Speed 9 Heating Capacity Function of Temperature Curve Name",
        "    HPACHeatCapFFF,          !- Speed 9 Heating Capacity Function of Air Flow Fraction Curve Name",
        "    HPACHeatEIRFT,           !- Speed 9 Energy Input Ratio Function of Temperature Curve Name",
        "    HPACHeatEIRFFF,          !- Speed 9 Energy Input Ratio Function of Air Flow Fraction Curve Name",
        "    9013.2,                  !- Speed 10 Reference Unit Gross Rated Heating Capacity {w}",
        "    5.0,                     !- Speed 10 Reference Unit Gross Rated Heating COP {dimensionless}",
        "    0.37752,                 !- Speed 10 Reference Unit Rated Air Flow Rate {m3/s}",
        "    HPACHeatCapFT,           !- Speed 10 Heating Capacity Function of Temperature Curve Name",
        "    HPACHeatCapFFF,          !- Speed 10 Heating Capacity Function of Air Flow Fraction Curve Name",
        "    HPACHeatEIRFT,           !- Speed 10 Energy Input Ratio Function of Temperature Curve Name",
        "    HPACHeatEIRFFF;          !- Speed 10 Energy Input Ratio Function of Air Flow Fraction Curve Name",

    });

    ASSERT_TRUE(process_idf(idf_objects));

    VariableSpeedCoils::GetVarSpeedCoilInput();

    ASSERT_EQ("HEATING COIL VARIABLESPEED", VariableSpeedCoils::VarSpeedCoil(1).Name); // Heating Coil Variable Speed
    ASSERT_EQ(-60.0, VariableSpeedCoils::VarSpeedCoil(1).MinOATCompressor);            // removed the minimum limit of -50.0C
}

TEST_F(EnergyPlusFixture, VariableSpeedCoils_Test_CalcTotCap_VSWSHP)
{
    std::string const idf_objects = delimited_string({
        "  Coil:Cooling:DX:VariableSpeed,",
        "    PSZ-AC_1:5_CoolC Standard 4-compressor IPAK,  !- Name",
        "    PSZ-AC_1:5_OA-PSZ-AC_1:5_CoolCNode,           !- Air Inlet Node Name",
        "    PSZ-AC_1:5_CoolC-PSZ-AC_1:5_HeatCNode,        !- Air Outlet Node Name",
        "    5,                       !- Number of Speeds {dimensionless}",
        "    5,                       !- Nominal Speed Level {dimensionless}",
        "    autosize,                !- Rated Total Cooling Capacity At Selected Nominal Speed Level {w}",
        "    autosize,                !- Rated Volumetric Air Flow Rate At Selected Nominal Speed Level {m3/s}",
        "    0,                       !- Nominal Time for Condensate to Begin Leaving the Coil {s}",
        "    0,                       !- Initial Moisture Evaporation Rate Divided by Steady-State AC Latent Capacity {dimensionless}",
        "    PLF Curve,               !- Energy Part Load Fraction Curve Name",
        "    PSZ-AC_1:5 OA Node,      !- Condenser Air Inlet Node Name",
        "    AirCooled,               !- Condenser Type",
        "    ,                        !- Evaporative Condenser Pump Rated Power Consumption {W}",
        "    ,                        !- Crankcase Heater Capacity {W}",
        "    10,                      !- Maximum Outdoor Dry-Bulb Temperature for Crankcase Heater Operation {C}",
        "    ,                        !- Minimum Outdoor Dry-Bulb Temperature for Compressor Operation {C}",
        "    ,                        !- Supply Water Storage Tank Name",
        "    ,                        !- Condensate Collection Water Storage Tank Name",
        "    ,                        !- Basin Heater Capacity {W/K}",
        "    2,                       !- Basin Heater Setpoint Temperature {C}",
        "    ,                        !- Basin Heater Operating Schedule Name",
        "    33861.72,                !- Speed 1 Reference Unit Total Cooling Capacity At Rated Conditions {w}",
        "    0.70,                    !- Speed 1 Reference Unit Sensible Heat Ratio At Rated Conditions {dimensionless}",
        "    4.328973,                !- Speed 1 Reference Unit COP At Rated Conditions {dimensionless}",
        "    1.396964,                !- Speed 1 Reference Unit Air Flow Rate At Rated Conditions {m3/s}",
        "    ,                        !- Speed 1 Reference Unit Condenser Flow Rate at Rated Conditions {m3/s}",
        "    ,                        !- Speed 1 Reference Unit Pad Effectiveness of Evap Precooling at Rated Conditions {dimensionless}",
        "    CapacityCurve,           !- Speed 1 Total Cooling Capacity Function of Temperature Curve Name",
        "    CAPFF Curve,             !- Speed 1 Total Cooling Capacity Function of Air Flow Fraction Curve Name",
        "    PowerCurve,              !- Speed 1 Energy Input Ratio Function of Temperature Curve Name",
        "    EIRFF Curve,             !- Speed 1 Energy Input Ratio Function of Air Flow Fraction Curve Name",
        "    35516.08,                !- Speed 2 Reference Unit Total Cooling Capacity At Rated Conditions {w}",
        "    0.78,                    !- Speed 2 Reference Unit Sensible Heat Ratio At Rated Conditions {dimensionless}",
        "    4.540061,                !- Speed 2 Reference Unit COP At Rated Conditions {dimensionless}",
        "    1.88779,                 !- Speed 2 Reference Unit Air Flow Rate At Rated Conditions {m3/s}",
        "    ,                        !- Speed 2 Reference Unit Condenser Flow Rate at Rated Conditions {m3/s}",
        "    ,                        !- Speed 2 Reference Unit Pad Effectiveness of Evap Precooling at Rated Conditions {dimensionless}",
        "    CapacityCurve,           !- Speed 2 Total Cooling Capacity Function of Temperature Curve Name",
        "    CAPFF Curve,             !- Speed 2 Total Cooling Capacity Function of Air Flow Fraction Curve Name",
        "    PowerCurve,              !- Speed 2 Energy Input Ratio Function of Temperature Curve Name",
        "    EIRFF Curve,             !- Speed 2 Energy Input Ratio Function of Air Flow Fraction Curve Name",
        "    65133.17,                !- Speed 3 Reference Unit Total Cooling Capacity At Rated Conditions {w}",
        "    0.70,                    !- Speed 3 Reference Unit Sensible Heat Ratio At Rated Conditions {dimensionless}",
        "    4.164418,                !- Speed 3 Reference Unit COP At Rated Conditions {dimensionless}",
        "    2.831685,                !- Speed 3 Reference Unit Air Flow Rate At Rated Conditions {m3/s}",
        "    ,                        !- Speed 3 Reference Unit Condenser Flow Rate at Rated Conditions {m3/s}",
        "    ,                        !- Speed 3 Reference Unit Pad Effectiveness of Evap Precooling at Rated Conditions {dimensionless}",
        "    CapacityCurve,           !- Speed 3 Total Cooling Capacity Function of Temperature Curve Name",
        "    CAPFF Curve,             !- Speed 3 Total Cooling Capacity Function of Air Flow Fraction Curve Name",
        "    PowerCurve,              !- Speed 3 Energy Input Ratio Function of Temperature Curve Name",
        "    EIRFF Curve,             !- Speed 3 Energy Input Ratio Function of Air Flow Fraction Curve Name",
        "    119583.3,                !- Speed 4 Reference Unit Total Cooling Capacity At Rated Conditions {w}",
        "    0.62,                    !- Speed 4 Reference Unit Sensible Heat Ratio At Rated Conditions {dimensionless}",
        "    3.469661,                !- Speed 4 Reference Unit COP At Rated Conditions {dimensionless}",
        "    3.553764,                !- Speed 4 Reference Unit Air Flow Rate At Rated Conditions {m3/s}",
        "    ,                        !- Speed 4 Reference Unit Condenser Flow Rate at Rated Conditions {m3/s}",
        "    ,                        !- Speed 4 Reference Unit Pad Effectiveness of Evap Precooling at Rated Conditions {dimensionless}",
        "    CapacityCurve,           !- Speed 4 Total Cooling Capacity Function of Temperature Curve Name",
        "    CAPFF Curve,             !- Speed 4 Total Cooling Capacity Function of Air Flow Fraction Curve Name",
        "    PowerCurve,              !- Speed 4 Energy Input Ratio Function of Temperature Curve Name",
        "    EIRFF Curve,             !- Speed 4 Energy Input Ratio Function of Air Flow Fraction Curve Name",
        "    132769.7,                !- Speed 5 Reference Unit Total Cooling Capacity At Rated Conditions {w}",
        "    0.69,                    !- Speed 5 Reference Unit Sensible Heat Ratio At Rated Conditions {dimensionless}",
        "    3.822957,                !- Speed 5 Reference Unit COP At Rated Conditions {dimensionless}",
        "    5.66336932,              !- Speed 5 Reference Unit Air Flow Rate At Rated Conditions {m3/s}",
        "    ,                        !- Speed 5 Reference Unit Condenser Flow Rate at Rated Conditions {m3/s}",
        "    ,                        !- Speed 5 Reference Unit Pad Effectiveness of Evap Precooling at Rated Conditions {dimensionless}",
        "    CapacityCurve,           !- Speed 5 Total Cooling Capacity Function of Temperature Curve Name",
        "    CAPFF Curve,             !- Speed 5 Total Cooling Capacity Function of Air Flow Fraction Curve Name",
        "    PowerCurve,              !- Speed 5 Energy Input Ratio Function of Temperature Curve Name",
        "    EIRFF Curve;             !- Speed 5 Energy Input Ratio Function of Air Flow Fraction Curve Name",

        "Curve:Biquadratic,",
        "    CapacityCurve, 1, 0, 0, 0, 0, 0, 10, 25.5, 7.2, 48.8, , , Temperature, Temperature, Dimensionless;",
        "Curve:Biquadratic,",
        "    PowerCurve, 1, 0, 0, 0, 0, 0, 10, 25.5, 7.2, 48.8, , , Temperature, Temperature, Dimensionless;",
        "Curve:Cubic,",
        "    CAPFF Curve, 1, 0, 0, 0, 0, 1, , , Dimensionless, Dimensionless;",
        "Curve:Cubic,",
        "    EIRFF Curve, 1, 0, 0, 0, 0, 1, , , Dimensionless, Dimensionless;",
        "Curve:Quadratic,",
        "    PLF Curve, 0.85, 0.8333, 0.0, 0.0, 0.3, 0.85, 1.0, Dimensionless, Dimensionless;",

    });

    ASSERT_TRUE(process_idf(idf_objects));

    VariableSpeedCoils::GetVarSpeedCoilInput();

    Real64 LSInletDBTemp = 24.0; // conditions at 24 DB / 20 Wb found at http://www.sugartech.co.za/psychro/index.php
    Real64 LSInletHumRat = 0.013019367;
    Real64 LSInletEnth = 57256.90248;
    Real64 LSInletWBTemp = 20.0;
    Real64 AirMassFlowRatio = VariableSpeedCoils::VarSpeedCoil(1).MSRatedAirVolFlowRate(1);
    Real64 WaterMassFlowRatio = 0.0;
    Real64 LSMassFlowRate = 1.45;
    Real64 CBFSpeed = 0.000001;
    Real64 MSRatedTotCap = VariableSpeedCoils::VarSpeedCoil(1).MSRatedTotCap(1);
    int MSCapFTemp = VariableSpeedCoils::VarSpeedCoil(1).MSCCapFTemp(1);
    int MSCapAirFFlow = VariableSpeedCoils::VarSpeedCoil(1).MSCCapAirFFlow(1);
    int MSCapWaterFFlow = VariableSpeedCoils::VarSpeedCoil(1).MSCCapWaterFFlow(1);
    Real64 QLoadTotal = 0.0;
    Real64 QLoadTotal1 = 0.0;
    Real64 QLoadTotal2 = 0.0;
    Real64 SHR = VariableSpeedCoils::VarSpeedCoil(1).MSRatedSHR(1);
    Real64 SSInletTemp = 24.0;
    Real64 InletAirPressure = 101320.0;

    VariableSpeedCoils::CalcTotCapSHR_VSWSHP(LSInletDBTemp, LSInletHumRat, LSInletEnth, LSInletWBTemp, AirMassFlowRatio, WaterMassFlowRatio,
                                             LSMassFlowRate, CBFSpeed, MSRatedTotCap, MSCapFTemp, MSCapAirFFlow, MSCapWaterFFlow, 0.0, 0, 0, 0,
                                             QLoadTotal1, QLoadTotal2, QLoadTotal, SHR, SSInletTemp, InletAirPressure, 0.0, 1,
                                             VariableSpeedCoils::VarSpeedCoil(1).capModFacTotal);

    // same calculations as in CalcTotCapSHR_VSWSHP (except CapFTemp term is 1 so no need to add that calc here)
    Real64 hDelta = MSRatedTotCap / LSMassFlowRate;                      // Change in air enthalpy across the cooling coil [J/kg]
    Real64 hADP = LSInletEnth - hDelta / (1.0 - CBFSpeed);               // Apparatus dew point enthalpy [J/kg]
    Real64 tADP = Psychrometrics::PsyTsatFnHPb(hADP, InletAirPressure);  // Apparatus dew point temperature [C]
    Real64 wADP = Psychrometrics::PsyWFnTdbH(tADP, hADP);                // Apparatus dew point humidity ratio [kg/kg]
    Real64 hTinwADP = Psychrometrics::PsyHFnTdbW(LSInletDBTemp, wADP);   // Enthalpy at inlet dry-bulb and wADP [J/kg]
    Real64 SHRCalc = min((hTinwADP - hADP) / (LSInletEnth - hADP), 1.0); // temporary calculated value of SHR

    // expect SHR to be < 1
    EXPECT_NEAR(SHR, 0.5275102, 0.000001);
    EXPECT_NEAR(SHR, SHRCalc, 0.000001);
    EXPECT_NEAR(QLoadTotal1, 33861.7200, 0.00001);
    EXPECT_NEAR(QLoadTotal2, 0.0, 0.00001);
    EXPECT_NEAR(QLoadTotal, 33861.7200, 0.00001);

    Real64 OutletTemp = LSInletDBTemp - (1.0 - CBFSpeed) * (LSInletDBTemp - tADP);
    Real64 OutletHumRat = LSInletHumRat - (1.0 - CBFSpeed) * (LSInletHumRat - wADP);
    Real64 OutletEnthalpy = LSInletEnth - hDelta;
    Real64 OutletAirRH = Psychrometrics::PsyRhFnTdbWPb(OutletTemp, OutletHumRat, InletAirPressure);

    // outlet conditions should be very near the saturation curve
    EXPECT_NEAR(OutletTemp, tADP, 0.0001);
    EXPECT_NEAR(OutletHumRat, wADP, 0.00000001); // CalcTotCapSHR_VSWSHP provides correct SHR even when CBF is very small
    EXPECT_NEAR(OutletEnthalpy, hADP, 0.1);
    EXPECT_NEAR(OutletAirRH, 0.999, 0.001);
    EXPECT_LT(OutletAirRH, 1.0);
}
} // namespace EnergyPlus
