// EnergyPlus, Copyright (c) 1996-2019, The Board of Trustees of the University of Illinois,
// The Regents of the University of California, through Lawrence Berkeley National Laboratory
// (subject to receipt of any required approvals from the U.S. Dept. of Energy), Oak Ridge
// National Laboratory, managed by UT-Battelle, Alliance for Sustainable Energy, LLC, and other
// contributors. All rights reserved.
//
// NOTICE: This Software was developed under funding from the U.S. Department of Energy and the
// U.S. Government consequently retains certain rights. As such, the U.S. Government has been
// granted for itself and others acting on its behalf a paid-up, nonexclusive, irrevocable,
// worldwide license in the Software to reproduce, distribute copies to the public, prepare
// derivative works, and perform publicly and display publicly, and to permit others to do so.
//
// Redistribution and use in source and binary forms, with or without modification, are permitted
// provided that the following conditions are met:
//
// (1) Redistributions of source code must retain the above copyright notice, this list of
//     conditions and the following disclaimer.
//
// (2) Redistributions in binary form must reproduce the above copyright notice, this list of
//     conditions and the following disclaimer in the documentation and/or other materials
//     provided with the distribution.
//
// (3) Neither the name of the University of California, Lawrence Berkeley National Laboratory,
//     the University of Illinois, U.S. Dept. of Energy nor the names of its contributors may be
//     used to endorse or promote products derived from this software without specific prior
//     written permission.
//
// (4) Use of EnergyPlus(TM) Name. If Licensee (i) distributes the software in stand-alone form
//     without changes from the version obtained under this License, or (ii) Licensee makes a
//     reference solely to the software portion of its product, Licensee must refer to the
//     software as "EnergyPlus version X" software, where "X" is the version number Licensee
//     obtained under this License and may not use a different name for the software. Except as
//     specifically required in this Section (4), Licensee shall not use in a company name, a
//     product name, in advertising, publicity, or other promotional activities any name, trade
//     name, trademark, logo, or other designation of "EnergyPlus", "E+", "e+" or confusingly
//     similar designation, without the U.S. Department of Energy's prior written consent.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.

// EnergyPlus::Dehumidifier:Desiccant:System Unit Tests

// Google Test Headers
#include <gtest/gtest.h>

// EnergyPlus Headers
#include <ElectricPowerServiceManager.hh>
#include <EnergyPlus/BranchInputManager.hh>
#include <EnergyPlus/DataEnvironment.hh>
#include <EnergyPlus/DataGlobals.hh>
#include <EnergyPlus/DataHVACGlobals.hh>
#include <EnergyPlus/DataSizing.hh>
#include <EnergyPlus/DesiccantDehumidifiers.hh>
#include <EnergyPlus/HeatBalanceManager.hh>
#include <EnergyPlus/HeatingCoils.hh>
#include <EnergyPlus/OutputProcessor.hh>
#include <EnergyPlus/OutputReportPredefined.hh>
#include <EnergyPlus/OutputReportTabular.hh>
#include <EnergyPlus/Psychrometrics.hh>
#include <EnergyPlus/SimulationManager.hh>
#include <EnergyPlus/SizingManager.hh>
#include <EnergyPlus/WaterCoils.hh>
#include <General.hh>

#include "Fixtures/EnergyPlusFixture.hh"

using namespace EnergyPlus;
using namespace EnergyPlus::DesiccantDehumidifiers;
using namespace EnergyPlus::BranchInputManager;
using namespace EnergyPlus::DataGlobals;
using namespace EnergyPlus::DataSizing;
using namespace EnergyPlus::DataEnvironment;
using namespace EnergyPlus::HeatBalanceManager;
using namespace EnergyPlus::HeatingCoils;
using namespace EnergyPlus::OutputProcessor;
using namespace EnergyPlus::OutputReportTabular;
using namespace EnergyPlus::OutputReportPredefined;
using namespace EnergyPlus::Psychrometrics;
using namespace EnergyPlus::WaterCoils;
using namespace ObjexxFCL;

namespace EnergyPlus {

TEST_F(EnergyPlusFixture, DesiccantDehum_OnOASystemTest)
{

    std::string CompName("");
    bool FirstHVACIteration(true);
    int DesicDehumNum(1);
    int CompIndex(1);
    int CoilIndex(0);
    int loop(0);
    Real64 RegCoilDesInletTemp(0.0);
    Real64 RegCoilDesOutletTemp(0.0);
    Real64 RegCoilInletAirMassFlowRate(0.0);
    Real64 RegCoilCapacity(0.0);

    std::string const idf_objects = delimited_string({

        "  Building,",
        "    Building,                !- Name",
        "    30.,                     !- North Axis {deg}",
        "    City,                    !- Terrain",
        "    0.04,                    !- Loads Convergence Tolerance Value",
        "    0.4,                     !- Temperature Convergence Tolerance Value {deltaC}",
        "    FullExterior,            !- Solar Distribution",
        "    40,                      !- Maximum Number of Warmup Days",
        "    6;                       !- Minimum Number of Warmup Days",

        "  Timestep,6;",

        "  SurfaceConvectionAlgorithm:Inside,Simple;",

        "  SurfaceConvectionAlgorithm:Outside,SimpleCombined;",

        "  HeatBalanceAlgorithm,ConductionTransferFunction;",

        "  SimulationControl,",
        "    Yes,                     !- Do Zone Sizing Calculation",
        "    Yes,                     !- Do System Sizing Calculation",
        "    No,                      !- Do Plant Sizing Calculation",
        "    Yes,                     !- Run Simulation for Sizing Periods",
        "    No;                      !- Run Simulation for Weather File Run Periods",

        "  RunPeriod,",
        "    ,                        !- Name",
        "    1,                       !- Begin Month",
        "    1,                       !- Begin Day of Month",
        "    ,                        !- Begin Year",
        "    1,                       !- End Month",
        "    5,                       !- End Day of Month",
        "    ,                        !- End Year",
        "    Tuesday,                 !- Day of Week for Start Day",
        "    Yes,                     !- Use Weather File Holidays and Special Days",
        "    Yes,                     !- Use Weather File Daylight Saving Period",
        "    No,                      !- Apply Weekend Holiday Rule",
        "    Yes,                     !- Use Weather File Rain Indicators",
        "    Yes;                     !- Use Weather File Snow Indicators",

        "  Site:Location,",
        "    Miami Intl Ap FL USA WMO=722020,  !- Name",
        "    25.82,                   !- Latitude {deg}",
        "    -80.30,                  !- Longitude {deg}",
        "    -5.00,                   !- Time Zone {hr}",
        "    11.00;                   !- Elevation {m}",

        "  SizingPeriod:DesignDay,",
        "    Miami Intl Ap Ann Htg 99.6% Condns DB,  !- Name",
        "    1,                       !- Month",
        "    21,                      !- Day of Month",
        "    WinterDesignDay,         !- Day Type",
        "    8.7,                     !- Maximum Dry-Bulb Temperature {C}",
        "    0.0,                     !- Daily Dry-Bulb Temperature Range {deltaC}",
        "    ,                        !- Dry-Bulb Temperature Range Modifier Type",
        "    ,                        !- Dry-Bulb Temperature Range Modifier Day Schedule Name",
        "    Wetbulb,                 !- Humidity Condition Type",
        "    8.7,                     !- Wetbulb or DewPoint at Maximum Dry-Bulb {C}",
        "    ,                        !- Humidity Condition Day Schedule Name",
        "    ,                        !- Humidity Ratio at Maximum Dry-Bulb {kgWater/kgDryAir}",
        "    ,                        !- Enthalpy at Maximum Dry-Bulb {J/kg}",
        "    ,                        !- Daily Wet-Bulb Temperature Range {deltaC}",
        "    101217.,                 !- Barometric Pressure {Pa}",
        "    3.8,                     !- Wind Speed {m/s}",
        "    340,                     !- Wind Direction {deg}",
        "    No,                      !- Rain Indicator",
        "    No,                      !- Snow Indicator",
        "    No,                      !- Daylight Saving Time Indicator",
        "    ASHRAEClearSky,          !- Solar Model Indicator",
        "    ,                        !- Beam Solar Day Schedule Name",
        "    ,                        !- Diffuse Solar Day Schedule Name",
        "    ,                        !- ASHRAE Clear Sky Optical Depth for Beam Irradiance (taub) {dimensionless}",
        "    ,                        !- ASHRAE Clear Sky Optical Depth for Diffuse Irradiance (taud) {dimensionless}",
        "    0.00;                    !- Sky Clearness",

        "  SizingPeriod:DesignDay,",
        "    Miami Intl Ap Ann Clg .4% Condns DB=>MWB,  !- Name",
        "    7,                       !- Month",
        "    21,                      !- Day of Month",
        "    SummerDesignDay,         !- Day Type",
        "    33.2,                    !- Maximum Dry-Bulb Temperature {C}",
        "    6.7,                     !- Daily Dry-Bulb Temperature Range {deltaC}",
        "    ,                        !- Dry-Bulb Temperature Range Modifier Type",
        "    ,                        !- Dry-Bulb Temperature Range Modifier Day Schedule Name",
        "    Wetbulb,                 !- Humidity Condition Type",
        "    25.3,                    !- Wetbulb or DewPoint at Maximum Dry-Bulb {C}",
        "    ,                        !- Humidity Condition Day Schedule Name",
        "    ,                        !- Humidity Ratio at Maximum Dry-Bulb {kgWater/kgDryAir}",
        "    ,                        !- Enthalpy at Maximum Dry-Bulb {J/kg}",
        "    ,                        !- Daily Wet-Bulb Temperature Range {deltaC}",
        "    101217.,                 !- Barometric Pressure {Pa}",
        "    4.5,                     !- Wind Speed {m/s}",
        "    140,                     !- Wind Direction {deg}",
        "    No,                      !- Rain Indicator",
        "    No,                      !- Snow Indicator",
        "    No,                      !- Daylight Saving Time Indicator",
        "    ASHRAEClearSky,          !- Solar Model Indicator",
        "    ,                        !- Beam Solar Day Schedule Name",
        "    ,                        !- Diffuse Solar Day Schedule Name",
        "    ,                        !- ASHRAE Clear Sky Optical Depth for Beam Irradiance (taub) {dimensionless}",
        "    ,                        !- ASHRAE Clear Sky Optical Depth for Diffuse Irradiance (taud) {dimensionless}",
        "    1.00;                    !- Sky Clearness",

        "  Site:GroundTemperature:BuildingSurface,21.0,21.5,22.0,22.2,22.2,22.2,22.2,22.2,22.2,22.0,21.5,21.0;",

        "  Material,",
        "    WD10,                    !- Name",
        "    MediumSmooth,            !- Roughness",
        "    0.667,                   !- Thickness {m}",
        "    0.115,                   !- Conductivity {W/m-K}",
        "    513,                     !- Density {kg/m3}",
        "    1381,                    !- Specific Heat {J/kg-K}",
        "    0.9,                     !- Thermal Absorptance",
        "    0.78,                    !- Solar Absorptance",
        "    0.78;                    !- Visible Absorptance",

        "  Material,",
        "    RG01,                    !- Name",
        "    Rough,                   !- Roughness",
        "    1.2700000E-02,           !- Thickness {m}",
        "    1.442000,                !- Conductivity {W/m-K}",
        "    881.0000,                !- Density {kg/m3}",
        "    1674.000,                !- Specific Heat {J/kg-K}",
        "    0.9000000,               !- Thermal Absorptance",
        "    0.6500000,               !- Solar Absorptance",
        "    0.6500000;               !- Visible Absorptance",

        "  Material,",
        "    BR01,                    !- Name",
        "    VeryRough,               !- Roughness",
        "    9.4999997E-03,           !- Thickness {m}",
        "    0.1620000,               !- Conductivity {W/m-K}",
        "    1121.000,                !- Density {kg/m3}",
        "    1464.000,                !- Specific Heat {J/kg-K}",
        "    0.9000000,               !- Thermal Absorptance",
        "    0.7000000,               !- Solar Absorptance",
        "    0.7000000;               !- Visible Absorptance",

        "  Material,",
        "    IN46,                    !- Name",
        "    VeryRough,               !- Roughness",
        "    7.6200001E-02,           !- Thickness {m}",
        "    2.3000000E-02,           !- Conductivity {W/m-K}",
        "    24.00000,                !- Density {kg/m3}",
        "    1590.000,                !- Specific Heat {J/kg-K}",
        "    0.9000000,               !- Thermal Absorptance",
        "    0.5000000,               !- Solar Absorptance",
        "    0.5000000;               !- Visible Absorptance",

        "  Material,",
        "    WD01,                    !- Name",
        "    MediumSmooth,            !- Roughness",
        "    1.9099999E-02,           !- Thickness {m}",
        "    0.1150000,               !- Conductivity {W/m-K}",
        "    513.0000,                !- Density {kg/m3}",
        "    1381.000,                !- Specific Heat {J/kg-K}",
        "    0.9000000,               !- Thermal Absorptance",
        "    0.7800000,               !- Solar Absorptance",
        "    0.7800000;               !- Visible Absorptance",

        "  Material,",
        "    PW03,                    !- Name",
        "    MediumSmooth,            !- Roughness",
        "    1.2700000E-02,           !- Thickness {m}",
        "    0.1150000,               !- Conductivity {W/m-K}",
        "    545.0000,                !- Density {kg/m3}",
        "    1213.000,                !- Specific Heat {J/kg-K}",
        "    0.9000000,               !- Thermal Absorptance",
        "    0.7800000,               !- Solar Absorptance",
        "    0.7800000;               !- Visible Absorptance",

        "  Material,",
        "    IN02,                    !- Name",
        "    Rough,                   !- Roughness",
        "    9.0099998E-02,           !- Thickness {m}",
        "    4.3000001E-02,           !- Conductivity {W/m-K}",
        "    10.00000,                !- Density {kg/m3}",
        "    837.0000,                !- Specific Heat {J/kg-K}",
        "    0.9000000,               !- Thermal Absorptance",
        "    0.7500000,               !- Solar Absorptance",
        "    0.7500000;               !- Visible Absorptance",

        "  Material,",
        "    GP01,                    !- Name",
        "    MediumSmooth,            !- Roughness",
        "    1.2700000E-02,           !- Thickness {m}",
        "    0.1600000,               !- Conductivity {W/m-K}",
        "    801.0000,                !- Density {kg/m3}",
        "    837.0000,                !- Specific Heat {J/kg-K}",
        "    0.9000000,               !- Thermal Absorptance",
        "    0.7500000,               !- Solar Absorptance",
        "    0.7500000;               !- Visible Absorptance",

        "  Material,",
        "    GP02,                    !- Name",
        "    MediumSmooth,            !- Roughness",
        "    1.5900001E-02,           !- Thickness {m}",
        "    0.1600000,               !- Conductivity {W/m-K}",
        "    801.0000,                !- Density {kg/m3}",
        "    837.0000,                !- Specific Heat {J/kg-K}",
        "    0.9000000,               !- Thermal Absorptance",
        "    0.7500000,               !- Solar Absorptance",
        "    0.7500000;               !- Visible Absorptance",

        "  Material,",
        "    CC03,                    !- Name",
        "    MediumRough,             !- Roughness",
        "    0.1016000,               !- Thickness {m}",
        "    1.310000,                !- Conductivity {W/m-K}",
        "    2243.000,                !- Density {kg/m3}",
        "    837.0000,                !- Specific Heat {J/kg-K}",
        "    0.9000000,               !- Thermal Absorptance",
        "    0.6500000,               !- Solar Absorptance",
        "    0.6500000;               !- Visible Absorptance",

        "  Material:NoMass,",
        "    CP01,                    !- Name",
        "    Rough,                   !- Roughness",
        "    0.3670000,               !- Thermal Resistance {m2-K/W}",
        "    0.9000000,               !- Thermal Absorptance",
        "    0.7500000,               !- Solar Absorptance",
        "    0.7500000;               !- Visible Absorptance",

        "  Material:NoMass,",
        "    MAT-SB-U,                !- Name",
        "    Rough,                   !- Roughness",
        "    0.117406666,             !- Thermal Resistance {m2-K/W}",
        "    0.65,                    !- Thermal Absorptance",
        "    0.65,                    !- Solar Absorptance",
        "    0.65;                    !- Visible Absorptance",

        "  Material:NoMass,",
        "    MAT-CLNG-1,              !- Name",
        "    Rough,                   !- Roughness",
        "    0.652259290,             !- Thermal Resistance {m2-K/W}",
        "    0.65,                    !- Thermal Absorptance",
        "    0.65,                    !- Solar Absorptance",
        "    0.65;                    !- Visible Absorptance",

        "  Material:NoMass,",
        "    MAT-FLOOR-1,             !- Name",
        "    Rough,                   !- Roughness",
        "    3.522199631,             !- Thermal Resistance {m2-K/W}",
        "    0.65,                    !- Thermal Absorptance",
        "    0.65,                    !- Solar Absorptance",
        "    0.65;                    !- Visible Absorptance",

        "  Material:AirGap,",
        "    AL21,                    !- Name",
        "    0.1570000;               !- Thermal Resistance {m2-K/W}",

        "  Material:AirGap,",
        "    AL23,                    !- Name",
        "    0.1530000;               !- Thermal Resistance {m2-K/W}",

        "  WindowMaterial:Glazing,",
        "    CLEAR 3MM,               !- Name",
        "    SpectralAverage,         !- Optical Data Type",
        "    ,                        !- Window Glass Spectral Data Set Name",
        "    0.003,                   !- Thickness {m}",
        "    0.837,                   !- Solar Transmittance at Normal Incidence",
        "    0.075,                   !- Front Side Solar Reflectance at Normal Incidence",
        "    0.075,                   !- Back Side Solar Reflectance at Normal Incidence",
        "    0.898,                   !- Visible Transmittance at Normal Incidence",
        "    0.081,                   !- Front Side Visible Reflectance at Normal Incidence",
        "    0.081,                   !- Back Side Visible Reflectance at Normal Incidence",
        "    0.0,                     !- Infrared Transmittance at Normal Incidence",
        "    0.84,                    !- Front Side Infrared Hemispherical Emissivity",
        "    0.84,                    !- Back Side Infrared Hemispherical Emissivity",
        "    0.9;                     !- Conductivity {W/m-K}",

        "  WindowMaterial:Glazing,",
        "    GREY 3MM,                !- Name",
        "    SpectralAverage,         !- Optical Data Type",
        "    ,                        !- Window Glass Spectral Data Set Name",
        "    0.003,                   !- Thickness {m}",
        "    0.626,                   !- Solar Transmittance at Normal Incidence",
        "    0.061,                   !- Front Side Solar Reflectance at Normal Incidence",
        "    0.061,                   !- Back Side Solar Reflectance at Normal Incidence",
        "    0.611,                   !- Visible Transmittance at Normal Incidence",
        "    0.061,                   !- Front Side Visible Reflectance at Normal Incidence",
        "    0.061,                   !- Back Side Visible Reflectance at Normal Incidence",
        "    0.0,                     !- Infrared Transmittance at Normal Incidence",
        "    0.84,                    !- Front Side Infrared Hemispherical Emissivity",
        "    0.84,                    !- Back Side Infrared Hemispherical Emissivity",
        "    0.9;                     !- Conductivity {W/m-K}",

        "  WindowMaterial:Glazing,",
        "    CLEAR 6MM,               !- Name",
        "    SpectralAverage,         !- Optical Data Type",
        "    ,                        !- Window Glass Spectral Data Set Name",
        "    0.006,                   !- Thickness {m}",
        "    0.775,                   !- Solar Transmittance at Normal Incidence",
        "    0.071,                   !- Front Side Solar Reflectance at Normal Incidence",
        "    0.071,                   !- Back Side Solar Reflectance at Normal Incidence",
        "    0.881,                   !- Visible Transmittance at Normal Incidence",
        "    0.080,                   !- Front Side Visible Reflectance at Normal Incidence",
        "    0.080,                   !- Back Side Visible Reflectance at Normal Incidence",
        "    0.0,                     !- Infrared Transmittance at Normal Incidence",
        "    0.84,                    !- Front Side Infrared Hemispherical Emissivity",
        "    0.84,                    !- Back Side Infrared Hemispherical Emissivity",
        "    0.9;                     !- Conductivity {W/m-K}",

        "  WindowMaterial:Glazing,",
        "    LoE CLEAR 6MM,           !- Name",
        "    SpectralAverage,         !- Optical Data Type",
        "    ,                        !- Window Glass Spectral Data Set Name",
        "    0.006,                   !- Thickness {m}",
        "    0.600,                   !- Solar Transmittance at Normal Incidence",
        "    0.170,                   !- Front Side Solar Reflectance at Normal Incidence",
        "    0.220,                   !- Back Side Solar Reflectance at Normal Incidence",
        "    0.840,                   !- Visible Transmittance at Normal Incidence",
        "    0.055,                   !- Front Side Visible Reflectance at Normal Incidence",
        "    0.078,                   !- Back Side Visible Reflectance at Normal Incidence",
        "    0.0,                     !- Infrared Transmittance at Normal Incidence",
        "    0.84,                    !- Front Side Infrared Hemispherical Emissivity",
        "    0.10,                    !- Back Side Infrared Hemispherical Emissivity",
        "    0.9;                     !- Conductivity {W/m-K}",

        "  WindowMaterial:Gas,",
        "    AIR 6MM,                 !- Name",
        "    Air,                     !- Gas Type",
        "    0.0063;                  !- Thickness {m}",

        "  WindowMaterial:Gas,",
        "    AIR 13MM,                !- Name",
        "    Air,                     !- Gas Type",
        "    0.0127;                  !- Thickness {m}",

        "  WindowMaterial:Gas,",
        "    ARGON 13MM,              !- Name",
        "    Argon,                   !- Gas Type",
        "    0.0127;                  !- Thickness {m}",

        "  Construction,",
        "    ROOF-1,                  !- Name",
        "    RG01,                    !- Outside Layer",
        "    BR01,                    !- Layer 2",
        "    IN46,                    !- Layer 3",
        "    WD01;                    !- Layer 4",

        "  Construction,",
        "    WALL-1,                  !- Name",
        "    WD01,                    !- Outside Layer",
        "    PW03,                    !- Layer 2",
        "    IN02,                    !- Layer 3",
        "    GP01;                    !- Layer 4",

        "  Construction,",
        "    CLNG-1,                  !- Name",
        "    MAT-CLNG-1;              !- Outside Layer",

        "  Construction,",
        "    SB-U,                    !- Name",
        "    MAT-SB-U;                !- Outside Layer",

        "  Construction,",
        "    FLOOR-1,                 !- Name",
        "    MAT-FLOOR-1;             !- Outside Layer",

        "  Construction,",
        "    FLOOR-SLAB-1,            !- Name",
        "    CC03,                    !- Outside Layer",
        "    CP01;                    !- Layer 2",

        "  Construction,",
        "    INT-WALL-1,              !- Name",
        "    GP02,                    !- Outside Layer",
        "    AL21,                    !- Layer 2",
        "    GP02;                    !- Layer 3",

        "  Construction,",
        "    Dbl Clr 3mm/13mm Air,    !- Name",
        "    CLEAR 3MM,               !- Outside Layer",
        "    AIR 13MM,                !- Layer 2",
        "    CLEAR 3MM;               !- Layer 3",

        "  Construction,",
        "    Dbl Clr 3mm/13mm Arg,    !- Name",
        "    CLEAR 3MM,               !- Outside Layer",
        "    ARGON 13MM,              !- Layer 2",
        "    CLEAR 3MM;               !- Layer 3",

        "  Construction,",
        "    Sgl Grey 3mm,            !- Name",
        "    GREY 3MM;                !- Outside Layer",

        "  Construction,",
        "    Dbl Clr 6mm/6mm Air,     !- Name",
        "    CLEAR 6MM,               !- Outside Layer",
        "    AIR 6MM,                 !- Layer 2",
        "    CLEAR 6MM;               !- Layer 3",

        "  Construction,",
        "    Dbl LoE (e2=.1) Clr 6mm/6mm Air,  !- Name",
        "    LoE CLEAR 6MM,           !- Outside Layer",
        "    AIR 6MM,                 !- Layer 2",
        "    CLEAR 6MM;               !- Layer 3",

        "  Zone,",
        "    PLENUM-1,                !- Name",
        "    0,                       !- Direction of Relative North {deg}",
        "    0,                       !- X Origin {m}",
        "    0,                       !- Y Origin {m}",
        "    0,                       !- Z Origin {m}",
        "    1,                       !- Type",
        "    1,                       !- Multiplier",
        "    0.609600067,             !- Ceiling Height {m}",
        "    283.2;                   !- Volume {m3}",

        "  Zone,",
        "    SPACE1-1,                !- Name",
        "    0,                       !- Direction of Relative North {deg}",
        "    0,                       !- X Origin {m}",
        "    0,                       !- Y Origin {m}",
        "    0,                       !- Z Origin {m}",
        "    1,                       !- Type",
        "    1,                       !- Multiplier",
        "    2.438400269,             !- Ceiling Height {m}",
        "    239.247360229;           !- Volume {m3}",

        "  Zone,",
        "    SPACE2-1,                !- Name",
        "    0,                       !- Direction of Relative North {deg}",
        "    0,                       !- X Origin {m}",
        "    0,                       !- Y Origin {m}",
        "    0,                       !- Z Origin {m}",
        "    1,                       !- Type",
        "    1,                       !- Multiplier",
        "    2.438400269,             !- Ceiling Height {m}",
        "    103.311355591;           !- Volume {m3}",

        "  Zone,",
        "    SPACE3-1,                !- Name",
        "    0,                       !- Direction of Relative North {deg}",
        "    0,                       !- X Origin {m}",
        "    0,                       !- Y Origin {m}",
        "    0,                       !- Z Origin {m}",
        "    1,                       !- Type",
        "    1,                       !- Multiplier",
        "    2.438400269,             !- Ceiling Height {m}",
        "    239.247360229;           !- Volume {m3}",

        "  Zone,",
        "    SPACE4-1,                !- Name",
        "    0,                       !- Direction of Relative North {deg}",
        "    0,                       !- X Origin {m}",
        "    0,                       !- Y Origin {m}",
        "    0,                       !- Z Origin {m}",
        "    1,                       !- Type",
        "    1,                       !- Multiplier",
        "    2.438400269,             !- Ceiling Height {m}",
        "    103.311355591;           !- Volume {m3}",

        "  Zone,",
        "    SPACE5-1,                !- Name",
        "    0,                       !- Direction of Relative North {deg}",
        "    0,                       !- X Origin {m}",
        "    0,                       !- Y Origin {m}",
        "    0,                       !- Z Origin {m}",
        "    1,                       !- Type",
        "    1,                       !- Multiplier",
        "    2.438400269,             !- Ceiling Height {m}",
        "    447.682556152;           !- Volume {m3}",

        "  GlobalGeometryRules,",
        "    UpperLeftCorner,         !- Starting Vertex Position",
        "    CounterClockWise,        !- Vertex Entry Direction",
        "    relative;                !- Coordinate System",

        "  BuildingSurface:Detailed,",
        "    WALL-1PF,                !- Name",
        "    WALL,                    !- Surface Type",
        "    WALL-1,                  !- Construction Name",
        "    PLENUM-1,                !- Zone Name",
        "    Outdoors,                !- Outside Boundary Condition",
        "    ,                        !- Outside Boundary Condition Object",
        "    SunExposed,              !- Sun Exposure",
        "    WindExposed,             !- Wind Exposure",
        "    0.50000,                 !- View Factor to Ground",
        "    4,                       !- Number of Vertices",
        "    0.0,0.0,3.0,  !- X,Y,Z ==> Vertex 1 {m}",
        "    0.0,0.0,2.4,  !- X,Y,Z ==> Vertex 2 {m}",
        "    30.5,0.0,2.4,  !- X,Y,Z ==> Vertex 3 {m}",
        "    30.5,0.0,3.0;  !- X,Y,Z ==> Vertex 4 {m}",

        "  BuildingSurface:Detailed,",
        "    WALL-1PR,                !- Name",
        "    WALL,                    !- Surface Type",
        "    WALL-1,                  !- Construction Name",
        "    PLENUM-1,                !- Zone Name",
        "    Outdoors,                !- Outside Boundary Condition",
        "    ,                        !- Outside Boundary Condition Object",
        "    SunExposed,              !- Sun Exposure",
        "    WindExposed,             !- Wind Exposure",
        "    0.50000,                 !- View Factor to Ground",
        "    4,                       !- Number of Vertices",
        "    30.5,0.0,3.0,  !- X,Y,Z ==> Vertex 1 {m}",
        "    30.5,0.0,2.4,  !- X,Y,Z ==> Vertex 2 {m}",
        "    30.5,15.2,2.4,  !- X,Y,Z ==> Vertex 3 {m}",
        "    30.5,15.2,3.0;  !- X,Y,Z ==> Vertex 4 {m}",

        "  BuildingSurface:Detailed,",
        "    WALL-1PB,                !- Name",
        "    WALL,                    !- Surface Type",
        "    WALL-1,                  !- Construction Name",
        "    PLENUM-1,                !- Zone Name",
        "    Outdoors,                !- Outside Boundary Condition",
        "    ,                        !- Outside Boundary Condition Object",
        "    SunExposed,              !- Sun Exposure",
        "    WindExposed,             !- Wind Exposure",
        "    0.50000,                 !- View Factor to Ground",
        "    4,                       !- Number of Vertices",
        "    30.5,15.2,3.0,  !- X,Y,Z ==> Vertex 1 {m}",
        "    30.5,15.2,2.4,  !- X,Y,Z ==> Vertex 2 {m}",
        "    0.0,15.2,2.4,  !- X,Y,Z ==> Vertex 3 {m}",
        "    0.0,15.2,3.0;  !- X,Y,Z ==> Vertex 4 {m}",

        "  BuildingSurface:Detailed,",
        "    WALL-1PL,                !- Name",
        "    WALL,                    !- Surface Type",
        "    WALL-1,                  !- Construction Name",
        "    PLENUM-1,                !- Zone Name",
        "    Outdoors,                !- Outside Boundary Condition",
        "    ,                        !- Outside Boundary Condition Object",
        "    SunExposed,              !- Sun Exposure",
        "    WindExposed,             !- Wind Exposure",
        "    0.50000,                 !- View Factor to Ground",
        "    4,                       !- Number of Vertices",
        "    0.0,15.2,3.0,  !- X,Y,Z ==> Vertex 1 {m}",
        "    0.0,15.2,2.4,  !- X,Y,Z ==> Vertex 2 {m}",
        "    0.0,0.0,2.4,  !- X,Y,Z ==> Vertex 3 {m}",
        "    0.0,0.0,3.0;  !- X,Y,Z ==> Vertex 4 {m}",

        "  BuildingSurface:Detailed,",
        "    TOP-1,                   !- Name",
        "    ROOF,                    !- Surface Type",
        "    ROOF-1,                  !- Construction Name",
        "    PLENUM-1,                !- Zone Name",
        "    Outdoors,                !- Outside Boundary Condition",
        "    ,                        !- Outside Boundary Condition Object",
        "    SunExposed,              !- Sun Exposure",
        "    WindExposed,             !- Wind Exposure",
        "    0.00000,                 !- View Factor to Ground",
        "    4,                       !- Number of Vertices",
        "    0.0,15.2,3.0,  !- X,Y,Z ==> Vertex 1 {m}",
        "    0.0,0.0,3.0,  !- X,Y,Z ==> Vertex 2 {m}",
        "    30.5,0.0,3.0,  !- X,Y,Z ==> Vertex 3 {m}",
        "    30.5,15.2,3.0;  !- X,Y,Z ==> Vertex 4 {m}",

        "  BuildingSurface:Detailed,",
        "    C1-1P,                   !- Name",
        "    FLOOR,                   !- Surface Type",
        "    CLNG-1,                  !- Construction Name",
        "    PLENUM-1,                !- Zone Name",
        "    Surface,                 !- Outside Boundary Condition",
        "    C1-1,                    !- Outside Boundary Condition Object",
        "    NoSun,                   !- Sun Exposure",
        "    NoWind,                  !- Wind Exposure",
        "    0.0,                     !- View Factor to Ground",
        "    4,                       !- Number of Vertices",
        "    26.8,3.7,2.4,  !- X,Y,Z ==> Vertex 1 {m}",
        "    30.5,0.0,2.4,  !- X,Y,Z ==> Vertex 2 {m}",
        "    0.0,0.0,2.4,  !- X,Y,Z ==> Vertex 3 {m}",
        "    3.7,3.7,2.4;  !- X,Y,Z ==> Vertex 4 {m}",

        "  BuildingSurface:Detailed,",
        "    C2-1P,                   !- Name",
        "    FLOOR,                   !- Surface Type",
        "    CLNG-1,                  !- Construction Name",
        "    PLENUM-1,                !- Zone Name",
        "    Surface,                 !- Outside Boundary Condition",
        "    C2-1,                    !- Outside Boundary Condition Object",
        "    NoSun,                   !- Sun Exposure",
        "    NoWind,                  !- Wind Exposure",
        "    0.0,                     !- View Factor to Ground",
        "    4,                       !- Number of Vertices",
        "    26.8,11.6,2.4,  !- X,Y,Z ==> Vertex 1 {m}",
        "    30.5,15.2,2.4,  !- X,Y,Z ==> Vertex 2 {m}",
        "    30.5,0.0,2.4,  !- X,Y,Z ==> Vertex 3 {m}",
        "    26.8,3.7,2.4;  !- X,Y,Z ==> Vertex 4 {m}",

        "  BuildingSurface:Detailed,",
        "    C3-1P,                   !- Name",
        "    FLOOR,                   !- Surface Type",
        "    CLNG-1,                  !- Construction Name",
        "    PLENUM-1,                !- Zone Name",
        "    Surface,                 !- Outside Boundary Condition",
        "    C3-1,                    !- Outside Boundary Condition Object",
        "    NoSun,                   !- Sun Exposure",
        "    NoWind,                  !- Wind Exposure",
        "    0.0,                     !- View Factor to Ground",
        "    4,                       !- Number of Vertices",
        "    26.8,11.6,2.4,  !- X,Y,Z ==> Vertex 1 {m}",
        "    3.7,11.6,2.4,  !- X,Y,Z ==> Vertex 2 {m}",
        "    0.0,15.2,2.4,  !- X,Y,Z ==> Vertex 3 {m}",
        "    30.5,15.2,2.4;  !- X,Y,Z ==> Vertex 4 {m}",

        "  BuildingSurface:Detailed,",
        "    C4-1P,                   !- Name",
        "    FLOOR,                   !- Surface Type",
        "    CLNG-1,                  !- Construction Name",
        "    PLENUM-1,                !- Zone Name",
        "    Surface,                 !- Outside Boundary Condition",
        "    C4-1,                    !- Outside Boundary Condition Object",
        "    NoSun,                   !- Sun Exposure",
        "    NoWind,                  !- Wind Exposure",
        "    0.0,                     !- View Factor to Ground",
        "    4,                       !- Number of Vertices",
        "    3.7,3.7,2.4,  !- X,Y,Z ==> Vertex 1 {m}",
        "    0.0,0.0,2.4,  !- X,Y,Z ==> Vertex 2 {m}",
        "    0.0,15.2,2.4,  !- X,Y,Z ==> Vertex 3 {m}",
        "    3.7,11.6,2.4;  !- X,Y,Z ==> Vertex 4 {m}",

        "  BuildingSurface:Detailed,",
        "    C5-1P,                   !- Name",
        "    FLOOR,                   !- Surface Type",
        "    CLNG-1,                  !- Construction Name",
        "    PLENUM-1,                !- Zone Name",
        "    Surface,                 !- Outside Boundary Condition",
        "    C5-1,                    !- Outside Boundary Condition Object",
        "    NoSun,                   !- Sun Exposure",
        "    NoWind,                  !- Wind Exposure",
        "    0.0,                     !- View Factor to Ground",
        "    4,                       !- Number of Vertices",
        "    26.8,11.6,2.4,  !- X,Y,Z ==> Vertex 1 {m}",
        "    26.8,3.7,2.4,  !- X,Y,Z ==> Vertex 2 {m}",
        "    3.7,3.7,2.4,  !- X,Y,Z ==> Vertex 3 {m}",
        "    3.7,11.6,2.4;  !- X,Y,Z ==> Vertex 4 {m}",

        "  BuildingSurface:Detailed,",
        "    FRONT-1,                 !- Name",
        "    WALL,                    !- Surface Type",
        "    WALL-1,                  !- Construction Name",
        "    SPACE1-1,                !- Zone Name",
        "    Outdoors,                !- Outside Boundary Condition",
        "    ,                        !- Outside Boundary Condition Object",
        "    SunExposed,              !- Sun Exposure",
        "    WindExposed,             !- Wind Exposure",
        "    0.50000,                 !- View Factor to Ground",
        "    4,                       !- Number of Vertices",
        "    0.0,0.0,2.4,  !- X,Y,Z ==> Vertex 1 {m}",
        "    0.0,0.0,0.0,  !- X,Y,Z ==> Vertex 2 {m}",
        "    30.5,0.0,0.0,  !- X,Y,Z ==> Vertex 3 {m}",
        "    30.5,0.0,2.4;  !- X,Y,Z ==> Vertex 4 {m}",

        "  BuildingSurface:Detailed,",
        "    C1-1,                    !- Name",
        "    CEILING,                 !- Surface Type",
        "    CLNG-1,                  !- Construction Name",
        "    SPACE1-1,                !- Zone Name",
        "    Surface,                 !- Outside Boundary Condition",
        "    C1-1P,                   !- Outside Boundary Condition Object",
        "    NoSun,                   !- Sun Exposure",
        "    NoWind,                  !- Wind Exposure",
        "    0.0,                     !- View Factor to Ground",
        "    4,                       !- Number of Vertices",
        "    3.7,3.7,2.4,  !- X,Y,Z ==> Vertex 1 {m}",
        "    0.0,0.0,2.4,  !- X,Y,Z ==> Vertex 2 {m}",
        "    30.5,0.0,2.4,  !- X,Y,Z ==> Vertex 3 {m}",
        "    26.8,3.7,2.4;  !- X,Y,Z ==> Vertex 4 {m}",

        "  BuildingSurface:Detailed,",
        "    F1-1,                    !- Name",
        "    FLOOR,                   !- Surface Type",
        "    FLOOR-SLAB-1,            !- Construction Name",
        "    SPACE1-1,                !- Zone Name",
        "    Ground,                  !- Outside Boundary Condition",
        "    ,                        !- Outside Boundary Condition Object",
        "    NoSun,                   !- Sun Exposure",
        "    NoWind,                  !- Wind Exposure",
        "    0.0,                     !- View Factor to Ground",
        "    4,                       !- Number of Vertices",
        "    26.8,3.7,0.0,  !- X,Y,Z ==> Vertex 1 {m}",
        "    30.5,0.0,0.0,  !- X,Y,Z ==> Vertex 2 {m}",
        "    0.0,0.0,0.0,  !- X,Y,Z ==> Vertex 3 {m}",
        "    3.7,3.7,0.0;  !- X,Y,Z ==> Vertex 4 {m}",

        "  BuildingSurface:Detailed,",
        "    SB12,                    !- Name",
        "    WALL,                    !- Surface Type",
        "    INT-WALL-1,              !- Construction Name",
        "    SPACE1-1,                !- Zone Name",
        "    Surface,                 !- Outside Boundary Condition",
        "    SB21,                    !- Outside Boundary Condition Object",
        "    NoSun,                   !- Sun Exposure",
        "    NoWind,                  !- Wind Exposure",
        "    0.0,                     !- View Factor to Ground",
        "    4,                       !- Number of Vertices",
        "    30.5,0.0,2.4,  !- X,Y,Z ==> Vertex 1 {m}",
        "    30.5,0.0,0.0,  !- X,Y,Z ==> Vertex 2 {m}",
        "    26.8,3.7,0.0,  !- X,Y,Z ==> Vertex 3 {m}",
        "    26.8,3.7,2.4;  !- X,Y,Z ==> Vertex 4 {m}",

        "  BuildingSurface:Detailed,",
        "    SB14,                    !- Name",
        "    WALL,                    !- Surface Type",
        "    INT-WALL-1,              !- Construction Name",
        "    SPACE1-1,                !- Zone Name",
        "    Surface,                 !- Outside Boundary Condition",
        "    SB41,                    !- Outside Boundary Condition Object",
        "    NoSun,                   !- Sun Exposure",
        "    NoWind,                  !- Wind Exposure",
        "    0.0,                     !- View Factor to Ground",
        "    4,                       !- Number of Vertices",
        "    3.7,3.7,2.4,  !- X,Y,Z ==> Vertex 1 {m}",
        "    3.7,3.7,0.0,  !- X,Y,Z ==> Vertex 2 {m}",
        "    0.0,0.0,0.0,  !- X,Y,Z ==> Vertex 3 {m}",
        "    0.0,0.0,2.4;  !- X,Y,Z ==> Vertex 4 {m}",

        "  BuildingSurface:Detailed,",
        "    SB15,                    !- Name",
        "    WALL,                    !- Surface Type",
        "    INT-WALL-1,              !- Construction Name",
        "    SPACE1-1,                !- Zone Name",
        "    Surface,                 !- Outside Boundary Condition",
        "    SB51,                    !- Outside Boundary Condition Object",
        "    NoSun,                   !- Sun Exposure",
        "    NoWind,                  !- Wind Exposure",
        "    0.0,                     !- View Factor to Ground",
        "    4,                       !- Number of Vertices",
        "    26.8,3.7,2.4,  !- X,Y,Z ==> Vertex 1 {m}",
        "    26.8,3.7,0.0,  !- X,Y,Z ==> Vertex 2 {m}",
        "    3.7,3.7,0.0,  !- X,Y,Z ==> Vertex 3 {m}",
        "    3.7,3.7,2.4;  !- X,Y,Z ==> Vertex 4 {m}",

        "  BuildingSurface:Detailed,",
        "    RIGHT-1,                 !- Name",
        "    WALL,                    !- Surface Type",
        "    WALL-1,                  !- Construction Name",
        "    SPACE2-1,                !- Zone Name",
        "    Outdoors,                !- Outside Boundary Condition",
        "    ,                        !- Outside Boundary Condition Object",
        "    SunExposed,              !- Sun Exposure",
        "    WindExposed,             !- Wind Exposure",
        "    0.50000,                 !- View Factor to Ground",
        "    4,                       !- Number of Vertices",
        "    30.5,0.0,2.4,  !- X,Y,Z ==> Vertex 1 {m}",
        "    30.5,0.0,0.0,  !- X,Y,Z ==> Vertex 2 {m}",
        "    30.5,15.2,0.0,  !- X,Y,Z ==> Vertex 3 {m}",
        "    30.5,15.2,2.4;  !- X,Y,Z ==> Vertex 4 {m}",

        "  BuildingSurface:Detailed,",
        "    C2-1,                    !- Name",
        "    CEILING,                 !- Surface Type",
        "    CLNG-1,                  !- Construction Name",
        "    SPACE2-1,                !- Zone Name",
        "    Surface,                 !- Outside Boundary Condition",
        "    C2-1P,                   !- Outside Boundary Condition Object",
        "    NoSun,                   !- Sun Exposure",
        "    NoWind,                  !- Wind Exposure",
        "    0.0,                     !- View Factor to Ground",
        "    4,                       !- Number of Vertices",
        "    26.8,3.7,2.4,  !- X,Y,Z ==> Vertex 1 {m}",
        "    30.5,0.0,2.4,  !- X,Y,Z ==> Vertex 2 {m}",
        "    30.5,15.2,2.4,  !- X,Y,Z ==> Vertex 3 {m}",
        "    26.8,11.6,2.4;  !- X,Y,Z ==> Vertex 4 {m}",

        "  BuildingSurface:Detailed,",
        "    F2-1,                    !- Name",
        "    FLOOR,                   !- Surface Type",
        "    FLOOR-SLAB-1,            !- Construction Name",
        "    SPACE2-1,                !- Zone Name",
        "    Ground,                  !- Outside Boundary Condition",
        "    ,                        !- Outside Boundary Condition Object",
        "    NoSun,                   !- Sun Exposure",
        "    NoWind,                  !- Wind Exposure",
        "    0.0,                     !- View Factor to Ground",
        "    4,                       !- Number of Vertices",
        "    26.8,11.6,0.0,  !- X,Y,Z ==> Vertex 1 {m}",
        "    30.5,15.2,0.0,  !- X,Y,Z ==> Vertex 2 {m}",
        "    30.5,0.0,0.0,  !- X,Y,Z ==> Vertex 3 {m}",
        "    26.8,3.7,0.0;  !- X,Y,Z ==> Vertex 4 {m}",

        "  BuildingSurface:Detailed,",
        "    SB21,                    !- Name",
        "    WALL,                    !- Surface Type",
        "    INT-WALL-1,              !- Construction Name",
        "    SPACE2-1,                !- Zone Name",
        "    Surface,                 !- Outside Boundary Condition",
        "    SB12,                    !- Outside Boundary Condition Object",
        "    NoSun,                   !- Sun Exposure",
        "    NoWind,                  !- Wind Exposure",
        "    0.0,                     !- View Factor to Ground",
        "    4,                       !- Number of Vertices",
        "    26.8,3.7,2.4,  !- X,Y,Z ==> Vertex 1 {m}",
        "    26.8,3.7,0.0,  !- X,Y,Z ==> Vertex 2 {m}",
        "    30.5,0.0,0.0,  !- X,Y,Z ==> Vertex 3 {m}",
        "    30.5,0.0,2.4;  !- X,Y,Z ==> Vertex 4 {m}",

        "  BuildingSurface:Detailed,",
        "    SB23,                    !- Name",
        "    WALL,                    !- Surface Type",
        "    INT-WALL-1,              !- Construction Name",
        "    SPACE2-1,                !- Zone Name",
        "    Surface,                 !- Outside Boundary Condition",
        "    SB32,                    !- Outside Boundary Condition Object",
        "    NoSun,                   !- Sun Exposure",
        "    NoWind,                  !- Wind Exposure",
        "    0.0,                     !- View Factor to Ground",
        "    4,                       !- Number of Vertices",
        "    30.5,15.2,2.4,  !- X,Y,Z ==> Vertex 1 {m}",
        "    30.5,15.2,0.0,  !- X,Y,Z ==> Vertex 2 {m}",
        "    26.8,11.6,0.0,  !- X,Y,Z ==> Vertex 3 {m}",
        "    26.8,11.6,2.4;  !- X,Y,Z ==> Vertex 4 {m}",

        "  BuildingSurface:Detailed,",
        "    SB25,                    !- Name",
        "    WALL,                    !- Surface Type",
        "    INT-WALL-1,              !- Construction Name",
        "    SPACE2-1,                !- Zone Name",
        "    Surface,                 !- Outside Boundary Condition",
        "    SB52,                    !- Outside Boundary Condition Object",
        "    NoSun,                   !- Sun Exposure",
        "    NoWind,                  !- Wind Exposure",
        "    0.0,                     !- View Factor to Ground",
        "    4,                       !- Number of Vertices",
        "    26.8,11.6,2.4,  !- X,Y,Z ==> Vertex 1 {m}",
        "    26.8,11.6,0.0,  !- X,Y,Z ==> Vertex 2 {m}",
        "    26.8,3.7,0.0,  !- X,Y,Z ==> Vertex 3 {m}",
        "    26.8,3.7,2.4;  !- X,Y,Z ==> Vertex 4 {m}",

        "  BuildingSurface:Detailed,",
        "    BACK-1,                  !- Name",
        "    WALL,                    !- Surface Type",
        "    WALL-1,                  !- Construction Name",
        "    SPACE3-1,                !- Zone Name",
        "    Outdoors,                !- Outside Boundary Condition",
        "    ,                        !- Outside Boundary Condition Object",
        "    SunExposed,              !- Sun Exposure",
        "    WindExposed,             !- Wind Exposure",
        "    0.50000,                 !- View Factor to Ground",
        "    4,                       !- Number of Vertices",
        "    30.5,15.2,2.4,  !- X,Y,Z ==> Vertex 1 {m}",
        "    30.5,15.2,0.0,  !- X,Y,Z ==> Vertex 2 {m}",
        "    0.0,15.2,0.0,  !- X,Y,Z ==> Vertex 3 {m}",
        "    0.0,15.2,2.4;  !- X,Y,Z ==> Vertex 4 {m}",

        "  BuildingSurface:Detailed,",
        "    C3-1,                    !- Name",
        "    CEILING,                 !- Surface Type",
        "    CLNG-1,                  !- Construction Name",
        "    SPACE3-1,                !- Zone Name",
        "    Surface,                 !- Outside Boundary Condition",
        "    C3-1P,                   !- Outside Boundary Condition Object",
        "    NoSun,                   !- Sun Exposure",
        "    NoWind,                  !- Wind Exposure",
        "    0.0,                     !- View Factor to Ground",
        "    4,                       !- Number of Vertices",
        "    30.5,15.2,2.4,  !- X,Y,Z ==> Vertex 1 {m}",
        "    0.0,15.2,2.4,  !- X,Y,Z ==> Vertex 2 {m}",
        "    3.7,11.6,2.4,  !- X,Y,Z ==> Vertex 3 {m}",
        "    26.8,11.6,2.4;  !- X,Y,Z ==> Vertex 4 {m}",

        "  BuildingSurface:Detailed,",
        "    F3-1,                    !- Name",
        "    FLOOR,                   !- Surface Type",
        "    FLOOR-SLAB-1,            !- Construction Name",
        "    SPACE3-1,                !- Zone Name",
        "    Ground,                  !- Outside Boundary Condition",
        "    ,                        !- Outside Boundary Condition Object",
        "    NoSun,                   !- Sun Exposure",
        "    NoWind,                  !- Wind Exposure",
        "    0.0,                     !- View Factor to Ground",
        "    4,                       !- Number of Vertices",
        "    26.8,11.6,0.0,  !- X,Y,Z ==> Vertex 1 {m}",
        "    3.7,11.6,0.0,  !- X,Y,Z ==> Vertex 2 {m}",
        "    0.0,15.2,0.0,  !- X,Y,Z ==> Vertex 3 {m}",
        "    30.5,15.2,0.0;  !- X,Y,Z ==> Vertex 4 {m}",

        "  BuildingSurface:Detailed,",
        "    SB32,                    !- Name",
        "    WALL,                    !- Surface Type",
        "    INT-WALL-1,              !- Construction Name",
        "    SPACE3-1,                !- Zone Name",
        "    Surface,                 !- Outside Boundary Condition",
        "    SB23,                    !- Outside Boundary Condition Object",
        "    NoSun,                   !- Sun Exposure",
        "    NoWind,                  !- Wind Exposure",
        "    0.0,                     !- View Factor to Ground",
        "    4,                       !- Number of Vertices",
        "    26.8,11.6,2.4,  !- X,Y,Z ==> Vertex 1 {m}",
        "    26.8,11.6,0.0,  !- X,Y,Z ==> Vertex 2 {m}",
        "    30.5,15.2,0.0,  !- X,Y,Z ==> Vertex 3 {m}",
        "    30.5,15.2,2.4;  !- X,Y,Z ==> Vertex 4 {m}",

        "  BuildingSurface:Detailed,",
        "    SB34,                    !- Name",
        "    WALL,                    !- Surface Type",
        "    INT-WALL-1,              !- Construction Name",
        "    SPACE3-1,                !- Zone Name",
        "    Surface,                 !- Outside Boundary Condition",
        "    SB43,                    !- Outside Boundary Condition Object",
        "    NoSun,                   !- Sun Exposure",
        "    NoWind,                  !- Wind Exposure",
        "    0.0,                     !- View Factor to Ground",
        "    4,                       !- Number of Vertices",
        "    0.0,15.2,2.4,  !- X,Y,Z ==> Vertex 1 {m}",
        "    0.0,15.2,0.0,  !- X,Y,Z ==> Vertex 2 {m}",
        "    3.7,11.6,0.0,  !- X,Y,Z ==> Vertex 3 {m}",
        "    3.7,11.6,2.4;  !- X,Y,Z ==> Vertex 4 {m}",

        "  BuildingSurface:Detailed,",
        "    SB35,                    !- Name",
        "    WALL,                    !- Surface Type",
        "    INT-WALL-1,              !- Construction Name",
        "    SPACE3-1,                !- Zone Name",
        "    Surface,                 !- Outside Boundary Condition",
        "    SB53,                    !- Outside Boundary Condition Object",
        "    NoSun,                   !- Sun Exposure",
        "    NoWind,                  !- Wind Exposure",
        "    0.0,                     !- View Factor to Ground",
        "    4,                       !- Number of Vertices",
        "    3.7,11.6,2.4,  !- X,Y,Z ==> Vertex 1 {m}",
        "    3.7,11.6,0.0,  !- X,Y,Z ==> Vertex 2 {m}",
        "    26.8,11.6,0.0,  !- X,Y,Z ==> Vertex 3 {m}",
        "    26.8,11.6,2.4;  !- X,Y,Z ==> Vertex 4 {m}",

        "  BuildingSurface:Detailed,",
        "    LEFT-1,                  !- Name",
        "    WALL,                    !- Surface Type",
        "    WALL-1,                  !- Construction Name",
        "    SPACE4-1,                !- Zone Name",
        "    Outdoors,                !- Outside Boundary Condition",
        "    ,                        !- Outside Boundary Condition Object",
        "    SunExposed,              !- Sun Exposure",
        "    WindExposed,             !- Wind Exposure",
        "    0.50000,                 !- View Factor to Ground",
        "    4,                       !- Number of Vertices",
        "    0.0,15.2,2.4,  !- X,Y,Z ==> Vertex 1 {m}",
        "    0.0,15.2,0.0,  !- X,Y,Z ==> Vertex 2 {m}",
        "    0.0,0.0,0.0,  !- X,Y,Z ==> Vertex 3 {m}",
        "    0.0,0.0,2.4;  !- X,Y,Z ==> Vertex 4 {m}",

        "  BuildingSurface:Detailed,",
        "    C4-1,                    !- Name",
        "    CEILING,                 !- Surface Type",
        "    CLNG-1,                  !- Construction Name",
        "    SPACE4-1,                !- Zone Name",
        "    Surface,                 !- Outside Boundary Condition",
        "    C4-1P,                   !- Outside Boundary Condition Object",
        "    NoSun,                   !- Sun Exposure",
        "    NoWind,                  !- Wind Exposure",
        "    0.0,                     !- View Factor to Ground",
        "    4,                       !- Number of Vertices",
        "    3.7,11.6,2.4,  !- X,Y,Z ==> Vertex 1 {m}",
        "    0.0,15.2,2.4,  !- X,Y,Z ==> Vertex 2 {m}",
        "    0.0,0.0,2.4,  !- X,Y,Z ==> Vertex 3 {m}",
        "    3.7,3.7,2.4;  !- X,Y,Z ==> Vertex 4 {m}",

        "  BuildingSurface:Detailed,",
        "    F4-1,                    !- Name",
        "    FLOOR,                   !- Surface Type",
        "    FLOOR-SLAB-1,            !- Construction Name",
        "    SPACE4-1,                !- Zone Name",
        "    Ground,                  !- Outside Boundary Condition",
        "    ,                        !- Outside Boundary Condition Object",
        "    NoSun,                   !- Sun Exposure",
        "    NoWind,                  !- Wind Exposure",
        "    0.0,                     !- View Factor to Ground",
        "    4,                       !- Number of Vertices",
        "    3.7,3.7,0.0,  !- X,Y,Z ==> Vertex 1 {m}",
        "    0.0,0.0,0.0,  !- X,Y,Z ==> Vertex 2 {m}",
        "    0.0,15.2,0.0,  !- X,Y,Z ==> Vertex 3 {m}",
        "    3.7,11.6,0.0;  !- X,Y,Z ==> Vertex 4 {m}",

        "  BuildingSurface:Detailed,",
        "    SB41,                    !- Name",
        "    WALL,                    !- Surface Type",
        "    INT-WALL-1,              !- Construction Name",
        "    SPACE4-1,                !- Zone Name",
        "    Surface,                 !- Outside Boundary Condition",
        "    SB14,                    !- Outside Boundary Condition Object",
        "    NoSun,                   !- Sun Exposure",
        "    NoWind,                  !- Wind Exposure",
        "    0.0,                     !- View Factor to Ground",
        "    4,                       !- Number of Vertices",
        "    0.0,0.0,2.4,  !- X,Y,Z ==> Vertex 1 {m}",
        "    0.0,0.0,0.0,  !- X,Y,Z ==> Vertex 2 {m}",
        "    3.7,3.7,0.0,  !- X,Y,Z ==> Vertex 3 {m}",
        "    3.7,3.7,2.4;  !- X,Y,Z ==> Vertex 4 {m}",

        "  BuildingSurface:Detailed,",
        "    SB43,                    !- Name",
        "    WALL,                    !- Surface Type",
        "    INT-WALL-1,              !- Construction Name",
        "    SPACE4-1,                !- Zone Name",
        "    Surface,                 !- Outside Boundary Condition",
        "    SB34,                    !- Outside Boundary Condition Object",
        "    NoSun,                   !- Sun Exposure",
        "    NoWind,                  !- Wind Exposure",
        "    0.0,                     !- View Factor to Ground",
        "    4,                       !- Number of Vertices",
        "    3.7,11.6,2.4,  !- X,Y,Z ==> Vertex 1 {m}",
        "    3.7,11.6,0.0,  !- X,Y,Z ==> Vertex 2 {m}",
        "    0.0,15.2,0.0,  !- X,Y,Z ==> Vertex 3 {m}",
        "    0.0,15.2,2.4;  !- X,Y,Z ==> Vertex 4 {m}",

        "  BuildingSurface:Detailed,",
        "    SB45,                    !- Name",
        "    WALL,                    !- Surface Type",
        "    INT-WALL-1,              !- Construction Name",
        "    SPACE4-1,                !- Zone Name",
        "    Surface,                 !- Outside Boundary Condition",
        "    SB54,                    !- Outside Boundary Condition Object",
        "    NoSun,                   !- Sun Exposure",
        "    NoWind,                  !- Wind Exposure",
        "    0.0,                     !- View Factor to Ground",
        "    4,                       !- Number of Vertices",
        "    3.7,3.7,2.4,  !- X,Y,Z ==> Vertex 1 {m}",
        "    3.7,3.7,0.0,  !- X,Y,Z ==> Vertex 2 {m}",
        "    3.7,11.6,0.0,  !- X,Y,Z ==> Vertex 3 {m}",
        "    3.7,11.6,2.4;  !- X,Y,Z ==> Vertex 4 {m}",

        "  BuildingSurface:Detailed,",
        "    C5-1,                    !- Name",
        "    CEILING,                 !- Surface Type",
        "    CLNG-1,                  !- Construction Name",
        "    SPACE5-1,                !- Zone Name",
        "    Surface,                 !- Outside Boundary Condition",
        "    C5-1P,                   !- Outside Boundary Condition Object",
        "    NoSun,                   !- Sun Exposure",
        "    NoWind,                  !- Wind Exposure",
        "    0.0,                     !- View Factor to Ground",
        "    4,                       !- Number of Vertices",
        "    3.7,11.6,2.4,  !- X,Y,Z ==> Vertex 1 {m}",
        "    3.7,3.7,2.4,  !- X,Y,Z ==> Vertex 2 {m}",
        "    26.8,3.7,2.4,  !- X,Y,Z ==> Vertex 3 {m}",
        "    26.8,11.6,2.4;  !- X,Y,Z ==> Vertex 4 {m}",

        "  BuildingSurface:Detailed,",
        "    F5-1,                    !- Name",
        "    FLOOR,                   !- Surface Type",
        "    FLOOR-SLAB-1,            !- Construction Name",
        "    SPACE5-1,                !- Zone Name",
        "    Ground,                  !- Outside Boundary Condition",
        "    ,                        !- Outside Boundary Condition Object",
        "    NoSun,                   !- Sun Exposure",
        "    NoWind,                  !- Wind Exposure",
        "    0.0,                     !- View Factor to Ground",
        "    4,                       !- Number of Vertices",
        "    26.8,11.6,0.0,  !- X,Y,Z ==> Vertex 1 {m}",
        "    26.8,3.7,0.0,  !- X,Y,Z ==> Vertex 2 {m}",
        "    3.7,3.7,0.0,  !- X,Y,Z ==> Vertex 3 {m}",
        "    3.7,11.6,0.0;  !- X,Y,Z ==> Vertex 4 {m}",

        "  BuildingSurface:Detailed,",
        "    SB51,                    !- Name",
        "    WALL,                    !- Surface Type",
        "    INT-WALL-1,              !- Construction Name",
        "    SPACE5-1,                !- Zone Name",
        "    Surface,                 !- Outside Boundary Condition",
        "    SB15,                    !- Outside Boundary Condition Object",
        "    NoSun,                   !- Sun Exposure",
        "    NoWind,                  !- Wind Exposure",
        "    0.0,                     !- View Factor to Ground",
        "    4,                       !- Number of Vertices",
        "    3.7,3.7,2.4,  !- X,Y,Z ==> Vertex 1 {m}",
        "    3.7,3.7,0.0,  !- X,Y,Z ==> Vertex 2 {m}",
        "    26.8,3.7,0.0,  !- X,Y,Z ==> Vertex 3 {m}",
        "    26.8,3.7,2.4;  !- X,Y,Z ==> Vertex 4 {m}",

        "  BuildingSurface:Detailed,",
        "    SB52,                    !- Name",
        "    WALL,                    !- Surface Type",
        "    INT-WALL-1,              !- Construction Name",
        "    SPACE5-1,                !- Zone Name",
        "    Surface,                 !- Outside Boundary Condition",
        "    SB25,                    !- Outside Boundary Condition Object",
        "    NoSun,                   !- Sun Exposure",
        "    NoWind,                  !- Wind Exposure",
        "    0.0,                     !- View Factor to Ground",
        "    4,                       !- Number of Vertices",
        "    26.8,3.7,2.4,  !- X,Y,Z ==> Vertex 1 {m}",
        "    26.8,3.7,0.0,  !- X,Y,Z ==> Vertex 2 {m}",
        "    26.8,11.6,0.0,  !- X,Y,Z ==> Vertex 3 {m}",
        "    26.8,11.6,2.4;  !- X,Y,Z ==> Vertex 4 {m}",

        "  BuildingSurface:Detailed,",
        "    SB53,                    !- Name",
        "    WALL,                    !- Surface Type",
        "    INT-WALL-1,              !- Construction Name",
        "    SPACE5-1,                !- Zone Name",
        "    Surface,                 !- Outside Boundary Condition",
        "    SB35,                    !- Outside Boundary Condition Object",
        "    NoSun,                   !- Sun Exposure",
        "    NoWind,                  !- Wind Exposure",
        "    0.0,                     !- View Factor to Ground",
        "    4,                       !- Number of Vertices",
        "    26.8,11.6,2.4,  !- X,Y,Z ==> Vertex 1 {m}",
        "    26.8,11.6,0.0,  !- X,Y,Z ==> Vertex 2 {m}",
        "    3.7,11.6,0.0,  !- X,Y,Z ==> Vertex 3 {m}",
        "    3.7,11.6,2.4;  !- X,Y,Z ==> Vertex 4 {m}",

        "  BuildingSurface:Detailed,",
        "    SB54,                    !- Name",
        "    WALL,                    !- Surface Type",
        "    INT-WALL-1,              !- Construction Name",
        "    SPACE5-1,                !- Zone Name",
        "    Surface,                 !- Outside Boundary Condition",
        "    SB45,                    !- Outside Boundary Condition Object",
        "    NoSun,                   !- Sun Exposure",
        "    NoWind,                  !- Wind Exposure",
        "    0.0,                     !- View Factor to Ground",
        "    4,                       !- Number of Vertices",
        "    3.7,11.6,2.4,  !- X,Y,Z ==> Vertex 1 {m}",
        "    3.7,11.6,0.0,  !- X,Y,Z ==> Vertex 2 {m}",
        "    3.7,3.7,0.0,  !- X,Y,Z ==> Vertex 3 {m}",
        "    3.7,3.7,2.4;  !- X,Y,Z ==> Vertex 4 {m}",

        "  FenestrationSurface:Detailed,",
        "    WF-1,                    !- Name",
        "    WINDOW,                  !- Surface Type",
        "    Dbl Clr 3mm/13mm Air,    !- Construction Name",
        "    FRONT-1,                 !- Building Surface Name",
        "    ,                        !- Outside Boundary Condition Object",
        "    0.50000,                 !- View Factor to Ground",
        "    ,                        !- Frame and Divider Name",
        "    1,                       !- Multiplier",
        "    4,                       !- Number of Vertices",
        "    3.0,0.0,2.1,  !- X,Y,Z ==> Vertex 1 {m}",
        "    3.0,0.0,0.9,  !- X,Y,Z ==> Vertex 2 {m}",
        "    16.8,0.0,0.9,  !- X,Y,Z ==> Vertex 3 {m}",
        "    16.8,0.0,2.1;  !- X,Y,Z ==> Vertex 4 {m}",

        "  FenestrationSurface:Detailed,",
        "    DF-1,                    !- Name",
        "    GLASSDOOR,               !- Surface Type",
        "    Sgl Grey 3mm,            !- Construction Name",
        "    FRONT-1,                 !- Building Surface Name",
        "    ,                        !- Outside Boundary Condition Object",
        "    0.50000,                 !- View Factor to Ground",
        "    ,                        !- Frame and Divider Name",
        "    1,                       !- Multiplier",
        "    4,                       !- Number of Vertices",
        "    21.3,0.0,2.1,  !- X,Y,Z ==> Vertex 1 {m}",
        "    21.3,0.0,0.0,  !- X,Y,Z ==> Vertex 2 {m}",
        "    23.8,0.0,0.0,  !- X,Y,Z ==> Vertex 3 {m}",
        "    23.8,0.0,2.1;  !- X,Y,Z ==> Vertex 4 {m}",

        "  FenestrationSurface:Detailed,",
        "    WR-1,                    !- Name",
        "    WINDOW,                  !- Surface Type",
        "    Dbl Clr 3mm/13mm Air,    !- Construction Name",
        "    RIGHT-1,                 !- Building Surface Name",
        "    ,                        !- Outside Boundary Condition Object",
        "    0.50000,                 !- View Factor to Ground",
        "    ,                        !- Frame and Divider Name",
        "    1,                       !- Multiplier",
        "    4,                       !- Number of Vertices",
        "    30.5,3.8,2.1,  !- X,Y,Z ==> Vertex 1 {m}",
        "    30.5,3.8,0.9,  !- X,Y,Z ==> Vertex 2 {m}",
        "    30.5,11.4,0.9,  !- X,Y,Z ==> Vertex 3 {m}",
        "    30.5,11.4,2.1;  !- X,Y,Z ==> Vertex 4 {m}",

        "  FenestrationSurface:Detailed,",
        "    WB-1,                    !- Name",
        "    WINDOW,                  !- Surface Type",
        "    Dbl Clr 3mm/13mm Air,    !- Construction Name",
        "    BACK-1,                  !- Building Surface Name",
        "    ,                        !- Outside Boundary Condition Object",
        "    0.50000,                 !- View Factor to Ground",
        "    ,                        !- Frame and Divider Name",
        "    1,                       !- Multiplier",
        "    4,                       !- Number of Vertices",
        "    27.4,15.2,2.1,  !- X,Y,Z ==> Vertex 1 {m}",
        "    27.4,15.2,0.9,  !- X,Y,Z ==> Vertex 2 {m}",
        "    13.7,15.2,0.9,  !- X,Y,Z ==> Vertex 3 {m}",
        "    13.7,15.2,2.1;  !- X,Y,Z ==> Vertex 4 {m}",

        "  FenestrationSurface:Detailed,",
        "    DB-1,                    !- Name",
        "    GLASSDOOR,               !- Surface Type",
        "    Sgl Grey 3mm,            !- Construction Name",
        "    BACK-1,                  !- Building Surface Name",
        "    ,                        !- Outside Boundary Condition Object",
        "    0.50000,                 !- View Factor to Ground",
        "    ,                        !- Frame and Divider Name",
        "    1,                       !- Multiplier",
        "    4,                       !- Number of Vertices",
        "    9.1,15.2,2.1,  !- X,Y,Z ==> Vertex 1 {m}",
        "    9.1,15.2,0.0,  !- X,Y,Z ==> Vertex 2 {m}",
        "    7.0,15.2,0.0,  !- X,Y,Z ==> Vertex 3 {m}",
        "    7.0,15.2,2.1;  !- X,Y,Z ==> Vertex 4 {m}",

        "  FenestrationSurface:Detailed,",
        "    WL-1,                    !- Name",
        "    WINDOW,                  !- Surface Type",
        "    Dbl Clr 3mm/13mm Air,    !- Construction Name",
        "    LEFT-1,                  !- Building Surface Name",
        "    ,                        !- Outside Boundary Condition Object",
        "    0.50000,                 !- View Factor to Ground",
        "    ,                        !- Frame and Divider Name",
        "    1,                       !- Multiplier",
        "    4,                       !- Number of Vertices",
        "    0.0,11.4,2.1,  !- X,Y,Z ==> Vertex 1 {m}",
        "    0.0,11.4,0.9,  !- X,Y,Z ==> Vertex 2 {m}",
        "    0.0,3.8,0.9,  !- X,Y,Z ==> Vertex 3 {m}",
        "    0.0,3.8,2.1;  !- X,Y,Z ==> Vertex 4 {m}",

        "  Shading:Zone:Detailed,",
        "    Main South Overhang,     !- Name",
        "    FRONT-1,                 !- Base Surface Name",
        "    ShadeTransSch,           !- Transmittance Schedule Name",
        "    4,                       !- Number of Vertices",
        "    0.0,-1.3,2.2,  !- X,Y,Z ==> Vertex 1 {m}",
        "    0.0,0.0,2.2,  !- X,Y,Z ==> Vertex 2 {m}",
        "    19.8,0.0,2.2,  !- X,Y,Z ==> Vertex 3 {m}",
        "    19.8,-1.3,2.2;  !- X,Y,Z ==> Vertex 4 {m}",

        "  Shading:Zone:Detailed,",
        "    South Door Overhang,     !- Name",
        "    FRONT-1,                 !- Base Surface Name",
        "    ShadeTransSch,           !- Transmittance Schedule Name",
        "    4,                       !- Number of Vertices",
        "    21.0,-2.0,2.6,  !- X,Y,Z ==> Vertex 1 {m}",
        "    21.0,0.0,2.6,  !- X,Y,Z ==> Vertex 2 {m}",
        "    24.1,0.0,2.6,  !- X,Y,Z ==> Vertex 3 {m}",
        "    24.1,-2.0,2.6;  !- X,Y,Z ==> Vertex 4 {m}",

        "  ScheduleTypeLimits,",
        "    Any Number;              !- Name",

        "  ScheduleTypeLimits,",
        "    Fraction,                !- Name",
        "    0.0,                     !- Lower Limit Value",
        "    1.0,                     !- Upper Limit Value",
        "    CONTINUOUS;              !- Numeric Type",

        "  ScheduleTypeLimits,",
        "    Temperature,             !- Name",
        "    -60,                     !- Lower Limit Value",
        "    200,                     !- Upper Limit Value",
        "    CONTINUOUS,              !- Numeric Type",
        "    Temperature;             !- Unit Type",

        "  ScheduleTypeLimits,",
        "    Control Type,            !- Name",
        "    0,                       !- Lower Limit Value",
        "    4,                       !- Upper Limit Value",
        "    DISCRETE;                !- Numeric Type",

        "  ScheduleTypeLimits,",
        "    On/Off,                  !- Name",
        "    0,                       !- Lower Limit Value",
        "    1,                       !- Upper Limit Value",
        "    DISCRETE;                !- Numeric Type",

        "  ScheduleTypeLimits,",
        "    FlowRate,                !- Name",
        "    0.0,                     !- Lower Limit Value",
        "    10,                      !- Upper Limit Value",
        "    CONTINUOUS;              !- Numeric Type",

        "  ScheduleTypeLimits,",
        "    Humidity,                !- Name",
        "    0,                       !- Lower Limit Value",
        "    100,                     !- Upper Limit Value",
        "    CONTINUOUS;              !- Numeric Type",

        "  Schedule:Compact,",
        "    OCCUPY-1,                !- Name",
        "    Fraction,                !- Schedule Type Limits Name",
        "    Through: 12/31,          !- Field 1",
        "    For: WeekDays SummerDesignDay CustomDay1 CustomDay2, !- Field 2",
        "    Until: 8:00,0.0,         !- Field 3",
        "    Until: 11:00,1.00,       !- Field 5",
        "    Until: 12:00,0.80,       !- Field 7",
        "    Until: 13:00,0.40,       !- Field 9",
        "    Until: 14:00,0.80,       !- Field 11",
        "    Until: 18:00,1.00,       !- Field 13",
        "    Until: 19:00,0.50,       !- Field 15",
        "    Until: 21:00,0.10,       !- Field 17",
        "    Until: 24:00,0.0,        !- Field 19",
        "    For: Weekends WinterDesignDay Holiday, !- Field 21",
        "    Until: 24:00,0.0;        !- Field 22",

        "  Schedule:Compact,",
        "    LIGHTS-1,                !- Name",
        "    Fraction,                !- Schedule Type Limits Name",
        "    Through: 12/31,          !- Field 1",
        "    For: WeekDays SummerDesignDay CustomDay1 CustomDay2, !- Field 2",
        "    Until: 8:00,0.05,        !- Field 3",
        "    Until: 9:00,0.9,         !- Field 5",
        "    Until: 10:00,0.95,       !- Field 7",
        "    Until: 11:00,1.00,       !- Field 9",
        "    Until: 12:00,0.95,       !- Field 11",
        "    Until: 13:00,0.8,        !- Field 13",
        "    Until: 14:00,0.9,        !- Field 15",
        "    Until: 18:00,1.00,       !- Field 17",
        "    Until: 19:00,0.60,       !- Field 19",
        "    Until: 21:00,0.20,       !- Field 21",
        "    Until: 24:00,0.05,       !- Field 23",
        "    For: Weekends WinterDesignDay Holiday, !- Field 25",
        "    Until: 24:00,0.05;       !- Field 26",

        "  Schedule:Compact,",
        "    EQUIP-1,                 !- Name",
        "    Fraction,                !- Schedule Type Limits Name",
        "    Through: 12/31,          !- Field 1",
        "    For: WeekDays SummerDesignDay CustomDay1 CustomDay2, !- Field 2",
        "    Until: 8:00,0.02,        !- Field 3",
        "    Until: 9:00,0.4,         !- Field 5",
        "    Until: 14:00,0.9,        !- Field 7",
        "    Until: 15:00,0.8,        !- Field 9",
        "    Until: 16:00,0.7,        !- Field 11",
        "    Until: 18:00,0.5,        !- Field 13",
        "    Until: 20:00,0.3,        !- Field 15",
        "    Until: 24:00,0.02,       !- Field 17",
        "    For: Weekends WinterDesignDay Holiday, !- Field 19",
        "    Until: 24:00,0.2;        !- Field 20",

        "  Schedule:Compact,",
        "    INFIL-SCH,               !- Name",
        "    Fraction,                !- Schedule Type Limits Name",
        "    Through: 3/31,           !- Field 1",
        "    For: WeekDays SummerDesignDay CustomDay1 CustomDay2, !- Field 2",
        "    Until: 24:00,1.0,        !- Field 3",
        "    For: AllOtherDays,       !- Field 5",
        "    Until: 24:00,1.0,        !- Field 6",
        "    Through: 9/30,           !- Field 8",
        "    For: AllDays,            !- Field 9",
        "    Until: 24:00,0.7,        !- Field 10",
        "    Through: 12/31,          !- Field 12",
        "    For: AllDays,            !- Field 13",
        "    Until: 24:00,1.0,        !- Field 14",
        "    For: AllOtherDays,       !- Field 16",
        "    Until: 24:00,0.2;        !- Field 17",

        "  Schedule:Compact,",
        "    ActSchd,                 !- Name",
        "    Any Number,              !- Schedule Type Limits Name",
        "    Through: 12/31,          !- Field 1",
        "    For: AllDays,            !- Field 2",
        "    Until: 24:00,117.239997864; !- Field 3",

        "  Schedule:Compact,",
        "    ShadeTransSch,           !- Name",
        "    Fraction,                !- Schedule Type Limits Name",
        "    Through: 12/31,          !- Field 1",
        "    For: AllDays,            !- Field 2",
        "    Until: 24:00,0.0;        !- Field 3",

        "  Schedule:Compact,",
        "    Htg-SetP-Sch,            !- Name",
        "    Temperature,             !- Schedule Type Limits Name",
        "    Through: 12/31,          !- Field 1",
        "    For: SummerDesignDay,    !- Field 2",
        "    Until: 24:00,12.8,       !- Field 3",
        "    For: WinterDesignDay,    !- Field 5",
        "    Until: 24:00,21.1,       !- Field 6",
        "    For: WeekEnds Holiday,   !- Field 8",
        "    Until: 7:00,12.8,        !- Field 9",
        "    Until: 13:00,21.1,       !- Field 11",
        "    Until: 24:00,12.8,       !- Field 13",
        "    For: AllOtherDays,       !- Field 15",
        "    Until: 7:00,12.8,        !- Field 16",
        "    Until: 18:00,21.1,       !- Field 18",
        "    Until: 24:00,12.8;       !- Field 20",

        "  Schedule:Compact,",
        "    PlenumHtg-SetP-Sch,      !- Name",
        "    Temperature,             !- Schedule Type Limits Name",
        "    Through: 12/31,          !- Field 1",
        "    For: AllDays,            !- Field 2",
        "    Until: 24:00,12.8;       !- Field 3",

        "  Schedule:Compact,",
        "    Clg-SetP-Sch,            !- Name",
        "    Temperature,             !- Schedule Type Limits Name",
        "    Through: 12/31,          !- Field 1",
        "    For: SummerDesignDay,    !- Field 2",
        "    Until: 24:00,23.9,       !- Field 3",
        "    For: WinterDesignDay,    !- Field 5",
        "    Until: 24:00,40.0,       !- Field 6",
        "    For: WeekEnds Holiday,   !- Field 8",
        "    Until: 7:00,40.0,        !- Field 9",
        "    Until: 13:00,23.9,       !- Field 11",
        "    Until: 24:00,32.2,       !- Field 13",
        "    For: AllOtherDays,       !- Field 15",
        "    Until: 7:00,40.0,        !- Field 16",
        "    Until: 18:00,23.9,       !- Field 18",
        "    Until: 24:00,40.0;       !- Field 20",

        "  Schedule:Compact,",
        "    PlenumClg-SetP-Sch,      !- Name",
        "    Temperature,             !- Schedule Type Limits Name",
        "    Through: 12/31,          !- Field 1",
        "    For: AllDays,            !- Field 2",
        "    Until: 24:00,40.0;       !- Field 3",

        "  Schedule:Compact,",
        "    Zone Control Type Sched, !- Name",
        "    Control Type,            !- Schedule Type Limits Name",
        "    Through: 12/31,          !- Field 1",
        "    For: SummerDesignDay,    !- Field 2",
        "    Until: 24:00,2,          !- Field 3",
        "    For: WinterDesignDay,    !- Field 5",
        "    Until: 24:00,1,          !- Field 6",
        "    For: AllOtherDays,       !- Field 8",
        "    Until: 24:00,4;          !- Field 9",

        "  Schedule:Compact,",
        "    Desiccant Control Type Sched,  !- Name",
        "    Control Type,            !- Schedule Type Limits Name",
        "    Through: 12/31,          !- Field 1",
        "    For: AllDays,            !- Field 2",
        "    Until: 24:00,4;          !- Field 3",

        "  Schedule:Compact,",
        "    Min OA Sched,            !- Name",
        "    Fraction,                !- Schedule Type Limits Name",
        "    Through: 12/31,          !- Field 1",
        "    For: Weekdays,           !- Field 2",
        "    Until: 7:00,0.02,        !- Field 3",
        "    Until: 17:00,1.0,        !- Field 5",
        "    Until: 24:00,0.0,        !- Field 7",
        "    For: AllOtherDays,       !- Field 9",
        "    Until: 24:00,0.02;       !- Field 10",

        "  Schedule:Compact,",
        "    FanAvailSched,           !- Name",
        "    Fraction,                !- Schedule Type Limits Name",
        "    Through: 3/31,           !- Field 1",
        "    For: AllDays,            !- Field 2",
        "    Until: 24:00,1.0,        !- Field 3",
        "    Through: 9/30,           !- Field 5",
        "    For: WeekDays,           !- Field 6",
        "    Until: 7:00,0.0,         !- Field 7",
        "    Until: 17:00,1.0,        !- Field 9",
        "    Until: 24:00,0.0,        !- Field 11",
        "    For: SummerDesignDay WinterDesignDay, !- Field 13",
        "    Until: 24:00,1.0,        !- Field 14",
        "    For: AllOtherDays,       !- Field 16",
        "    Until: 24:00,0.0,        !- Field 17",
        "    Through: 12/31,          !- Field 19",
        "    For: AllDays,            !- Field 20",
        "    Until: 24:00,1.0;        !- Field 21",

        "  Schedule:Compact,",
        "    Seasonal Reset Supply Air Temp Sch,  !- Name",
        "    Temperature,             !- Schedule Type Limits Name",
        "    Through: 3/31,           !- Field 1",
        "    For: AllDays,            !- Field 2",
        "    Until: 24:00,17.0,       !- Field 3",
        "    Through: 9/30,           !- Field 5",
        "    For: AllDays,            !- Field 6",
        "    Until: 24:00,15.0,       !- Field 7",
        "    Through: 12/31,          !- Field 9",
        "    For: AllDays,            !- Field 10",
        "    Until: 24:00,17.0;       !- Field 11",

        "  Schedule:Compact,",
        "    Constant,                !- Name",
        "    Any Number,              !- Schedule Type Limits Name",
        "    Through: 12/31,          !- Field 1",
        "    For: AllDays,            !- Field 2",
        "    Until: 24:00,1.0;        !- Field 3",

        "  People,",
        "    SPACE1-1 People 1,       !- Name",
        "    SPACE1-1,                !- Zone or ZoneList Name",
        "    OCCUPY-1,                !- Number of People Schedule Name",
        "    people,                  !- Number of People Calculation Method",
        "    11,                      !- Number of People",
        "    ,                        !- People per Zone Floor Area {person/m2}",
        "    ,                        !- Zone Floor Area per Person {m2/person}",
        "    0.3,                     !- Fraction Radiant",
        "    AutoCalculate,           !- Sensible Heat Fraction",
        "    ActSchd;                 !- Activity Level Schedule Name",

        "  People,",
        "    SPACE2-1 People 1,       !- Name",
        "    SPACE2-1,                !- Zone or ZoneList Name",
        "    OCCUPY-1,                !- Number of People Schedule Name",
        "    people,                  !- Number of People Calculation Method",
        "    5,                       !- Number of People",
        "    ,                        !- People per Zone Floor Area {person/m2}",
        "    ,                        !- Zone Floor Area per Person {m2/person}",
        "    0.3,                     !- Fraction Radiant",
        "    AutoCalculate,           !- Sensible Heat Fraction",
        "    ActSchd;                 !- Activity Level Schedule Name",

        "  People,",
        "    SPACE3-1 People 1,       !- Name",
        "    SPACE3-1,                !- Zone or ZoneList Name",
        "    OCCUPY-1,                !- Number of People Schedule Name",
        "    people,                  !- Number of People Calculation Method",
        "    11,                      !- Number of People",
        "    ,                        !- People per Zone Floor Area {person/m2}",
        "    ,                        !- Zone Floor Area per Person {m2/person}",
        "    0.3,                     !- Fraction Radiant",
        "    AutoCalculate,           !- Sensible Heat Fraction",
        "    ActSchd;                 !- Activity Level Schedule Name",

        "  People,",
        "    SPACE4-1 People 1,       !- Name",
        "    SPACE4-1,                !- Zone or ZoneList Name",
        "    OCCUPY-1,                !- Number of People Schedule Name",
        "    people,                  !- Number of People Calculation Method",
        "    5,                       !- Number of People",
        "    ,                        !- People per Zone Floor Area {person/m2}",
        "    ,                        !- Zone Floor Area per Person {m2/person}",
        "    0.3,                     !- Fraction Radiant",
        "    AutoCalculate,           !- Sensible Heat Fraction",
        "    ActSchd;                 !- Activity Level Schedule Name",

        "  Lights,",
        "    SPACE1-1 Lights 1,       !- Name",
        "    SPACE1-1,                !- Zone or ZoneList Name",
        "    LIGHTS-1,                !- Schedule Name",
        "    LightingLevel,           !- Design Level Calculation Method",
        "    1584,                    !- Lighting Level {W}",
        "    ,                        !- Watts per Zone Floor Area {W/m2}",
        "    ,                        !- Watts per Person {W/person}",
        "    0.2,                     !- Return Air Fraction",
        "    0.59,                    !- Fraction Radiant",
        "    0.2,                     !- Fraction Visible",
        "    0,                       !- Fraction Replaceable",
        "    GeneralLights;           !- End-Use Subcategory",

        "  Lights,",
        "    SPACE2-1 Lights 1,       !- Name",
        "    SPACE2-1,                !- Zone or ZoneList Name",
        "    LIGHTS-1,                !- Schedule Name",
        "    LightingLevel,           !- Design Level Calculation Method",
        "    684,                     !- Lighting Level {W}",
        "    ,                        !- Watts per Zone Floor Area {W/m2}",
        "    ,                        !- Watts per Person {W/person}",
        "    0.2,                     !- Return Air Fraction",
        "    0.59,                    !- Fraction Radiant",
        "    0.2,                     !- Fraction Visible",
        "    0,                       !- Fraction Replaceable",
        "    GeneralLights;           !- End-Use Subcategory",

        "  Lights,",
        "    SPACE3-1 Lights 1,       !- Name",
        "    SPACE3-1,                !- Zone or ZoneList Name",
        "    LIGHTS-1,                !- Schedule Name",
        "    LightingLevel,           !- Design Level Calculation Method",
        "    1584,                    !- Lighting Level {W}",
        "    ,                        !- Watts per Zone Floor Area {W/m2}",
        "    ,                        !- Watts per Person {W/person}",
        "    0.2,                     !- Return Air Fraction",
        "    0.59,                    !- Fraction Radiant",
        "    0.2,                     !- Fraction Visible",
        "    0,                       !- Fraction Replaceable",
        "    GeneralLights;           !- End-Use Subcategory",

        "  Lights,",
        "    SPACE4-1 Lights 1,       !- Name",
        "    SPACE4-1,                !- Zone or ZoneList Name",
        "    LIGHTS-1,                !- Schedule Name",
        "    LightingLevel,           !- Design Level Calculation Method",
        "    684,                     !- Lighting Level {W}",
        "    ,                        !- Watts per Zone Floor Area {W/m2}",
        "    ,                        !- Watts per Person {W/person}",
        "    0.2,                     !- Return Air Fraction",
        "    0.59,                    !- Fraction Radiant",
        "    0.2,                     !- Fraction Visible",
        "    0,                       !- Fraction Replaceable",
        "    GeneralLights;           !- End-Use Subcategory",

        "  ElectricEquipment,",
        "    SPACE1-1 ElecEq 1,       !- Name",
        "    SPACE1-1,                !- Zone or ZoneList Name",
        "    EQUIP-1,                 !- Schedule Name",
        "    EquipmentLevel,          !- Design Level Calculation Method",
        "    10560,                   !- Design Level {W}",
        "    ,                        !- Watts per Zone Floor Area {W/m2}",
        "    ,                        !- Watts per Person {W/person}",
        "    0.5,                     !- Fraction Latent",
        "    0.3,                     !- Fraction Radiant",
        "    0;                       !- Fraction Lost",

        "  ElectricEquipment,",
        "    SPACE2-1 ElecEq 1,       !- Name",
        "    SPACE2-1,                !- Zone or ZoneList Name",
        "    EQUIP-1,                 !- Schedule Name",
        "    EquipmentLevel,          !- Design Level Calculation Method",
        "    4560,                    !- Design Level {W}",
        "    ,                        !- Watts per Zone Floor Area {W/m2}",
        "    ,                        !- Watts per Person {W/person}",
        "    0.5,                     !- Fraction Latent",
        "    0.3,                     !- Fraction Radiant",
        "    0;                       !- Fraction Lost",

        "  ElectricEquipment,",
        "    SPACE3-1 ElecEq 1,       !- Name",
        "    SPACE3-1,                !- Zone or ZoneList Name",
        "    EQUIP-1,                 !- Schedule Name",
        "    EquipmentLevel,          !- Design Level Calculation Method",
        "    10560,                   !- Design Level {W}",
        "    ,                        !- Watts per Zone Floor Area {W/m2}",
        "    ,                        !- Watts per Person {W/person}",
        "    0.5,                     !- Fraction Latent",
        "    0.3,                     !- Fraction Radiant",
        "    0;                       !- Fraction Lost",

        "  ElectricEquipment,",
        "    SPACE4-1 ElecEq 1,       !- Name",
        "    SPACE4-1,                !- Zone or ZoneList Name",
        "    EQUIP-1,                 !- Schedule Name",
        "    EquipmentLevel,          !- Design Level Calculation Method",
        "    4560,                    !- Design Level {W}",
        "    ,                        !- Watts per Zone Floor Area {W/m2}",
        "    ,                        !- Watts per Person {W/person}",
        "    0.5,                     !- Fraction Latent",
        "    0.3,                     !- Fraction Radiant",
        "    0;                       !- Fraction Lost",

        "  ZoneInfiltration:DesignFlowRate,",
        "    SPACE1-1 Infil 1,        !- Name",
        "    SPACE1-1,                !- Zone or ZoneList Name",
        "    INFIL-SCH,               !- Schedule Name",
        "    flow/zone,               !- Design Flow Rate Calculation Method",
        "    0.0167,                  !- Design Flow Rate {m3/s}",
        "    ,                        !- Flow per Zone Floor Area {m3/s-m2}",
        "    ,                        !- Flow per Exterior Surface Area {m3/s-m2}",
        "    ,                        !- Air Changes per Hour {1/hr}",
        "    0,                       !- Constant Term Coefficient",
        "    0,                       !- Temperature Term Coefficient",
        "    0.2237,                  !- Velocity Term Coefficient",
        "    0;                       !- Velocity Squared Term Coefficient",

        "  ZoneInfiltration:DesignFlowRate,",
        "    SPACE2-1 Infil 1,        !- Name",
        "    SPACE2-1,                !- Zone or ZoneList Name",
        "    INFIL-SCH,               !- Schedule Name",
        "    flow/zone,               !- Design Flow Rate Calculation Method",
        "    0.00717,                 !- Design Flow Rate {m3/s}",
        "    ,                        !- Flow per Zone Floor Area {m3/s-m2}",
        "    ,                        !- Flow per Exterior Surface Area {m3/s-m2}",
        "    ,                        !- Air Changes per Hour {1/hr}",
        "    0,                       !- Constant Term Coefficient",
        "    0,                       !- Temperature Term Coefficient",
        "    0.2237,                  !- Velocity Term Coefficient",
        "    0;                       !- Velocity Squared Term Coefficient",

        "  ZoneInfiltration:DesignFlowRate,",
        "    SPACE3-1 Infil 1,        !- Name",
        "    SPACE3-1,                !- Zone or ZoneList Name",
        "    INFIL-SCH,               !- Schedule Name",
        "    flow/zone,               !- Design Flow Rate Calculation Method",
        "    0.0167,                  !- Design Flow Rate {m3/s}",
        "    ,                        !- Flow per Zone Floor Area {m3/s-m2}",
        "    ,                        !- Flow per Exterior Surface Area {m3/s-m2}",
        "    ,                        !- Air Changes per Hour {1/hr}",
        "    0,                       !- Constant Term Coefficient",
        "    0,                       !- Temperature Term Coefficient",
        "    0.2237,                  !- Velocity Term Coefficient",
        "    0;                       !- Velocity Squared Term Coefficient",

        "  ZoneInfiltration:DesignFlowRate,",
        "    SPACE4-1 Infil 1,        !- Name",
        "    SPACE4-1,                !- Zone or ZoneList Name",
        "    INFIL-SCH,               !- Schedule Name",
        "    flow/zone,               !- Design Flow Rate Calculation Method",
        "    0.00717,                 !- Design Flow Rate {m3/s}",
        "    ,                        !- Flow per Zone Floor Area {m3/s-m2}",
        "    ,                        !- Flow per Exterior Surface Area {m3/s-m2}",
        "    ,                        !- Air Changes per Hour {1/hr}",
        "    0,                       !- Constant Term Coefficient",
        "    0,                       !- Temperature Term Coefficient",
        "    0.2237,                  !- Velocity Term Coefficient",
        "    0;                       !- Velocity Squared Term Coefficient",

        "  Sizing:Zone,",
        "    SPACE1-1,                !- Zone or ZoneList Name",
        "    SupplyAirTemperature,    !- Zone Cooling Design Supply Air Temperature Input Method",
        "    14.,                     !- Zone Cooling Design Supply Air Temperature {C}",
        "    ,                        !- Zone Cooling Design Supply Air Temperature Difference {deltaC}",
        "    SupplyAirTemperature,    !- Zone Heating Design Supply Air Temperature Input Method",
        "    50.,                     !- Zone Heating Design Supply Air Temperature {C}",
        "    ,                        !- Zone Heating Design Supply Air Temperature Difference {deltaC}",
        "    0.009,                   !- Zone Cooling Design Supply Air Humidity Ratio {kgWater/kgDryAir}",
        "    0.004,                   !- Zone Heating Design Supply Air Humidity Ratio {kgWater/kgDryAir}",
        "    SZ DSOA SPACE1-1,        !- Design Specification Outdoor Air Object Name",
        "    0.0,                     !- Zone Heating Sizing Factor",
        "    0.0,                     !- Zone Cooling Sizing Factor",
        "    DesignDayWithLimit,      !- Cooling Design Air Flow Method",
        "    ,                        !- Cooling Design Air Flow Rate {m3/s}",
        "    ,                        !- Cooling Minimum Air Flow per Zone Floor Area {m3/s-m2}",
        "    ,                        !- Cooling Minimum Air Flow {m3/s}",
        "    ,                        !- Cooling Minimum Air Flow Fraction",
        "    DesignDay,               !- Heating Design Air Flow Method",
        "    ,                        !- Heating Design Air Flow Rate {m3/s}",
        "    ,                        !- Heating Maximum Air Flow per Zone Floor Area {m3/s-m2}",
        "    ,                        !- Heating Maximum Air Flow {m3/s}",
        "    ;                        !- Heating Maximum Air Flow Fraction",

        "  DesignSpecification:OutdoorAir,",
        "    SZ DSOA SPACE1-1,        !- Name",
        "    sum,                     !- Outdoor Air Method",
        "    0.00236,                 !- Outdoor Air Flow per Person {m3/s-person}",
        "    0.000305,                !- Outdoor Air Flow per Zone Floor Area {m3/s-m2}",
        "    0.0;                     !- Outdoor Air Flow per Zone {m3/s}",

        "  Sizing:Zone,",
        "    SPACE2-1,                !- Zone or ZoneList Name",
        "    SupplyAirTemperature,    !- Zone Cooling Design Supply Air Temperature Input Method",
        "    14.,                     !- Zone Cooling Design Supply Air Temperature {C}",
        "    ,                        !- Zone Cooling Design Supply Air Temperature Difference {deltaC}",
        "    SupplyAirTemperature,    !- Zone Heating Design Supply Air Temperature Input Method",
        "    50.,                     !- Zone Heating Design Supply Air Temperature {C}",
        "    ,                        !- Zone Heating Design Supply Air Temperature Difference {deltaC}",
        "    0.009,                   !- Zone Cooling Design Supply Air Humidity Ratio {kgWater/kgDryAir}",
        "    0.004,                   !- Zone Heating Design Supply Air Humidity Ratio {kgWater/kgDryAir}",
        "    SZ DSOA SPACE2-1,        !- Design Specification Outdoor Air Object Name",
        "    0.0,                     !- Zone Heating Sizing Factor",
        "    0.0,                     !- Zone Cooling Sizing Factor",
        "    DesignDayWithLimit,      !- Cooling Design Air Flow Method",
        "    ,                        !- Cooling Design Air Flow Rate {m3/s}",
        "    ,                        !- Cooling Minimum Air Flow per Zone Floor Area {m3/s-m2}",
        "    ,                        !- Cooling Minimum Air Flow {m3/s}",
        "    ,                        !- Cooling Minimum Air Flow Fraction",
        "    DesignDay,               !- Heating Design Air Flow Method",
        "    ,                        !- Heating Design Air Flow Rate {m3/s}",
        "    ,                        !- Heating Maximum Air Flow per Zone Floor Area {m3/s-m2}",
        "    ,                        !- Heating Maximum Air Flow {m3/s}",
        "    ;                        !- Heating Maximum Air Flow Fraction",

        "  DesignSpecification:OutdoorAir,",
        "    SZ DSOA SPACE2-1,        !- Name",
        "    sum,                     !- Outdoor Air Method",
        "    0.00236,                 !- Outdoor Air Flow per Person {m3/s-person}",
        "    0.000305,                !- Outdoor Air Flow per Zone Floor Area {m3/s-m2}",
        "    0.0;                     !- Outdoor Air Flow per Zone {m3/s}",

        "  Sizing:Zone,",
        "    SPACE3-1,                !- Zone or ZoneList Name",
        "    SupplyAirTemperature,    !- Zone Cooling Design Supply Air Temperature Input Method",
        "    14.,                     !- Zone Cooling Design Supply Air Temperature {C}",
        "    ,                        !- Zone Cooling Design Supply Air Temperature Difference {deltaC}",
        "    SupplyAirTemperature,    !- Zone Heating Design Supply Air Temperature Input Method",
        "    50.,                     !- Zone Heating Design Supply Air Temperature {C}",
        "    ,                        !- Zone Heating Design Supply Air Temperature Difference {deltaC}",
        "    0.009,                   !- Zone Cooling Design Supply Air Humidity Ratio {kgWater/kgDryAir}",
        "    0.004,                   !- Zone Heating Design Supply Air Humidity Ratio {kgWater/kgDryAir}",
        "    SZ DSOA SPACE3-1,        !- Design Specification Outdoor Air Object Name",
        "    0.0,                     !- Zone Heating Sizing Factor",
        "    0.0,                     !- Zone Cooling Sizing Factor",
        "    DesignDayWithLimit,      !- Cooling Design Air Flow Method",
        "    ,                        !- Cooling Design Air Flow Rate {m3/s}",
        "    ,                        !- Cooling Minimum Air Flow per Zone Floor Area {m3/s-m2}",
        "    ,                        !- Cooling Minimum Air Flow {m3/s}",
        "    ,                        !- Cooling Minimum Air Flow Fraction",
        "    DesignDay,               !- Heating Design Air Flow Method",
        "    ,                        !- Heating Design Air Flow Rate {m3/s}",
        "    ,                        !- Heating Maximum Air Flow per Zone Floor Area {m3/s-m2}",
        "    ,                        !- Heating Maximum Air Flow {m3/s}",
        "    ;                        !- Heating Maximum Air Flow Fraction",

        "  DesignSpecification:OutdoorAir,",
        "    SZ DSOA SPACE3-1,        !- Name",
        "    sum,                     !- Outdoor Air Method",
        "    0.00236,                 !- Outdoor Air Flow per Person {m3/s-person}",
        "    0.000305,                !- Outdoor Air Flow per Zone Floor Area {m3/s-m2}",
        "    0.0;                     !- Outdoor Air Flow per Zone {m3/s}",

        "  Sizing:Zone,",
        "    SPACE4-1,                !- Zone or ZoneList Name",
        "    SupplyAirTemperature,    !- Zone Cooling Design Supply Air Temperature Input Method",
        "    14.,                     !- Zone Cooling Design Supply Air Temperature {C}",
        "    ,                        !- Zone Cooling Design Supply Air Temperature Difference {deltaC}",
        "    SupplyAirTemperature,    !- Zone Heating Design Supply Air Temperature Input Method",
        "    50.,                     !- Zone Heating Design Supply Air Temperature {C}",
        "    ,                        !- Zone Heating Design Supply Air Temperature Difference {deltaC}",
        "    0.009,                   !- Zone Cooling Design Supply Air Humidity Ratio {kgWater/kgDryAir}",
        "    0.004,                   !- Zone Heating Design Supply Air Humidity Ratio {kgWater/kgDryAir}",
        "    SZ DSOA SPACE4-1,        !- Design Specification Outdoor Air Object Name",
        "    0.0,                     !- Zone Heating Sizing Factor",
        "    0.0,                     !- Zone Cooling Sizing Factor",
        "    DesignDayWithLimit,      !- Cooling Design Air Flow Method",
        "    ,                        !- Cooling Design Air Flow Rate {m3/s}",
        "    ,                        !- Cooling Minimum Air Flow per Zone Floor Area {m3/s-m2}",
        "    ,                        !- Cooling Minimum Air Flow {m3/s}",
        "    ,                        !- Cooling Minimum Air Flow Fraction",
        "    DesignDay,               !- Heating Design Air Flow Method",
        "    ,                        !- Heating Design Air Flow Rate {m3/s}",
        "    ,                        !- Heating Maximum Air Flow per Zone Floor Area {m3/s-m2}",
        "    ,                        !- Heating Maximum Air Flow {m3/s}",
        "    ;                        !- Heating Maximum Air Flow Fraction",

        "  DesignSpecification:OutdoorAir,",
        "    SZ DSOA SPACE4-1,        !- Name",
        "    sum,                     !- Outdoor Air Method",
        "    0.00236,                 !- Outdoor Air Flow per Person {m3/s-person}",
        "    0.000305,                !- Outdoor Air Flow per Zone Floor Area {m3/s-m2}",
        "    0.0;                     !- Outdoor Air Flow per Zone {m3/s}",

        "  Sizing:System,",
        "    VAV Sys 1,               !- AirLoop Name",
        "    sensible,                !- Type of Load to Size On",
        "    autosize,                !- Design Outdoor Air Flow Rate {m3/s}",
        "    0.3,                     !- Central Heating Maximum System Air Flow Ratio",
        "    4.5,                     !- Preheat Design Temperature {C}",
        "    0.008,                   !- Preheat Design Humidity Ratio {kgWater/kgDryAir}",
        "    11.0,                    !- Precool Design Temperature {C}",
        "    0.008,                   !- Precool Design Humidity Ratio {kgWater/kgDryAir}",
        "    12.8,                    !- Central Cooling Design Supply Air Temperature {C}",
        "    16.7,                    !- Central Heating Design Supply Air Temperature {C}",
        "    noncoincident,           !- Type of Zone Sum to Use",
        "    no,                      !- 100% Outdoor Air in Cooling",
        "    no,                      !- 100% Outdoor Air in Heating",
        "    0.008,                   !- Central Cooling Design Supply Air Humidity Ratio {kgWater/kgDryAir}",
        "    0.008,                   !- Central Heating Design Supply Air Humidity Ratio {kgWater/kgDryAir}",
        "    DesignDay,               !- Cooling Design Air Flow Method",
        "    0,                       !- Cooling Design Air Flow Rate {m3/s}",
        "    ,                        !- Supply Air Flow Rate Per Floor Area During Cooling Operation {m3/s-m2}",
        "    ,                        !- Fraction of Autosized Design Cooling Supply Air Flow Rate",
        "    ,                        !- Design Supply Air Flow Rate Per Unit Cooling Capacity {m3/s-W}",
        "    DesignDay,               !- Heating Design Air Flow Method",
        "    0,                       !- Heating Design Air Flow Rate {m3/s}",
        "    ,                        !- Supply Air Flow Rate Per Floor Area During Heating Operation {m3/s-m2}",
        "    ,                        !- Fraction of Autosized Design Heating Supply Air Flow Rate",
        "    ,                        !- Fraction of Autosized Design Cooling Supply Air Flow Rate",
        "    ,                        !- Design Supply Air Flow Rate Per Unit Heating Capacity {m3/s-W}",
        "    ,                        !- System Outdoor Air Method",
        "    1.0,                     !- Zone Maximum Outdoor Air Fraction {dimensionless}",
        "    CoolingDesignCapacity,   !- Cooling Design Capacity Method",
        "    autosize,                !- Cooling Design Capacity {W}",
        "    ,                        !- Cooling Design Capacity Per Floor Area {W/m2}",
        "    ,                        !- Fraction of Autosized Cooling Design Capacity",
        "    HeatingDesignCapacity,   !- Heating Design Capacity Method",
        "    autosize,                !- Heating Design Capacity {W}",
        "    ,                        !- Heating Design Capacity Per Floor Area {W/m2}",
        "    ,                        !- Fraction of Autosized Heating Design Capacity",
        "    VAV;                     !- Central Cooling Capacity Control Method",

        "  Curve:Biquadratic,",
        "    VarSpeedCoolCapFT,       !- Name",
        "    0.476428E+00,            !- Coefficient1 Constant",
        "    0.401147E-01,            !- Coefficient2 x",
        "    0.226411E-03,            !- Coefficient3 x**2",
        "    -0.827136E-03,           !- Coefficient4 y",
        "    -0.732240E-05,           !- Coefficient5 y**2",
        "    -0.446278E-03,           !- Coefficient6 x*y",
        "    12.77778,                !- Minimum Value of x",
        "    23.88889,                !- Maximum Value of x",
        "    23.88889,                !- Minimum Value of y",
        "    46.11111,                !- Maximum Value of y",
        "    ,                        !- Minimum Curve Output",
        "    ,                        !- Maximum Curve Output",
        "    Temperature,             !- Input Unit Type for X",
        "    Temperature,             !- Input Unit Type for Y",
        "    Dimensionless;           !- Output Unit Type",

        "  Curve:Biquadratic,",
        "    VarSpeedCoolEIRFT,       !- Name",
        "    0.632475E+00,            !- Coefficient1 Constant",
        "    -0.121321E-01,           !- Coefficient2 x",
        "    0.507773E-03,            !- Coefficient3 x**2",
        "    0.155377E-01,            !- Coefficient4 y",
        "    0.272840E-03,            !- Coefficient5 y**2",
        "    -0.679201E-03,           !- Coefficient6 x*y",
        "    12.77778,                !- Minimum Value of x",
        "    23.88889,                !- Maximum Value of x",
        "    23.88889,                !- Minimum Value of y",
        "    46.11111,                !- Maximum Value of y",
        "    ,                        !- Minimum Curve Output",
        "    ,                        !- Maximum Curve Output",
        "    Temperature,             !- Input Unit Type for X",
        "    Temperature,             !- Input Unit Type for Y",
        "    Dimensionless;           !- Output Unit Type",

        "  Curve:Biquadratic,",
        "    VarSpeedCoolCapLSFT,     !- Name",
        "    0.476428E+00,            !- Coefficient1 Constant",
        "    0.401147E-01,            !- Coefficient2 x",
        "    0.226411E-03,            !- Coefficient3 x**2",
        "    -0.827136E-03,           !- Coefficient4 y",
        "    -0.732240E-05,           !- Coefficient5 y**2",
        "    -0.446278E-03,           !- Coefficient6 x*y",
        "    12.77778,                !- Minimum Value of x",
        "    23.88889,                !- Maximum Value of x",
        "    23.88889,                !- Minimum Value of y",
        "    46.11111,                !- Maximum Value of y",
        "    ,                        !- Minimum Curve Output",
        "    ,                        !- Maximum Curve Output",
        "    Temperature,             !- Input Unit Type for X",
        "    Temperature,             !- Input Unit Type for Y",
        "    Dimensionless;           !- Output Unit Type",

        "  Curve:Biquadratic,",
        "    VarSpeedCoolEIRLSFT,     !- Name",
        "    0.774645E+00,            !- Coefficient1 Constant",
        "    -0.343731E-01,           !- Coefficient2 x",
        "    0.783173E-03,            !- Coefficient3 x**2",
        "    0.146596E-01,            !- Coefficient4 y",
        "    0.488851E-03,            !- Coefficient5 y**2",
        "    -0.752036E-03,           !- Coefficient6 x*y",
        "    12.77778,                !- Minimum Value of x",
        "    23.88889,                !- Maximum Value of x",
        "    23.88889,                !- Minimum Value of y",
        "    46.11111,                !- Maximum Value of y",
        "    ,                        !- Minimum Curve Output",
        "    ,                        !- Maximum Curve Output",
        "    Temperature,             !- Input Unit Type for X",
        "    Temperature,             !- Input Unit Type for Y",
        "    Dimensionless;           !- Output Unit Type",

        "  Curve:Biquadratic,",
        "    HPACCoolCapFT,           !- Name",
        "    0.942587793,             !- Coefficient1 Constant",
        "    0.009543347,             !- Coefficient2 x",
        "    0.00068377,              !- Coefficient3 x**2",
        "    -0.011042676,            !- Coefficient4 y",
        "    0.000005249,             !- Coefficient5 y**2",
        "    -0.00000972,             !- Coefficient6 x*y",
        "    -100,                    !- Minimum Value of x",
        "    100,                     !- Maximum Value of x",
        "    -100,                    !- Minimum Value of y",
        "    100,                     !- Maximum Value of y",
        "    ,                        !- Minimum Curve Output",
        "    ,                        !- Maximum Curve Output",
        "    Temperature,             !- Input Unit Type for X",
        "    Temperature,             !- Input Unit Type for Y",
        "    Dimensionless;           !- Output Unit Type",

        "  Curve:Biquadratic,",
        "    HPACCOOLEIRFT,           !- Name",
        "    0.342414409,             !- Coefficient1 Constant",
        "    0.034885008,             !- Coefficient2 x",
        "    -0.0006237,              !- Coefficient3 x**2",
        "    0.004977216,             !- Coefficient4 y",
        "    0.000437951,             !- Coefficient5 y**2",
        "    -0.000728028,            !- Coefficient6 x*y",
        "    -100,                    !- Minimum Value of x",
        "    100,                     !- Maximum Value of x",
        "    -100,                    !- Minimum Value of y",
        "    100,                     !- Maximum Value of y",
        "    ,                        !- Minimum Curve Output",
        "    ,                        !- Maximum Curve Output",
        "    Temperature,             !- Input Unit Type for X",
        "    Temperature,             !- Input Unit Type for Y",
        "    Dimensionless;           !- Output Unit Type",

        "  Curve:Quadratic,",
        "    HPACCoolCapFFF,          !- Name",
        "    0.8,                     !- Coefficient1 Constant",
        "    0.2,                     !- Coefficient2 x",
        "    0.0,                     !- Coefficient3 x**2",
        "    0.0,                     !- Minimum Value of x",
        "    1.0;                     !- Maximum Value of x",

        "  Curve:Quadratic,",
        "    HPACCOOLEIRFFF,          !- Name",
        "    1.1552,                  !- Coefficient1 Constant",
        "    -0.1808,                 !- Coefficient2 x",
        "    0.0256,                  !- Coefficient3 x**2",
        "    0.0,                     !- Minimum Value of x",
        "    1.0;                     !- Maximum Value of x",

        "  Curve:Quadratic,",
        "    HPACCOOLPLFFPLR,         !- Name",
        "    0.75,                    !- Coefficient1 Constant",
        "    0.25,                    !- Coefficient2 x",
        "    0.0,                     !- Coefficient3 x**2",
        "    0.0,                     !- Minimum Value of x",
        "    1.0;                     !- Maximum Value of x",

        "  Curve:Cubic,",
        "    PackagedRatedCoolCapFFlow,  !- Name",
        "    0.47278589,              !- Coefficient1 Constant",
        "    1.2433415,               !- Coefficient2 x",
        "    -1.0387055,              !- Coefficient3 x**2",
        "    0.32257813,              !- Coefficient4 x**3",
        "    0.5,                     !- Minimum Value of x",
        "    1.5,                     !- Maximum Value of x",
        "    ,                        !- Minimum Curve Output",
        "    ,                        !- Maximum Curve Output",
        "    Dimensionless,           !- Input Unit Type for X",
        "    Dimensionless;           !- Output Unit Type",

        "  Curve:Cubic,",
        "    PackagedRatedCoolEIRFFlow,  !- Name",
        "    1.0079484,               !- Coefficient1 Constant",
        "    0.34544129,              !- Coefficient2 x",
        "    -.6922891,               !- Coefficient3 x**2",
        "    0.33889943,              !- Coefficient4 x**3",
        "    0.5,                     !- Minimum Value of x",
        "    1.5,                     !- Maximum Value of x",
        "    ,                        !- Minimum Curve Output",
        "    ,                        !- Maximum Curve Output",
        "    Dimensionless,           !- Input Unit Type for X",
        "    Dimensionless;           !- Output Unit Type",

        "  Curve:Cubic,",
        "    EXHAUSTFANPLF,           !- Name",
        "    0,                       !- Coefficient1 Constant",
        "    1,                       !- Coefficient2 x",
        "    0.0,                     !- Coefficient3 x**2",
        "    0.0,                     !- Coefficient4 x**3",
        "    0.0,                     !- Minimum Value of x",
        "    1.0;                     !- Maximum Value of x",

        "  Curve:Quadratic,",
        "    VarSpeedCyclingPLFFPLR,  !- Name",
        "    0.85,                    !- Coefficient1 Constant",
        "    0.15,                    !- Coefficient2 x",
        "    0.0,                     !- Coefficient3 x**2",
        "    0.0,                     !- Minimum Value of x",
        "    1.0;                     !- Maximum Value of x",

        "  NodeList,",
        "    OutsideAirInletNodes,    !- Name",
        "    Outside Air Inlet Node;  !- Node 1 Name",

        "  NodeList,",
        "    SPACE1-1 In Nodes,       !- Name",
        "    SPACE1-1 In Node;        !- Node 1 Name",

        "  NodeList,",
        "    SPACE2-1 In Nodes,       !- Name",
        "    SPACE2-1 In Node;        !- Node 1 Name",

        "  NodeList,",
        "    SPACE3-1 In Nodes,       !- Name",
        "    SPACE3-1 In Node;        !- Node 1 Name",

        "  NodeList,",
        "    SPACE4-1 In Nodes,       !- Name",
        "    SPACE4-1 In Node;        !- Node 1 Name",

        "  NodeList,",
        "    Supply Air Temp Nodes 1, !- Name",
        "    VAV Sys 1 Outlet Node;   !- Node 1 Name",

        "  NodeList,",
        "    Supply Fan Upstream Nodes,  !- Name",
        "    Mixed Air Node 1,        !- Node 1 Name",
        "    Main Cooling Coil 1 Outlet Node,  !- Node 2 Name",
        "    Main Heating Coil 1 Outlet Node;  !- Node 3 Name",

        "  BranchList,",
        "    VAV Sys 1 Branches,      !- Name",
        "    VAV Sys 1 Main Branch;   !- Branch 1 Name",

        "  Branch,",
        "    VAV Sys 1 Main Branch,   !- Name",
        "    ,                        !- Pressure Drop Curve Name",
        "    AirLoopHVAC:OutdoorAirSystem,  !- Component 1 Object Type",
        "    OA Sys 1,                !- Component 1 Name",
        "    VAV Sys 1 Inlet Node,    !- Component 1 Inlet Node Name",
        "    Mixed Air Node 1,        !- Component 1 Outlet Node Name",
        "    CoilSystem:Cooling:DX,   !- Component 2 Object Type",
        "    DX Cooling Coil System 1,!- Component 2 Name",
        "    Mixed Air Node 1,        !- Component 2 Inlet Node Name",
        "    Main Cooling Coil 1 Outlet Node,  !- Component 2 Outlet Node Name",
        "    Coil:Heating:Fuel,        !- Component 3 Object Type",
        "    Main Heating Coil 1,     !- Component 3 Name",
        "    Main Cooling Coil 1 Outlet Node,  !- Component 3 Inlet Node Name",
        "    Main Heating Coil 1 Outlet Node,  !- Component 3 Outlet Node Name",
        "    Fan:VariableVolume,      !- Component 4 Object Type",
        "    Supply Fan 1,            !- Component 4 Name",
        "    Main Heating Coil 1 Outlet Node,  !- Component 4 Inlet Node Name",
        "    VAV Sys 1 Outlet Node;   !- Component 4 Outlet Node Name",

        "  AirLoopHVAC,",
        "    VAV Sys 1,               !- Name",
        "    ,                        !- Controller List Name",
        "    VAV Sys 1 Avail List,    !- Availability Manager List Name",
        "    autosize,                !- Design Supply Air Flow Rate {m3/s}",
        "    VAV Sys 1 Branches,      !- Branch List Name",
        "    ,                        !- Connector List Name",
        "    VAV Sys 1 Inlet Node,    !- Supply Side Inlet Node Name",
        "    PLENUM-1 Out Node,       !- Demand Side Outlet Node Name",
        "    Zone Eq In Node,         !- Demand Side Inlet Node Names",
        "    VAV Sys 1 Outlet Node;   !- Supply Side Outlet Node Names",

        "  AirLoopHVAC:ControllerList,",
        "    OA Sys 1 Controllers,    !- Name",
        "    Controller:OutdoorAir,   !- Controller 1 Object Type",
        "    OA Controller 1;         !- Controller 1 Name",

        "  AirLoopHVAC:OutdoorAirSystem:EquipmentList,",
        "    OA Sys 1 Equipment,      !- Name",
        "    Dehumidifier:Desiccant:System,  !- Component 1 Object Type",
        "    OA Desiccant System,     !- Component 1 Name",
        "    HeatExchanger:AirToAir:FlatPlate,  !- Component 2 Object Type",
        "    OA Heat Recovery,        !- Component 2 Name",
        "    OutdoorAir:Mixer,        !- Component 3 Object Type",
        "    OA Mixing Box 1;         !- Component 3 Name",

        "  AirLoopHVAC:OutdoorAirSystem,",
        "    OA Sys 1,                !- Name",
        "    OA Sys 1 Controllers,    !- Controller List Name",
        "    OA Sys 1 Equipment,      !- Outdoor Air Equipment List Name",
        "    VAV Sys 1 Avail List;    !- Availability Manager List Name",

        "  OutdoorAir:Node,",
        "    Main Cooling Coil 1 Condenser Node,  !- Name",
        "    -1.0;                    !- Height Above Ground {m}",

        "  OutdoorAir:NodeList,",
        "    OutsideAirInletNodes;    !- Node or NodeList Name 1",

        "  OutdoorAir:NodeList,",
        "    Desiccant DXSystem Outside Air Inlet Node,  !- Node or NodeList Name 1",
        "    Desiccant Outside Air Inlet Node 2;  !- Node or NodeList Name 2",

        "  OutdoorAir:NodeList,",
        "    Regen Coil Inlet Node;   !- Node or NodeList Name 1",

        "  OutdoorAir:NodeList,",
        "    Condenser Inlet Node;    !- Node or NodeList Name 1",

        "  OutdoorAir:Mixer,",
        "    OA Mixing Box 1,         !- Name",
        "    Mixed Air Node 1,        !- Mixed Air Node Name",
        "    OA Heat Recovery Outlet Node,  !- Outdoor Air Stream Node Name",
        "    Relief Air Outlet Node,  !- Relief Air Stream Node Name",
        "    VAV Sys 1 Inlet Node;    !- Return Air Stream Node Name",

        "  AvailabilityManagerAssignmentList,",
        "    VAV Sys 1 Avail List,    !- Name",
        "    AvailabilityManager:Scheduled,  !- Availability Manager 1 Object Type",
        "    VAV Sys 1 Avail;         !- Availability Manager 1 Name",

        "  AvailabilityManagerAssignmentList,",
        "    Desiccant Outdoor Air Avail List,  !- Name",
        "    AvailabilityManager:Scheduled,  !- Availability Manager 1 Object Type",
        "    Desiccant Outdoor Air Avail;  !- Availability Manager 1 Name",

        "  AvailabilityManager:Scheduled,",
        "    VAV Sys 1 Avail,         !- Name",
        "    FanAvailSched;           !- Schedule Name",

        "  AvailabilityManager:Scheduled,",
        "    Desiccant Outdoor Air Avail,  !- Name",
        "    FanAvailSched;           !- Schedule Name",

        "  SetpointManager:Scheduled,",
        "    Supply Air Temp Manager 1,  !- Name",
        "    Temperature,             !- Control Variable",
        "    Seasonal Reset Supply Air Temp Sch,  !- Schedule Name",
        "    Supply Air Temp Nodes 1; !- Setpoint Node or NodeList Name",

        "  SetpointManager:MixedAir,",
        "    Mixed Air and Coil Exit Temp Manager 1,  !- Name",
        "    Temperature,             !- Control Variable",
        "    VAV Sys 1 Outlet Node,   !- Reference Setpoint Node Name",
        "    Main Heating Coil 1 Outlet Node,  !- Fan Inlet Node Name",
        "    VAV Sys 1 Outlet Node,   !- Fan Outlet Node Name",
        "    Supply Fan Upstream Nodes;  !- Setpoint Node or NodeList Name",

        "  SetpointManager:MixedAir,",
        "    Adjusted Cooling Supply Air Temp Manager,  !- Name",
        "    Temperature,             !- Control Variable",
        "    Desiccant Supply Fan Outlet Node,  !- Reference Setpoint Node Name",
        "    HX Process Outlet Node,  !- Fan Inlet Node Name",
        "    Desiccant Supply Fan Outlet Node,  !- Fan Outlet Node Name",
        "    HX Process Outlet Node;  !- Setpoint Node or NodeList Name",

        "  Controller:OutdoorAir,",
        "    OA Controller 1,         !- Name",
        "    Relief Air Outlet Node,  !- Relief Air Outlet Node Name",
        "    VAV Sys 1 Inlet Node,    !- Return Air Node Name",
        "    Mixed Air Node 1,        !- Mixed Air Node Name",
        "    Outside Air Inlet Node,  !- Actuator Node Name",
        "    autosize,                !- Minimum Outdoor Air Flow Rate {m3/s}",
        "    autosize,                !- Maximum Outdoor Air Flow Rate {m3/s}",
        "    FixedDryBulb,            !- Economizer Control Type",
        "    ModulateFlow,            !- Economizer Control Action Type",
        "    19.,                     !- Economizer Maximum Limit Dry-Bulb Temperature {C}",
        "    ,                        !- Economizer Maximum Limit Enthalpy {J/kg}",
        "    ,                        !- Economizer Maximum Limit Dewpoint Temperature {C}",
        "    ,                        !- Electronic Enthalpy Limit Curve Name",
        "    4.,                      !- Economizer Minimum Limit Dry-Bulb Temperature {C}",
        "    NoLockout,               !- Lockout Type",
        "    FixedMinimum,            !- Minimum Limit Type",
        "    Min OA Sched;            !- Minimum Outdoor Air Schedule Name",

        "  ZoneHVAC:EquipmentConnections,",
        "    SPACE1-1,                !- Zone Name",
        "    SPACE1-1 Eq,             !- Zone Conditioning Equipment List Name",
        "    SPACE1-1 In Nodes,       !- Zone Air Inlet Node or NodeList Name",
        "    ,                        !- Zone Air Exhaust Node or NodeList Name",
        "    SPACE1-1 Node,           !- Zone Air Node Name",
        "    SPACE1-1 Out Node;       !- Zone Return Air Node Name",

        "  ZoneHVAC:EquipmentConnections,",
        "    SPACE2-1,                !- Zone Name",
        "    SPACE2-1 Eq,             !- Zone Conditioning Equipment List Name",
        "    SPACE2-1 In Nodes,       !- Zone Air Inlet Node or NodeList Name",
        "    ,                        !- Zone Air Exhaust Node or NodeList Name",
        "    SPACE2-1 Node,           !- Zone Air Node Name",
        "    SPACE2-1 Out Node;       !- Zone Return Air Node Name",

        "  ZoneHVAC:EquipmentConnections,",
        "    SPACE3-1,                !- Zone Name",
        "    SPACE3-1 Eq,             !- Zone Conditioning Equipment List Name",
        "    SPACE3-1 In Nodes,       !- Zone Air Inlet Node or NodeList Name",
        "    ,                        !- Zone Air Exhaust Node or NodeList Name",
        "    SPACE3-1 Node,           !- Zone Air Node Name",
        "    SPACE3-1 Out Node;       !- Zone Return Air Node Name",

        "  ZoneHVAC:EquipmentConnections,",
        "    SPACE4-1,                !- Zone Name",
        "    SPACE4-1 Eq,             !- Zone Conditioning Equipment List Name",
        "    SPACE4-1 In Nodes,       !- Zone Air Inlet Node or NodeList Name",
        "    ,                        !- Zone Air Exhaust Node or NodeList Name",
        "    SPACE4-1 Node,           !- Zone Air Node Name",
        "    SPACE4-1 Out Node;       !- Zone Return Air Node Name",

        "  ZoneHVAC:EquipmentList,",
        "    SPACE1-1 Eq,             !- Name",
        "    SequentialLoad,          !- Load Distribution Scheme",
        "    ZoneHVAC:AirDistributionUnit,  !- Zone Equipment 1 Object Type",
        "    SPACE1-1 ATU,            !- Zone Equipment 1 Name",
        "    1,                       !- Zone Equipment 1 Cooling Sequence",
        "    1;                       !- Zone Equipment 1 Heating or No-Load Sequence",

        "  ZoneHVAC:EquipmentList,",
        "    SPACE2-1 Eq,             !- Name",
        "    SequentialLoad,          !- Load Distribution Scheme",
        "    ZoneHVAC:AirDistributionUnit,  !- Zone Equipment 1 Object Type",
        "    SPACE2-1 ATU,            !- Zone Equipment 1 Name",
        "    1,                       !- Zone Equipment 1 Cooling Sequence",
        "    1;                       !- Zone Equipment 1 Heating or No-Load Sequence",

        "  ZoneHVAC:EquipmentList,",
        "    SPACE3-1 Eq,             !- Name",
        "    SequentialLoad,          !- Load Distribution Scheme",
        "    ZoneHVAC:AirDistributionUnit,  !- Zone Equipment 1 Object Type",
        "    SPACE3-1 ATU,            !- Zone Equipment 1 Name",
        "    1,                       !- Zone Equipment 1 Cooling Sequence",
        "    1;                       !- Zone Equipment 1 Heating or No-Load Sequence",

        "  ZoneHVAC:EquipmentList,",
        "    SPACE4-1 Eq,             !- Name",
        "    SequentialLoad,          !- Load Distribution Scheme",
        "    ZoneHVAC:AirDistributionUnit,  !- Zone Equipment 1 Object Type",
        "    SPACE4-1 ATU,            !- Zone Equipment 1 Name",
        "    1,                       !- Zone Equipment 1 Cooling Sequence",
        "    1;                       !- Zone Equipment 1 Heating or No-Load Sequence",

        "  ZoneHVAC:AirDistributionUnit,",
        "    SPACE1-1 ATU,            !- Name",
        "    SPACE1-1 In Node,        !- Air Distribution Unit Outlet Node Name",
        "    AirTerminal:SingleDuct:VAV:Reheat,  !- Air Terminal Object Type",
        "    SPACE1-1 VAV Reheat;     !- Air Terminal Name",

        "  ZoneHVAC:AirDistributionUnit,",
        "    SPACE2-1 ATU,            !- Name",
        "    SPACE2-1 In Node,        !- Air Distribution Unit Outlet Node Name",
        "    AirTerminal:SingleDuct:VAV:Reheat,  !- Air Terminal Object Type",
        "    SPACE2-1 VAV Reheat;     !- Air Terminal Name",

        "  ZoneHVAC:AirDistributionUnit,",
        "    SPACE3-1 ATU,            !- Name",
        "    SPACE3-1 In Node,        !- Air Distribution Unit Outlet Node Name",
        "    AirTerminal:SingleDuct:VAV:Reheat,  !- Air Terminal Object Type",
        "    SPACE3-1 VAV Reheat;     !- Air Terminal Name",

        "  ZoneHVAC:AirDistributionUnit,",
        "    SPACE4-1 ATU,            !- Name",
        "    SPACE4-1 In Node,        !- Air Distribution Unit Outlet Node Name",
        "    AirTerminal:SingleDuct:VAV:Reheat,  !- Air Terminal Object Type",
        "    SPACE4-1 VAV Reheat;     !- Air Terminal Name",

        "  CoilSystem:Cooling:DX,",
        "    DX Cooling Coil System 1,!- Name",
        "    FanAvailSched,           !- Availability Schedule Name",
        "    Mixed Air Node 1,        !- DX Cooling Coil System Inlet Node Name",
        "    Main Cooling Coil 1 Outlet Node,  !- DX Cooling Coil System Outlet Node Name",
        "    Main Cooling Coil 1 Outlet Node,  !- DX Cooling Coil System Sensor Node Name",
        "    Coil:Cooling:DX:TwoSpeed,!- Cooling Coil Object Type",
        "    Main Cooling Coil 1;     !- Cooling Coil Name",

        "  AirTerminal:SingleDuct:VAV:Reheat,",
        "    SPACE1-1 VAV Reheat,     !- Name",
        "    FanAvailSched,           !- Availability Schedule Name",
        "    SPACE1-1 Zone Coil Air In Node,  !- Damper Air Outlet Node Name",
        "    SPACE1-1 ATU In Node,    !- Air Inlet Node Name",
        "    autosize,                !- Maximum Air Flow Rate {m3/s}",
        "    Constant,                !- Zone Minimum Air Flow Input Method",
        "    0.3,                     !- Constant Minimum Air Flow Fraction",
        "    ,                        !- Fixed Minimum Air Flow Rate {m3/s}",
        "    ,                        !- Minimum Air Flow Fraction Schedule Name",
        "    Coil:Heating:Fuel,        !- Reheat Coil Object Type",
        "    SPACE1-1 Zone Coil,      !- Reheat Coil Name",
        "    0.0,                     !- Maximum Hot Water or Steam Flow Rate {m3/s}",
        "    0.0,                     !- Minimum Hot Water or Steam Flow Rate {m3/s}",
        "    SPACE1-1 In Node,        !- Air Outlet Node Name",
        "    0.001,                   !- Convergence Tolerance",
        "    Reverse,                 !- Damper Heating Action",
        "    AUTOCALCULATE,           !- Maximum Flow per Zone Floor Area During Reheat {m3/s-m2}",
        "    AUTOCALCULATE;           !- Maximum Flow Fraction During Reheat",

        "  AirTerminal:SingleDuct:VAV:Reheat,",
        "    SPACE2-1 VAV Reheat,     !- Name",
        "    FanAvailSched,           !- Availability Schedule Name",
        "    SPACE2-1 Zone Coil Air In Node,  !- Damper Air Outlet Node Name",
        "    SPACE2-1 ATU In Node,    !- Air Inlet Node Name",
        "    autosize,                !- Maximum Air Flow Rate {m3/s}",
        "    Constant,                !- Zone Minimum Air Flow Input Method",
        "    0.3,                     !- Constant Minimum Air Flow Fraction",
        "    ,                        !- Fixed Minimum Air Flow Rate {m3/s}",
        "    ,                        !- Minimum Air Flow Fraction Schedule Name",
        "    Coil:Heating:Fuel,        !- Reheat Coil Object Type",
        "    SPACE2-1 Zone Coil,      !- Reheat Coil Name",
        "    0.0,                     !- Maximum Hot Water or Steam Flow Rate {m3/s}",
        "    0.0,                     !- Minimum Hot Water or Steam Flow Rate {m3/s}",
        "    SPACE2-1 In Node,        !- Air Outlet Node Name",
        "    0.001,                   !- Convergence Tolerance",
        "    Reverse,                 !- Damper Heating Action",
        "    AUTOCALCULATE,           !- Maximum Flow per Zone Floor Area During Reheat {m3/s-m2}",
        "    AUTOCALCULATE;           !- Maximum Flow Fraction During Reheat",

        "  AirTerminal:SingleDuct:VAV:Reheat,",
        "    SPACE3-1 VAV Reheat,     !- Name",
        "    FanAvailSched,           !- Availability Schedule Name",
        "    SPACE3-1 Zone Coil Air In Node,  !- Damper Air Outlet Node Name",
        "    SPACE3-1 ATU In Node,    !- Air Inlet Node Name",
        "    autosize,                !- Maximum Air Flow Rate {m3/s}",
        "    Constant,                !- Zone Minimum Air Flow Input Method",
        "    0.3,                     !- Constant Minimum Air Flow Fraction",
        "    ,                        !- Fixed Minimum Air Flow Rate {m3/s}",
        "    ,                        !- Minimum Air Flow Fraction Schedule Name",
        "    Coil:Heating:Fuel,        !- Reheat Coil Object Type",
        "    SPACE3-1 Zone Coil,      !- Reheat Coil Name",
        "    0.0,                     !- Maximum Hot Water or Steam Flow Rate {m3/s}",
        "    0.0,                     !- Minimum Hot Water or Steam Flow Rate {m3/s}",
        "    SPACE3-1 In Node,        !- Air Outlet Node Name",
        "    0.001,                   !- Convergence Tolerance",
        "    Reverse,                 !- Damper Heating Action",
        "    AUTOCALCULATE,           !- Maximum Flow per Zone Floor Area During Reheat {m3/s-m2}",
        "    AUTOCALCULATE;           !- Maximum Flow Fraction During Reheat",

        "  AirTerminal:SingleDuct:VAV:Reheat,",
        "    SPACE4-1 VAV Reheat,     !- Name",
        "    FanAvailSched,           !- Availability Schedule Name",
        "    SPACE4-1 Zone Coil Air In Node,  !- Damper Air Outlet Node Name",
        "    SPACE4-1 ATU In Node,    !- Air Inlet Node Name",
        "    autosize,                !- Maximum Air Flow Rate {m3/s}",
        "    Constant,                !- Zone Minimum Air Flow Input Method",
        "    0.3,                     !- Constant Minimum Air Flow Fraction",
        "    ,                        !- Fixed Minimum Air Flow Rate {m3/s}",
        "    ,                        !- Minimum Air Flow Fraction Schedule Name",
        "    Coil:Heating:Fuel,        !- Reheat Coil Object Type",
        "    SPACE4-1 Zone Coil,      !- Reheat Coil Name",
        "    0.0,                     !- Maximum Hot Water or Steam Flow Rate {m3/s}",
        "    0.0,                     !- Minimum Hot Water or Steam Flow Rate {m3/s}",
        "    SPACE4-1 In Node,        !- Air Outlet Node Name",
        "    0.001,                   !- Convergence Tolerance",
        "    Reverse,                 !- Damper Heating Action",
        "    AUTOCALCULATE,           !- Maximum Flow per Zone Floor Area During Reheat {m3/s-m2}",
        "    AUTOCALCULATE;           !- Maximum Flow Fraction During Reheat",

        "  ZoneControl:Thermostat,",
        "    SPACE1-1 Control,        !- Name",
        "    SPACE1-1,                !- Zone or ZoneList Name",
        "    Zone Control Type Sched, !- Control Type Schedule Name",
        "    ThermostatSetpoint:SingleCooling,  !- Control 1 Object Type",
        "    CoolingSetPoint,         !- Control 1 Name",
        "    ThermostatSetpoint:SingleHeating,  !- Control 2 Object Type",
        "    HeatingSetpoint,         !- Control 2 Name",
        "    ThermostatSetpoint:DualSetpoint,  !- Control 3 Object Type",
        "    DualSetPoint;            !- Control 3 Name",

        "  ZoneControl:Thermostat,",
        "    SPACE2-1 Control,        !- Name",
        "    SPACE2-1,                !- Zone or ZoneList Name",
        "    Zone Control Type Sched, !- Control Type Schedule Name",
        "    ThermostatSetpoint:SingleCooling,  !- Control 1 Object Type",
        "    CoolingSetPoint,         !- Control 1 Name",
        "    ThermostatSetpoint:SingleHeating,  !- Control 2 Object Type",
        "    HeatingSetpoint,         !- Control 2 Name",
        "    ThermostatSetpoint:DualSetpoint,  !- Control 3 Object Type",
        "    DualSetPoint;            !- Control 3 Name",

        "  ZoneControl:Thermostat,",
        "    SPACE3-1 Control,        !- Name",
        "    SPACE3-1,                !- Zone or ZoneList Name",
        "    Zone Control Type Sched, !- Control Type Schedule Name",
        "    ThermostatSetpoint:SingleCooling,  !- Control 1 Object Type",
        "    CoolingSetPoint,         !- Control 1 Name",
        "    ThermostatSetpoint:SingleHeating,  !- Control 2 Object Type",
        "    HeatingSetpoint,         !- Control 2 Name",
        "    ThermostatSetpoint:DualSetpoint,  !- Control 3 Object Type",
        "    DualSetPoint;            !- Control 3 Name",

        "  ZoneControl:Thermostat,",
        "    SPACE4-1 Control,        !- Name",
        "    SPACE4-1,                !- Zone or ZoneList Name",
        "    Zone Control Type Sched, !- Control Type Schedule Name",
        "    ThermostatSetpoint:SingleCooling,  !- Control 1 Object Type",
        "    CoolingSetPoint,         !- Control 1 Name",
        "    ThermostatSetpoint:SingleHeating,  !- Control 2 Object Type",
        "    HeatingSetpoint,         !- Control 2 Name",
        "    ThermostatSetpoint:DualSetpoint,  !- Control 3 Object Type",
        "    DualSetPoint;            !- Control 3 Name",

        "  Schedule:Compact,",
        "    Relative Humidity Sched, !- Name",
        "    Any Number,              !- Schedule Type Limits Name",
        "    Through: 12/31,          !- Field 1",
        "    For: AllDays,            !- Field 2",
        "    Until: 24:00,45.0;       !- Field 3",

        "  ThermostatSetpoint:SingleHeating,",
        "    HeatingSetpoint,         !- Name",
        "    Htg-SetP-Sch;            !- Setpoint Temperature Schedule Name",

        "  ThermostatSetpoint:SingleHeating,",
        "    PlenumHeatingSetpoint,   !- Name",
        "    PlenumHtg-SetP-Sch;      !- Setpoint Temperature Schedule Name",

        "  ThermostatSetpoint:SingleCooling,",
        "    CoolingSetpoint,         !- Name",
        "    Clg-SetP-Sch;            !- Setpoint Temperature Schedule Name",

        "  ThermostatSetpoint:SingleCooling,",
        "    PlenumCoolingSetpoint,   !- Name",
        "    PlenumClg-SetP-Sch;      !- Setpoint Temperature Schedule Name",

        "  ThermostatSetpoint:DualSetpoint,",
        "    DualSetPoint,            !- Name",
        "    Htg-SetP-Sch,            !- Heating Setpoint Temperature Schedule Name",
        "    Clg-SetP-Sch;            !- Cooling Setpoint Temperature Schedule Name",

        "  AirLoopHVAC:SupplyPath,",
        "    Zone Supply Air Path 1,  !- Name",
        "    Zone Eq In Node,         !- Supply Air Path Inlet Node Name",
        "    AirLoopHVAC:ZoneSplitter,!- Component 1 Object Type",
        "    Zone Supply Air Splitter 1;  !- Component 1 Name",

        "  AirLoopHVAC:ReturnPath,",
        "    ReturnAirPath1,          !- Name",
        "    PLENUM-1 Out Node,       !- Return Air Path Outlet Node Name",
        "    AirLoopHVAC:ReturnPlenum,!- Component 1 Object Type",
        "    Return-Plenum-1;         !- Component 1 Name",

        "  AirLoopHVAC:ReturnPlenum,",
        "    Return-Plenum-1,         !- Name",
        "    PLENUM-1,                !- Zone Name",
        "    PLENUM-1 Node,           !- Zone Node Name",
        "    PLENUM-1 Out Node,       !- Outlet Node Name",
        "    ,                        !- Induced Air Outlet Node or NodeList Name",
        "    SPACE1-1 Out Node,       !- Inlet 1 Node Name",
        "    SPACE2-1 Out Node,       !- Inlet 2 Node Name",
        "    SPACE3-1 Out Node,       !- Inlet 3 Node Name",
        "    SPACE4-1 Out Node;       !- Inlet 4 Node Name",

        "  AirLoopHVAC:ZoneSplitter,",
        "    Zone Supply Air Splitter 1,  !- Name",
        "    Zone Eq In Node,         !- Inlet Node Name",
        "    SPACE1-1 ATU In Node,    !- Outlet 1 Node Name",
        "    SPACE2-1 ATU In Node,    !- Outlet 2 Node Name",
        "    SPACE3-1 ATU In Node,    !- Outlet 3 Node Name",
        "    SPACE4-1 ATU In Node;    !- Outlet 4 Node Name",

        "  Coil:Heating:Fuel,",
        "    SPACE1-1 Zone Coil,      !- Name",
        "    FanAvailSched,           !- Availability Schedule Name",
        "    Gas,                     !- Fuel Type",
        "    0.8,                     !- Gas Burner Efficiency",
        "    autosize,                !- Nominal Capacity {W}",
        "    SPACE1-1 Zone Coil Air In Node,  !- Air Inlet Node Name",
        "    SPACE1-1 In Node;        !- Air Outlet Node Name",

        "  Coil:Heating:Fuel,",
        "    SPACE2-1 Zone Coil,      !- Name",
        "    FanAvailSched,           !- Availability Schedule Name",
        "    Gas,                     !- Fuel Type",
        "    0.8,                     !- Gas Burner Efficiency",
        "    autosize,                !- Nominal Capacity {W}",
        "    SPACE2-1 Zone Coil Air In Node,  !- Air Inlet Node Name",
        "    SPACE2-1 In Node;        !- Air Outlet Node Name",

        "  Coil:Heating:Fuel,",
        "    SPACE3-1 Zone Coil,      !- Name",
        "    FanAvailSched,           !- Availability Schedule Name",
        "    Gas,                     !- Fuel Type",
        "    0.8,                     !- Gas Burner Efficiency",
        "    autosize,                !- Nominal Capacity {W}",
        "    SPACE3-1 Zone Coil Air In Node,  !- Air Inlet Node Name",
        "    SPACE3-1 In Node;        !- Air Outlet Node Name",

        "  Coil:Heating:Fuel,",
        "    SPACE4-1 Zone Coil,      !- Name",
        "    FanAvailSched,           !- Availability Schedule Name",
        "    Gas,                     !- Fuel Type",
        "    0.8,                     !- Gas Burner Efficiency",
        "    autosize,                !- Nominal Capacity {W}",
        "    SPACE4-1 Zone Coil Air In Node,  !- Air Inlet Node Name",
        "    SPACE4-1 In Node;        !- Air Outlet Node Name",

        "  Coil:Heating:Fuel,",
        "    Main heating Coil 1,     !- Name",
        "    FanAvailSched,           !- Availability Schedule Name",
        "    Gas,                     !- Fuel Type",
        "    0.8,                     !- Gas Burner Efficiency",
        "    autosize,                !- Nominal Capacity {W}",
        "    Main Cooling Coil 1 Outlet Node,  !- Air Inlet Node Name",
        "    Main Heating Coil 1 Outlet Node,  !- Air Outlet Node Name",
        "    Main Heating Coil 1 Outlet Node;  !- Temperature Setpoint Node Name",

        "  Coil:Cooling:DX:TwoSpeed,",
        "    Main Cooling Coil 1,     !- Name",
        "    FanAvailSched,           !- Availability Schedule Name",
        "    autosize,                !- High Speed Gross Rated Total Cooling Capacity {W}",
        "    autosize,                !- Gross Rated Sensible Heat Ratio",
        "    3.0,                     !- High Speed Gross Rated Cooling COP {W/W}",
        "    autosize,                !- High Speed Rated Air Flow Rate {m3/s}",
        "    ,                        !- Unit Internal Static Air Pressure {Pa}",
        "    Mixed Air Node 1,        !- Air Inlet Node Name",
        "    Main Cooling Coil 1 Outlet Node,  !- Air Outlet Node Name",
        "    VarSpeedCoolCapFT,       !- Total Cooling Capacity Function of Temperature Curve Name",
        "    PackagedRatedCoolCapFFlow,  !- Total Cooling Capacity Function of Flow Fraction Curve Name",
        "    VarSpeedCoolEIRFT,       !- Energy Input Ratio Function of Temperature Curve Name",
        "    PackagedRatedCoolEIRFFlow,  !- Energy Input Ratio Function of Flow Fraction Curve Name",
        "    VarSpeedCyclingPLFFPLR,  !- Part Load Fraction Correlation Curve Name",
        "    autosize,                !- Low Speed Gross Rated Total Cooling Capacity {W}",
        "    autosize,                !- Low Speed Gross Rated Sensible Heat Ratio",
        "    4.2,                     !- Low Speed Gross Rated Cooling COP {W/W}",
        "    autosize,                !- Low Speed Rated Air Flow Rate {m3/s}",
        "    VarSpeedCoolCapLSFT,     !- Low Speed Total Cooling Capacity Function of Temperature Curve Name",
        "    VarSpeedCoolEIRLSFT,     !- Low Speed Energy Input Ratio Function of Temperature Curve Name",
        "    Main Cooling Coil 1 Condenser Node;  !- Condenser Air Inlet Node Name",

        "  Coil:Cooling:DX:SingleSpeed,",
        "    Desiccant DXSystem Cooling Coil,  !- Name",
        "    FanAvailSched,           !- Availability Schedule Name",
        "    autosize,                !- High Speed Rated Air Flow Rate {m3/s}",
        "    autosize,                !- Gross Rated Sensible Heat Ratio",
        "    3.5,                     !- Gross Rated Cooling COP {W/W}",
        "    autosize,                !- High Speed Rated Air Flow Rate {m3/s}",
        "    ,                        !- Rated Evaporator Fan Power Per Volume Flow Rate {W/(m3/s)}",
        "    Desiccant DXSystem Mixed Air Node,  !- Air Inlet Node Name",
        "    HX Process Inlet Node,   !- Air Outlet Node Name",
        "    HPACCoolCapFT,           !- Total Cooling Capacity Function of Temperature Curve Name",
        "    HPACCoolCapFFF,          !- Total Cooling Capacity Function of Flow Fraction Curve Name",
        "    HPACCOOLEIRFT,           !- Energy Input Ratio Function of Temperature Curve Name",
        "    HPACCOOLEIRFFF,          !- Energy Input Ratio Function of Flow Fraction Curve Name",
        "    HPACCOOLPLFFPLR,         !- Part Load Fraction Correlation Curve Name",
        "   ,                         !- Minimum Outdoor Dry-Bulb Temperature for Compressor Operation {C}",
        "    1000,                    !- Nominal Time for Condensate Removal to Begin {s}",
        "    0.4,                     !- Ratio of Initial Moisture Evaporation Rate and Steady State Latent Capacity {dimensionless}",
        "    4,                       !- Maximum Cycling Rate {cycles/hr}",
        "    45,                      !- Latent Capacity Time Constant {s}",
        "    Condenser Inlet Node,    !- Condenser Air Inlet Node Name",
        "    AirCooled;               !- Condenser Type",

        "  Fan:VariableVolume,",
        "    Supply Fan 1,            !- Name",
        "    FanAvailSched,           !- Availability Schedule Name",
        "    0.7,                     !- Fan Total Efficiency",
        "    600.0,                   !- Pressure Rise {Pa}",
        "    autosize,                !- Maximum Flow Rate {m3/s}",
        "    FixedFlowRate,           !- Fan Power Minimum Flow Rate Input Method",
        "    ,                        !- Fan Power Minimum Flow Fraction",
        "    0.35326,                 !- Fan Power Minimum Air Flow Rate {m3/s}",
        "    0.9,                     !- Motor Efficiency",
        "    1.0,                     !- Motor In Airstream Fraction",
        "    0.0015302446,            !- Fan Power Coefficient 1",
        "    0.0052080574,            !- Fan Power Coefficient 2",
        "    1.1086242,               !- Fan Power Coefficient 3",
        "    -0.11635563,             !- Fan Power Coefficient 4",
        "    0.000,                   !- Fan Power Coefficient 5",
        "    Main Heating Coil 1 Outlet Node,  !- Air Inlet Node Name",
        "    VAV Sys 1 Outlet Node;   !- Air Outlet Node Name",

        "  HeatExchanger:AirToAir:FlatPlate,",
        "    OA Heat Recovery,        !- Name",
        "    FanAvailSched,           !- Availability Schedule Name",
        "    CounterFlow,             !- Flow Arrangement Type",
        "    Yes,                     !- Economizer Lockout",
        "    1.0,                     !- Ratio of Supply to Secondary hA Values",
        "    autosize,                !- Nominal Supply Air Flow Rate {m3/s}",
        "    5.0,                     !- Nominal Supply Air Inlet Temperature {C}",
        "    15.0,                    !- Nominal Supply Air Outlet Temperature {C}",
        "    autosize,                !- Nominal Secondary Air Flow Rate {m3/s}",
        "    20.0,                    !- Nominal Secondary Air Inlet Temperature {C}",
        "    0.0,                     !- Nominal Electric Power {W}",
        "    OA Desiccant Process Outlet Node,  !- Supply Air Inlet Node Name",
        "    OA Heat Recovery Outlet Node,  !- Supply Air Outlet Node Name",
        "    Relief Air Outlet Node,  !- Secondary Air Inlet Node Name",
        "    OA Heat Recovery Secondary Outlet Node;  !- Secondary Air Outlet Node Name",

        "  Fan:OnOff,",
        "    OA Desiccant Regen Fan,  !- Name",
        "    FanAvailSched,           !- Availability Schedule Name",
        "    0.7,                     !- Fan Total Efficiency",
        "    205.5,                   !- Pressure Rise {Pa}",
        "    autosize,                !- Maximum Flow Rate {m3/s}",
        "    0.9,                     !- Motor Efficiency",
        "    1.0,                     !- Motor In Airstream Fraction",
        "    OA HX Regen Outlet Node, !- Air Inlet Node Name",
        "    OA Regen Fan Outlet Node;!- Air Outlet Node Name",

        "  Coil:Heating:Fuel,",
        "    OA Desiccant Regen Coil, !- Name",
        "    FanAvailSched,           !- Availability Schedule Name",
        "    Gas,                     !- Fuel Type",
        "    0.80,                    !- Gas Burner Efficiency",
        "    autosize,                !- Nominal Capacity {W}",
        "    OA Heat Recovery Secondary Outlet Node,  !- Air Inlet Node Name",
        "    OA HX Regen Inlet Node;  !- Air Outlet Node Name",

        "  Dehumidifier:Desiccant:System,",
        "    OA Desiccant System,     !- Name",
        "    FanAvailSched,           !- Availability Schedule Name",
        "    HeatExchanger:Desiccant:BalancedFlow,  !- Desiccant Heat Exchanger Object Type",
        "    OA Desiccant Heat Exchanger,  !- Desiccant Heat Exchanger Name",
        "    OA Desiccant Process Outlet Node,  !- Sensor Node Name",
        "    Fan:OnOff,               !- Regeneration Air Fan Object Type",
        "    OA Desiccant Regen Fan,  !- Regeneration Air Fan Name",
        "    DrawThrough,             !- Regeneration Air Fan Placement",
        "    Coil:Heating:Fuel,        !- Regeneration Air Heater Object Type",
        "    OA Desiccant Regen Coil, !- Regeneration Air Heater Name",
        "    46.111111;               !- Regeneration Inlet Air Setpoint Temperature {C}",

        "  SetpointManager:OutdoorAirPretreat,",
        "    DXSystem 1 Desiccant Set Point Manager,  !- Name",
        "    MaximumHumidityRatio,    !- Control Variable",
        "    -99,                     !- Minimum Setpoint Temperature {C}",
        "    99,                      !- Maximum Setpoint Temperature {C}",
        "    0.00001,                 !- Minimum Setpoint Humidity Ratio {kgWater/kgDryAir}",
        "    1.0,                     !- Maximum Setpoint Humidity Ratio {kgWater/kgDryAir}",
        "    Mixed Air Node 1,        !- Reference Setpoint Node Name",
        "    Mixed Air Node 1,        !- Mixed Air Stream Node Name",
        "    OA Desiccant Process Outlet Node,  !- Outdoor Air Stream Node Name",
        "    VAV Sys 1 Inlet Node,    !- Return Air Stream Node Name",
        "    OA Desiccant Process Outlet Node;  !- Setpoint Node or NodeList Name",

        "  SetpointManager:SingleZone:Humidity:Maximum,",
        "    SPACE1-1 Humidistat Manager,  !- Name",
        "    Mixed Air Node 1,        !- Setpoint Node or NodeList Name",
        "    SPACE1-1 Node;           !- Control Zone Air Node Name",

        "  ZoneControl:Humidistat,",
        "    SPACE1-1 Humidistat,     !- Name",
        "    SPACE1-1,                !- Zone Name",
        "    Relative Humidity Sched; !- Humidifying Relative Humidity Setpoint Schedule Name",

        "  HeatExchanger:Desiccant:BalancedFlow,",
        "    OA Desiccant Heat Exchanger,  !- Name",
        "    FanAvailSched,           !- Availability Schedule Name",
        "    OA HX Regen Inlet Node,  !- Regeneration Air Inlet Node Name",
        "    OA HX Regen Outlet Node, !- Regeneration Air Outlet Node Name",
        "    Outside Air Inlet Node,  !- Process Air Inlet Node Name",
        "    OA Desiccant Process Outlet Node,  !- Process Air Outlet Node Name",
        "    HeatExchanger:Desiccant:BalancedFlow:PerformanceDataType1,  !- Heat Exchanger Performance Object Type",
        "    HXDesPerf2;              !- Heat Exchanger Performance Name",

        "  HeatExchanger:Desiccant:BalancedFlow:PerformanceDataType1,",
        "    HXDesPerf2,              !- Name",
        "    2.5893,                  !- Nominal Air Flow Rate {m3/s}",
        "    4.8000,                  !- Nominal Air Face Velocity {m/s}",
        "    50.0,                    !- Nominal Electric Power {W}",
        "    -3.0000E+00,             !- Temperature Equation Coefficient 1",
        "    0.00000E+01,             !- Temperature Equation Coefficient 2",
        "    1.00000E+00,             !- Temperature Equation Coefficient 3",
        "    0.00000E+00,             !- Temperature Equation Coefficient 4",
        "    0.00000E+00,             !- Temperature Equation Coefficient 5",
        "    0.00000E+00,             !- Temperature Equation Coefficient 6",
        "    0.00000E+00,             !- Temperature Equation Coefficient 7",
        "    0.00000E+00,             !- Temperature Equation Coefficient 8",
        "    0.000000,                !- Minimum Regeneration Inlet Air Humidity Ratio for Temperature Equation {kgWater/kgDryAir}",
        "    1.000000,                !- Maximum Regeneration Inlet Air Humidity Ratio for Temperature Equation {kgWater/kgDryAir}",
        "    46.111110,               !- Minimum Regeneration Inlet Air Temperature for Temperature Equation {C}",
        "    46.111112,               !- Maximum Regeneration Inlet Air Temperature for Temperature Equation {C}",
        "    0.000000,                !- Minimum Process Inlet Air Humidity Ratio for Temperature Equation {kgWater/kgDryAir}",
        "    1.000000,                !- Maximum Process Inlet Air Humidity Ratio for Temperature Equation {kgWater/kgDryAir}",
        "    -100.000,                !- Minimum Process Inlet Air Temperature for Temperature Equation {C}",
        "    100.0000,                !- Maximum Process Inlet Air Temperature for Temperature Equation {C}",
        "    0.001,                   !- Minimum Regeneration Air Velocity for Temperature Equation {m/s}",
        "    4.826,                   !- Maximum Regeneration Air Velocity for Temperature Equation {m/s}",
        "    43.111110,               !- Minimum Regeneration Outlet Air Temperature for Temperature Equation {C}",
        "    43.111112,               !- Maximum Regeneration Outlet Air Temperature for Temperature Equation {C}",
        "    5.0,                     !- Minimum Regeneration Inlet Air Relative Humidity for Temperature Equation {percent}",
        "    45.0,                    !- Maximum Regeneration Inlet Air Relative Humidity for Temperature Equation {percent}",
        "    0.0,                     !- Minimum Process Inlet Air Relative Humidity for Temperature Equation {percent}",
        "    100.0,                   !- Maximum Process Inlet Air Relative Humidity for Temperature Equation {percent}",
        "    0.10000E-02,             !- Humidity Ratio Equation Coefficient 1",
        "    1.00000E+00,             !- Humidity Ratio Equation Coefficient 2",
        "    0.00000E+00,             !- Humidity Ratio Equation Coefficient 3",
        "    0.00000E+00,             !- Humidity Ratio Equation Coefficient 4",
        "    0.00000E+00,             !- Humidity Ratio Equation Coefficient 5",
        "    0.00000E+00,             !- Humidity Ratio Equation Coefficient 6",
        "    0.00000E+00,             !- Humidity Ratio Equation Coefficient 7",
        "    0.00000E+00,             !- Humidity Ratio Equation Coefficient 8",
        "    0.000000,                !- Minimum Regeneration Inlet Air Humidity Ratio for Humidity Ratio Equation {kgWater/kgDryAir}",
        "    1.000000,                !- Maximum Regeneration Inlet Air Humidity Ratio for Humidity Ratio Equation {kgWater/kgDryAir}",
        "    46.111110,               !- Minimum Regeneration Inlet Air Temperature for Humidity Ratio Equation {C}",
        "    46.111112,               !- Maximum Regeneration Inlet Air Temperature for Humidity Ratio Equation {C}",
        "    0.000000,                !- Minimum Process Inlet Air Humidity Ratio for Humidity Ratio Equation {kgWater/kgDryAir}",
        "    1.000000,                !- Maximum Process Inlet Air Humidity Ratio for Humidity Ratio Equation {kgWater/kgDryAir}",
        "    -100.000,                !- Minimum Process Inlet Air Temperature for Humidity Ratio Equation {C}",
        "    100.0000,                !- Maximum Process Inlet Air Temperature for Humidity Ratio Equation {C}",
        "    0.001,                   !- Minimum Regeneration Air Velocity for Humidity Ratio Equation {m/s}",
        "    4.826,                   !- Maximum Regeneration Air Velocity for Humidity Ratio Equation {m/s}",
        "    0.000000,                !- Minimum Regeneration Outlet Air Humidity Ratio for Humidity Ratio Equation {kgWater/kgDryAir}",
        "    1.000000,                !- Maximum Regeneration Outlet Air Humidity Ratio for Humidity Ratio Equation {kgWater/kgDryAir}",
        "    5.0,                     !- Minimum Regeneration Inlet Air Relative Humidity for Humidity Ratio Equation {percent}",
        "    45.0,                    !- Maximum Regeneration Inlet Air Relative Humidity for Humidity Ratio Equation {percent}",
        "    0.0,                     !- Minimum Process Inlet Air Relative Humidity for Humidity Ratio Equation {percent}",
        "    100.0;                   !- Maximum Process Inlet Air Relative Humidity for Humidity Ratio Equation {percent}",

    });

    ASSERT_TRUE(process_idf(idf_objects));

    OutputProcessor::TimeValue.allocate(2);
    DataGlobals::DDOnlySimulation = true;

    SimulationManager::GetProjectData();
    OutputReportPredefined::SetPredefinedTables();
    createFacilityElectricPowerServiceObject();
    SetPreConstructionInputParameters(); // establish array bounds for constructions early
    BranchInputManager::ManageBranchInput();
    BeginSimFlag = true;
    BeginEnvrnFlag = true;
    ZoneSizingCalc = true;
    SysSizingCalc = true;
    SizingManager::ManageSizing();

    DataSizing::CurSysNum = 1;
    DataSizing::CurOASysNum = 1;

    GetDesiccantDehumidifierInput();
    EXPECT_EQ(1, NumDesicDehums);
    EXPECT_EQ("OA DESICCANT SYSTEM", DesicDehum(DesicDehumNum).Name);
    EXPECT_EQ("OA DESICCANT REGEN COIL", DesicDehum(DesicDehumNum).RegenCoilName);

    CompName = DesicDehum(DesicDehumNum).Name;
    CompIndex = NumGenericDesicDehums;
    SimDesiccantDehumidifier(CompName, FirstHVACIteration, CompIndex);

    RegCoilDesInletTemp = FinalSysSizing(DataSizing::CurSysNum).HeatRetTemp;
    RegCoilDesOutletTemp = DesicDehum(DesicDehumNum).RegenSetPointTemp;
    RegCoilInletAirMassFlowRate = FinalSysSizing(DataSizing::CurSysNum).DesOutAirVolFlow * DataEnvironment::StdRhoAir;
    RegCoilCapacity = RegCoilInletAirMassFlowRate * PsyCpAirFnWTdb(0.0, 20.0) * (RegCoilDesOutletTemp - RegCoilDesInletTemp);

    for (loop = 1; loop <= NumHeatingCoils; ++loop) {
        if (HeatingCoil(loop).Name == DesicDehum(DesicDehumNum).RegenCoilName) {
            CoilIndex = loop;
        }
    }
    EXPECT_EQ(RegCoilCapacity, HeatingCoil(CoilIndex).NominalCapacity);
}

TEST_F(EnergyPlusFixture, DesiccantDehum_OnPrimaryAirSystemTest)
{

    std::string CompName("");
    bool FirstHVACIteration(true);
    int DesicDehumNum(1);
    int CompIndex(1);
    int CoilIndex(0);
    int loop(0);
    Real64 RegCoilDesInletTemp(0.0);
    Real64 RegCoilDesOutletTemp(0.0);
    Real64 RegCoilInletAirMassFlowRate(0.0);
    Real64 RegCoilCapacity(0.0);

    std::string const idf_objects = delimited_string({

        "  Timestep,6;",

        "  Building,",
        "    Bldg,                    !- Name",
        "    0.0,                     !- North Axis {deg}",
        "    Suburbs,                 !- Terrain",
        "    0.05,                    !- Loads Convergence Tolerance Value",
        "    0.05,                    !- Temperature Convergence Tolerance Value {deltaC}",
        "    MinimalShadowing,        !- Solar Distribution",
        "    30,                      !- Maximum Number of Warmup Days",
        "    6;                       !- Minimum Number of Warmup Days",

        "  SurfaceConvectionAlgorithm:Inside,Simple;",

        "  SurfaceConvectionAlgorithm:Outside,SimpleCombined;",

        "  HeatBalanceAlgorithm,ConductionTransferFunction;",

        "  SimulationControl,",
        "    Yes,                     !- Do Zone Sizing Calculation",
        "    Yes,                     !- Do System Sizing Calculation",
        "    No,                      !- Do Plant Sizing Calculation",
        "    Yes,                     !- Run Simulation for Sizing Periods",
        "    No;                      !- Run Simulation for Weather File Run Periods",

        "  RunPeriod,",
        "    ,                        !- Name",
        "    1,                       !- Begin Month",
        "    1,                       !- Begin Day of Month",
        "    ,                        !- Begin Year",
        "    1,                       !- End Month",
        "    5,                       !- End Day of Month",
        "    ,                        !- End Year",
        "    Tuesday,                 !- Day of Week for Start Day",
        "    Yes,                     !- Use Weather File Holidays and Special Days",
        "    Yes,                     !- Use Weather File Daylight Saving Period",
        "    No,                      !- Apply Weekend Holiday Rule",
        "    Yes,                     !- Use Weather File Rain Indicators",
        "    Yes;                     !- Use Weather File Snow Indicators",

        "  Site:Location,",
        "    Miami Intl Ap FL USA WMO=722020,  !- Name",
        "    25.82,                   !- Latitude {deg}",
        "    -80.30,                  !- Longitude {deg}",
        "    -5.00,                   !- Time Zone {hr}",
        "    11.00;                   !- Elevation {m}",

        "  SizingPeriod:DesignDay,",
        "    Miami Intl Ap Ann Htg 99.6% Condns DB,  !- Name",
        "    1,                       !- Month",
        "    21,                      !- Day of Month",
        "    WinterDesignDay,         !- Day Type",
        "    8.7,                     !- Maximum Dry-Bulb Temperature {C}",
        "    0.0,                     !- Daily Dry-Bulb Temperature Range {deltaC}",
        "    ,                        !- Dry-Bulb Temperature Range Modifier Type",
        "    ,                        !- Dry-Bulb Temperature Range Modifier Day Schedule Name",
        "    Wetbulb,                 !- Humidity Condition Type",
        "    8.7,                     !- Wetbulb or DewPoint at Maximum Dry-Bulb {C}",
        "    ,                        !- Humidity Condition Day Schedule Name",
        "    ,                        !- Humidity Ratio at Maximum Dry-Bulb {kgWater/kgDryAir}",
        "    ,                        !- Enthalpy at Maximum Dry-Bulb {J/kg}",
        "    ,                        !- Daily Wet-Bulb Temperature Range {deltaC}",
        "    101217.,                 !- Barometric Pressure {Pa}",
        "    3.8,                     !- Wind Speed {m/s}",
        "    340,                     !- Wind Direction {deg}",
        "    No,                      !- Rain Indicator",
        "    No,                      !- Snow Indicator",
        "    No,                      !- Daylight Saving Time Indicator",
        "    ASHRAEClearSky,          !- Solar Model Indicator",
        "    ,                        !- Beam Solar Day Schedule Name",
        "    ,                        !- Diffuse Solar Day Schedule Name",
        "    ,                        !- ASHRAE Clear Sky Optical Depth for Beam Irradiance (taub) {dimensionless}",
        "    ,                        !- ASHRAE Clear Sky Optical Depth for Diffuse Irradiance (taud) {dimensionless}",
        "    0.00;                    !- Sky Clearness",

        "  SizingPeriod:DesignDay,",
        "    Miami Intl Ap Ann Clg .4% Condns DB=>MWB,  !- Name",
        "    7,                       !- Month",
        "    21,                      !- Day of Month",
        "    SummerDesignDay,         !- Day Type",
        "    33.2,                    !- Maximum Dry-Bulb Temperature {C}",
        "    6.7,                     !- Daily Dry-Bulb Temperature Range {deltaC}",
        "    ,                        !- Dry-Bulb Temperature Range Modifier Type",
        "    ,                        !- Dry-Bulb Temperature Range Modifier Day Schedule Name",
        "    Wetbulb,                 !- Humidity Condition Type",
        "    25.3,                    !- Wetbulb or DewPoint at Maximum Dry-Bulb {C}",
        "    ,                        !- Humidity Condition Day Schedule Name",
        "    ,                        !- Humidity Ratio at Maximum Dry-Bulb {kgWater/kgDryAir}",
        "    ,                        !- Enthalpy at Maximum Dry-Bulb {J/kg}",
        "    ,                        !- Daily Wet-Bulb Temperature Range {deltaC}",
        "    101217.,                 !- Barometric Pressure {Pa}",
        "    4.5,                     !- Wind Speed {m/s}",
        "    140,                     !- Wind Direction {deg}",
        "    No,                      !- Rain Indicator",
        "    No,                      !- Snow Indicator",
        "    No,                      !- Daylight Saving Time Indicator",
        "    ASHRAEClearSky,          !- Solar Model Indicator",
        "    ,                        !- Beam Solar Day Schedule Name",
        "    ,                        !- Diffuse Solar Day Schedule Name",
        "    ,                        !- ASHRAE Clear Sky Optical Depth for Beam Irradiance (taub) {dimensionless}",
        "    ,                        !- ASHRAE Clear Sky Optical Depth for Diffuse Irradiance (taud) {dimensionless}",
        "    1.00;                    !- Sky Clearness",

        "  Site:GroundTemperature:BuildingSurface,21.0,21.5,22.0,22.2,22.2,22.2,22.2,22.2,22.2,22.0,21.5,21.0;",

        "  Material:NoMass,",
        "    R13LAYER,                !- Name",
        "    Rough,                   !- Roughness",
        "    2.290965,                !- Thermal Resistance {m2-K/W}",
        "    0.9000000,               !- Thermal Absorptance",
        "    0.7500000,               !- Solar Absorptance",
        "    0.7500000;               !- Visible Absorptance",

        "  Material:NoMass,",
        "    R31LAYER,                !- Name",
        "    Rough,                   !- Roughness",
        "    5.456,                   !- Thermal Resistance {m2-K/W}",
        "    0.9000000,               !- Thermal Absorptance",
        "    0.7500000,               !- Solar Absorptance",
        "    0.7500000;               !- Visible Absorptance",

        "  Material,",
        "    C5 - 4 IN HW CONCRETE,   !- Name",
        "    MediumRough,             !- Roughness",
        "    0.1014984,               !- Thickness {m}",
        "    1.729577,                !- Conductivity {W/m-K}",
        "    2242.585,                !- Density {kg/m3}",
        "    836.8000,                !- Specific Heat {J/kg-K}",
        "    0.9000000,               !- Thermal Absorptance",
        "    0.6500000,               !- Solar Absorptance",
        "    0.6500000;               !- Visible Absorptance",

        "  Construction,",
        "    R13WALL,                 !- Name",
        "    R13LAYER;                !- Outside Layer",

        "  Construction,",
        "    FLOOR,                   !- Name",
        "    C5 - 4 IN HW CONCRETE;   !- Outside Layer",

        "  Construction,",
        "    ROOF31,                  !- Name",
        "    R31LAYER;                !- Outside Layer",

        "  Zone,",
        "    Main Zone,               !- Name",
        "    0,                       !- Direction of Relative North {deg}",
        "    0,                       !- X Origin {m}",
        "    0,                       !- Y Origin {m}",
        "    0,                       !- Z Origin {m}",
        "    1,                       !- Type",
        "    1,                       !- Multiplier",
        "    autocalculate,           !- Ceiling Height {m}",
        "    autocalculate;           !- Volume {m3}",

        "  GlobalGeometryRules,",
        "    UpperLeftCorner,         !- Starting Vertex Position",
        "    CounterClockWise,        !- Vertex Entry Direction",
        "    relative;                !- Coordinate System",

        "  BuildingSurface:Detailed,",
        "    Zn001:Wall001,           !- Name",
        "    Wall,                    !- Surface Type",
        "    R13WALL,                 !- Construction Name",
        "    Main Zone,               !- Zone Name",
        "    Outdoors,                !- Outside Boundary Condition",
        "    ,                        !- Outside Boundary Condition Object",
        "    SunExposed,              !- Sun Exposure",
        "    WindExposed,             !- Wind Exposure",
        "    0.5000000,               !- View Factor to Ground",
        "    4,                       !- Number of Vertices",
        "    0,0,4.572000,  !- X,Y,Z ==> Vertex 1 {m}",
        "    0,0,0,  !- X,Y,Z ==> Vertex 2 {m}",
        "    15.24000,0,0,  !- X,Y,Z ==> Vertex 3 {m}",
        "    15.24000,0,4.572000;  !- X,Y,Z ==> Vertex 4 {m}",

        "  BuildingSurface:Detailed,",
        "    Zn001:Wall002,           !- Name",
        "    Wall,                    !- Surface Type",
        "    R13WALL,                 !- Construction Name",
        "    Main Zone,               !- Zone Name",
        "    Outdoors,                !- Outside Boundary Condition",
        "    ,                        !- Outside Boundary Condition Object",
        "    SunExposed,              !- Sun Exposure",
        "    WindExposed,             !- Wind Exposure",
        "    0.5000000,               !- View Factor to Ground",
        "    4,                       !- Number of Vertices",
        "    15.24000,0,4.572000,  !- X,Y,Z ==> Vertex 1 {m}",
        "    15.24000,0,0,  !- X,Y,Z ==> Vertex 2 {m}",
        "    15.24000,15.24000,0,  !- X,Y,Z ==> Vertex 3 {m}",
        "    15.24000,15.24000,4.572000;  !- X,Y,Z ==> Vertex 4 {m}",

        "  BuildingSurface:Detailed,",
        "    Zn001:Wall003,           !- Name",
        "    Wall,                    !- Surface Type",
        "    R13WALL,                 !- Construction Name",
        "    Main Zone,               !- Zone Name",
        "    Outdoors,                !- Outside Boundary Condition",
        "    ,                        !- Outside Boundary Condition Object",
        "    SunExposed,              !- Sun Exposure",
        "    WindExposed,             !- Wind Exposure",
        "    0.5000000,               !- View Factor to Ground",
        "    4,                       !- Number of Vertices",
        "    15.24000,15.24000,4.572000,  !- X,Y,Z ==> Vertex 1 {m}",
        "    15.24000,15.24000,0,  !- X,Y,Z ==> Vertex 2 {m}",
        "    0,15.24000,0,  !- X,Y,Z ==> Vertex 3 {m}",
        "    0,15.24000,4.572000;  !- X,Y,Z ==> Vertex 4 {m}",

        "  BuildingSurface:Detailed,",
        "    Zn001:Wall004,           !- Name",
        "    Wall,                    !- Surface Type",
        "    R13WALL,                 !- Construction Name",
        "    Main Zone,               !- Zone Name",
        "    Outdoors,                !- Outside Boundary Condition",
        "    ,                        !- Outside Boundary Condition Object",
        "    SunExposed,              !- Sun Exposure",
        "    WindExposed,             !- Wind Exposure",
        "    0.5000000,               !- View Factor to Ground",
        "    4,                       !- Number of Vertices",
        "    0,15.24000,4.572000,  !- X,Y,Z ==> Vertex 1 {m}",
        "    0,15.24000,0,  !- X,Y,Z ==> Vertex 2 {m}",
        "    0,0,0,  !- X,Y,Z ==> Vertex 3 {m}",
        "    0,0,4.572000;  !- X,Y,Z ==> Vertex 4 {m}",

        "  BuildingSurface:Detailed,",
        "    Zn001:Flr001,            !- Name",
        "    Floor,                   !- Surface Type",
        "    FLOOR,                   !- Construction Name",
        "    Main Zone,               !- Zone Name",
        "    Surface,                 !- Outside Boundary Condition",
        "    Zn001:Flr001,            !- Outside Boundary Condition Object",
        "    NoSun,                   !- Sun Exposure",
        "    NoWind,                  !- Wind Exposure",
        "    1.000000,                !- View Factor to Ground",
        "    4,                       !- Number of Vertices",
        "    15.24000,0.000000,0.0,  !- X,Y,Z ==> Vertex 1 {m}",
        "    0.000000,0.000000,0.0,  !- X,Y,Z ==> Vertex 2 {m}",
        "    0.000000,15.24000,0.0,  !- X,Y,Z ==> Vertex 3 {m}",
        "    15.24000,15.24000,0.0;  !- X,Y,Z ==> Vertex 4 {m}",

        "  BuildingSurface:Detailed,",
        "    Zn001:Roof001,           !- Name",
        "    Roof,                    !- Surface Type",
        "    ROOF31,                  !- Construction Name",
        "    Main Zone,               !- Zone Name",
        "    Outdoors,                !- Outside Boundary Condition",
        "    ,                        !- Outside Boundary Condition Object",
        "    SunExposed,              !- Sun Exposure",
        "    WindExposed,             !- Wind Exposure",
        "    0,                       !- View Factor to Ground",
        "    4,                       !- Number of Vertices",
        "    0.000000,15.24000,4.572,  !- X,Y,Z ==> Vertex 1 {m}",
        "    0.000000,0.000000,4.572,  !- X,Y,Z ==> Vertex 2 {m}",
        "    15.24000,0.000000,4.572,  !- X,Y,Z ==> Vertex 3 {m}",
        "    15.24000,15.24000,4.572;  !- X,Y,Z ==> Vertex 4 {m}",

        "  ScheduleTypeLimits,",
        "    Any Number;              !- Name",

        "  ScheduleTypeLimits,",
        "    Fraction,                !- Name",
        "    0.0,                     !- Lower Limit Value",
        "    1.0,                     !- Upper Limit Value",
        "    CONTINUOUS;              !- Numeric Type",

        "  ScheduleTypeLimits,",
        "    Temperature,             !- Name",
        "    -60,                     !- Lower Limit Value",
        "    200,                     !- Upper Limit Value",
        "    CONTINUOUS,              !- Numeric Type",
        "    Temperature;             !- Unit Type",

        "  ScheduleTypeLimits,",
        "    Control Type,            !- Name",
        "    0,                       !- Lower Limit Value",
        "    4,                       !- Upper Limit Value",
        "    DISCRETE;                !- Numeric Type",

        "  ScheduleTypeLimits,",
        "    On/Off,                  !- Name",
        "    0,                       !- Lower Limit Value",
        "    1,                       !- Upper Limit Value",
        "    DISCRETE;                !- Numeric Type",

        "  ScheduleTypeLimits,",
        "    FlowRate,                !- Name",
        "    0.0,                     !- Lower Limit Value",
        "    10,                      !- Upper Limit Value",
        "    CONTINUOUS;              !- Numeric Type",

        "  ScheduleTypeLimits,",
        "    Humidity,                !- Name",
        "    0,                       !- Lower Limit Value",
        "    100,                     !- Upper Limit Value",
        "    CONTINUOUS;              !- Numeric Type",

        "  Schedule:Compact,",
        "    OCCUPY-1,                !- Name",
        "    Fraction,                !- Schedule Type Limits Name",
        "    Through: 12/31,          !- Field 1",
        "    For: WeekDays SummerDesignDay CustomDay1 CustomDay2, !- Field 2",
        "    Until: 8:00,0.0,         !- Field 3",
        "    Until: 11:00,1.00,       !- Field 5",
        "    Until: 12:00,0.80,       !- Field 7",
        "    Until: 13:00,0.40,       !- Field 9",
        "    Until: 14:00,0.80,       !- Field 11",
        "    Until: 18:00,1.00,       !- Field 13",
        "    Until: 19:00,0.50,       !- Field 15",
        "    Until: 21:00,0.10,       !- Field 17",
        "    Until: 24:00,0.0,        !- Field 19",
        "    For: Weekends WinterDesignDay Holiday, !- Field 21",
        "    Until: 24:00,0.0;        !- Field 22",

        "  Schedule:Compact,",
        "    LIGHTS-1,                !- Name",
        "    Fraction,                !- Schedule Type Limits Name",
        "    Through: 12/31,          !- Field 1",
        "    For: WeekDays SummerDesignDay CustomDay1 CustomDay2, !- Field 2",
        "    Until: 8:00,0.05,        !- Field 3",
        "    Until: 9:00,0.9,         !- Field 5",
        "    Until: 10:00,0.95,       !- Field 7",
        "    Until: 11:00,1.00,       !- Field 9",
        "    Until: 12:00,0.95,       !- Field 11",
        "    Until: 13:00,0.8,        !- Field 13",
        "    Until: 14:00,0.9,        !- Field 15",
        "    Until: 18:00,1.00,       !- Field 17",
        "    Until: 19:00,0.60,       !- Field 19",
        "    Until: 21:00,0.20,       !- Field 21",
        "    Until: 24:00,0.05,       !- Field 23",
        "    For: Weekends WinterDesignDay Holiday, !- Field 25",
        "    Until: 24:00,0.05;       !- Field 26",

        "  Schedule:Compact,",
        "    EQUIP-1,                 !- Name",
        "    Fraction,                !- Schedule Type Limits Name",
        "    Through: 12/31,          !- Field 1",
        "    For: WeekDays SummerDesignDay CustomDay1 CustomDay2, !- Field 2",
        "    Until: 8:00,0.02,        !- Field 3",
        "    Until: 9:00,0.4,         !- Field 5",
        "    Until: 14:00,0.9,        !- Field 7",
        "    Until: 15:00,0.8,        !- Field 9",
        "    Until: 16:00,0.7,        !- Field 11",
        "    Until: 18:00,0.5,        !- Field 13",
        "    Until: 20:00,0.3,        !- Field 15",
        "    Until: 24:00,0.02,       !- Field 17",
        "    For: Weekends WinterDesignDay Holiday, !- Field 19",
        "    Until: 24:00,0.2;        !- Field 20",

        "  Schedule:Compact,",
        "    INFIL-SCH,               !- Name",
        "    Fraction,                !- Schedule Type Limits Name",
        "    Through: 3/31,           !- Field 1",
        "    For: WeekDays SummerDesignDay CustomDay1 CustomDay2, !- Field 2",
        "    Until: 24:00,1.0,        !- Field 3",
        "    For: AllOtherDays,       !- Field 5",
        "    Until: 24:00,1.0,        !- Field 6",
        "    Through: 9/30,           !- Field 8",
        "    For: AllDays,            !- Field 9",
        "    Until: 24:00,0.7,        !- Field 10",
        "    Through: 12/31,          !- Field 12",
        "    For: AllDays,            !- Field 13",
        "    Until: 24:00,1.0,        !- Field 14",
        "    For: AllOtherDays,       !- Field 16",
        "    Until: 24:00,0.2;        !- Field 17",

        "  Schedule:Compact,",
        "    ActSchd,                 !- Name",
        "    Any Number,              !- Schedule Type Limits Name",
        "    Through: 12/31,          !- Field 1",
        "    For: AllDays,            !- Field 2",
        "    Until: 24:00,117.239997864; !- Field 3",

        "  Schedule:Compact,",
        "    ShadeTransSch,           !- Name",
        "    Fraction,                !- Schedule Type Limits Name",
        "    Through: 12/31,          !- Field 1",
        "    For: AllDays,            !- Field 2",
        "    Until: 24:00,0.0;        !- Field 3",

        "  Schedule:Compact,",
        "    Htg-SetP-Sch,            !- Name",
        "    Temperature,             !- Schedule Type Limits Name",
        "    Through: 12/31,          !- Field 1",
        "    For: SummerDesignDay,    !- Field 2",
        "    Until: 24:00,12.8,       !- Field 3",
        "    For: WinterDesignDay,    !- Field 5",
        "    Until: 24:00,21.1,       !- Field 6",
        "    For: WeekEnds Holiday,   !- Field 8",
        "    Until: 7:00,12.8,        !- Field 9",
        "    Until: 13:00,21.1,       !- Field 11",
        "    Until: 24:00,12.8,       !- Field 13",
        "    For: AllOtherDays,       !- Field 15",
        "    Until: 7:00,12.8,        !- Field 16",
        "    Until: 18:00,21.1,       !- Field 18",
        "    Until: 24:00,12.8;       !- Field 20",

        "  Schedule:Compact,",
        "    PlenumHtg-SetP-Sch,      !- Name",
        "    Temperature,             !- Schedule Type Limits Name",
        "    Through: 12/31,          !- Field 1",
        "    For: AllDays,            !- Field 2",
        "    Until: 24:00,12.8;       !- Field 3",

        "  Schedule:Compact,",
        "    Clg-SetP-Sch,            !- Name",
        "    Temperature,             !- Schedule Type Limits Name",
        "    Through: 12/31,          !- Field 1",
        "    For: SummerDesignDay,    !- Field 2",
        "    Until: 24:00,23.9,       !- Field 3",
        "    For: WinterDesignDay,    !- Field 5",
        "    Until: 24:00,40.0,       !- Field 6",
        "    For: WeekEnds Holiday,   !- Field 8",
        "    Until: 7:00,40.0,        !- Field 9",
        "    Until: 13:00,23.9,       !- Field 11",
        "    Until: 24:00,32.2,       !- Field 13",
        "    For: AllOtherDays,       !- Field 15",
        "    Until: 7:00,40.0,        !- Field 16",
        "    Until: 18:00,23.9,       !- Field 18",
        "    Until: 24:00,40.0;       !- Field 20",

        "  Schedule:Compact,",
        "    PlenumClg-SetP-Sch,      !- Name",
        "    Temperature,             !- Schedule Type Limits Name",
        "    Through: 12/31,          !- Field 1",
        "    For: AllDays,            !- Field 2",
        "    Until: 24:00,40.0;       !- Field 3",

        "  Schedule:Compact,",
        "    Zone Control Type Sched, !- Name",
        "    Control Type,            !- Schedule Type Limits Name",
        "    Through: 12/31,          !- Field 1",
        "    For: SummerDesignDay,    !- Field 2",
        "    Until: 24:00,2,          !- Field 3",
        "    For: WinterDesignDay,    !- Field 5",
        "    Until: 24:00,1,          !- Field 6",
        "    For: AllOtherDays,       !- Field 8",
        "    Until: 24:00,4;          !- Field 9",

        "  Schedule:Compact,",
        "    Desiccant Control Type Sched,  !- Name",
        "    Control Type,            !- Schedule Type Limits Name",
        "    Through: 12/31,          !- Field 1",
        "    For: AllDays,            !- Field 2",
        "    Until: 24:00,4;          !- Field 3",

        "  Schedule:Compact,",
        "    Min OA Sched,            !- Name",
        "    Fraction,                !- Schedule Type Limits Name",
        "    Through: 12/31,          !- Field 1",
        "    For: Weekdays,           !- Field 2",
        "    Until: 7:00,0.02,        !- Field 3",
        "    Until: 17:00,1.0,        !- Field 5",
        "    Until: 24:00,0.0,        !- Field 7",
        "    For: AllOtherDays,       !- Field 9",
        "    Until: 24:00,0.02;       !- Field 10",

        "  Schedule:Compact,",
        "    FanAvailSched,           !- Name",
        "    Fraction,                !- Schedule Type Limits Name",
        "    Through: 3/31,           !- Field 1",
        "    For: AllDays,            !- Field 2",
        "    Until: 24:00,1.0,        !- Field 3",
        "    Through: 9/30,           !- Field 5",
        "    For: WeekDays,           !- Field 6",
        "    Until: 7:00,0.0,         !- Field 7",
        "    Until: 17:00,1.0,        !- Field 9",
        "    Until: 24:00,0.0,        !- Field 11",
        "    For: SummerDesignDay WinterDesignDay, !- Field 13",
        "    Until: 24:00,1.0,        !- Field 14",
        "    For: AllOtherDays,       !- Field 16",
        "    Until: 24:00,0.0,        !- Field 17",
        "    Through: 12/31,          !- Field 19",
        "    For: AllDays,            !- Field 20",
        "    Until: 24:00,1.0;        !- Field 21",

        "  Schedule:Compact,",
        "    Seasonal Reset Supply Air Temp Sch,  !- Name",
        "    Temperature,             !- Schedule Type Limits Name",
        "    Through: 3/31,           !- Field 1",
        "    For: AllDays,            !- Field 2",
        "    Until: 24:00,17.0,       !- Field 3",
        "    Through: 9/30,           !- Field 5",
        "    For: AllDays,            !- Field 6",
        "    Until: 24:00,15.0,       !- Field 7",
        "    Through: 12/31,          !- Field 9",
        "    For: AllDays,            !- Field 10",
        "    Until: 24:00,17.0;       !- Field 11",

        "  Schedule:Compact,",
        "    Constant,                !- Name",
        "    Any Number,              !- Schedule Type Limits Name",
        "    Through: 12/31,          !- Field 1",
        "    For: AllDays,            !- Field 2",
        "    Until: 24:00,1.0;        !- Field 3",

        "  People,",
        "    Main Zone People 1,       !- Name",
        "    Main Zone,                !- Zone or ZoneList Name",
        "    OCCUPY-1,                !- Number of People Schedule Name",
        "    people,                  !- Number of People Calculation Method",
        "    20,                      !- Number of People",
        "    ,                        !- People per Zone Floor Area {person/m2}",
        "    ,                        !- Zone Floor Area per Person {m2/person}",
        "    0.3,                     !- Fraction Radiant",
        "    AutoCalculate,           !- Sensible Heat Fraction",
        "    ActSchd;                 !- Activity Level Schedule Name",

        "  Lights,",
        "    Main Zone Lights 1,       !- Name",
        "    Main Zone,                !- Zone or ZoneList Name",
        "    LIGHTS-1,                !- Schedule Name",
        "    LightingLevel,           !- Design Level Calculation Method",
        "    2964,                    !- Lighting Level {W}",
        "    ,                        !- Watts per Zone Floor Area {W/m2}",
        "    ,                        !- Watts per Person {W/person}",
        "    0.2,                     !- Return Air Fraction",
        "    0.59,                    !- Fraction Radiant",
        "    0.2,                     !- Fraction Visible",
        "    0,                       !- Fraction Replaceable",
        "    GeneralLights;           !- End-Use Subcategory",

        "  ElectricEquipment,",
        "    Main Zone ElecEq 1,       !- Name",
        "    Main Zone,                !- Zone or ZoneList Name",
        "    EQUIP-1,                 !- Schedule Name",
        "    EquipmentLevel,          !- Design Level Calculation Method",
        "    19760,                   !- Design Level {W}",
        "    ,                        !- Watts per Zone Floor Area {W/m2}",
        "    ,                        !- Watts per Person {W/person}",
        "    0.5,                     !- Fraction Latent",
        "    0.3,                     !- Fraction Radiant",
        "    0;                       !- Fraction Lost",

        "  ZoneInfiltration:DesignFlowRate,",
        "    Main Zone Infil 1,        !- Name",
        "    Main Zone,                !- Zone or ZoneList Name",
        "    INFIL-SCH,               !- Schedule Name",
        "    flow/zone,               !- Design Flow Rate Calculation Method",
        "    0.031089,                !- Design Flow Rate {m3/s}",
        "    ,                        !- Flow per Zone Floor Area {m3/s-m2}",
        "    ,                        !- Flow per Exterior Surface Area {m3/s-m2}",
        "    ,                        !- Air Changes per Hour {1/hr}",
        "    0,                       !- Constant Term Coefficient",
        "    0,                       !- Temperature Term Coefficient",
        "    0.2237,                  !- Velocity Term Coefficient",
        "    0;                       !- Velocity Squared Term Coefficient",

        "  Sizing:Zone,",
        "    Main Zone,                !- Zone or ZoneList Name",
        "    SupplyAirTemperature,    !- Zone Cooling Design Supply Air Temperature Input Method",
        "    14.,                     !- Zone Cooling Design Supply Air Temperature {C}",
        "    ,                        !- Zone Cooling Design Supply Air Temperature Difference {deltaC}",
        "    SupplyAirTemperature,    !- Zone Heating Design Supply Air Temperature Input Method",
        "    50.,                     !- Zone Heating Design Supply Air Temperature {C}",
        "    ,                        !- Zone Heating Design Supply Air Temperature Difference {deltaC}",
        "    0.009,                   !- Zone Cooling Design Supply Air Humidity Ratio {kgWater/kgDryAir}",
        "    0.004,                   !- Zone Heating Design Supply Air Humidity Ratio {kgWater/kgDryAir}",
        "    SZ DSOA Main Zone,        !- Design Specification Outdoor Air Object Name",
        "    0.0,                     !- Zone Heating Sizing Factor",
        "    0.0,                     !- Zone Cooling Sizing Factor",
        "    DesignDayWithLimit,      !- Cooling Design Air Flow Method",
        "    ,                        !- Cooling Design Air Flow Rate {m3/s}",
        "    ,                        !- Cooling Minimum Air Flow per Zone Floor Area {m3/s-m2}",
        "    ,                        !- Cooling Minimum Air Flow {m3/s}",
        "    ,                        !- Cooling Minimum Air Flow Fraction",
        "    DesignDay,               !- Heating Design Air Flow Method",
        "    ,                        !- Heating Design Air Flow Rate {m3/s}",
        "    ,                        !- Heating Maximum Air Flow per Zone Floor Area {m3/s-m2}",
        "    ,                        !- Heating Maximum Air Flow {m3/s}",
        "    ;                        !- Heating Maximum Air Flow Fraction",

        "  DesignSpecification:OutdoorAir,",
        "    SZ DSOA Main Zone,        !- Name",
        "    sum,                     !- Outdoor Air Method",
        "    0.00236,                 !- Outdoor Air Flow per Person {m3/s-person}",
        "    0.000305,                !- Outdoor Air Flow per Zone Floor Area {m3/s-m2}",
        "    0.0;                     !- Outdoor Air Flow per Zone {m3/s}",

        "  Sizing:System,",
        "    Desiccant DXSystem,      !- AirLoop Name",
        "    sensible,                !- Type of Load to Size On",
        "    autosize,                !- Design Outdoor Air Flow Rate {m3/s}",
        "    0.3,                     !- Central Heating Maximum System Air Flow Ratio",
        "    4.5,                     !- Preheat Design Temperature {C}",
        "    0.008,                   !- Preheat Design Humidity Ratio {kgWater/kgDryAir}",
        "    11.0,                    !- Precool Design Temperature {C}",
        "    0.008,                   !- Precool Design Humidity Ratio {kgWater/kgDryAir}",
        "    12.8,                    !- Central Cooling Design Supply Air Temperature {C}",
        "    16.7,                    !- Central Heating Design Supply Air Temperature {C}",
        "    noncoincident,           !- Type of Zone Sum to Use",
        "    no,                      !- 100% Outdoor Air in Cooling",
        "    no,                      !- 100% Outdoor Air in Heating",
        "    0.008,                   !- Central Cooling Design Supply Air Humidity Ratio {kgWater/kgDryAir}",
        "    0.008,                   !- Central Heating Design Supply Air Humidity Ratio {kgWater/kgDryAir}",
        "    DesignDay,               !- Cooling Design Air Flow Method",
        "    0,                       !- Cooling Design Air Flow Rate {m3/s}",
        "    ,                        !- Supply Air Flow Rate Per Floor Area During Cooling Operation {m3/s-m2}",
        "    ,                        !- Fraction of Autosized Design Cooling Supply Air Flow Rate",
        "    ,                        !- Design Supply Air Flow Rate Per Unit Cooling Capacity {m3/s-W}",
        "    DesignDay,               !- Heating Design Air Flow Method",
        "    0,                       !- Heating Design Air Flow Rate {m3/s}",
        "    ,                        !- Supply Air Flow Rate Per Floor Area During Heating Operation {m3/s-m2}",
        "    ,                        !- Fraction of Autosized Design Heating Supply Air Flow Rate",
        "    ,                        !- Fraction of Autosized Design Cooling Supply Air Flow Rate",
        "    ,                        !- Design Supply Air Flow Rate Per Unit Heating Capacity {m3/s-W}",
        "    ,                        !- System Outdoor Air Method",
        "    1.0,                     !- Zone Maximum Outdoor Air Fraction {dimensionless}",
        "    CoolingDesignCapacity,   !- Cooling Design Capacity Method",
        "    autosize,                !- Cooling Design Capacity {W}",
        "    ,                        !- Cooling Design Capacity Per Floor Area {W/m2}",
        "    ,                        !- Fraction of Autosized Cooling Design Capacity",
        "    HeatingDesignCapacity,   !- Heating Design Capacity Method",
        "    autosize,                !- Heating Design Capacity {W}",
        "    ,                        !- Heating Design Capacity Per Floor Area {W/m2}",
        "    ,                        !- Fraction of Autosized Heating Design Capacity",
        "    VAV;                     !- Central Cooling Capacity Control Method",

        "  Curve:Biquadratic,",
        "    HPACCoolCapFT,           !- Name",
        "    0.942587793,             !- Coefficient1 Constant",
        "    0.009543347,             !- Coefficient2 x",
        "    0.00068377,              !- Coefficient3 x**2",
        "    -0.011042676,            !- Coefficient4 y",
        "    0.000005249,             !- Coefficient5 y**2",
        "    -0.00000972,             !- Coefficient6 x*y",
        "    -100,                    !- Minimum Value of x",
        "    100,                     !- Maximum Value of x",
        "    -100,                    !- Minimum Value of y",
        "    100,                     !- Maximum Value of y",
        "    ,                        !- Minimum Curve Output",
        "    ,                        !- Maximum Curve Output",
        "    Temperature,             !- Input Unit Type for X",
        "    Temperature,             !- Input Unit Type for Y",
        "    Dimensionless;           !- Output Unit Type",

        "  Curve:Biquadratic,",
        "    HPACCOOLEIRFT,           !- Name",
        "    0.342414409,             !- Coefficient1 Constant",
        "    0.034885008,             !- Coefficient2 x",
        "    -0.0006237,              !- Coefficient3 x**2",
        "    0.004977216,             !- Coefficient4 y",
        "    0.000437951,             !- Coefficient5 y**2",
        "    -0.000728028,            !- Coefficient6 x*y",
        "    -100,                    !- Minimum Value of x",
        "    100,                     !- Maximum Value of x",
        "    -100,                    !- Minimum Value of y",
        "    100,                     !- Maximum Value of y",
        "    ,                        !- Minimum Curve Output",
        "    ,                        !- Maximum Curve Output",
        "    Temperature,             !- Input Unit Type for X",
        "    Temperature,             !- Input Unit Type for Y",
        "    Dimensionless;           !- Output Unit Type",

        "  Curve:Quadratic,",
        "    HPACCoolCapFFF,          !- Name",
        "    0.8,                     !- Coefficient1 Constant",
        "    0.2,                     !- Coefficient2 x",
        "    0.0,                     !- Coefficient3 x**2",
        "    0.0,                     !- Minimum Value of x",
        "    1.0;                     !- Maximum Value of x",

        "  Curve:Quadratic,",
        "    HPACCOOLEIRFFF,          !- Name",
        "    1.1552,                  !- Coefficient1 Constant",
        "    -0.1808,                 !- Coefficient2 x",
        "    0.0256,                  !- Coefficient3 x**2",
        "    0.0,                     !- Minimum Value of x",
        "    1.0;                     !- Maximum Value of x",

        "  Curve:Quadratic,",
        "    HPACCOOLPLFFPLR,         !- Name",
        "    0.75,                    !- Coefficient1 Constant",
        "    0.25,                    !- Coefficient2 x",
        "    0.0,                     !- Coefficient3 x**2",
        "    0.0,                     !- Minimum Value of x",
        "    1.0;                     !- Maximum Value of x",

        "  Curve:Cubic,",
        "    PackagedRatedCoolCapFFlow,  !- Name",
        "    0.47278589,              !- Coefficient1 Constant",
        "    1.2433415,               !- Coefficient2 x",
        "    -1.0387055,              !- Coefficient3 x**2",
        "    0.32257813,              !- Coefficient4 x**3",
        "    0.5,                     !- Minimum Value of x",
        "    1.5,                     !- Maximum Value of x",
        "    ,                        !- Minimum Curve Output",
        "    ,                        !- Maximum Curve Output",
        "    Dimensionless,           !- Input Unit Type for X",
        "    Dimensionless;           !- Output Unit Type",

        "  Curve:Cubic,",
        "    PackagedRatedCoolEIRFFlow,  !- Name",
        "    1.0079484,               !- Coefficient1 Constant",
        "    0.34544129,              !- Coefficient2 x",
        "    -.6922891,               !- Coefficient3 x**2",
        "    0.33889943,              !- Coefficient4 x**3",
        "    0.5,                     !- Minimum Value of x",
        "    1.5,                     !- Maximum Value of x",
        "    ,                        !- Minimum Curve Output",
        "    ,                        !- Maximum Curve Output",
        "    Dimensionless,           !- Input Unit Type for X",
        "    Dimensionless;           !- Output Unit Type",

        "  Curve:Cubic,",
        "    EXHAUSTFANPLF,           !- Name",
        "    0,                       !- Coefficient1 Constant",
        "    1,                       !- Coefficient2 x",
        "    0.0,                     !- Coefficient3 x**2",
        "    0.0,                     !- Coefficient4 x**3",
        "    0.0,                     !- Minimum Value of x",
        "    1.0;                     !- Maximum Value of x",

        "  NodeList,",
        "    Main Zone In Nodes,       !- Name",
        "    Main Zone In Node;        !- Node 1 Name",

        "  NodeList,",
        "    Desiccant DX Air Loop Setpoint Nodes,  !- Name",
        "    HX Process Outlet Node,  !- Node 1 Name",
        "    Desiccant DXSystem Mixed Air Node;  !- Node 2 Name",

        "  NodeList,",
        "    Regen Setpoint Nodes,    !- Name",
        "    HX Regen Inlet Node;     !- Node 1 Name",

        "  BranchList,",
        "    Desiccant Air Loop Branches,  !- Name",
        "    Desiccant Air Loop Main Branch;  !- Branch 1 Name",

        "  Branch,",
        "    Desiccant Air Loop Main Branch,  !- Name",
        "    ,                        !- Pressure Drop Curve Name",
        "    AirLoopHVAC:OutdoorAirSystem,  !- Component 1 Object Type",
        "    Desiccant DXSystem OA System,  !- Component 1 Name",
        "    Desiccant DXSystem Air Loop Inlet Node,  !- Component 1 Inlet Node Name",
        "    Desiccant DXSystem Mixed Air Node,  !- Component 1 Outlet Node Name",
        "    CoilSystem:Cooling:DX,   !- Component 2 Object Type",
        "    Desiccant DXSystem Cooling Coil Sys,  !- Component 2 Name",
        "    Desiccant DXSystem Mixed Air Node,  !- Component 2 Inlet Node Name",
        "    HX Process Inlet Node,   !- Component 2 Outlet Node Name",
        "    Dehumidifier:Desiccant:System,  !- Component 3 Object Type",
        "    Desiccant 1,             !- Component 3 Name",
        "    HX Process Inlet Node,   !- Component 3 Inlet Node Name",
        "    HX Process Outlet Node,  !- Component 3 Outlet Node Name",
        "    Fan:ConstantVolume,      !- Component 4 Object Type",
        "    Desiccant DXSystem Supply Fan,  !- Component 4 Name",
        "    HX Process Outlet Node,  !- Component 4 Inlet Node Name",
        "    Desiccant Supply Fan Outlet Node,  !- Component 4 Outlet Node Name",
        "    Coil:Heating:Fuel,        !- Component 5 Object Type",
        "    Desiccant Process Heating Coil,  !- Component 5 Name",
        "    Desiccant Supply Fan Outlet Node,  !- Component 5 Inlet Node Name",
        "    Desiccant DXSystem Air Loop Outlet Node;  !- Component 5 Outlet Node Name",

        "  AirLoopHVAC,",
        "    Desiccant DXSystem,      !- Name",
        "    ,                        !- Controller List Name",
        "    Desiccant Avail List,    !- Availability Manager List Name",
        "    autosize,                !- Design Supply Air Flow Rate {m3/s}",
        "    Desiccant Air Loop Branches,  !- Branch List Name",
        "    ,                        !- Connector List Name",
        "    Desiccant DXSystem Air Loop Inlet Node,  !- Supply Side Inlet Node Name",
        "    Desiccant DXSystem Return Air Mixer Outlet Node,  !- Demand Side Outlet Node Name",
        "    Desiccant DXSystem Zone Equipment Inlet Node,  !- Demand Side Inlet Node Names",
        "    Desiccant DXSystem Air Loop Outlet Node;  !- Supply Side Outlet Node Names",

        "  AirLoopHVAC:ControllerList,",
        "    Desiccant DXSystem OA System Controllers,  !- Name",
        "    Controller:OutdoorAir,   !- Controller 1 Object Type",
        "    Desiccant OA Controller; !- Controller 1 Name",

        "  AirLoopHVAC:OutdoorAirSystem:EquipmentList,",
        "    Desiccant DXSystem OA System Equipment,  !- Name",
        "    OutdoorAir:Mixer,        !- Component 1 Object Type",
        "    Desiccant DXSystem OA Mixing Box;  !- Component 1 Name",

        "  AirLoopHVAC:OutdoorAirSystem,",
        "    Desiccant DXSystem OA System,  !- Name",
        "    Desiccant DXSystem OA System Controllers,  !- Controller List Name",
        "    Desiccant DXSystem OA System Equipment,  !- Outdoor Air Equipment List Name",
        "    Desiccant Outdoor Air Avail List;  !- Availability Manager List Name",

        "  OutdoorAir:NodeList,",
        "    OutsideAirInletNodes;    !- Node or NodeList Name 1",

        "  OutdoorAir:NodeList,",
        "    Desiccant DXSystem Outside Air Inlet Node,  !- Node or NodeList Name 1",
        "    Desiccant Outside Air Inlet Node 2;  !- Node or NodeList Name 2",

        "  OutdoorAir:NodeList,",
        "    Regen Coil Inlet Node;   !- Node or NodeList Name 1",

        "  OutdoorAir:NodeList,",
        "    Condenser Inlet Node;    !- Node or NodeList Name 1",

        "  OutdoorAir:Mixer,",
        "    Desiccant DXSystem OA Mixing Box,  !- Name",
        "    Desiccant DXSystem Mixed Air Node,  !- Mixed Air Node Name",
        "    Desiccant DXSystem Outside Air Inlet Node,  !- Outdoor Air Stream Node Name",
        "    Desiccant DXSystem Relief Air Outlet Node,  !- Relief Air Stream Node Name",
        "    Desiccant DXSystem Air Loop Inlet Node;  !- Return Air Stream Node Name",

        "  AvailabilityManagerAssignmentList,",
        "    Desiccant Avail List,    !- Name",
        "    AvailabilityManager:Scheduled,  !- Availability Manager 1 Object Type",
        "    Desiccant DXSystem Avail;!- Availability Manager 1 Name",

        "  AvailabilityManagerAssignmentList,",
        "    Desiccant Outdoor Air Avail List,  !- Name",
        "    AvailabilityManager:Scheduled,  !- Availability Manager 1 Object Type",
        "    Desiccant Outdoor Air Avail;  !- Availability Manager 1 Name",

        "  AvailabilityManager:Scheduled,",
        "    Desiccant DXSystem Avail,!- Name",
        "    FanAvailSched;           !- Schedule Name",

        "  AvailabilityManager:Scheduled,",
        "    Desiccant Outdoor Air Avail,  !- Name",
        "    FanAvailSched;           !- Schedule Name",

        "  SetpointManager:SingleZone:Heating,",
        "    Heating Supply Air Temp Manager,  !- Name",
        "    Temperature,             !- Control Variable",
        "    -99.,                    !- Minimum Supply Air Temperature {C}",
        "    99.,                     !- Maximum Supply Air Temperature {C}",
        "    Main Zone,                !- Control Zone Name",
        "    Main Zone Node,           !- Zone Node Name",
        "    Main Zone In Node,        !- Zone Inlet Node Name",
        "    Desiccant DXSystem Air Loop Outlet Node;  !- Setpoint Node or NodeList Name",

        "  SetpointManager:SingleZone:Cooling,",
        "    Cooling Supply Air Temp Manager,  !- Name",
        "    Temperature,             !- Control Variable",
        "    -99.,                    !- Minimum Supply Air Temperature {C}",
        "    99.,                     !- Maximum Supply Air Temperature {C}",
        "    Main Zone,                !- Control Zone Name",
        "    Main Zone Node,           !- Zone Node Name",
        "    Main Zone In Node,        !- Zone Inlet Node Name",
        "    Desiccant Supply Fan Outlet Node;  !- Setpoint Node or NodeList Name",

        "  SetpointManager:SingleZone:Humidity:Maximum,",
        "    Zone Max Humidity Manager,  !- Name",
        "    HX Process Outlet Node,  !- Setpoint Node or NodeList Name",
        "    Main Zone Node;           !- Control Zone Air Node Name",

        "  SetpointManager:MixedAir,",
        "    Adjusted Cooling Supply Air Temp Manager,  !- Name",
        "    Temperature,             !- Control Variable",
        "    Desiccant Supply Fan Outlet Node,  !- Reference Setpoint Node Name",
        "    HX Process Outlet Node,  !- Fan Inlet Node Name",
        "    Desiccant Supply Fan Outlet Node,  !- Fan Outlet Node Name",
        "    HX Process Outlet Node;  !- Setpoint Node or NodeList Name",

        "  Controller:OutdoorAir,",
        "    Desiccant OA Controller, !- Name",
        "    Desiccant DXSystem Relief Air Outlet Node,  !- Relief Air Outlet Node Name",
        "    Desiccant DXSystem Air Loop Inlet Node,  !- Return Air Node Name",
        "    Desiccant DXSystem Mixed Air Node,  !- Mixed Air Node Name",
        "    Desiccant DXSystem Outside Air Inlet Node,  !- Actuator Node Name",
        "    autosize,                !- Minimum Outdoor Air Flow Rate {m3/s}",
        "    autosize,                !- Maximum Outdoor Air Flow Rate {m3/s}",
        "    NoEconomizer,            !- Economizer Control Type",
        "    ModulateFlow,            !- Economizer Control Action Type",
        "    100.,                    !- Economizer Maximum Limit Dry-Bulb Temperature {C}",
        "    ,                        !- Economizer Maximum Limit Enthalpy {J/kg}",
        "    ,                        !- Economizer Maximum Limit Dewpoint Temperature {C}",
        "    ,                        !- Electronic Enthalpy Limit Curve Name",
        "    ,                        !- Economizer Minimum Limit Dry-Bulb Temperature {C}",
        "    NoLockout,               !- Lockout Type",
        "    FixedMinimum,            !- Minimum Limit Type",
        "    Min OA Sched,            !- Minimum Outdoor Air Schedule Name",
        "    ,                        !- Minimum Fraction of Outdoor Air Schedule Name",
        "    ,                        !- Maximum Fraction of Outdoor Air Schedule Name",
        "    0;                       !- Mechanical Ventilation Controller Name",

        "  ZoneHVAC:EquipmentConnections,",
        "    Main Zone,                !- Zone Name",
        "    Main Zone Eq,             !- Zone Conditioning Equipment List Name",
        "    Main Zone In Nodes,       !- Zone Air Inlet Node or NodeList Name",
        "    ,                         !- Zone Air Exhaust Node or NodeList Name",
        "    Main Zone Node,           !- Zone Air Node Name",
        "    Main Zone Out Node;       !- Zone Return Air Node Name",

        "  ZoneHVAC:EquipmentList,",
        "    Main Zone Eq,             !- Name",
        "    SequentialLoad,          !- Load Distribution Scheme",
        "    ZoneHVAC:AirDistributionUnit,  !- Zone Equipment 1 Object Type",
        "    Main Zone Direct Air ADU,     !- Zone Equipment 1 Name",
        "    1,                        !- Zone Equipment 1 Cooling Sequence",
        "    1;                        !- Zone Equipment 1 Heating or No-Load Sequence",

        "  ZoneHVAC:AirDistributionUnit,",
        "    Main Zone Direct Air ADU, !- Name",
        "    Main Zone In Node,       !- Air Distribution Unit Outlet Node Name",
        "    AirTerminal:SingleDuct:ConstantVolume:NoReheat,  !- Air Terminal Object Type",
        "    Main Zone Direct Air;    !- Air Terminal Name",

        "  AirTerminal:SingleDuct:ConstantVolume:NoReheat,",
        "    Main Zone Direct Air,    !- Name",
        "    Constant,                !- Availability Schedule Name",
        "    Main Zone In Node 2AT,   !- Air Inlet Node Name",
        "    Main Zone In Node,       !- Air Outlet Node Name",
        "    autosize;                !- Maximum Air Flow Rate {m3/s}",

        "  CoilSystem:Cooling:DX,",
        "    Desiccant DXSystem Cooling Coil Sys,  !- Name",
        "    FanAvailSched,           !- Availability Schedule Name",
        "    Desiccant DXSystem Mixed Air Node,  !- DX Cooling Coil System Inlet Node Name",
        "    HX Process Inlet Node,   !- DX Cooling Coil System Outlet Node Name",
        "    HX Process Outlet Node,  !- DX Cooling Coil System Sensor Node Name",
        "    Coil:Cooling:DX:SingleSpeed,  !- Cooling Coil Object Type",
        "    Desiccant DXSystem Cooling Coil;  !- Cooling Coil Name",

        "  ZoneControl:Thermostat,",
        "    Main Zone Control,        !- Name",
        "    Main Zone,                !- Zone or ZoneList Name",
        "    Desiccant Control Type Sched,  !- Control Type Schedule Name",
        "    ThermostatSetpoint:DualSetpoint,  !- Control 1 Object Type",
        "    DualSetPoint;            !- Control 1 Name",

        "  ZoneControl:Humidistat,",
        "    Main Zone Humidistat,     !- Name",
        "    Main Zone,                !- Zone Name",
        "    Relative Humidity Sched; !- Humidifying Relative Humidity Setpoint Schedule Name",

        "  Schedule:Compact,",
        "    Relative Humidity Sched, !- Name",
        "    Any Number,              !- Schedule Type Limits Name",
        "    Through: 12/31,          !- Field 1",
        "    For: AllDays,            !- Field 2",
        "    Until: 24:00,45.0;       !- Field 3",

        "  ThermostatSetpoint:SingleHeating,",
        "    HeatingSetpoint,         !- Name",
        "    Htg-SetP-Sch;            !- Setpoint Temperature Schedule Name",

        "  ThermostatSetpoint:SingleHeating,",
        "    PlenumHeatingSetpoint,   !- Name",
        "    PlenumHtg-SetP-Sch;      !- Setpoint Temperature Schedule Name",

        "  ThermostatSetpoint:SingleCooling,",
        "    CoolingSetpoint,         !- Name",
        "    Clg-SetP-Sch;            !- Setpoint Temperature Schedule Name",

        "  ThermostatSetpoint:SingleCooling,",
        "    PlenumCoolingSetpoint,   !- Name",
        "    PlenumClg-SetP-Sch;      !- Setpoint Temperature Schedule Name",

        "  ThermostatSetpoint:DualSetpoint,",
        "    DualSetPoint,            !- Name",
        "    Htg-SetP-Sch,            !- Heating Setpoint Temperature Schedule Name",
        "    Clg-SetP-Sch;            !- Cooling Setpoint Temperature Schedule Name",

        "  AirLoopHVAC:SupplyPath,",
        "    Desiccant DXSystem Supply Path,  !- Name",
        "    Desiccant DXSystem Zone Equipment Inlet Node,  !- Supply Air Path Inlet Node Name",
        "    AirLoopHVAC:ZoneSplitter,!- Component 1 Object Type",
        "    Desiccant DXSystem Zone Supply Air Splitter;  !- Component 1 Name",

        "  AirLoopHVAC:ReturnPath,",
        "    Desiccant DXSystem Return Path,  !- Name",
        "    Desiccant DXSystem Return Air Mixer Outlet Node,  !- Return Air Path Outlet Node Name",
        "    AirLoopHVAC:ZoneMixer,   !- Component 1 Object Type",
        "    Desiccant DXSystem Zone Return Air Mixer;  !- Component 1 Name",

        "  AirLoopHVAC:ZoneSplitter,",
        "    Desiccant DXSystem Zone Supply Air Splitter,  !- Name",
        "    Desiccant DXSystem Zone Equipment Inlet Node,  !- Inlet Node Name",
        "    Main Zone In Node 2AT;      !- Outlet 1 Node Name",

        "  AirLoopHVAC:ZoneMixer,",
        "    Desiccant DXSystem Zone Return Air Mixer,  !- Name",
        "    Desiccant DXSystem Return Air Mixer Outlet Node,  !- Outlet Node Name",
        "    Main Zone Out Node;       !- Inlet 1 Node Name",

        "  Coil:Heating:Fuel,",
        "    Desiccant Process Heating Coil,  !- Name",
        "    Constant,                !- Availability Schedule Name",
        "    Gas,                     !- Fuel Type",
        "    0.8,                     !- Gas Burner Efficiency",
        "    autosize,                !- Nominal Capacity {W}",
        "    Desiccant Supply Fan Outlet Node,  !- Air Inlet Node Name",
        "    Desiccant DXSystem Air Loop Outlet Node,  !- Air Outlet Node Name",
        "    Desiccant DXSystem Air Loop Outlet Node;  !- Temperature Setpoint Node Name",

        "  Coil:Heating:Fuel,",
        "    Desiccant Regen Coil,    !- Name",
        "    FanAvailSched,           !- Availability Schedule Name",
        "    Gas,                     !- Fuel Type",
        "    1,                       !- Gas Burner Efficiency",
        "    autosize,                !- Nominal Capacity {W}",
        "    Regen Coil Inlet Node,   !- Air Inlet Node Name",
        "    HX Regen Inlet Node;     !- Air Outlet Node Name",

        "  Coil:Cooling:DX:SingleSpeed,",
        "    Desiccant DXSystem Cooling Coil,  !- Name",
        "    FanAvailSched,           !- Availability Schedule Name",
        "    autosize,                !- High Speed Rated Air Flow Rate {m3/s}",
        "    autosize,                !- Gross Rated Sensible Heat Ratio",
        "    3.5,                     !- Gross Rated Cooling COP {W/W}",
        "    autosize,                !- High Speed Rated Air Flow Rate {m3/s}",
        "    ,                        !- Rated Evaporator Fan Power Per Volume Flow Rate {W/(m3/s)}",
        "    Desiccant DXSystem Mixed Air Node,  !- Air Inlet Node Name",
        "    HX Process Inlet Node,   !- Air Outlet Node Name",
        "    HPACCoolCapFT,           !- Total Cooling Capacity Function of Temperature Curve Name",
        "    HPACCoolCapFFF,          !- Total Cooling Capacity Function of Flow Fraction Curve Name",
        "    HPACCOOLEIRFT,           !- Energy Input Ratio Function of Temperature Curve Name",
        "    HPACCOOLEIRFFF,          !- Energy Input Ratio Function of Flow Fraction Curve Name",
        "    HPACCOOLPLFFPLR,         !- Part Load Fraction Correlation Curve Name",
        "   ,                         !- Minimum Outdoor Dry-Bulb Temperature for Compressor Operation {C}",
        "    1000,                    !- Nominal Time for Condensate Removal to Begin {s}",
        "    0.4,                     !- Ratio of Initial Moisture Evaporation Rate and Steady State Latent Capacity {dimensionless}",
        "    4,                       !- Maximum Cycling Rate {cycles/hr}",
        "    45,                      !- Latent Capacity Time Constant {s}",
        "    Condenser Inlet Node,    !- Condenser Air Inlet Node Name",
        "    AirCooled;               !- Condenser Type",

        "  Fan:ConstantVolume,",
        "    Desiccant DXSystem Supply Fan,  !- Name",
        "    FanAvailSched,           !- Availability Schedule Name",
        "    0.7,                     !- Fan Total Efficiency",
        "    571.66,                  !- Pressure Rise {Pa}",
        "    autosize,                !- Maximum Flow Rate {m3/s}",
        "    0.9,                     !- Motor Efficiency",
        "    1.0,                     !- Motor In Airstream Fraction",
        "    HX Process Outlet Node,  !- Air Inlet Node Name",
        "    Desiccant Supply Fan Outlet Node;  !- Air Outlet Node Name",

        "  Fan:ConstantVolume,",
        "    Desiccant Regen Fan,     !- Name",
        "    FanAvailSched,           !- Availability Schedule Name",
        "    0.7,                     !- Fan Total Efficiency",
        "    205.5,                   !- Pressure Rise {Pa}",
        "    autosize,                !- Maximum Flow Rate {m3/s}",
        "    0.9,                     !- Motor Efficiency",
        "    1.0,                     !- Motor In Airstream Fraction",
        "    HX Regen Outlet Node,    !- Air Inlet Node Name",
        "    Regen Fan Outlet Node;   !- Air Outlet Node Name",

        "  Dehumidifier:Desiccant:System,",
        "    Desiccant 1,             !- Name",
        "    FanAvailSched,           !- Availability Schedule Name",
        "    HeatExchanger:Desiccant:BalancedFlow,  !- Desiccant Heat Exchanger Object Type",
        "    Desiccant Heat Exchanger 1,  !- Desiccant Heat Exchanger Name",
        "    HX Process Outlet Node,  !- Sensor Node Name",
        "    Fan:ConstantVolume,      !- Regeneration Air Fan Object Type",
        "    Desiccant Regen Fan,     !- Regeneration Air Fan Name",
        "    DrawThrough,             !- Regeneration Air Fan Placement",
        "    Coil:Heating:Fuel,        !- Regeneration Air Heater Object Type",
        "    Desiccant Regen Coil,    !- Regeneration Air Heater Name",
        "    46.111111,               !- Regeneration Inlet Air Setpoint Temperature {C}",
        "    Coil:Cooling:DX:SingleSpeed,  !- Companion Cooling Coil Object Type",
        "    Desiccant DXSystem Cooling Coil,  !- Companion Cooling Coil Name",
        "    Yes,                     !- Companion Cooling Coil Upstream of Dehumidifier Process Inlet",
        "    Yes,                     !- Companion Coil Regeneration Air Heating",
        "    0.87931,                 !- Exhaust Fan Maximum Flow Rate {m3/s}",
        "    50,                      !- Exhaust Fan Maximum Power {W}",
        "    EXHAUSTFANPLF;           !- Exhaust Fan Power Curve Name",

        "  HeatExchanger:Desiccant:BalancedFlow,",
        "    Desiccant Heat Exchanger 1,  !- Name",
        "    FanAvailSched,           !- Availability Schedule Name",
        "    HX Regen Inlet Node,     !- Regeneration Air Inlet Node Name",
        "    HX Regen Outlet Node,    !- Regeneration Air Outlet Node Name",
        "    HX Process Inlet Node,   !- Process Air Inlet Node Name",
        "    HX Process Outlet Node,  !- Process Air Outlet Node Name",
        "    HeatExchanger:Desiccant:BalancedFlow:PerformanceDataType1,  !- Heat Exchanger Performance Object Type",
        "    HXDesPerf1;              !- Heat Exchanger Performance Name",

        "  HeatExchanger:Desiccant:BalancedFlow:PerformanceDataType1,",
        "    HXDesPerf1,              !- Name",
        "    0.87931,                 !- Nominal Air Flow Rate {m3/s}",
        "    3.25,                    !- Nominal Air Face Velocity {m/s}",
        "    50.0,                    !- Nominal Electric Power {W}",
        "    -2.53636E+00,            !- Temperature Equation Coefficient 1",
        "    2.13247E+01,             !- Temperature Equation Coefficient 2",
        "    9.23308E-01,             !- Temperature Equation Coefficient 3",
        "    9.43276E+02,             !- Temperature Equation Coefficient 4",
        "    -5.92367E+01,            !- Temperature Equation Coefficient 5",
        "    -4.27465E-02,            !- Temperature Equation Coefficient 6",
        "    1.12204E+02,             !- Temperature Equation Coefficient 7",
        "    7.78252E-01,             !- Temperature Equation Coefficient 8",
        "    0.007143,                !- Minimum Regeneration Inlet Air Humidity Ratio for Temperature Equation {kgWater/kgDryAir}",
        "    0.024286,                !- Maximum Regeneration Inlet Air Humidity Ratio for Temperature Equation {kgWater/kgDryAir}",
        "    46.111110,               !- Minimum Regeneration Inlet Air Temperature for Temperature Equation {C}",
        "    46.111112,               !- Maximum Regeneration Inlet Air Temperature for Temperature Equation {C}",
        "    0.005000,                !- Minimum Process Inlet Air Humidity Ratio for Temperature Equation {kgWater/kgDryAir}",
        "    0.015714,                !- Maximum Process Inlet Air Humidity Ratio for Temperature Equation {kgWater/kgDryAir}",
        "    4.583333,                !- Minimum Process Inlet Air Temperature for Temperature Equation {C}",
        "    21.83333,                !- Maximum Process Inlet Air Temperature for Temperature Equation {C}",
        "    2.286,                   !- Minimum Regeneration Air Velocity for Temperature Equation {m/s}",
        "    4.826,                   !- Maximum Regeneration Air Velocity for Temperature Equation {m/s}",
        "    35.0,                    !- Minimum Regeneration Outlet Air Temperature for Temperature Equation {C}",
        "    50.0,                    !- Maximum Regeneration Outlet Air Temperature for Temperature Equation {C}",
        "    5.0,                     !- Minimum Regeneration Inlet Air Relative Humidity for Temperature Equation {percent}",
        "    45.0,                    !- Maximum Regeneration Inlet Air Relative Humidity for Temperature Equation {percent}",
        "    80.0,                    !- Minimum Process Inlet Air Relative Humidity for Temperature Equation {percent}",
        "    100.0,                   !- Maximum Process Inlet Air Relative Humidity for Temperature Equation {percent}",
        "    -2.25547E+01,            !- Humidity Ratio Equation Coefficient 1",
        "    9.76839E-01,             !- Humidity Ratio Equation Coefficient 2",
        "    4.89176E-01,             !- Humidity Ratio Equation Coefficient 3",
        "    -6.30019E-02,            !- Humidity Ratio Equation Coefficient 4",
        "    1.20773E-02,             !- Humidity Ratio Equation Coefficient 5",
        "    5.17134E-05,             !- Humidity Ratio Equation Coefficient 6",
        "    4.94917E-02,             !- Humidity Ratio Equation Coefficient 7",
        "    -2.59417E-04,            !- Humidity Ratio Equation Coefficient 8",
        "    0.007143,                !- Minimum Regeneration Inlet Air Humidity Ratio for Humidity Ratio Equation {kgWater/kgDryAir}",
        "    0.024286,                !- Maximum Regeneration Inlet Air Humidity Ratio for Humidity Ratio Equation {kgWater/kgDryAir}",
        "    46.111110,               !- Minimum Regeneration Inlet Air Temperature for Humidity Ratio Equation {C}",
        "    46.111112,               !- Maximum Regeneration Inlet Air Temperature for Humidity Ratio Equation {C}",
        "    0.005000,                !- Minimum Process Inlet Air Humidity Ratio for Humidity Ratio Equation {kgWater/kgDryAir}",
        "    0.015714,                !- Maximum Process Inlet Air Humidity Ratio for Humidity Ratio Equation {kgWater/kgDryAir}",
        "    4.583333,                !- Minimum Process Inlet Air Temperature for Humidity Ratio Equation {C}",
        "    21.83333,                !- Maximum Process Inlet Air Temperature for Humidity Ratio Equation {C}",
        "    2.286,                   !- Minimum Regeneration Air Velocity for Humidity Ratio Equation {m/s}",
        "    4.826,                   !- Maximum Regeneration Air Velocity for Humidity Ratio Equation {m/s}",
        "    0.007914,                !- Minimum Regeneration Outlet Air Humidity Ratio for Humidity Ratio Equation {kgWater/kgDryAir}",
        "    0.026279,                !- Maximum Regeneration Outlet Air Humidity Ratio for Humidity Ratio Equation {kgWater/kgDryAir}",
        "    5.0,                     !- Minimum Regeneration Inlet Air Relative Humidity for Humidity Ratio Equation {percent}",
        "    45.0,                    !- Maximum Regeneration Inlet Air Relative Humidity for Humidity Ratio Equation {percent}",
        "    80.0,                    !- Minimum Process Inlet Air Relative Humidity for Humidity Ratio Equation {percent}",
        "    100.0;                   !- Maximum Process Inlet Air Relative Humidity for Humidity Ratio Equation {percent}",
    });

    ASSERT_TRUE(process_idf(idf_objects));

    OutputProcessor::TimeValue.allocate(2);
    DataGlobals::DDOnlySimulation = true;

    SimulationManager::GetProjectData();
    OutputReportPredefined::SetPredefinedTables();
    createFacilityElectricPowerServiceObject();
    SetPreConstructionInputParameters(); // establish array bounds for constructions early
    BranchInputManager::ManageBranchInput();
    BeginSimFlag = true;
    BeginEnvrnFlag = true;
    ZoneSizingCalc = true;
    SysSizingCalc = true;
    SizingManager::ManageSizing();

    DataSizing::CurSysNum = 1;
    DataSizing::CurOASysNum = 0;

    GetDesiccantDehumidifierInput();
    EXPECT_EQ(1, NumDesicDehums);
    EXPECT_EQ("DESICCANT 1", DesicDehum(DesicDehumNum).Name);
    EXPECT_EQ("DESICCANT REGEN COIL", DesicDehum(DesicDehumNum).RegenCoilName);

    CompName = DesicDehum(DesicDehumNum).Name;
    CompIndex = NumGenericDesicDehums;
    SimDesiccantDehumidifier(CompName, FirstHVACIteration, CompIndex);

    RegCoilDesInletTemp = FinalSysSizing(DataSizing::CurSysNum).HeatOutTemp;
    RegCoilDesOutletTemp = DesicDehum(DesicDehumNum).RegenSetPointTemp;
    RegCoilInletAirMassFlowRate = FinalSysSizing(DataSizing::CurSysNum).DesMainVolFlow * DataEnvironment::StdRhoAir;
    RegCoilCapacity = RegCoilInletAirMassFlowRate * PsyCpAirFnWTdb(0.0, 20.0) * (RegCoilDesOutletTemp - RegCoilDesInletTemp);

    for (loop = 1; loop <= NumHeatingCoils; ++loop) {
        if (HeatingCoil(loop).Name == DesicDehum(DesicDehumNum).RegenCoilName) {
            CoilIndex = loop;
        }
    }
    EXPECT_EQ(RegCoilCapacity, HeatingCoil(CoilIndex).NominalCapacity);
}

TEST_F(EnergyPlusFixture, DesiccantDehum_RegenAirHeaterHWCoilSizingTest)
{

    std::string CompName("");
    bool FirstHVACIteration(true);
    int DesicDehumNum(1);
    int CompIndex(1);
    int CoilIndex(0);
    int loop(0);
    Real64 RegCoilDesInletTemp(0.0);
    Real64 RegCoilDesOutletTemp(0.0);
    Real64 RegCoilInletAirMassFlowRate(0.0);
    Real64 RegCoilCapacity(0.0);

    std::string const idf_objects = delimited_string({

        "  Timestep,6;",

        "  Building,",
        "    Bldg,                    !- Name",
        "    0.0,                     !- North Axis {deg}",
        "    Suburbs,                 !- Terrain",
        "    0.05,                    !- Loads Convergence Tolerance Value",
        "    0.05,                    !- Temperature Convergence Tolerance Value {deltaC}",
        "    MinimalShadowing,        !- Solar Distribution",
        "    30,                      !- Maximum Number of Warmup Days",
        "    6;                       !- Minimum Number of Warmup Days",

        "  SurfaceConvectionAlgorithm:Inside,Simple;",

        "  SurfaceConvectionAlgorithm:Outside,SimpleCombined;",

        "  HeatBalanceAlgorithm,ConductionTransferFunction;",

        "  SimulationControl,",
        "    Yes,                     !- Do Zone Sizing Calculation",
        "    Yes,                     !- Do System Sizing Calculation",
        "    No,                      !- Do Plant Sizing Calculation",
        "    Yes,                     !- Run Simulation for Sizing Periods",
        "    No;                      !- Run Simulation for Weather File Run Periods",

        "  RunPeriod,",
        "    ,                        !- Name",
        "    1,                       !- Begin Month",
        "    1,                       !- Begin Day of Month",
        "    ,                        !- Begin Year",
        "    1,                       !- End Month",
        "    5,                       !- End Day of Month",
        "    ,                        !- End Year",
        "    Tuesday,                 !- Day of Week for Start Day",
        "    Yes,                     !- Use Weather File Holidays and Special Days",
        "    Yes,                     !- Use Weather File Daylight Saving Period",
        "    No,                      !- Apply Weekend Holiday Rule",
        "    Yes,                     !- Use Weather File Rain Indicators",
        "    Yes;                     !- Use Weather File Snow Indicators",

        "  Site:Location,",
        "    Miami Intl Ap FL USA WMO=722020,  !- Name",
        "    25.82,                   !- Latitude {deg}",
        "    -80.30,                  !- Longitude {deg}",
        "    -5.00,                   !- Time Zone {hr}",
        "    11.00;                   !- Elevation {m}",

        "  SizingPeriod:DesignDay,",
        "    Miami Intl Ap Ann Htg 99.6% Condns DB,  !- Name",
        "    1,                       !- Month",
        "    21,                      !- Day of Month",
        "    WinterDesignDay,         !- Day Type",
        "    8.7,                     !- Maximum Dry-Bulb Temperature {C}",
        "    0.0,                     !- Daily Dry-Bulb Temperature Range {deltaC}",
        "    ,                        !- Dry-Bulb Temperature Range Modifier Type",
        "    ,                        !- Dry-Bulb Temperature Range Modifier Day Schedule Name",
        "    Wetbulb,                 !- Humidity Condition Type",
        "    8.7,                     !- Wetbulb or DewPoint at Maximum Dry-Bulb {C}",
        "    ,                        !- Humidity Condition Day Schedule Name",
        "    ,                        !- Humidity Ratio at Maximum Dry-Bulb {kgWater/kgDryAir}",
        "    ,                        !- Enthalpy at Maximum Dry-Bulb {J/kg}",
        "    ,                        !- Daily Wet-Bulb Temperature Range {deltaC}",
        "    101217.,                 !- Barometric Pressure {Pa}",
        "    3.8,                     !- Wind Speed {m/s}",
        "    340,                     !- Wind Direction {deg}",
        "    No,                      !- Rain Indicator",
        "    No,                      !- Snow Indicator",
        "    No,                      !- Daylight Saving Time Indicator",
        "    ASHRAEClearSky,          !- Solar Model Indicator",
        "    ,                        !- Beam Solar Day Schedule Name",
        "    ,                        !- Diffuse Solar Day Schedule Name",
        "    ,                        !- ASHRAE Clear Sky Optical Depth for Beam Irradiance (taub) {dimensionless}",
        "    ,                        !- ASHRAE Clear Sky Optical Depth for Diffuse Irradiance (taud) {dimensionless}",
        "    0.00;                    !- Sky Clearness",

        "  SizingPeriod:DesignDay,",
        "    Miami Intl Ap Ann Clg .4% Condns DB=>MWB,  !- Name",
        "    7,                       !- Month",
        "    21,                      !- Day of Month",
        "    SummerDesignDay,         !- Day Type",
        "    33.2,                    !- Maximum Dry-Bulb Temperature {C}",
        "    6.7,                     !- Daily Dry-Bulb Temperature Range {deltaC}",
        "    ,                        !- Dry-Bulb Temperature Range Modifier Type",
        "    ,                        !- Dry-Bulb Temperature Range Modifier Day Schedule Name",
        "    Wetbulb,                 !- Humidity Condition Type",
        "    25.3,                    !- Wetbulb or DewPoint at Maximum Dry-Bulb {C}",
        "    ,                        !- Humidity Condition Day Schedule Name",
        "    ,                        !- Humidity Ratio at Maximum Dry-Bulb {kgWater/kgDryAir}",
        "    ,                        !- Enthalpy at Maximum Dry-Bulb {J/kg}",
        "    ,                        !- Daily Wet-Bulb Temperature Range {deltaC}",
        "    101217.,                 !- Barometric Pressure {Pa}",
        "    4.5,                     !- Wind Speed {m/s}",
        "    140,                     !- Wind Direction {deg}",
        "    No,                      !- Rain Indicator",
        "    No,                      !- Snow Indicator",
        "    No,                      !- Daylight Saving Time Indicator",
        "    ASHRAEClearSky,          !- Solar Model Indicator",
        "    ,                        !- Beam Solar Day Schedule Name",
        "    ,                        !- Diffuse Solar Day Schedule Name",
        "    ,                        !- ASHRAE Clear Sky Optical Depth for Beam Irradiance (taub) {dimensionless}",
        "    ,                        !- ASHRAE Clear Sky Optical Depth for Diffuse Irradiance (taud) {dimensionless}",
        "    1.00;                    !- Sky Clearness",

        "  Site:GroundTemperature:BuildingSurface,21.0,21.5,22.0,22.2,22.2,22.2,22.2,22.2,22.2,22.0,21.5,21.0;",

        "  Material:NoMass,",
        "    R13LAYER,                !- Name",
        "    Rough,                   !- Roughness",
        "    2.290965,                !- Thermal Resistance {m2-K/W}",
        "    0.9000000,               !- Thermal Absorptance",
        "    0.7500000,               !- Solar Absorptance",
        "    0.7500000;               !- Visible Absorptance",

        "  Material:NoMass,",
        "    R31LAYER,                !- Name",
        "    Rough,                   !- Roughness",
        "    5.456,                   !- Thermal Resistance {m2-K/W}",
        "    0.9000000,               !- Thermal Absorptance",
        "    0.7500000,               !- Solar Absorptance",
        "    0.7500000;               !- Visible Absorptance",

        "  Material,",
        "    C5 - 4 IN HW CONCRETE,   !- Name",
        "    MediumRough,             !- Roughness",
        "    0.1014984,               !- Thickness {m}",
        "    1.729577,                !- Conductivity {W/m-K}",
        "    2242.585,                !- Density {kg/m3}",
        "    836.8000,                !- Specific Heat {J/kg-K}",
        "    0.9000000,               !- Thermal Absorptance",
        "    0.6500000,               !- Solar Absorptance",
        "    0.6500000;               !- Visible Absorptance",

        "  Construction,",
        "    R13WALL,                 !- Name",
        "    R13LAYER;                !- Outside Layer",

        "  Construction,",
        "    FLOOR,                   !- Name",
        "    C5 - 4 IN HW CONCRETE;   !- Outside Layer",

        "  Construction,",
        "    ROOF31,                  !- Name",
        "    R31LAYER;                !- Outside Layer",

        "  Zone,",
        "    Main Zone,               !- Name",
        "    0,                       !- Direction of Relative North {deg}",
        "    0,                       !- X Origin {m}",
        "    0,                       !- Y Origin {m}",
        "    0,                       !- Z Origin {m}",
        "    1,                       !- Type",
        "    1,                       !- Multiplier",
        "    autocalculate,           !- Ceiling Height {m}",
        "    autocalculate;           !- Volume {m3}",

        "  GlobalGeometryRules,",
        "    UpperLeftCorner,         !- Starting Vertex Position",
        "    CounterClockWise,        !- Vertex Entry Direction",
        "    relative;                !- Coordinate System",

        "  BuildingSurface:Detailed,",
        "    Zn001:Wall001,           !- Name",
        "    Wall,                    !- Surface Type",
        "    R13WALL,                 !- Construction Name",
        "    Main Zone,               !- Zone Name",
        "    Outdoors,                !- Outside Boundary Condition",
        "    ,                        !- Outside Boundary Condition Object",
        "    SunExposed,              !- Sun Exposure",
        "    WindExposed,             !- Wind Exposure",
        "    0.5000000,               !- View Factor to Ground",
        "    4,                       !- Number of Vertices",
        "    0,0,4.572000,  !- X,Y,Z ==> Vertex 1 {m}",
        "    0,0,0,  !- X,Y,Z ==> Vertex 2 {m}",
        "    15.24000,0,0,  !- X,Y,Z ==> Vertex 3 {m}",
        "    15.24000,0,4.572000;  !- X,Y,Z ==> Vertex 4 {m}",

        "  BuildingSurface:Detailed,",
        "    Zn001:Wall002,           !- Name",
        "    Wall,                    !- Surface Type",
        "    R13WALL,                 !- Construction Name",
        "    Main Zone,               !- Zone Name",
        "    Outdoors,                !- Outside Boundary Condition",
        "    ,                        !- Outside Boundary Condition Object",
        "    SunExposed,              !- Sun Exposure",
        "    WindExposed,             !- Wind Exposure",
        "    0.5000000,               !- View Factor to Ground",
        "    4,                       !- Number of Vertices",
        "    15.24000,0,4.572000,  !- X,Y,Z ==> Vertex 1 {m}",
        "    15.24000,0,0,  !- X,Y,Z ==> Vertex 2 {m}",
        "    15.24000,15.24000,0,  !- X,Y,Z ==> Vertex 3 {m}",
        "    15.24000,15.24000,4.572000;  !- X,Y,Z ==> Vertex 4 {m}",

        "  BuildingSurface:Detailed,",
        "    Zn001:Wall003,           !- Name",
        "    Wall,                    !- Surface Type",
        "    R13WALL,                 !- Construction Name",
        "    Main Zone,               !- Zone Name",
        "    Outdoors,                !- Outside Boundary Condition",
        "    ,                        !- Outside Boundary Condition Object",
        "    SunExposed,              !- Sun Exposure",
        "    WindExposed,             !- Wind Exposure",
        "    0.5000000,               !- View Factor to Ground",
        "    4,                       !- Number of Vertices",
        "    15.24000,15.24000,4.572000,  !- X,Y,Z ==> Vertex 1 {m}",
        "    15.24000,15.24000,0,  !- X,Y,Z ==> Vertex 2 {m}",
        "    0,15.24000,0,  !- X,Y,Z ==> Vertex 3 {m}",
        "    0,15.24000,4.572000;  !- X,Y,Z ==> Vertex 4 {m}",

        "  BuildingSurface:Detailed,",
        "    Zn001:Wall004,           !- Name",
        "    Wall,                    !- Surface Type",
        "    R13WALL,                 !- Construction Name",
        "    Main Zone,               !- Zone Name",
        "    Outdoors,                !- Outside Boundary Condition",
        "    ,                        !- Outside Boundary Condition Object",
        "    SunExposed,              !- Sun Exposure",
        "    WindExposed,             !- Wind Exposure",
        "    0.5000000,               !- View Factor to Ground",
        "    4,                       !- Number of Vertices",
        "    0,15.24000,4.572000,  !- X,Y,Z ==> Vertex 1 {m}",
        "    0,15.24000,0,  !- X,Y,Z ==> Vertex 2 {m}",
        "    0,0,0,  !- X,Y,Z ==> Vertex 3 {m}",
        "    0,0,4.572000;  !- X,Y,Z ==> Vertex 4 {m}",

        "  BuildingSurface:Detailed,",
        "    Zn001:Flr001,            !- Name",
        "    Floor,                   !- Surface Type",
        "    FLOOR,                   !- Construction Name",
        "    Main Zone,               !- Zone Name",
        "    Surface,                 !- Outside Boundary Condition",
        "    Zn001:Flr001,            !- Outside Boundary Condition Object",
        "    NoSun,                   !- Sun Exposure",
        "    NoWind,                  !- Wind Exposure",
        "    1.000000,                !- View Factor to Ground",
        "    4,                       !- Number of Vertices",
        "    15.24000,0.000000,0.0,  !- X,Y,Z ==> Vertex 1 {m}",
        "    0.000000,0.000000,0.0,  !- X,Y,Z ==> Vertex 2 {m}",
        "    0.000000,15.24000,0.0,  !- X,Y,Z ==> Vertex 3 {m}",
        "    15.24000,15.24000,0.0;  !- X,Y,Z ==> Vertex 4 {m}",

        "  BuildingSurface:Detailed,",
        "    Zn001:Roof001,           !- Name",
        "    Roof,                    !- Surface Type",
        "    ROOF31,                  !- Construction Name",
        "    Main Zone,               !- Zone Name",
        "    Outdoors,                !- Outside Boundary Condition",
        "    ,                        !- Outside Boundary Condition Object",
        "    SunExposed,              !- Sun Exposure",
        "    WindExposed,             !- Wind Exposure",
        "    0,                       !- View Factor to Ground",
        "    4,                       !- Number of Vertices",
        "    0.000000,15.24000,4.572,  !- X,Y,Z ==> Vertex 1 {m}",
        "    0.000000,0.000000,4.572,  !- X,Y,Z ==> Vertex 2 {m}",
        "    15.24000,0.000000,4.572,  !- X,Y,Z ==> Vertex 3 {m}",
        "    15.24000,15.24000,4.572;  !- X,Y,Z ==> Vertex 4 {m}",

        "  ScheduleTypeLimits,",
        "    Any Number;              !- Name",

        "  ScheduleTypeLimits,",
        "    Fraction,                !- Name",
        "    0.0,                     !- Lower Limit Value",
        "    1.0,                     !- Upper Limit Value",
        "    CONTINUOUS;              !- Numeric Type",

        "  ScheduleTypeLimits,",
        "    Temperature,             !- Name",
        "    -60,                     !- Lower Limit Value",
        "    200,                     !- Upper Limit Value",
        "    CONTINUOUS,              !- Numeric Type",
        "    Temperature;             !- Unit Type",

        "  ScheduleTypeLimits,",
        "    Control Type,            !- Name",
        "    0,                       !- Lower Limit Value",
        "    4,                       !- Upper Limit Value",
        "    DISCRETE;                !- Numeric Type",

        "  ScheduleTypeLimits,",
        "    On/Off,                  !- Name",
        "    0,                       !- Lower Limit Value",
        "    1,                       !- Upper Limit Value",
        "    DISCRETE;                !- Numeric Type",

        "  ScheduleTypeLimits,",
        "    FlowRate,                !- Name",
        "    0.0,                     !- Lower Limit Value",
        "    10,                      !- Upper Limit Value",
        "    CONTINUOUS;              !- Numeric Type",

        "  ScheduleTypeLimits,",
        "    Humidity,                !- Name",
        "    0,                       !- Lower Limit Value",
        "    100,                     !- Upper Limit Value",
        "    CONTINUOUS;              !- Numeric Type",

        "  Schedule:Compact,",
        "    OCCUPY-1,                !- Name",
        "    Fraction,                !- Schedule Type Limits Name",
        "    Through: 12/31,          !- Field 1",
        "    For: WeekDays SummerDesignDay CustomDay1 CustomDay2, !- Field 2",
        "    Until: 8:00,0.0,         !- Field 3",
        "    Until: 11:00,1.00,       !- Field 5",
        "    Until: 12:00,0.80,       !- Field 7",
        "    Until: 13:00,0.40,       !- Field 9",
        "    Until: 14:00,0.80,       !- Field 11",
        "    Until: 18:00,1.00,       !- Field 13",
        "    Until: 19:00,0.50,       !- Field 15",
        "    Until: 21:00,0.10,       !- Field 17",
        "    Until: 24:00,0.0,        !- Field 19",
        "    For: Weekends WinterDesignDay Holiday, !- Field 21",
        "    Until: 24:00,0.0;        !- Field 22",

        "  Schedule:Compact,",
        "    LIGHTS-1,                !- Name",
        "    Fraction,                !- Schedule Type Limits Name",
        "    Through: 12/31,          !- Field 1",
        "    For: WeekDays SummerDesignDay CustomDay1 CustomDay2, !- Field 2",
        "    Until: 8:00,0.05,        !- Field 3",
        "    Until: 9:00,0.9,         !- Field 5",
        "    Until: 10:00,0.95,       !- Field 7",
        "    Until: 11:00,1.00,       !- Field 9",
        "    Until: 12:00,0.95,       !- Field 11",
        "    Until: 13:00,0.8,        !- Field 13",
        "    Until: 14:00,0.9,        !- Field 15",
        "    Until: 18:00,1.00,       !- Field 17",
        "    Until: 19:00,0.60,       !- Field 19",
        "    Until: 21:00,0.20,       !- Field 21",
        "    Until: 24:00,0.05,       !- Field 23",
        "    For: Weekends WinterDesignDay Holiday, !- Field 25",
        "    Until: 24:00,0.05;       !- Field 26",

        "  Schedule:Compact,",
        "    EQUIP-1,                 !- Name",
        "    Fraction,                !- Schedule Type Limits Name",
        "    Through: 12/31,          !- Field 1",
        "    For: WeekDays SummerDesignDay CustomDay1 CustomDay2, !- Field 2",
        "    Until: 8:00,0.02,        !- Field 3",
        "    Until: 9:00,0.4,         !- Field 5",
        "    Until: 14:00,0.9,        !- Field 7",
        "    Until: 15:00,0.8,        !- Field 9",
        "    Until: 16:00,0.7,        !- Field 11",
        "    Until: 18:00,0.5,        !- Field 13",
        "    Until: 20:00,0.3,        !- Field 15",
        "    Until: 24:00,0.02,       !- Field 17",
        "    For: Weekends WinterDesignDay Holiday, !- Field 19",
        "    Until: 24:00,0.2;        !- Field 20",

        "  Schedule:Compact,",
        "    INFIL-SCH,               !- Name",
        "    Fraction,                !- Schedule Type Limits Name",
        "    Through: 3/31,           !- Field 1",
        "    For: WeekDays SummerDesignDay CustomDay1 CustomDay2, !- Field 2",
        "    Until: 24:00,1.0,        !- Field 3",
        "    For: AllOtherDays,       !- Field 5",
        "    Until: 24:00,1.0,        !- Field 6",
        "    Through: 9/30,           !- Field 8",
        "    For: AllDays,            !- Field 9",
        "    Until: 24:00,0.7,        !- Field 10",
        "    Through: 12/31,          !- Field 12",
        "    For: AllDays,            !- Field 13",
        "    Until: 24:00,1.0,        !- Field 14",
        "    For: AllOtherDays,       !- Field 16",
        "    Until: 24:00,0.2;        !- Field 17",

        "  Schedule:Compact,",
        "    ActSchd,                 !- Name",
        "    Any Number,              !- Schedule Type Limits Name",
        "    Through: 12/31,          !- Field 1",
        "    For: AllDays,            !- Field 2",
        "    Until: 24:00,117.239997864; !- Field 3",

        "  Schedule:Compact,",
        "    ShadeTransSch,           !- Name",
        "    Fraction,                !- Schedule Type Limits Name",
        "    Through: 12/31,          !- Field 1",
        "    For: AllDays,            !- Field 2",
        "    Until: 24:00,0.0;        !- Field 3",

        "  Schedule:Compact,",
        "    Htg-SetP-Sch,            !- Name",
        "    Temperature,             !- Schedule Type Limits Name",
        "    Through: 12/31,          !- Field 1",
        "    For: SummerDesignDay,    !- Field 2",
        "    Until: 24:00,12.8,       !- Field 3",
        "    For: WinterDesignDay,    !- Field 5",
        "    Until: 24:00,21.1,       !- Field 6",
        "    For: WeekEnds Holiday,   !- Field 8",
        "    Until: 7:00,12.8,        !- Field 9",
        "    Until: 13:00,21.1,       !- Field 11",
        "    Until: 24:00,12.8,       !- Field 13",
        "    For: AllOtherDays,       !- Field 15",
        "    Until: 7:00,12.8,        !- Field 16",
        "    Until: 18:00,21.1,       !- Field 18",
        "    Until: 24:00,12.8;       !- Field 20",

        "  Schedule:Compact,",
        "    PlenumHtg-SetP-Sch,      !- Name",
        "    Temperature,             !- Schedule Type Limits Name",
        "    Through: 12/31,          !- Field 1",
        "    For: AllDays,            !- Field 2",
        "    Until: 24:00,12.8;       !- Field 3",

        "  Schedule:Compact,",
        "    Clg-SetP-Sch,            !- Name",
        "    Temperature,             !- Schedule Type Limits Name",
        "    Through: 12/31,          !- Field 1",
        "    For: SummerDesignDay,    !- Field 2",
        "    Until: 24:00,23.9,       !- Field 3",
        "    For: WinterDesignDay,    !- Field 5",
        "    Until: 24:00,40.0,       !- Field 6",
        "    For: WeekEnds Holiday,   !- Field 8",
        "    Until: 7:00,40.0,        !- Field 9",
        "    Until: 13:00,23.9,       !- Field 11",
        "    Until: 24:00,32.2,       !- Field 13",
        "    For: AllOtherDays,       !- Field 15",
        "    Until: 7:00,40.0,        !- Field 16",
        "    Until: 18:00,23.9,       !- Field 18",
        "    Until: 24:00,40.0;       !- Field 20",

        "  Schedule:Compact,",
        "    PlenumClg-SetP-Sch,      !- Name",
        "    Temperature,             !- Schedule Type Limits Name",
        "    Through: 12/31,          !- Field 1",
        "    For: AllDays,            !- Field 2",
        "    Until: 24:00,40.0;       !- Field 3",

        "  Schedule:Compact,",
        "    Zone Control Type Sched, !- Name",
        "    Control Type,            !- Schedule Type Limits Name",
        "    Through: 12/31,          !- Field 1",
        "    For: SummerDesignDay,    !- Field 2",
        "    Until: 24:00,2,          !- Field 3",
        "    For: WinterDesignDay,    !- Field 5",
        "    Until: 24:00,1,          !- Field 6",
        "    For: AllOtherDays,       !- Field 8",
        "    Until: 24:00,4;          !- Field 9",

        "  Schedule:Compact,",
        "    Desiccant Control Type Sched,  !- Name",
        "    Control Type,            !- Schedule Type Limits Name",
        "    Through: 12/31,          !- Field 1",
        "    For: AllDays,            !- Field 2",
        "    Until: 24:00,4;          !- Field 3",

        "  Schedule:Compact,",
        "    Min OA Sched,            !- Name",
        "    Fraction,                !- Schedule Type Limits Name",
        "    Through: 12/31,          !- Field 1",
        "    For: Weekdays,           !- Field 2",
        "    Until: 7:00,0.02,        !- Field 3",
        "    Until: 17:00,1.0,        !- Field 5",
        "    Until: 24:00,0.0,        !- Field 7",
        "    For: AllOtherDays,       !- Field 9",
        "    Until: 24:00,0.02;       !- Field 10",

        "  Schedule:Compact,",
        "    FanAvailSched,           !- Name",
        "    Fraction,                !- Schedule Type Limits Name",
        "    Through: 3/31,           !- Field 1",
        "    For: AllDays,            !- Field 2",
        "    Until: 24:00,1.0,        !- Field 3",
        "    Through: 9/30,           !- Field 5",
        "    For: WeekDays,           !- Field 6",
        "    Until: 7:00,0.0,         !- Field 7",
        "    Until: 17:00,1.0,        !- Field 9",
        "    Until: 24:00,0.0,        !- Field 11",
        "    For: SummerDesignDay WinterDesignDay, !- Field 13",
        "    Until: 24:00,1.0,        !- Field 14",
        "    For: AllOtherDays,       !- Field 16",
        "    Until: 24:00,0.0,        !- Field 17",
        "    Through: 12/31,          !- Field 19",
        "    For: AllDays,            !- Field 20",
        "    Until: 24:00,1.0;        !- Field 21",

        "  Schedule:Compact,",
        "    Seasonal Reset Supply Air Temp Sch,  !- Name",
        "    Temperature,             !- Schedule Type Limits Name",
        "    Through: 3/31,           !- Field 1",
        "    For: AllDays,            !- Field 2",
        "    Until: 24:00,17.0,       !- Field 3",
        "    Through: 9/30,           !- Field 5",
        "    For: AllDays,            !- Field 6",
        "    Until: 24:00,15.0,       !- Field 7",
        "    Through: 12/31,          !- Field 9",
        "    For: AllDays,            !- Field 10",
        "    Until: 24:00,17.0;       !- Field 11",

        "  Schedule:Compact,",
        "    Constant,                !- Name",
        "    Any Number,              !- Schedule Type Limits Name",
        "    Through: 12/31,          !- Field 1",
        "    For: AllDays,            !- Field 2",
        "    Until: 24:00,1.0;        !- Field 3",

        "  People,",
        "    Main Zone People 1,       !- Name",
        "    Main Zone,                !- Zone or ZoneList Name",
        "    OCCUPY-1,                !- Number of People Schedule Name",
        "    people,                  !- Number of People Calculation Method",
        "    20,                      !- Number of People",
        "    ,                        !- People per Zone Floor Area {person/m2}",
        "    ,                        !- Zone Floor Area per Person {m2/person}",
        "    0.3,                     !- Fraction Radiant",
        "    AutoCalculate,           !- Sensible Heat Fraction",
        "    ActSchd;                 !- Activity Level Schedule Name",

        "  Lights,",
        "    Main Zone Lights 1,       !- Name",
        "    Main Zone,                !- Zone or ZoneList Name",
        "    LIGHTS-1,                !- Schedule Name",
        "    LightingLevel,           !- Design Level Calculation Method",
        "    2964,                    !- Lighting Level {W}",
        "    ,                        !- Watts per Zone Floor Area {W/m2}",
        "    ,                        !- Watts per Person {W/person}",
        "    0.2,                     !- Return Air Fraction",
        "    0.59,                    !- Fraction Radiant",
        "    0.2,                     !- Fraction Visible",
        "    0,                       !- Fraction Replaceable",
        "    GeneralLights;           !- End-Use Subcategory",

        "  ElectricEquipment,",
        "    Main Zone ElecEq 1,       !- Name",
        "    Main Zone,                !- Zone or ZoneList Name",
        "    EQUIP-1,                 !- Schedule Name",
        "    EquipmentLevel,          !- Design Level Calculation Method",
        "    19760,                   !- Design Level {W}",
        "    ,                        !- Watts per Zone Floor Area {W/m2}",
        "    ,                        !- Watts per Person {W/person}",
        "    0.5,                     !- Fraction Latent",
        "    0.3,                     !- Fraction Radiant",
        "    0;                       !- Fraction Lost",

        "  ZoneInfiltration:DesignFlowRate,",
        "    Main Zone Infil 1,        !- Name",
        "    Main Zone,                !- Zone or ZoneList Name",
        "    INFIL-SCH,               !- Schedule Name",
        "    flow/zone,               !- Design Flow Rate Calculation Method",
        "    0.031089,                !- Design Flow Rate {m3/s}",
        "    ,                        !- Flow per Zone Floor Area {m3/s-m2}",
        "    ,                        !- Flow per Exterior Surface Area {m3/s-m2}",
        "    ,                        !- Air Changes per Hour {1/hr}",
        "    0,                       !- Constant Term Coefficient",
        "    0,                       !- Temperature Term Coefficient",
        "    0.2237,                  !- Velocity Term Coefficient",
        "    0;                       !- Velocity Squared Term Coefficient",

        "  Sizing:Zone,",
        "    Main Zone,                !- Zone or ZoneList Name",
        "    SupplyAirTemperature,    !- Zone Cooling Design Supply Air Temperature Input Method",
        "    14.,                     !- Zone Cooling Design Supply Air Temperature {C}",
        "    ,                        !- Zone Cooling Design Supply Air Temperature Difference {deltaC}",
        "    SupplyAirTemperature,    !- Zone Heating Design Supply Air Temperature Input Method",
        "    50.,                     !- Zone Heating Design Supply Air Temperature {C}",
        "    ,                        !- Zone Heating Design Supply Air Temperature Difference {deltaC}",
        "    0.009,                   !- Zone Cooling Design Supply Air Humidity Ratio {kgWater/kgDryAir}",
        "    0.004,                   !- Zone Heating Design Supply Air Humidity Ratio {kgWater/kgDryAir}",
        "    SZ DSOA Main Zone,        !- Design Specification Outdoor Air Object Name",
        "    0.0,                     !- Zone Heating Sizing Factor",
        "    0.0,                     !- Zone Cooling Sizing Factor",
        "    DesignDayWithLimit,      !- Cooling Design Air Flow Method",
        "    ,                        !- Cooling Design Air Flow Rate {m3/s}",
        "    ,                        !- Cooling Minimum Air Flow per Zone Floor Area {m3/s-m2}",
        "    ,                        !- Cooling Minimum Air Flow {m3/s}",
        "    ,                        !- Cooling Minimum Air Flow Fraction",
        "    DesignDay,               !- Heating Design Air Flow Method",
        "    ,                        !- Heating Design Air Flow Rate {m3/s}",
        "    ,                        !- Heating Maximum Air Flow per Zone Floor Area {m3/s-m2}",
        "    ,                        !- Heating Maximum Air Flow {m3/s}",
        "    ;                        !- Heating Maximum Air Flow Fraction",

        "  DesignSpecification:OutdoorAir,",
        "    SZ DSOA Main Zone,        !- Name",
        "    sum,                     !- Outdoor Air Method",
        "    0.00236,                 !- Outdoor Air Flow per Person {m3/s-person}",
        "    0.000305,                !- Outdoor Air Flow per Zone Floor Area {m3/s-m2}",
        "    0.0;                     !- Outdoor Air Flow per Zone {m3/s}",

        "  Sizing:System,",
        "    Desiccant DXSystem,      !- AirLoop Name",
        "    sensible,                !- Type of Load to Size On",
        "    autosize,                !- Design Outdoor Air Flow Rate {m3/s}",
        "    0.3,                     !- Central Heating Maximum System Air Flow Ratio",
        "    4.5,                     !- Preheat Design Temperature {C}",
        "    0.008,                   !- Preheat Design Humidity Ratio {kgWater/kgDryAir}",
        "    11.0,                    !- Precool Design Temperature {C}",
        "    0.008,                   !- Precool Design Humidity Ratio {kgWater/kgDryAir}",
        "    12.8,                    !- Central Cooling Design Supply Air Temperature {C}",
        "    16.7,                    !- Central Heating Design Supply Air Temperature {C}",
        "    noncoincident,           !- Type of Zone Sum to Use",
        "    no,                      !- 100% Outdoor Air in Cooling",
        "    no,                      !- 100% Outdoor Air in Heating",
        "    0.008,                   !- Central Cooling Design Supply Air Humidity Ratio {kgWater/kgDryAir}",
        "    0.008,                   !- Central Heating Design Supply Air Humidity Ratio {kgWater/kgDryAir}",
        "    DesignDay,               !- Cooling Design Air Flow Method",
        "    0,                       !- Cooling Design Air Flow Rate {m3/s}",
        "    ,                        !- Supply Air Flow Rate Per Floor Area During Cooling Operation {m3/s-m2}",
        "    ,                        !- Fraction of Autosized Design Cooling Supply Air Flow Rate",
        "    ,                        !- Design Supply Air Flow Rate Per Unit Cooling Capacity {m3/s-W}",
        "    DesignDay,               !- Heating Design Air Flow Method",
        "    0,                       !- Heating Design Air Flow Rate {m3/s}",
        "    ,                        !- Supply Air Flow Rate Per Floor Area During Heating Operation {m3/s-m2}",
        "    ,                        !- Fraction of Autosized Design Heating Supply Air Flow Rate",
        "    ,                        !- Fraction of Autosized Design Cooling Supply Air Flow Rate",
        "    ,                        !- Design Supply Air Flow Rate Per Unit Heating Capacity {m3/s-W}",
        "    ,                        !- System Outdoor Air Method",
        "    1.0,                     !- Zone Maximum Outdoor Air Fraction {dimensionless}",
        "    CoolingDesignCapacity,   !- Cooling Design Capacity Method",
        "    autosize,                !- Cooling Design Capacity {W}",
        "    ,                        !- Cooling Design Capacity Per Floor Area {W/m2}",
        "    ,                        !- Fraction of Autosized Cooling Design Capacity",
        "    HeatingDesignCapacity,   !- Heating Design Capacity Method",
        "    autosize,                !- Heating Design Capacity {W}",
        "    ,                        !- Heating Design Capacity Per Floor Area {W/m2}",
        "    ,                        !- Fraction of Autosized Heating Design Capacity",
        "    VAV;                     !- Central Cooling Capacity Control Method",

        "  Curve:Biquadratic,",
        "    HPACCoolCapFT,           !- Name",
        "    0.942587793,             !- Coefficient1 Constant",
        "    0.009543347,             !- Coefficient2 x",
        "    0.00068377,              !- Coefficient3 x**2",
        "    -0.011042676,            !- Coefficient4 y",
        "    0.000005249,             !- Coefficient5 y**2",
        "    -0.00000972,             !- Coefficient6 x*y",
        "    -100,                    !- Minimum Value of x",
        "    100,                     !- Maximum Value of x",
        "    -100,                    !- Minimum Value of y",
        "    100,                     !- Maximum Value of y",
        "    ,                        !- Minimum Curve Output",
        "    ,                        !- Maximum Curve Output",
        "    Temperature,             !- Input Unit Type for X",
        "    Temperature,             !- Input Unit Type for Y",
        "    Dimensionless;           !- Output Unit Type",

        "  Curve:Biquadratic,",
        "    HPACCOOLEIRFT,           !- Name",
        "    0.342414409,             !- Coefficient1 Constant",
        "    0.034885008,             !- Coefficient2 x",
        "    -0.0006237,              !- Coefficient3 x**2",
        "    0.004977216,             !- Coefficient4 y",
        "    0.000437951,             !- Coefficient5 y**2",
        "    -0.000728028,            !- Coefficient6 x*y",
        "    -100,                    !- Minimum Value of x",
        "    100,                     !- Maximum Value of x",
        "    -100,                    !- Minimum Value of y",
        "    100,                     !- Maximum Value of y",
        "    ,                        !- Minimum Curve Output",
        "    ,                        !- Maximum Curve Output",
        "    Temperature,             !- Input Unit Type for X",
        "    Temperature,             !- Input Unit Type for Y",
        "    Dimensionless;           !- Output Unit Type",

        "  Curve:Quadratic,",
        "    HPACCoolCapFFF,          !- Name",
        "    0.8,                     !- Coefficient1 Constant",
        "    0.2,                     !- Coefficient2 x",
        "    0.0,                     !- Coefficient3 x**2",
        "    0.0,                     !- Minimum Value of x",
        "    1.0;                     !- Maximum Value of x",

        "  Curve:Quadratic,",
        "    HPACCOOLEIRFFF,          !- Name",
        "    1.1552,                  !- Coefficient1 Constant",
        "    -0.1808,                 !- Coefficient2 x",
        "    0.0256,                  !- Coefficient3 x**2",
        "    0.0,                     !- Minimum Value of x",
        "    1.0;                     !- Maximum Value of x",

        "  Curve:Quadratic,",
        "    HPACCOOLPLFFPLR,         !- Name",
        "    0.75,                    !- Coefficient1 Constant",
        "    0.25,                    !- Coefficient2 x",
        "    0.0,                     !- Coefficient3 x**2",
        "    0.0,                     !- Minimum Value of x",
        "    1.0;                     !- Maximum Value of x",

        "  Curve:Cubic,",
        "    PackagedRatedCoolCapFFlow,  !- Name",
        "    0.47278589,              !- Coefficient1 Constant",
        "    1.2433415,               !- Coefficient2 x",
        "    -1.0387055,              !- Coefficient3 x**2",
        "    0.32257813,              !- Coefficient4 x**3",
        "    0.5,                     !- Minimum Value of x",
        "    1.5,                     !- Maximum Value of x",
        "    ,                        !- Minimum Curve Output",
        "    ,                        !- Maximum Curve Output",
        "    Dimensionless,           !- Input Unit Type for X",
        "    Dimensionless;           !- Output Unit Type",

        "  Curve:Cubic,",
        "    PackagedRatedCoolEIRFFlow,  !- Name",
        "    1.0079484,               !- Coefficient1 Constant",
        "    0.34544129,              !- Coefficient2 x",
        "    -.6922891,               !- Coefficient3 x**2",
        "    0.33889943,              !- Coefficient4 x**3",
        "    0.5,                     !- Minimum Value of x",
        "    1.5,                     !- Maximum Value of x",
        "    ,                        !- Minimum Curve Output",
        "    ,                        !- Maximum Curve Output",
        "    Dimensionless,           !- Input Unit Type for X",
        "    Dimensionless;           !- Output Unit Type",

        "  Curve:Cubic,",
        "    EXHAUSTFANPLF,           !- Name",
        "    0,                       !- Coefficient1 Constant",
        "    1,                       !- Coefficient2 x",
        "    0.0,                     !- Coefficient3 x**2",
        "    0.0,                     !- Coefficient4 x**3",
        "    0.0,                     !- Minimum Value of x",
        "    1.0;                     !- Maximum Value of x",

        "  NodeList,",
        "    Main Zone In Nodes,       !- Name",
        "    Main Zone In Node;        !- Node 1 Name",

        "  NodeList,",
        "    Desiccant DX Air Loop Setpoint Nodes,  !- Name",
        "    HX Process Outlet Node,  !- Node 1 Name",
        "    Desiccant DXSystem Mixed Air Node;  !- Node 2 Name",

        "  NodeList,",
        "    Regen Setpoint Nodes,    !- Name",
        "    HX Regen Inlet Node;     !- Node 1 Name",

        "  BranchList,",
        "    Desiccant Air Loop Branches,  !- Name",
        "    Desiccant Air Loop Main Branch;  !- Branch 1 Name",

        "  Branch,",
        "    Desiccant Air Loop Main Branch,  !- Name",
        "    ,                        !- Pressure Drop Curve Name",
        "    AirLoopHVAC:OutdoorAirSystem,  !- Component 1 Object Type",
        "    Desiccant DXSystem OA System,  !- Component 1 Name",
        "    Desiccant DXSystem Air Loop Inlet Node,  !- Component 1 Inlet Node Name",
        "    Desiccant DXSystem Mixed Air Node,  !- Component 1 Outlet Node Name",
        "    CoilSystem:Cooling:DX,   !- Component 2 Object Type",
        "    Desiccant DXSystem Cooling Coil Sys,  !- Component 2 Name",
        "    Desiccant DXSystem Mixed Air Node,  !- Component 2 Inlet Node Name",
        "    HX Process Inlet Node,   !- Component 2 Outlet Node Name",
        "    Dehumidifier:Desiccant:System,  !- Component 3 Object Type",
        "    Desiccant 1,             !- Component 3 Name",
        "    HX Process Inlet Node,   !- Component 3 Inlet Node Name",
        "    HX Process Outlet Node,  !- Component 3 Outlet Node Name",
        "    Fan:ConstantVolume,      !- Component 4 Object Type",
        "    Desiccant DXSystem Supply Fan,  !- Component 4 Name",
        "    HX Process Outlet Node,  !- Component 4 Inlet Node Name",
        "    Desiccant Supply Fan Outlet Node,  !- Component 4 Outlet Node Name",
        "    Coil:Heating:Fuel,        !- Component 5 Object Type",
        "    Desiccant Process Heating Coil,  !- Component 5 Name",
        "    Desiccant Supply Fan Outlet Node,  !- Component 5 Inlet Node Name",
        "    Desiccant DXSystem Air Loop Outlet Node;  !- Component 5 Outlet Node Name",

        "  AirLoopHVAC,",
        "    Desiccant DXSystem,      !- Name",
        "    ,                        !- Controller List Name",
        "    Desiccant Avail List,    !- Availability Manager List Name",
        "    autosize,                !- Design Supply Air Flow Rate {m3/s}",
        "    Desiccant Air Loop Branches,  !- Branch List Name",
        "    ,                        !- Connector List Name",
        "    Desiccant DXSystem Air Loop Inlet Node,  !- Supply Side Inlet Node Name",
        "    Desiccant DXSystem Return Air Mixer Outlet Node,  !- Demand Side Outlet Node Name",
        "    Desiccant DXSystem Zone Equipment Inlet Node,  !- Demand Side Inlet Node Names",
        "    Desiccant DXSystem Air Loop Outlet Node;  !- Supply Side Outlet Node Names",

        "  AirLoopHVAC:ControllerList,",
        "    Desiccant DXSystem OA System Controllers,  !- Name",
        "    Controller:OutdoorAir,   !- Controller 1 Object Type",
        "    Desiccant OA Controller; !- Controller 1 Name",

        "  AirLoopHVAC:OutdoorAirSystem:EquipmentList,",
        "    Desiccant DXSystem OA System Equipment,  !- Name",
        "    OutdoorAir:Mixer,        !- Component 1 Object Type",
        "    Desiccant DXSystem OA Mixing Box;  !- Component 1 Name",

        "  AirLoopHVAC:OutdoorAirSystem,",
        "    Desiccant DXSystem OA System,  !- Name",
        "    Desiccant DXSystem OA System Controllers,  !- Controller List Name",
        "    Desiccant DXSystem OA System Equipment,  !- Outdoor Air Equipment List Name",
        "    Desiccant Outdoor Air Avail List;  !- Availability Manager List Name",

        "  OutdoorAir:NodeList,",
        "    OutsideAirInletNodes;    !- Node or NodeList Name 1",

        "  OutdoorAir:NodeList,",
        "    Desiccant DXSystem Outside Air Inlet Node,  !- Node or NodeList Name 1",
        "    Desiccant Outside Air Inlet Node 2;  !- Node or NodeList Name 2",

        "  OutdoorAir:NodeList,",
        "    Regen Coil Inlet Node;   !- Node or NodeList Name 1",

        "  OutdoorAir:NodeList,",
        "    Condenser Inlet Node;    !- Node or NodeList Name 1",

        "  OutdoorAir:Mixer,",
        "    Desiccant DXSystem OA Mixing Box,  !- Name",
        "    Desiccant DXSystem Mixed Air Node,  !- Mixed Air Node Name",
        "    Desiccant DXSystem Outside Air Inlet Node,  !- Outdoor Air Stream Node Name",
        "    Desiccant DXSystem Relief Air Outlet Node,  !- Relief Air Stream Node Name",
        "    Desiccant DXSystem Air Loop Inlet Node;  !- Return Air Stream Node Name",

        "  AvailabilityManagerAssignmentList,",
        "    Desiccant Avail List,    !- Name",
        "    AvailabilityManager:Scheduled,  !- Availability Manager 1 Object Type",
        "    Desiccant DXSystem Avail;!- Availability Manager 1 Name",

        "  AvailabilityManagerAssignmentList,",
        "    Desiccant Outdoor Air Avail List,  !- Name",
        "    AvailabilityManager:Scheduled,  !- Availability Manager 1 Object Type",
        "    Desiccant Outdoor Air Avail;  !- Availability Manager 1 Name",

        "  AvailabilityManager:Scheduled,",
        "    Desiccant DXSystem Avail,!- Name",
        "    FanAvailSched;           !- Schedule Name",

        "  AvailabilityManager:Scheduled,",
        "    Desiccant Outdoor Air Avail,  !- Name",
        "    FanAvailSched;           !- Schedule Name",

        "  SetpointManager:SingleZone:Heating,",
        "    Heating Supply Air Temp Manager,  !- Name",
        "    Temperature,             !- Control Variable",
        "    -99.,                    !- Minimum Supply Air Temperature {C}",
        "    99.,                     !- Maximum Supply Air Temperature {C}",
        "    Main Zone,                !- Control Zone Name",
        "    Main Zone Node,           !- Zone Node Name",
        "    Main Zone In Node,        !- Zone Inlet Node Name",
        "    Desiccant DXSystem Air Loop Outlet Node;  !- Setpoint Node or NodeList Name",

        "  SetpointManager:SingleZone:Cooling,",
        "    Cooling Supply Air Temp Manager,  !- Name",
        "    Temperature,             !- Control Variable",
        "    -99.,                    !- Minimum Supply Air Temperature {C}",
        "    99.,                     !- Maximum Supply Air Temperature {C}",
        "    Main Zone,                !- Control Zone Name",
        "    Main Zone Node,           !- Zone Node Name",
        "    Main Zone In Node,        !- Zone Inlet Node Name",
        "    Desiccant Supply Fan Outlet Node;  !- Setpoint Node or NodeList Name",

        "  SetpointManager:SingleZone:Humidity:Maximum,",
        "    Zone Max Humidity Manager,  !- Name",
        "    HX Process Outlet Node,  !- Setpoint Node or NodeList Name",
        "    Main Zone Node;           !- Control Zone Air Node Name",

        "  SetpointManager:MixedAir,",
        "    Adjusted Cooling Supply Air Temp Manager,  !- Name",
        "    Temperature,             !- Control Variable",
        "    Desiccant Supply Fan Outlet Node,  !- Reference Setpoint Node Name",
        "    HX Process Outlet Node,  !- Fan Inlet Node Name",
        "    Desiccant Supply Fan Outlet Node,  !- Fan Outlet Node Name",
        "    HX Process Outlet Node;  !- Setpoint Node or NodeList Name",

        "  Controller:OutdoorAir,",
        "    Desiccant OA Controller, !- Name",
        "    Desiccant DXSystem Relief Air Outlet Node,  !- Relief Air Outlet Node Name",
        "    Desiccant DXSystem Air Loop Inlet Node,  !- Return Air Node Name",
        "    Desiccant DXSystem Mixed Air Node,  !- Mixed Air Node Name",
        "    Desiccant DXSystem Outside Air Inlet Node,  !- Actuator Node Name",
        "    autosize,                !- Minimum Outdoor Air Flow Rate {m3/s}",
        "    autosize,                !- Maximum Outdoor Air Flow Rate {m3/s}",
        "    NoEconomizer,            !- Economizer Control Type",
        "    ModulateFlow,            !- Economizer Control Action Type",
        "    100.,                    !- Economizer Maximum Limit Dry-Bulb Temperature {C}",
        "    ,                        !- Economizer Maximum Limit Enthalpy {J/kg}",
        "    ,                        !- Economizer Maximum Limit Dewpoint Temperature {C}",
        "    ,                        !- Electronic Enthalpy Limit Curve Name",
        "    ,                        !- Economizer Minimum Limit Dry-Bulb Temperature {C}",
        "    NoLockout,               !- Lockout Type",
        "    FixedMinimum,            !- Minimum Limit Type",
        "    Min OA Sched,            !- Minimum Outdoor Air Schedule Name",
        "    ,                        !- Minimum Fraction of Outdoor Air Schedule Name",
        "    ,                        !- Maximum Fraction of Outdoor Air Schedule Name",
        "    0;                       !- Mechanical Ventilation Controller Name",

        "  ZoneHVAC:EquipmentConnections,",
        "    Main Zone,                !- Zone Name",
        "    Main Zone Eq,             !- Zone Conditioning Equipment List Name",
        "    Main Zone In Nodes,       !- Zone Air Inlet Node or NodeList Name",
        "    ,                        !- Zone Air Exhaust Node or NodeList Name",
        "    Main Zone Node,           !- Zone Air Node Name",
        "    Main Zone Out Node;       !- Zone Return Air Node Name",

        "  ZoneHVAC:EquipmentList,",
        "    Main Zone Eq,             !- Name",
        "    SequentialLoad,          !- Load Distribution Scheme",
        "    ZoneHVAC:AirDistributionUnit,  !- Zone Equipment 1 Object Type",
        "    Main Zone Direct Air ADU,     !- Zone Equipment 1 Name",
        "    1,                        !- Zone Equipment 1 Cooling Sequence",
        "    1;                        !- Zone Equipment 1 Heating or No-Load Sequence",

        "  ZoneHVAC:AirDistributionUnit,",
        "    Main Zone Direct Air ADU, !- Name",
        "    Main Zone In Node,       !- Air Distribution Unit Outlet Node Name",
        "    AirTerminal:SingleDuct:ConstantVolume:NoReheat,  !- Air Terminal Object Type",
        "    Main Zone Direct Air;    !- Air Terminal Name",

        "  AirTerminal:SingleDuct:ConstantVolume:NoReheat,",
        "    Main Zone Direct Air,    !- Name",
        "    Constant,                !- Availability Schedule Name",
        "    Main Zone In Node 2AT,   !- Air Inlet Node Name",
        "    Main Zone In Node,       !- Air Outlet Node Name",
        "    autosize;                !- Maximum Air Flow Rate {m3/s}",

        "  CoilSystem:Cooling:DX,",
        "    Desiccant DXSystem Cooling Coil Sys,  !- Name",
        "    FanAvailSched,           !- Availability Schedule Name",
        "    Desiccant DXSystem Mixed Air Node,  !- DX Cooling Coil System Inlet Node Name",
        "    HX Process Inlet Node,   !- DX Cooling Coil System Outlet Node Name",
        "    HX Process Outlet Node,  !- DX Cooling Coil System Sensor Node Name",
        "    Coil:Cooling:DX:SingleSpeed,  !- Cooling Coil Object Type",
        "    Desiccant DXSystem Cooling Coil;  !- Cooling Coil Name",

        "  ZoneControl:Thermostat,",
        "    Main Zone Control,        !- Name",
        "    Main Zone,                !- Zone or ZoneList Name",
        "    Desiccant Control Type Sched,  !- Control Type Schedule Name",
        "    ThermostatSetpoint:DualSetpoint,  !- Control 1 Object Type",
        "    DualSetPoint;            !- Control 1 Name",

        "  ZoneControl:Humidistat,",
        "    Main Zone Humidistat,     !- Name",
        "    Main Zone,                !- Zone Name",
        "    Relative Humidity Sched; !- Humidifying Relative Humidity Setpoint Schedule Name",

        "  Schedule:Compact,",
        "    Relative Humidity Sched, !- Name",
        "    Any Number,              !- Schedule Type Limits Name",
        "    Through: 12/31,          !- Field 1",
        "    For: AllDays,            !- Field 2",
        "    Until: 24:00,45.0;       !- Field 3",

        "  ThermostatSetpoint:SingleHeating,",
        "    HeatingSetpoint,         !- Name",
        "    Htg-SetP-Sch;            !- Setpoint Temperature Schedule Name",

        "  ThermostatSetpoint:SingleHeating,",
        "    PlenumHeatingSetpoint,   !- Name",
        "    PlenumHtg-SetP-Sch;      !- Setpoint Temperature Schedule Name",

        "  ThermostatSetpoint:SingleCooling,",
        "    CoolingSetpoint,         !- Name",
        "    Clg-SetP-Sch;            !- Setpoint Temperature Schedule Name",

        "  ThermostatSetpoint:SingleCooling,",
        "    PlenumCoolingSetpoint,   !- Name",
        "    PlenumClg-SetP-Sch;      !- Setpoint Temperature Schedule Name",

        "  ThermostatSetpoint:DualSetpoint,",
        "    DualSetPoint,            !- Name",
        "    Htg-SetP-Sch,            !- Heating Setpoint Temperature Schedule Name",
        "    Clg-SetP-Sch;            !- Cooling Setpoint Temperature Schedule Name",

        "  AirLoopHVAC:SupplyPath,",
        "    Desiccant DXSystem Supply Path,  !- Name",
        "    Desiccant DXSystem Zone Equipment Inlet Node,  !- Supply Air Path Inlet Node Name",
        "    AirLoopHVAC:ZoneSplitter,!- Component 1 Object Type",
        "    Desiccant DXSystem Zone Supply Air Splitter;  !- Component 1 Name",

        "  AirLoopHVAC:ReturnPath,",
        "    Desiccant DXSystem Return Path,  !- Name",
        "    Desiccant DXSystem Return Air Mixer Outlet Node,  !- Return Air Path Outlet Node Name",
        "    AirLoopHVAC:ZoneMixer,   !- Component 1 Object Type",
        "    Desiccant DXSystem Zone Return Air Mixer;  !- Component 1 Name",

        "  AirLoopHVAC:ZoneSplitter,",
        "    Desiccant DXSystem Zone Supply Air Splitter,  !- Name",
        "    Desiccant DXSystem Zone Equipment Inlet Node,  !- Inlet Node Name",
        "    Main Zone In Node 2AT;        !- Outlet 1 Node Name",

        "  AirLoopHVAC:ZoneMixer,",
        "    Desiccant DXSystem Zone Return Air Mixer,  !- Name",
        "    Desiccant DXSystem Return Air Mixer Outlet Node,  !- Outlet Node Name",
        "    Main Zone Out Node;       !- Inlet 1 Node Name",

        "  Coil:Heating:Fuel,",
        "    Desiccant Process Heating Coil,  !- Name",
        "    Constant,                !- Availability Schedule Name",
        "    Gas,                     !- Fuel Type",
        "    0.8,                     !- Gas Burner Efficiency",
        "    autosize,                !- Nominal Capacity {W}",
        "    Desiccant Supply Fan Outlet Node,  !- Air Inlet Node Name",
        "    Desiccant DXSystem Air Loop Outlet Node,  !- Air Outlet Node Name",
        "    Desiccant DXSystem Air Loop Outlet Node;  !- Temperature Setpoint Node Name",

        "  Coil:Cooling:DX:SingleSpeed,",
        "    Desiccant DXSystem Cooling Coil,  !- Name",
        "    FanAvailSched,           !- Availability Schedule Name",
        "    autosize,                !- High Speed Rated Air Flow Rate {m3/s}",
        "    autosize,                !- Gross Rated Sensible Heat Ratio",
        "    3.5,                     !- Gross Rated Cooling COP {W/W}",
        "    autosize,                !- High Speed Rated Air Flow Rate {m3/s}",
        "    ,                        !- Rated Evaporator Fan Power Per Volume Flow Rate {W/(m3/s)}",
        "    Desiccant DXSystem Mixed Air Node,  !- Air Inlet Node Name",
        "    HX Process Inlet Node,   !- Air Outlet Node Name",
        "    HPACCoolCapFT,           !- Total Cooling Capacity Function of Temperature Curve Name",
        "    HPACCoolCapFFF,          !- Total Cooling Capacity Function of Flow Fraction Curve Name",
        "    HPACCOOLEIRFT,           !- Energy Input Ratio Function of Temperature Curve Name",
        "    HPACCOOLEIRFFF,          !- Energy Input Ratio Function of Flow Fraction Curve Name",
        "    HPACCOOLPLFFPLR,         !- Part Load Fraction Correlation Curve Name",
        "   ,                         !- Minimum Outdoor Dry-Bulb Temperature for Compressor Operation {C}",
        "    1000,                    !- Nominal Time for Condensate Removal to Begin {s}",
        "    0.4,                     !- Ratio of Initial Moisture Evaporation Rate and Steady State Latent Capacity {dimensionless}",
        "    4,                       !- Maximum Cycling Rate {cycles/hr}",
        "    45,                      !- Latent Capacity Time Constant {s}",
        "    Condenser Inlet Node,    !- Condenser Air Inlet Node Name",
        "    AirCooled;               !- Condenser Type",

        "  Fan:ConstantVolume,",
        "    Desiccant DXSystem Supply Fan,  !- Name",
        "    FanAvailSched,           !- Availability Schedule Name",
        "    0.7,                     !- Fan Total Efficiency",
        "    571.66,                  !- Pressure Rise {Pa}",
        "    autosize,                !- Maximum Flow Rate {m3/s}",
        "    0.9,                     !- Motor Efficiency",
        "    1.0,                     !- Motor In Airstream Fraction",
        "    HX Process Outlet Node,  !- Air Inlet Node Name",
        "    Desiccant Supply Fan Outlet Node;  !- Air Outlet Node Name",

        "  Fan:ConstantVolume,",
        "    Desiccant Regen Fan,     !- Name",
        "    FanAvailSched,           !- Availability Schedule Name",
        "    0.7,                     !- Fan Total Efficiency",
        "    205.5,                   !- Pressure Rise {Pa}",
        "    autosize,                !- Maximum Flow Rate {m3/s}",
        "    0.9,                     !- Motor Efficiency",
        "    1.0,                     !- Motor In Airstream Fraction",
        "    HX Regen Outlet Node,    !- Air Inlet Node Name",
        "    Regen Fan Outlet Node;   !- Air Outlet Node Name",

        "  Dehumidifier:Desiccant:System,",
        "    Desiccant 1,             !- Name",
        "    FanAvailSched,           !- Availability Schedule Name",
        "    HeatExchanger:Desiccant:BalancedFlow,  !- Desiccant Heat Exchanger Object Type",
        "    Desiccant Heat Exchanger 1,  !- Desiccant Heat Exchanger Name",
        "    HX Process Outlet Node,  !- Sensor Node Name",
        "    Fan:ConstantVolume,      !- Regeneration Air Fan Object Type",
        "    Desiccant Regen Fan,     !- Regeneration Air Fan Name",
        "    DrawThrough,             !- Regeneration Air Fan Placement",
        "    Coil:Heating:Water,      !- Regeneration Air Heater Object Type",
        "    Desiccant Regen Coil,    !- Regeneration Air Heater Name",
        "    46.111111,               !- Regeneration Inlet Air Setpoint Temperature {C}",
        "    Coil:Cooling:DX:SingleSpeed,  !- Companion Cooling Coil Object Type",
        "    Desiccant DXSystem Cooling Coil,  !- Companion Cooling Coil Name",
        "    Yes,                     !- Companion Cooling Coil Upstream of Dehumidifier Process Inlet",
        "    Yes,                     !- Companion Coil Regeneration Air Heating",
        "    0.87931,                 !- Exhaust Fan Maximum Flow Rate {m3/s}",
        "    50,                      !- Exhaust Fan Maximum Power {W}",
        "    EXHAUSTFANPLF;           !- Exhaust Fan Power Curve Name",

        "  HeatExchanger:Desiccant:BalancedFlow,",
        "    Desiccant Heat Exchanger 1,  !- Name",
        "    FanAvailSched,           !- Availability Schedule Name",
        "    HX Regen Inlet Node,     !- Regeneration Air Inlet Node Name",
        "    HX Regen Outlet Node,    !- Regeneration Air Outlet Node Name",
        "    HX Process Inlet Node,   !- Process Air Inlet Node Name",
        "    HX Process Outlet Node,  !- Process Air Outlet Node Name",
        "    HeatExchanger:Desiccant:BalancedFlow:PerformanceDataType1,  !- Heat Exchanger Performance Object Type",
        "    HXDesPerf1;              !- Heat Exchanger Performance Name",

        "  HeatExchanger:Desiccant:BalancedFlow:PerformanceDataType1,",
        "    HXDesPerf1,              !- Name",
        "    0.87931,                 !- Nominal Air Flow Rate {m3/s}",
        "    3.25,                    !- Nominal Air Face Velocity {m/s}",
        "    50.0,                    !- Nominal Electric Power {W}",
        "    -2.53636E+00,            !- Temperature Equation Coefficient 1",
        "    2.13247E+01,             !- Temperature Equation Coefficient 2",
        "    9.23308E-01,             !- Temperature Equation Coefficient 3",
        "    9.43276E+02,             !- Temperature Equation Coefficient 4",
        "    -5.92367E+01,            !- Temperature Equation Coefficient 5",
        "    -4.27465E-02,            !- Temperature Equation Coefficient 6",
        "    1.12204E+02,             !- Temperature Equation Coefficient 7",
        "    7.78252E-01,             !- Temperature Equation Coefficient 8",
        "    0.007143,                !- Minimum Regeneration Inlet Air Humidity Ratio for Temperature Equation {kgWater/kgDryAir}",
        "    0.024286,                !- Maximum Regeneration Inlet Air Humidity Ratio for Temperature Equation {kgWater/kgDryAir}",
        "    46.111110,               !- Minimum Regeneration Inlet Air Temperature for Temperature Equation {C}",
        "    46.111112,               !- Maximum Regeneration Inlet Air Temperature for Temperature Equation {C}",
        "    0.005000,                !- Minimum Process Inlet Air Humidity Ratio for Temperature Equation {kgWater/kgDryAir}",
        "    0.015714,                !- Maximum Process Inlet Air Humidity Ratio for Temperature Equation {kgWater/kgDryAir}",
        "    4.583333,                !- Minimum Process Inlet Air Temperature for Temperature Equation {C}",
        "    21.83333,                !- Maximum Process Inlet Air Temperature for Temperature Equation {C}",
        "    2.286,                   !- Minimum Regeneration Air Velocity for Temperature Equation {m/s}",
        "    4.826,                   !- Maximum Regeneration Air Velocity for Temperature Equation {m/s}",
        "    35.0,                    !- Minimum Regeneration Outlet Air Temperature for Temperature Equation {C}",
        "    50.0,                    !- Maximum Regeneration Outlet Air Temperature for Temperature Equation {C}",
        "    5.0,                     !- Minimum Regeneration Inlet Air Relative Humidity for Temperature Equation {percent}",
        "    45.0,                    !- Maximum Regeneration Inlet Air Relative Humidity for Temperature Equation {percent}",
        "    80.0,                    !- Minimum Process Inlet Air Relative Humidity for Temperature Equation {percent}",
        "    100.0,                   !- Maximum Process Inlet Air Relative Humidity for Temperature Equation {percent}",
        "    -2.25547E+01,            !- Humidity Ratio Equation Coefficient 1",
        "    9.76839E-01,             !- Humidity Ratio Equation Coefficient 2",
        "    4.89176E-01,             !- Humidity Ratio Equation Coefficient 3",
        "    -6.30019E-02,            !- Humidity Ratio Equation Coefficient 4",
        "    1.20773E-02,             !- Humidity Ratio Equation Coefficient 5",
        "    5.17134E-05,             !- Humidity Ratio Equation Coefficient 6",
        "    4.94917E-02,             !- Humidity Ratio Equation Coefficient 7",
        "    -2.59417E-04,            !- Humidity Ratio Equation Coefficient 8",
        "    0.007143,                !- Minimum Regeneration Inlet Air Humidity Ratio for Humidity Ratio Equation {kgWater/kgDryAir}",
        "    0.024286,                !- Maximum Regeneration Inlet Air Humidity Ratio for Humidity Ratio Equation {kgWater/kgDryAir}",
        "    46.111110,               !- Minimum Regeneration Inlet Air Temperature for Humidity Ratio Equation {C}",
        "    46.111112,               !- Maximum Regeneration Inlet Air Temperature for Humidity Ratio Equation {C}",
        "    0.005000,                !- Minimum Process Inlet Air Humidity Ratio for Humidity Ratio Equation {kgWater/kgDryAir}",
        "    0.015714,                !- Maximum Process Inlet Air Humidity Ratio for Humidity Ratio Equation {kgWater/kgDryAir}",
        "    4.583333,                !- Minimum Process Inlet Air Temperature for Humidity Ratio Equation {C}",
        "    21.83333,                !- Maximum Process Inlet Air Temperature for Humidity Ratio Equation {C}",
        "    2.286,                   !- Minimum Regeneration Air Velocity for Humidity Ratio Equation {m/s}",
        "    4.826,                   !- Maximum Regeneration Air Velocity for Humidity Ratio Equation {m/s}",
        "    0.007914,                !- Minimum Regeneration Outlet Air Humidity Ratio for Humidity Ratio Equation {kgWater/kgDryAir}",
        "    0.026279,                !- Maximum Regeneration Outlet Air Humidity Ratio for Humidity Ratio Equation {kgWater/kgDryAir}",
        "    5.0,                     !- Minimum Regeneration Inlet Air Relative Humidity for Humidity Ratio Equation {percent}",
        "    45.0,                    !- Maximum Regeneration Inlet Air Relative Humidity for Humidity Ratio Equation {percent}",
        "    80.0,                    !- Minimum Process Inlet Air Relative Humidity for Humidity Ratio Equation {percent}",
        "    100.0;                   !- Maximum Process Inlet Air Relative Humidity for Humidity Ratio Equation {percent}",

        "  Sizing:Plant,",
        "    Hot Water Loop,          !- Plant or Condenser Loop Name",
        "    Heating,                 !- Loop Type",
        "    82.2222222222222,        !- Design Loop Exit Temperature {C}",
        "    22.2222222222222;        !- Loop Design Temperature Difference {deltaC}",

        "  PlantLoop,",
        "    Hot Water Loop,          !- Name",
        "    Water,                   !- Fluid Type",
        "    ,                        !- User Defined Fluid Type",
        "    Hot Loop Operation,      !- Plant Equipment Operation Scheme Name",
        "    HW Supply Outlet Node,   !- Loop Temperature Setpoint Node Name",
        "    100,                     !- Maximum Loop Temperature {C}",
        "    10,                      !- Minimum Loop Temperature {C}",
        "    Autosize,                !- Maximum Loop Flow Rate {m3/s}",
        "    0.0,                     !- Minimum Loop Flow Rate {m3/s}",
        "    autocalculate,           !- Plant Loop Volume {m3}",
        "    HW Supply Inlet Node,    !- Plant Side Inlet Node Name",
        "    HW Supply Outlet Node,   !- Plant Side Outlet Node Name",
        "    Heating Supply Side Branches,  !- Plant Side Branch List Name",
        "    Heating Supply Side Connectors,  !- Plant Side Connector List Name",
        "    HW Demand Inlet Node,    !- Demand Side Inlet Node Name",
        "    HW Demand Outlet Node,   !- Demand Side Outlet Node Name",
        "    Heating Demand Side Branches,  !- Demand Side Branch List Name",
        "    Heating Demand Side Connectors,  !- Demand Side Connector List Name",
        "    Optimal,                 !- Load Distribution Scheme",
        "    ,                        !- Availability Manager List Name",
        "    ,                        !- Plant Loop Demand Calculation Scheme",
        "    ,                        !- Common Pipe Simulation",
        "    ,                        !- Pressure Simulation Type",
        "    2.0;                     !- Loop Circulation Time {minutes}",

        "  SetpointManager:Scheduled,",
        "    Hot Water Loop Setpoint Manager,  !- Name",
        "    Temperature,             !- Control Variable",
        "    HW Loop Temp Schedule,   !- Schedule Name",
        "    Hot Water Loop Setpoint Node List;  !- Setpoint Node or NodeList Name",

        "  NodeList,",
        "    Hot Water Loop Setpoint Node List,  !- Name",
        "    HW Supply Outlet Node;   !- Node 1 Name",

        "  Schedule:Compact,",
        "    HW LOOP TEMP SCHEDULE,   !- Name",
        "    TEMPERATURE,             !- Schedule Type Limits Name",
        "    Through: 12/31,          !- Field 1",
        "    For: Alldays,            !- Field 2",
        "    Until: 24:00,82.00;      !- Field 3",

        "  BranchList,",
        "    Heating Supply Side Branches,  !- Name",
        "    Heating Supply Inlet Branch,  !- Branch 1 Name",
        "    Heating Purchased Hot Water Branch,  !- Branch 2 Name",
        "    Heating Supply Bypass Branch,  !- Branch 3 Name",
        "    Heating Supply Outlet Branch;  !- Branch 4 Name",

        "  ConnectorList,",
        "    Heating Supply Side Connectors,  !- Name",
        "    Connector:Splitter,      !- Connector 1 Object Type",
        "    Heating Supply Splitter, !- Connector 1 Name",
        "    Connector:Mixer,         !- Connector 2 Object Type",
        "    Heating Supply Mixer;    !- Connector 2 Name",

        "  Connector:Splitter,",
        "    Heating Supply Splitter, !- Name",
        "    Heating Supply Inlet Branch,  !- Inlet Branch Name",
        "    Heating Purchased Hot Water Branch,  !- Outlet Branch 1 Name",
        "    Heating Supply Bypass Branch;  !- Outlet Branch 2 Name",

        "  Connector:Mixer,",
        "    Heating Supply Mixer,    !- Name",
        "    Heating Supply Outlet Branch,  !- Outlet Branch Name",
        "    Heating Purchased Hot Water Branch,  !- Inlet Branch 1 Name",
        "    Heating Supply Bypass Branch;  !- Inlet Branch 2 Name",

        "  Branch,",
        "    Heating Supply Inlet Branch,  !- Name",
        "    ,                        !- Pressure Drop Curve Name",
        "    Pump:VariableSpeed,      !- Component 1 Object Type",
        "    HW Circ Pump,            !- Component 1 Name",
        "    HW Supply Inlet Node,    !- Component 1 Inlet Node Name",
        "    HW Pump Outlet Node;     !- Component 1 Outlet Node Name",

        "  Branch,",
        "    Heating Purchased Hot Water Branch,  !- Name",
        "    ,                        !- Pressure Drop Curve Name",
        "    DistrictHeating,         !- Component 1 Object Type",
        "    Purchased Heating,       !- Component 1 Name",
        "    Purchased Heat Inlet Node,  !- Component 1 Inlet Node Name",
        "    Purchased Heat Outlet Node;  !- Component 1 Outlet Node Name",

        "  Branch,",
        "    Heating Supply Bypass Branch,  !- Name",
        "    ,                        !- Pressure Drop Curve Name",
        "    Pipe:Adiabatic,          !- Component 1 Object Type",
        "    Heating Supply Side Bypass,  !- Component 1 Name",
        "    Heating Supply Bypass Inlet Node,  !- Component 1 Inlet Node Name",
        "    Heating Supply Bypass Outlet Node;  !- Component 1 Outlet Node Name",

        "  Pipe:Adiabatic,",
        "    Heating Supply Side Bypass,  !- Name",
        "    Heating Supply Bypass Inlet Node,  !- Inlet Node Name",
        "    Heating Supply Bypass Outlet Node;  !- Outlet Node Name",

        "  Branch,",
        "    Heating Supply Outlet Branch,  !- Name",
        "    ,                        !- Pressure Drop Curve Name",
        "    Pipe:Adiabatic,          !- Component 1 Object Type",
        "    Heating Supply Outlet,   !- Component 1 Name",
        "    Heating Supply Exit Pipe Inlet Node,  !- Component 1 Inlet Node Name",
        "    HW Supply Outlet Node;   !- Component 1 Outlet Node Name",

        "  Pipe:Adiabatic,",
        "    Heating Supply Outlet,   !- Name",
        "    Heating Supply Exit Pipe Inlet Node,  !- Inlet Node Name",
        "    HW Supply Outlet Node;   !- Outlet Node Name",

        "  Pump:VariableSpeed,",
        "    HW Circ Pump,            !- Name",
        "    HW Supply Inlet Node,    !- Inlet Node Name",
        "    HW Pump Outlet Node,     !- Outlet Node Name",
        "    autosize,                !- Rated Flow Rate {m3/s}",
        "    179352,                  !- Rated Pump Head {Pa}",
        "    autosize,                !- Rated Power Consumption {W}",
        "    0.9,                     !- Motor Efficiency",
        "    0.0,                     !- Fraction of Motor Inefficiencies to Fluid Stream",
        "    0,                       !- Coefficient 1 of the Part Load Performance Curve",
        "    1,                       !- Coefficient 2 of the Part Load Performance Curve",
        "    0,                       !- Coefficient 3 of the Part Load Performance Curve",
        "    0,                       !- Coefficient 4 of the Part Load Performance Curve",
        "    0,                       !- Minimum Flow Rate {m3/s}",
        "    INTERMITTENT;            !- Pump Control Type",

        "  BranchList,",
        "    Heating Demand Side Branches,  !- Name",
        "    ZonesHWInletBranch,      !- Branch 1 Name",
        "    RegenCoilHWBranch,       !- Branch 2 Name",
        "    ZonesHWBypassBranch,     !- Branch 5 Name",
        "    ZonesHWOutletBranch;     !- Branch 6 Name",

        "  ConnectorList,",
        "    Heating Demand Side Connectors,  !- Name",
        "    Connector:Splitter,      !- Connector 1 Object Type",
        "    Zones HW Splitter,       !- Connector 1 Name",
        "    Connector:Mixer,         !- Connector 2 Object Type",
        "    Zones HW Mixer;          !- Connector 2 Name",

        "  Connector:Splitter,",
        "    Zones HW Splitter,       !- Name",
        "    ZonesHWInletBranch,      !- Inlet Branch Name",
        "    RegenCoilHWBranch,           !- Outlet Branch 1 Name",
        "    ZonesHWBypassBranch;     !- Outlet Branch 4 Name",

        "  Connector:Mixer,",
        "    Zones HW Mixer,          !- Name",
        "    ZonesHWOutletBranch,     !- Outlet Branch Name",
        "    RegenCoilHWBranch,       !- Inlet Branch 1 Name",
        "    ZonesHWBypassBranch;     !- Inlet Branch 4 Name",

        "  Branch,",
        "    ZonesHWInletBranch,      !- Name",
        "    ,                        !- Pressure Drop Curve Name",
        "    Pipe:Adiabatic,          !- Component 1 Object Type",
        "    ZonesHWInletPipe,        !- Component 1 Name",
        "    HW Demand Inlet Node,    !- Component 1 Inlet Node Name",
        "    HW Demand Entrance Pipe Outlet Node;  !- Component 1 Outlet Node Name",

        "  Pipe:Adiabatic,",
        "    ZonesHWInletPipe,        !- Name",
        "    HW Demand Inlet Node,    !- Inlet Node Name",
        "    HW Demand Entrance Pipe Outlet Node;  !- Outlet Node Name",

        "  Branch,",
        "    ZonesHWOutletBranch,     !- Name",
        "    ,                        !- Pressure Drop Curve Name",
        "    Pipe:Adiabatic,          !- Component 1 Object Type",
        "    ZonesHWOutletPipe,       !- Component 1 Name",
        "    HW Demand Exit Pipe Inlet Node,  !- Component 1 Inlet Node Name",
        "    HW Demand Outlet Node;   !- Component 1 Outlet Node Name",

        "  Pipe:Adiabatic,",
        "    ZonesHWOutletPipe,       !- Name",
        "    HW Demand Exit Pipe Inlet Node,  !- Inlet Node Name",
        "    HW Demand Outlet Node;   !- Outlet Node Name",

        "  Branch,",
        "    RegenCoilHWBranch,       !- Name",
        "    ,                        !- Pressure Drop Curve Name",
        "    Coil:Heating:Water,      !- Component 1 Object Type",
        "    Desiccant Regen Coil,    !- Component 1 Name",
        "    RegenCoilHWInletNode,    !- Component 1 Inlet Node Name",
        "    RegenCoilHWOutletNode;   !- Component 1 Outlet Node Name",

        "  Coil:Heating:Water,",
        "    Desiccant Regen Coil,    !- Name",
        "    FanAvailSched,           !- Availability Schedule Name",
        "    autosize,                !- U-Factor Times Area Value {W/K}",
        "    autosize,                !- Maximum Water Flow Rate {m3/s}",
        "    RegenCoilHWInletNode,    !- Water Inlet Node Name",
        "    RegenCoilHWOutletNode,   !- Water Outlet Node Name",
        "    Regen Coil Inlet Node,   !- Air Inlet Node Name",
        "    HX Regen Inlet Node,     !- Air Outlet Node Name",
        "    UFactorTimesAreaAndDesignWaterFlowRate,  !- Performance Input Method",
        "    autosize,                !- Rated Capacity {W}",
        "    82.2,                    !- Rated Inlet Water Temperature {C}",
        "    16.6,                    !- Rated Inlet Air Temperature {C}",
        "    71.1,                    !- Rated Outlet Water Temperature {C}",
        "    32.2,                    !- Rated Outlet Air Temperature {C}",
        "    ;                        !- Rated Ratio for Air and Water Convection",

        "  Branch,",
        "    ZonesHWBypassBranch,     !- Name",
        "    ,                        !- Pressure Drop Curve Name",
        "    Pipe:Adiabatic,          !- Component 1 Object Type",
        "    ZonesHWBypassPipe,       !- Component 1 Name",
        "    ZonesHWBypassInletNode,  !- Component 1 Inlet Node Name",
        "    ZonesHWBypassOutletNode; !- Component 1 Outlet Node Name",

        "  Pipe:Adiabatic,",
        "    ZonesHWBypassPipe,       !- Name",
        "    ZonesHWBypassInletNode,  !- Inlet Node Name",
        "    ZonesHWBypassOutletNode; !- Outlet Node Name",

        "  PlantEquipmentOperationSchemes,",
        "    Hot Loop Operation,      !- Name",
        "    PlantEquipmentOperation:HeatingLoad,  !- Control Scheme 1 Object Type",
        "    Purchased Heating Only,  !- Control Scheme 1 Name",
        "    Constant;                !- Control Scheme 1 Schedule Name",

        "  PlantEquipmentOperation:HeatingLoad,",
        "    Purchased Heating Only,  !- Name",
        "    0,                       !- Load Range 1 Lower Limit {W}",
        "    1000000,                 !- Load Range 1 Upper Limit {W}",
        "    heating plant;           !- Range 1 Equipment List Name",

        "  PlantEquipmentList,",
        "    heating plant,           !- Name",
        "    DistrictHeating,         !- Equipment 1 Object Type",
        "    Purchased Heating;       !- Equipment 1 Name",

        "  DistrictHeating,",
        "    Purchased Heating,       !- Name",
        "    Purchased Heat Inlet Node,  !- Hot Water Inlet Node Name",
        "    Purchased Heat Outlet Node,  !- Hot Water Outlet Node Name",
        "    1000000;                 !- Nominal Capacity {W}",

    });

    ASSERT_TRUE(process_idf(idf_objects));

    OutputProcessor::TimeValue.allocate(2);
    DataGlobals::DDOnlySimulation = true;

    SimulationManager::GetProjectData();
    OutputReportPredefined::SetPredefinedTables();
    createFacilityElectricPowerServiceObject();
    SetPreConstructionInputParameters(); // establish array bounds for constructions early
    BranchInputManager::ManageBranchInput();
    BeginSimFlag = true;
    BeginEnvrnFlag = true;
    ZoneSizingCalc = true;
    SysSizingCalc = true;
    SizingManager::ManageSizing();

    DataSizing::CurSysNum = 1;
    DataSizing::CurOASysNum = 0;

    GetDesiccantDehumidifierInput();
    EXPECT_EQ(1, NumDesicDehums);
    EXPECT_EQ(1, NumGenericDesicDehums);
    EXPECT_EQ("DESICCANT 1", DesicDehum(DesicDehumNum).Name);
    EXPECT_EQ("DESICCANT REGEN COIL", DesicDehum(DesicDehumNum).RegenCoilName);
    EXPECT_EQ("COIL:HEATING:WATER", DesicDehum(DesicDehumNum).RegenCoilType);

    CompName = DesicDehum(DesicDehumNum).Name;
    CompIndex = NumGenericDesicDehums;
    // set design parameters and calculate HW coil design capacity
    RegCoilDesInletTemp = 8.5;
    DataSizing::FinalSysSizing(DataSizing::CurSysNum).HeatOutTemp = RegCoilDesInletTemp;
    RegCoilDesOutletTemp = DesiccantDehumidifiers::DesicDehum(DesicDehumNum).RegenSetPointTemp;
    DataSizing::FinalSysSizing(DataSizing::CurSysNum).DesMainVolFlow = 1.07;
    RegCoilInletAirMassFlowRate = DataEnvironment::StdRhoAir * DataSizing::FinalSysSizing(DataSizing::CurSysNum).DesMainVolFlow;
    RegCoilCapacity = RegCoilInletAirMassFlowRate * PsyCpAirFnWTdb(0.0, 20.0) * (RegCoilDesOutletTemp - RegCoilDesInletTemp);

    // simulate to determine HW coil design capacity
    SimDesiccantDehumidifier(CompName, FirstHVACIteration, CompIndex);
    for (loop = 1; loop <= NumWaterCoils; ++loop) {
        if (WaterCoil(loop).Name == DesicDehum(DesicDehumNum).RegenCoilName) {
            CoilIndex = loop;
        }
    }
    // verify results
    EXPECT_EQ(RegCoilCapacity, WaterCoil(CoilIndex).DesWaterHeatingCoilRate);
}

TEST_F(EnergyPlusFixture, DesiccantDehum_VSCoolingCoilOnPrimaryAirSystemTest)
{

    // this unit test is derived from DesiccantDehum_OnPrimaryAirSystemTest but uses a VS dx coil instead

    std::string CompName("");
    bool FirstHVACIteration(true);
    int DesicDehumNum(1);
    int CompIndex(1);
    int CoilIndex(0);
    int loop(0);
    Real64 RegCoilDesInletTemp(0.0);
    Real64 RegCoilDesOutletTemp(0.0);
    Real64 RegCoilInletAirMassFlowRate(0.0);
    Real64 RegCoilCapacity(0.0);

    std::string const idf_objects = delimited_string({

        "  Version,9.1;",

        "  Timestep,6;",

        "  Building,",
        "    Bldg,                    !- Name",
        "    0.0,                     !- North Axis {deg}",
        "    Suburbs,                 !- Terrain",
        "    0.05,                    !- Loads Convergence Tolerance Value",
        "    0.05,                    !- Temperature Convergence Tolerance Value {deltaC}",
        "    MinimalShadowing,        !- Solar Distribution",
        "    30,                      !- Maximum Number of Warmup Days",
        "    6;                       !- Minimum Number of Warmup Days",

        "  SurfaceConvectionAlgorithm:Inside,Simple;",

        "  SurfaceConvectionAlgorithm:Outside,SimpleCombined;",

        "  HeatBalanceAlgorithm,ConductionTransferFunction;",

        "  SimulationControl,",
        "    Yes,                     !- Do Zone Sizing Calculation",
        "    Yes,                     !- Do System Sizing Calculation",
        "    No,                      !- Do Plant Sizing Calculation",
        "    Yes,                     !- Run Simulation for Sizing Periods",
        "    No;                      !- Run Simulation for Weather File Run Periods",

        "  RunPeriod,",
        "    ,                        !- Name",
        "    1,                       !- Begin Month",
        "    1,                       !- Begin Day of Month",
        "    ,                        !- Begin Year",
        "    1,                       !- End Month",
        "    5,                       !- End Day of Month",
        "    ,                        !- End Year",
        "    Tuesday,                 !- Day of Week for Start Day",
        "    Yes,                     !- Use Weather File Holidays and Special Days",
        "    Yes,                     !- Use Weather File Daylight Saving Period",
        "    No,                      !- Apply Weekend Holiday Rule",
        "    Yes,                     !- Use Weather File Rain Indicators",
        "    Yes;                     !- Use Weather File Snow Indicators",

        "  Site:Location,",
        "    Miami Intl Ap FL USA WMO=722020,  !- Name",
        "    25.82,                   !- Latitude {deg}",
        "    -80.30,                  !- Longitude {deg}",
        "    -5.00,                   !- Time Zone {hr}",
        "    11.00;                   !- Elevation {m}",

        "  SizingPeriod:DesignDay,",
        "    Miami Intl Ap Ann Htg 99.6% Condns DB,  !- Name",
        "    1,                       !- Month",
        "    21,                      !- Day of Month",
        "    WinterDesignDay,         !- Day Type",
        "    8.7,                     !- Maximum Dry-Bulb Temperature {C}",
        "    0.0,                     !- Daily Dry-Bulb Temperature Range {deltaC}",
        "    ,                        !- Dry-Bulb Temperature Range Modifier Type",
        "    ,                        !- Dry-Bulb Temperature Range Modifier Day Schedule Name",
        "    Wetbulb,                 !- Humidity Condition Type",
        "    8.7,                     !- Wetbulb or DewPoint at Maximum Dry-Bulb {C}",
        "    ,                        !- Humidity Condition Day Schedule Name",
        "    ,                        !- Humidity Ratio at Maximum Dry-Bulb {kgWater/kgDryAir}",
        "    ,                        !- Enthalpy at Maximum Dry-Bulb {J/kg}",
        "    ,                        !- Daily Wet-Bulb Temperature Range {deltaC}",
        "    101217.,                 !- Barometric Pressure {Pa}",
        "    3.8,                     !- Wind Speed {m/s}",
        "    340,                     !- Wind Direction {deg}",
        "    No,                      !- Rain Indicator",
        "    No,                      !- Snow Indicator",
        "    No,                      !- Daylight Saving Time Indicator",
        "    ASHRAEClearSky,          !- Solar Model Indicator",
        "    ,                        !- Beam Solar Day Schedule Name",
        "    ,                        !- Diffuse Solar Day Schedule Name",
        "    ,                        !- ASHRAE Clear Sky Optical Depth for Beam Irradiance (taub) {dimensionless}",
        "    ,                        !- ASHRAE Clear Sky Optical Depth for Diffuse Irradiance (taud) {dimensionless}",
        "    0.00;                    !- Sky Clearness",

        "  SizingPeriod:DesignDay,",
        "    Miami Intl Ap Ann Clg .4% Condns DB=>MWB,  !- Name",
        "    7,                       !- Month",
        "    21,                      !- Day of Month",
        "    SummerDesignDay,         !- Day Type",
        "    33.2,                    !- Maximum Dry-Bulb Temperature {C}",
        "    6.7,                     !- Daily Dry-Bulb Temperature Range {deltaC}",
        "    ,                        !- Dry-Bulb Temperature Range Modifier Type",
        "    ,                        !- Dry-Bulb Temperature Range Modifier Day Schedule Name",
        "    Wetbulb,                 !- Humidity Condition Type",
        "    25.3,                    !- Wetbulb or DewPoint at Maximum Dry-Bulb {C}",
        "    ,                        !- Humidity Condition Day Schedule Name",
        "    ,                        !- Humidity Ratio at Maximum Dry-Bulb {kgWater/kgDryAir}",
        "    ,                        !- Enthalpy at Maximum Dry-Bulb {J/kg}",
        "    ,                        !- Daily Wet-Bulb Temperature Range {deltaC}",
        "    101217.,                 !- Barometric Pressure {Pa}",
        "    4.5,                     !- Wind Speed {m/s}",
        "    140,                     !- Wind Direction {deg}",
        "    No,                      !- Rain Indicator",
        "    No,                      !- Snow Indicator",
        "    No,                      !- Daylight Saving Time Indicator",
        "    ASHRAEClearSky,          !- Solar Model Indicator",
        "    ,                        !- Beam Solar Day Schedule Name",
        "    ,                        !- Diffuse Solar Day Schedule Name",
        "    ,                        !- ASHRAE Clear Sky Optical Depth for Beam Irradiance (taub) {dimensionless}",
        "    ,                        !- ASHRAE Clear Sky Optical Depth for Diffuse Irradiance (taud) {dimensionless}",
        "    1.00;                    !- Sky Clearness",

        "  Site:GroundTemperature:BuildingSurface,21.0,21.5,22.0,22.2,22.2,22.2,22.2,22.2,22.2,22.0,21.5,21.0;",

        "  Material:NoMass,",
        "    R13LAYER,                !- Name",
        "    Rough,                   !- Roughness",
        "    2.290965,                !- Thermal Resistance {m2-K/W}",
        "    0.9000000,               !- Thermal Absorptance",
        "    0.7500000,               !- Solar Absorptance",
        "    0.7500000;               !- Visible Absorptance",

        "  Material:NoMass,",
        "    R31LAYER,                !- Name",
        "    Rough,                   !- Roughness",
        "    5.456,                   !- Thermal Resistance {m2-K/W}",
        "    0.9000000,               !- Thermal Absorptance",
        "    0.7500000,               !- Solar Absorptance",
        "    0.7500000;               !- Visible Absorptance",

        "  Material,",
        "    C5 - 4 IN HW CONCRETE,   !- Name",
        "    MediumRough,             !- Roughness",
        "    0.1014984,               !- Thickness {m}",
        "    1.729577,                !- Conductivity {W/m-K}",
        "    2242.585,                !- Density {kg/m3}",
        "    836.8000,                !- Specific Heat {J/kg-K}",
        "    0.9000000,               !- Thermal Absorptance",
        "    0.6500000,               !- Solar Absorptance",
        "    0.6500000;               !- Visible Absorptance",

        "  Construction,",
        "    R13WALL,                 !- Name",
        "    R13LAYER;                !- Outside Layer",

        "  Construction,",
        "    FLOOR,                   !- Name",
        "    C5 - 4 IN HW CONCRETE;   !- Outside Layer",

        "  Construction,",
        "    ROOF31,                  !- Name",
        "    R31LAYER;                !- Outside Layer",

        "  Zone,",
        "    Main Zone,               !- Name",
        "    0,                       !- Direction of Relative North {deg}",
        "    0,                       !- X Origin {m}",
        "    0,                       !- Y Origin {m}",
        "    0,                       !- Z Origin {m}",
        "    1,                       !- Type",
        "    1,                       !- Multiplier",
        "    autocalculate,           !- Ceiling Height {m}",
        "    autocalculate;           !- Volume {m3}",

        "  GlobalGeometryRules,",
        "    UpperLeftCorner,         !- Starting Vertex Position",
        "    CounterClockWise,        !- Vertex Entry Direction",
        "    relative;                !- Coordinate System",

        "  BuildingSurface:Detailed,",
        "    Zn001:Wall001,           !- Name",
        "    Wall,                    !- Surface Type",
        "    R13WALL,                 !- Construction Name",
        "    Main Zone,               !- Zone Name",
        "    Outdoors,                !- Outside Boundary Condition",
        "    ,                        !- Outside Boundary Condition Object",
        "    SunExposed,              !- Sun Exposure",
        "    WindExposed,             !- Wind Exposure",
        "    0.5000000,               !- View Factor to Ground",
        "    4,                       !- Number of Vertices",
        "    0,0,4.572000,  !- X,Y,Z ==> Vertex 1 {m}",
        "    0,0,0,  !- X,Y,Z ==> Vertex 2 {m}",
        "    15.24000,0,0,  !- X,Y,Z ==> Vertex 3 {m}",
        "    15.24000,0,4.572000;  !- X,Y,Z ==> Vertex 4 {m}",

        "  BuildingSurface:Detailed,",
        "    Zn001:Wall002,           !- Name",
        "    Wall,                    !- Surface Type",
        "    R13WALL,                 !- Construction Name",
        "    Main Zone,               !- Zone Name",
        "    Outdoors,                !- Outside Boundary Condition",
        "    ,                        !- Outside Boundary Condition Object",
        "    SunExposed,              !- Sun Exposure",
        "    WindExposed,             !- Wind Exposure",
        "    0.5000000,               !- View Factor to Ground",
        "    4,                       !- Number of Vertices",
        "    15.24000,0,4.572000,  !- X,Y,Z ==> Vertex 1 {m}",
        "    15.24000,0,0,  !- X,Y,Z ==> Vertex 2 {m}",
        "    15.24000,15.24000,0,  !- X,Y,Z ==> Vertex 3 {m}",
        "    15.24000,15.24000,4.572000;  !- X,Y,Z ==> Vertex 4 {m}",

        "  BuildingSurface:Detailed,",
        "    Zn001:Wall003,           !- Name",
        "    Wall,                    !- Surface Type",
        "    R13WALL,                 !- Construction Name",
        "    Main Zone,               !- Zone Name",
        "    Outdoors,                !- Outside Boundary Condition",
        "    ,                        !- Outside Boundary Condition Object",
        "    SunExposed,              !- Sun Exposure",
        "    WindExposed,             !- Wind Exposure",
        "    0.5000000,               !- View Factor to Ground",
        "    4,                       !- Number of Vertices",
        "    15.24000,15.24000,4.572000,  !- X,Y,Z ==> Vertex 1 {m}",
        "    15.24000,15.24000,0,  !- X,Y,Z ==> Vertex 2 {m}",
        "    0,15.24000,0,  !- X,Y,Z ==> Vertex 3 {m}",
        "    0,15.24000,4.572000;  !- X,Y,Z ==> Vertex 4 {m}",

        "  BuildingSurface:Detailed,",
        "    Zn001:Wall004,           !- Name",
        "    Wall,                    !- Surface Type",
        "    R13WALL,                 !- Construction Name",
        "    Main Zone,               !- Zone Name",
        "    Outdoors,                !- Outside Boundary Condition",
        "    ,                        !- Outside Boundary Condition Object",
        "    SunExposed,              !- Sun Exposure",
        "    WindExposed,             !- Wind Exposure",
        "    0.5000000,               !- View Factor to Ground",
        "    4,                       !- Number of Vertices",
        "    0,15.24000,4.572000,  !- X,Y,Z ==> Vertex 1 {m}",
        "    0,15.24000,0,  !- X,Y,Z ==> Vertex 2 {m}",
        "    0,0,0,  !- X,Y,Z ==> Vertex 3 {m}",
        "    0,0,4.572000;  !- X,Y,Z ==> Vertex 4 {m}",

        "  BuildingSurface:Detailed,",
        "    Zn001:Flr001,            !- Name",
        "    Floor,                   !- Surface Type",
        "    FLOOR,                   !- Construction Name",
        "    Main Zone,               !- Zone Name",
        "    Surface,                 !- Outside Boundary Condition",
        "    Zn001:Flr001,            !- Outside Boundary Condition Object",
        "    NoSun,                   !- Sun Exposure",
        "    NoWind,                  !- Wind Exposure",
        "    1.000000,                !- View Factor to Ground",
        "    4,                       !- Number of Vertices",
        "    15.24000,0.000000,0.0,  !- X,Y,Z ==> Vertex 1 {m}",
        "    0.000000,0.000000,0.0,  !- X,Y,Z ==> Vertex 2 {m}",
        "    0.000000,15.24000,0.0,  !- X,Y,Z ==> Vertex 3 {m}",
        "    15.24000,15.24000,0.0;  !- X,Y,Z ==> Vertex 4 {m}",

        "  BuildingSurface:Detailed,",
        "    Zn001:Roof001,           !- Name",
        "    Roof,                    !- Surface Type",
        "    ROOF31,                  !- Construction Name",
        "    Main Zone,               !- Zone Name",
        "    Outdoors,                !- Outside Boundary Condition",
        "    ,                        !- Outside Boundary Condition Object",
        "    SunExposed,              !- Sun Exposure",
        "    WindExposed,             !- Wind Exposure",
        "    0,                       !- View Factor to Ground",
        "    4,                       !- Number of Vertices",
        "    0.000000,15.24000,4.572,  !- X,Y,Z ==> Vertex 1 {m}",
        "    0.000000,0.000000,4.572,  !- X,Y,Z ==> Vertex 2 {m}",
        "    15.24000,0.000000,4.572,  !- X,Y,Z ==> Vertex 3 {m}",
        "    15.24000,15.24000,4.572;  !- X,Y,Z ==> Vertex 4 {m}",

        "  ScheduleTypeLimits,",
        "    Any Number;              !- Name",

        "  ScheduleTypeLimits,",
        "    Fraction,                !- Name",
        "    0.0,                     !- Lower Limit Value",
        "    1.0,                     !- Upper Limit Value",
        "    CONTINUOUS;              !- Numeric Type",

        "  ScheduleTypeLimits,",
        "    Temperature,             !- Name",
        "    -60,                     !- Lower Limit Value",
        "    200,                     !- Upper Limit Value",
        "    CONTINUOUS,              !- Numeric Type",
        "    Temperature;             !- Unit Type",

        "  ScheduleTypeLimits,",
        "    Control Type,            !- Name",
        "    0,                       !- Lower Limit Value",
        "    4,                       !- Upper Limit Value",
        "    DISCRETE;                !- Numeric Type",

        "  ScheduleTypeLimits,",
        "    On/Off,                  !- Name",
        "    0,                       !- Lower Limit Value",
        "    1,                       !- Upper Limit Value",
        "    DISCRETE;                !- Numeric Type",

        "  ScheduleTypeLimits,",
        "    FlowRate,                !- Name",
        "    0.0,                     !- Lower Limit Value",
        "    10,                      !- Upper Limit Value",
        "    CONTINUOUS;              !- Numeric Type",

        "  ScheduleTypeLimits,",
        "    Humidity,                !- Name",
        "    0,                       !- Lower Limit Value",
        "    100,                     !- Upper Limit Value",
        "    CONTINUOUS;              !- Numeric Type",

        "  Schedule:Compact,",
        "    OCCUPY-1,                !- Name",
        "    Fraction,                !- Schedule Type Limits Name",
        "    Through: 12/31,          !- Field 1",
        "    For: WeekDays SummerDesignDay CustomDay1 CustomDay2, !- Field 2",
        "    Until: 8:00,0.0,         !- Field 3",
        "    Until: 11:00,1.00,       !- Field 5",
        "    Until: 12:00,0.80,       !- Field 7",
        "    Until: 13:00,0.40,       !- Field 9",
        "    Until: 14:00,0.80,       !- Field 11",
        "    Until: 18:00,1.00,       !- Field 13",
        "    Until: 19:00,0.50,       !- Field 15",
        "    Until: 21:00,0.10,       !- Field 17",
        "    Until: 24:00,0.0,        !- Field 19",
        "    For: Weekends WinterDesignDay Holiday, !- Field 21",
        "    Until: 24:00,0.0;        !- Field 22",

        "  Schedule:Compact,",
        "    LIGHTS-1,                !- Name",
        "    Fraction,                !- Schedule Type Limits Name",
        "    Through: 12/31,          !- Field 1",
        "    For: WeekDays SummerDesignDay CustomDay1 CustomDay2, !- Field 2",
        "    Until: 8:00,0.05,        !- Field 3",
        "    Until: 9:00,0.9,         !- Field 5",
        "    Until: 10:00,0.95,       !- Field 7",
        "    Until: 11:00,1.00,       !- Field 9",
        "    Until: 12:00,0.95,       !- Field 11",
        "    Until: 13:00,0.8,        !- Field 13",
        "    Until: 14:00,0.9,        !- Field 15",
        "    Until: 18:00,1.00,       !- Field 17",
        "    Until: 19:00,0.60,       !- Field 19",
        "    Until: 21:00,0.20,       !- Field 21",
        "    Until: 24:00,0.05,       !- Field 23",
        "    For: Weekends WinterDesignDay Holiday, !- Field 25",
        "    Until: 24:00,0.05;       !- Field 26",

        "  Schedule:Compact,",
        "    EQUIP-1,                 !- Name",
        "    Fraction,                !- Schedule Type Limits Name",
        "    Through: 12/31,          !- Field 1",
        "    For: WeekDays SummerDesignDay CustomDay1 CustomDay2, !- Field 2",
        "    Until: 8:00,0.02,        !- Field 3",
        "    Until: 9:00,0.4,         !- Field 5",
        "    Until: 14:00,0.9,        !- Field 7",
        "    Until: 15:00,0.8,        !- Field 9",
        "    Until: 16:00,0.7,        !- Field 11",
        "    Until: 18:00,0.5,        !- Field 13",
        "    Until: 20:00,0.3,        !- Field 15",
        "    Until: 24:00,0.02,       !- Field 17",
        "    For: Weekends WinterDesignDay Holiday, !- Field 19",
        "    Until: 24:00,0.2;        !- Field 20",

        "  Schedule:Compact,",
        "    INFIL-SCH,               !- Name",
        "    Fraction,                !- Schedule Type Limits Name",
        "    Through: 3/31,           !- Field 1",
        "    For: WeekDays SummerDesignDay CustomDay1 CustomDay2, !- Field 2",
        "    Until: 24:00,1.0,        !- Field 3",
        "    For: AllOtherDays,       !- Field 5",
        "    Until: 24:00,1.0,        !- Field 6",
        "    Through: 9/30,           !- Field 8",
        "    For: AllDays,            !- Field 9",
        "    Until: 24:00,0.7,        !- Field 10",
        "    Through: 12/31,          !- Field 12",
        "    For: AllDays,            !- Field 13",
        "    Until: 24:00,1.0,        !- Field 14",
        "    For: AllOtherDays,       !- Field 16",
        "    Until: 24:00,0.2;        !- Field 17",

        "  Schedule:Compact,",
        "    ActSchd,                 !- Name",
        "    Any Number,              !- Schedule Type Limits Name",
        "    Through: 12/31,          !- Field 1",
        "    For: AllDays,            !- Field 2",
        "    Until: 24:00,117.239997864; !- Field 3",

        "  Schedule:Compact,",
        "    ShadeTransSch,           !- Name",
        "    Fraction,                !- Schedule Type Limits Name",
        "    Through: 12/31,          !- Field 1",
        "    For: AllDays,            !- Field 2",
        "    Until: 24:00,0.0;        !- Field 3",

        "  Schedule:Compact,",
        "    Htg-SetP-Sch,            !- Name",
        "    Temperature,             !- Schedule Type Limits Name",
        "    Through: 12/31,          !- Field 1",
        "    For: SummerDesignDay,    !- Field 2",
        "    Until: 24:00,12.8,       !- Field 3",
        "    For: WinterDesignDay,    !- Field 5",
        "    Until: 24:00,21.1,       !- Field 6",
        "    For: WeekEnds Holiday,   !- Field 8",
        "    Until: 7:00,12.8,        !- Field 9",
        "    Until: 13:00,21.1,       !- Field 11",
        "    Until: 24:00,12.8,       !- Field 13",
        "    For: AllOtherDays,       !- Field 15",
        "    Until: 7:00,12.8,        !- Field 16",
        "    Until: 18:00,21.1,       !- Field 18",
        "    Until: 24:00,12.8;       !- Field 20",

        "  Schedule:Compact,",
        "    PlenumHtg-SetP-Sch,      !- Name",
        "    Temperature,             !- Schedule Type Limits Name",
        "    Through: 12/31,          !- Field 1",
        "    For: AllDays,            !- Field 2",
        "    Until: 24:00,12.8;       !- Field 3",

        "  Schedule:Compact,",
        "    Clg-SetP-Sch,            !- Name",
        "    Temperature,             !- Schedule Type Limits Name",
        "    Through: 12/31,          !- Field 1",
        "    For: SummerDesignDay,    !- Field 2",
        "    Until: 24:00,23.9,       !- Field 3",
        "    For: WinterDesignDay,    !- Field 5",
        "    Until: 24:00,40.0,       !- Field 6",
        "    For: WeekEnds Holiday,   !- Field 8",
        "    Until: 7:00,40.0,        !- Field 9",
        "    Until: 13:00,23.9,       !- Field 11",
        "    Until: 24:00,32.2,       !- Field 13",
        "    For: AllOtherDays,       !- Field 15",
        "    Until: 7:00,40.0,        !- Field 16",
        "    Until: 18:00,23.9,       !- Field 18",
        "    Until: 24:00,40.0;       !- Field 20",

        "  Schedule:Compact,",
        "    PlenumClg-SetP-Sch,      !- Name",
        "    Temperature,             !- Schedule Type Limits Name",
        "    Through: 12/31,          !- Field 1",
        "    For: AllDays,            !- Field 2",
        "    Until: 24:00,40.0;       !- Field 3",

        "  Schedule:Compact,",
        "    Zone Control Type Sched, !- Name",
        "    Control Type,            !- Schedule Type Limits Name",
        "    Through: 12/31,          !- Field 1",
        "    For: SummerDesignDay,    !- Field 2",
        "    Until: 24:00,2,          !- Field 3",
        "    For: WinterDesignDay,    !- Field 5",
        "    Until: 24:00,1,          !- Field 6",
        "    For: AllOtherDays,       !- Field 8",
        "    Until: 24:00,4;          !- Field 9",

        "  Schedule:Compact,",
        "    Desiccant Control Type Sched,  !- Name",
        "    Control Type,            !- Schedule Type Limits Name",
        "    Through: 12/31,          !- Field 1",
        "    For: AllDays,            !- Field 2",
        "    Until: 24:00,4;          !- Field 3",

        "  Schedule:Compact,",
        "    Min OA Sched,            !- Name",
        "    Fraction,                !- Schedule Type Limits Name",
        "    Through: 12/31,          !- Field 1",
        "    For: Weekdays,           !- Field 2",
        "    Until: 7:00,0.02,        !- Field 3",
        "    Until: 17:00,1.0,        !- Field 5",
        "    Until: 24:00,0.0,        !- Field 7",
        "    For: AllOtherDays,       !- Field 9",
        "    Until: 24:00,0.02;       !- Field 10",

        "  Schedule:Compact,",
        "    FanAvailSched,           !- Name",
        "    Fraction,                !- Schedule Type Limits Name",
        "    Through: 3/31,           !- Field 1",
        "    For: AllDays,            !- Field 2",
        "    Until: 24:00,1.0,        !- Field 3",
        "    Through: 9/30,           !- Field 5",
        "    For: WeekDays,           !- Field 6",
        "    Until: 7:00,0.0,         !- Field 7",
        "    Until: 17:00,1.0,        !- Field 9",
        "    Until: 24:00,0.0,        !- Field 11",
        "    For: SummerDesignDay WinterDesignDay, !- Field 13",
        "    Until: 24:00,1.0,        !- Field 14",
        "    For: AllOtherDays,       !- Field 16",
        "    Until: 24:00,0.0,        !- Field 17",
        "    Through: 12/31,          !- Field 19",
        "    For: AllDays,            !- Field 20",
        "    Until: 24:00,1.0;        !- Field 21",

        "  Schedule:Compact,",
        "    Seasonal Reset Supply Air Temp Sch,  !- Name",
        "    Temperature,             !- Schedule Type Limits Name",
        "    Through: 3/31,           !- Field 1",
        "    For: AllDays,            !- Field 2",
        "    Until: 24:00,17.0,       !- Field 3",
        "    Through: 9/30,           !- Field 5",
        "    For: AllDays,            !- Field 6",
        "    Until: 24:00,15.0,       !- Field 7",
        "    Through: 12/31,          !- Field 9",
        "    For: AllDays,            !- Field 10",
        "    Until: 24:00,17.0;       !- Field 11",

        "  Schedule:Compact,",
        "    Constant,                !- Name",
        "    Any Number,              !- Schedule Type Limits Name",
        "    Through: 12/31,          !- Field 1",
        "    For: AllDays,            !- Field 2",
        "    Until: 24:00,1.0;        !- Field 3",

        "  People,",
        "    Main Zone People 1,       !- Name",
        "    Main Zone,                !- Zone or ZoneList Name",
        "    OCCUPY-1,                !- Number of People Schedule Name",
        "    people,                  !- Number of People Calculation Method",
        "    20,                      !- Number of People",
        "    ,                        !- People per Zone Floor Area {person/m2}",
        "    ,                        !- Zone Floor Area per Person {m2/person}",
        "    0.3,                     !- Fraction Radiant",
        "    AutoCalculate,           !- Sensible Heat Fraction",
        "    ActSchd;                 !- Activity Level Schedule Name",

        "  Lights,",
        "    Main Zone Lights 1,       !- Name",
        "    Main Zone,                !- Zone or ZoneList Name",
        "    LIGHTS-1,                !- Schedule Name",
        "    LightingLevel,           !- Design Level Calculation Method",
        "    2964,                    !- Lighting Level {W}",
        "    ,                        !- Watts per Zone Floor Area {W/m2}",
        "    ,                        !- Watts per Person {W/person}",
        "    0.2,                     !- Return Air Fraction",
        "    0.59,                    !- Fraction Radiant",
        "    0.2,                     !- Fraction Visible",
        "    0,                       !- Fraction Replaceable",
        "    GeneralLights;           !- End-Use Subcategory",

        "  ElectricEquipment,",
        "    Main Zone ElecEq 1,       !- Name",
        "    Main Zone,                !- Zone or ZoneList Name",
        "    EQUIP-1,                 !- Schedule Name",
        "    EquipmentLevel,          !- Design Level Calculation Method",
        "    19760,                   !- Design Level {W}",
        "    ,                        !- Watts per Zone Floor Area {W/m2}",
        "    ,                        !- Watts per Person {W/person}",
        "    0.5,                     !- Fraction Latent",
        "    0.3,                     !- Fraction Radiant",
        "    0;                       !- Fraction Lost",

        "  ZoneInfiltration:DesignFlowRate,",
        "    Main Zone Infil 1,        !- Name",
        "    Main Zone,                !- Zone or ZoneList Name",
        "    INFIL-SCH,               !- Schedule Name",
        "    flow/zone,               !- Design Flow Rate Calculation Method",
        "    0.031089,                !- Design Flow Rate {m3/s}",
        "    ,                        !- Flow per Zone Floor Area {m3/s-m2}",
        "    ,                        !- Flow per Exterior Surface Area {m3/s-m2}",
        "    ,                        !- Air Changes per Hour {1/hr}",
        "    0,                       !- Constant Term Coefficient",
        "    0,                       !- Temperature Term Coefficient",
        "    0.2237,                  !- Velocity Term Coefficient",
        "    0;                       !- Velocity Squared Term Coefficient",

        "  Sizing:Zone,",
        "    Main Zone,                !- Zone or ZoneList Name",
        "    SupplyAirTemperature,    !- Zone Cooling Design Supply Air Temperature Input Method",
        "    14.,                     !- Zone Cooling Design Supply Air Temperature {C}",
        "    ,                        !- Zone Cooling Design Supply Air Temperature Difference {deltaC}",
        "    SupplyAirTemperature,    !- Zone Heating Design Supply Air Temperature Input Method",
        "    50.,                     !- Zone Heating Design Supply Air Temperature {C}",
        "    ,                        !- Zone Heating Design Supply Air Temperature Difference {deltaC}",
        "    0.009,                   !- Zone Cooling Design Supply Air Humidity Ratio {kgWater/kgDryAir}",
        "    0.004,                   !- Zone Heating Design Supply Air Humidity Ratio {kgWater/kgDryAir}",
        "    SZ DSOA Main Zone,        !- Design Specification Outdoor Air Object Name",
        "    0.0,                     !- Zone Heating Sizing Factor",
        "    0.0,                     !- Zone Cooling Sizing Factor",
        "    DesignDayWithLimit,      !- Cooling Design Air Flow Method",
        "    ,                        !- Cooling Design Air Flow Rate {m3/s}",
        "    ,                        !- Cooling Minimum Air Flow per Zone Floor Area {m3/s-m2}",
        "    ,                        !- Cooling Minimum Air Flow {m3/s}",
        "    ,                        !- Cooling Minimum Air Flow Fraction",
        "    DesignDay,               !- Heating Design Air Flow Method",
        "    ,                        !- Heating Design Air Flow Rate {m3/s}",
        "    ,                        !- Heating Maximum Air Flow per Zone Floor Area {m3/s-m2}",
        "    ,                        !- Heating Maximum Air Flow {m3/s}",
        "    ;                        !- Heating Maximum Air Flow Fraction",

        "  DesignSpecification:OutdoorAir,",
        "    SZ DSOA Main Zone,        !- Name",
        "    sum,                     !- Outdoor Air Method",
        "    0.00236,                 !- Outdoor Air Flow per Person {m3/s-person}",
        "    0.000305,                !- Outdoor Air Flow per Zone Floor Area {m3/s-m2}",
        "    0.0;                     !- Outdoor Air Flow per Zone {m3/s}",

        "  Sizing:System,",
        "    Desiccant DXSystem,      !- AirLoop Name",
        "    sensible,                !- Type of Load to Size On",
        "    autosize,                !- Design Outdoor Air Flow Rate {m3/s}",
        "    0.3,                     !- Central Heating Maximum System Air Flow Ratio",
        "    4.5,                     !- Preheat Design Temperature {C}",
        "    0.008,                   !- Preheat Design Humidity Ratio {kgWater/kgDryAir}",
        "    11.0,                    !- Precool Design Temperature {C}",
        "    0.008,                   !- Precool Design Humidity Ratio {kgWater/kgDryAir}",
        "    12.8,                    !- Central Cooling Design Supply Air Temperature {C}",
        "    16.7,                    !- Central Heating Design Supply Air Temperature {C}",
        "    noncoincident,           !- Type of Zone Sum to Use",
        "    no,                      !- 100% Outdoor Air in Cooling",
        "    no,                      !- 100% Outdoor Air in Heating",
        "    0.008,                   !- Central Cooling Design Supply Air Humidity Ratio {kgWater/kgDryAir}",
        "    0.008,                   !- Central Heating Design Supply Air Humidity Ratio {kgWater/kgDryAir}",
        "    DesignDay,               !- Cooling Design Air Flow Method",
        "    0,                       !- Cooling Design Air Flow Rate {m3/s}",
        "    ,                        !- Supply Air Flow Rate Per Floor Area During Cooling Operation {m3/s-m2}",
        "    ,                        !- Fraction of Autosized Design Cooling Supply Air Flow Rate",
        "    ,                        !- Design Supply Air Flow Rate Per Unit Cooling Capacity {m3/s-W}",
        "    DesignDay,               !- Heating Design Air Flow Method",
        "    0,                       !- Heating Design Air Flow Rate {m3/s}",
        "    ,                        !- Supply Air Flow Rate Per Floor Area During Heating Operation {m3/s-m2}",
        "    ,                        !- Fraction of Autosized Design Heating Supply Air Flow Rate",
        "    ,                        !- Fraction of Autosized Design Cooling Supply Air Flow Rate",
        "    ,                        !- Design Supply Air Flow Rate Per Unit Heating Capacity {m3/s-W}",
        "    ,                        !- System Outdoor Air Method",
        "    1.0,                     !- Zone Maximum Outdoor Air Fraction {dimensionless}",
        "    CoolingDesignCapacity,   !- Cooling Design Capacity Method",
        "    autosize,                !- Cooling Design Capacity {W}",
        "    ,                        !- Cooling Design Capacity Per Floor Area {W/m2}",
        "    ,                        !- Fraction of Autosized Cooling Design Capacity",
        "    HeatingDesignCapacity,   !- Heating Design Capacity Method",
        "    autosize,                !- Heating Design Capacity {W}",
        "    ,                        !- Heating Design Capacity Per Floor Area {W/m2}",
        "    ,                        !- Fraction of Autosized Heating Design Capacity",
        "    VAV;                     !- Central Cooling Capacity Control Method",

        "  Curve:Biquadratic,",
        "    HPACCoolCapFT,           !- Name",
        "    0.942587793,             !- Coefficient1 Constant",
        "    0.009543347,             !- Coefficient2 x",
        "    0.00068377,              !- Coefficient3 x**2",
        "    -0.011042676,            !- Coefficient4 y",
        "    0.000005249,             !- Coefficient5 y**2",
        "    -0.00000972,             !- Coefficient6 x*y",
        "    -100,                    !- Minimum Value of x",
        "    100,                     !- Maximum Value of x",
        "    -100,                    !- Minimum Value of y",
        "    100,                     !- Maximum Value of y",
        "    ,                        !- Minimum Curve Output",
        "    ,                        !- Maximum Curve Output",
        "    Temperature,             !- Input Unit Type for X",
        "    Temperature,             !- Input Unit Type for Y",
        "    Dimensionless;           !- Output Unit Type",

        "  Curve:Biquadratic,",
        "    HPACCOOLEIRFT,           !- Name",
        "    0.342414409,             !- Coefficient1 Constant",
        "    0.034885008,             !- Coefficient2 x",
        "    -0.0006237,              !- Coefficient3 x**2",
        "    0.004977216,             !- Coefficient4 y",
        "    0.000437951,             !- Coefficient5 y**2",
        "    -0.000728028,            !- Coefficient6 x*y",
        "    -100,                    !- Minimum Value of x",
        "    100,                     !- Maximum Value of x",
        "    -100,                    !- Minimum Value of y",
        "    100,                     !- Maximum Value of y",
        "    ,                        !- Minimum Curve Output",
        "    ,                        !- Maximum Curve Output",
        "    Temperature,             !- Input Unit Type for X",
        "    Temperature,             !- Input Unit Type for Y",
        "    Dimensionless;           !- Output Unit Type",

        "  Curve:Quadratic,",
        "    HPACCoolCapFFF,          !- Name",
        "    0.8,                     !- Coefficient1 Constant",
        "    0.2,                     !- Coefficient2 x",
        "    0.0,                     !- Coefficient3 x**2",
        "    0.0,                     !- Minimum Value of x",
        "    1.0;                     !- Maximum Value of x",

        "  Curve:Quadratic,",
        "    HPACCOOLEIRFFF,          !- Name",
        "    1.1552,                  !- Coefficient1 Constant",
        "    -0.1808,                 !- Coefficient2 x",
        "    0.0256,                  !- Coefficient3 x**2",
        "    0.0,                     !- Minimum Value of x",
        "    1.0;                     !- Maximum Value of x",

        "  Curve:Quadratic,",
        "    HPACCOOLPLFFPLR,         !- Name",
        "    0.75,                    !- Coefficient1 Constant",
        "    0.25,                    !- Coefficient2 x",
        "    0.0,                     !- Coefficient3 x**2",
        "    0.0,                     !- Minimum Value of x",
        "    1.0;                     !- Maximum Value of x",

        "  Curve:Cubic,",
        "    PackagedRatedCoolCapFFlow,  !- Name",
        "    0.47278589,              !- Coefficient1 Constant",
        "    1.2433415,               !- Coefficient2 x",
        "    -1.0387055,              !- Coefficient3 x**2",
        "    0.32257813,              !- Coefficient4 x**3",
        "    0.5,                     !- Minimum Value of x",
        "    1.5,                     !- Maximum Value of x",
        "    ,                        !- Minimum Curve Output",
        "    ,                        !- Maximum Curve Output",
        "    Dimensionless,           !- Input Unit Type for X",
        "    Dimensionless;           !- Output Unit Type",

        "  Curve:Cubic,",
        "    PackagedRatedCoolEIRFFlow,  !- Name",
        "    1.0079484,               !- Coefficient1 Constant",
        "    0.34544129,              !- Coefficient2 x",
        "    -.6922891,               !- Coefficient3 x**2",
        "    0.33889943,              !- Coefficient4 x**3",
        "    0.5,                     !- Minimum Value of x",
        "    1.5,                     !- Maximum Value of x",
        "    ,                        !- Minimum Curve Output",
        "    ,                        !- Maximum Curve Output",
        "    Dimensionless,           !- Input Unit Type for X",
        "    Dimensionless;           !- Output Unit Type",

        "  Curve:Cubic,",
        "    EXHAUSTFANPLF,           !- Name",
        "    0,                       !- Coefficient1 Constant",
        "    1,                       !- Coefficient2 x",
        "    0.0,                     !- Coefficient3 x**2",
        "    0.0,                     !- Coefficient4 x**3",
        "    0.0,                     !- Minimum Value of x",
        "    1.0;                     !- Maximum Value of x",

        "  NodeList,",
        "    Main Zone In Nodes,       !- Name",
        "    Main Zone In Node;        !- Node 1 Name",

        "  NodeList,",
        "    Desiccant DX Air Loop Setpoint Nodes,  !- Name",
        "    HX Process Outlet Node,  !- Node 1 Name",
        "    Desiccant DXSystem Mixed Air Node;  !- Node 2 Name",

        "  NodeList,",
        "    Regen Setpoint Nodes,    !- Name",
        "    HX Regen Inlet Node;     !- Node 1 Name",

        "  BranchList,",
        "    Desiccant Air Loop Branches,  !- Name",
        "    Desiccant Air Loop Main Branch;  !- Branch 1 Name",

        "  Branch,",
        "    Desiccant Air Loop Main Branch,  !- Name",
        "    ,                        !- Pressure Drop Curve Name",
        "    AirLoopHVAC:OutdoorAirSystem,  !- Component 1 Object Type",
        "    Desiccant DXSystem OA System,  !- Component 1 Name",
        "    Desiccant DXSystem Air Loop Inlet Node,  !- Component 1 Inlet Node Name",
        "    Desiccant DXSystem Mixed Air Node,  !- Component 1 Outlet Node Name",
        "    CoilSystem:Cooling:DX,   !- Component 2 Object Type",
        "    Desiccant DXSystem Cooling Coil Sys,  !- Component 2 Name",
        "    Desiccant DXSystem Mixed Air Node,  !- Component 2 Inlet Node Name",
        "    HX Process Inlet Node,   !- Component 2 Outlet Node Name",
        "    Dehumidifier:Desiccant:System,  !- Component 3 Object Type",
        "    Desiccant 1,             !- Component 3 Name",
        "    HX Process Inlet Node,   !- Component 3 Inlet Node Name",
        "    HX Process Outlet Node,  !- Component 3 Outlet Node Name",
        "    Fan:ConstantVolume,      !- Component 4 Object Type",
        "    Desiccant DXSystem Supply Fan,  !- Component 4 Name",
        "    HX Process Outlet Node,  !- Component 4 Inlet Node Name",
        "    Desiccant Supply Fan Outlet Node,  !- Component 4 Outlet Node Name",
        "    Coil:Heating:Fuel,        !- Component 5 Object Type",
        "    Desiccant Process Heating Coil,  !- Component 5 Name",
        "    Desiccant Supply Fan Outlet Node,  !- Component 5 Inlet Node Name",
        "    Desiccant DXSystem Air Loop Outlet Node;  !- Component 5 Outlet Node Name",

        "  AirLoopHVAC,",
        "    Desiccant DXSystem,      !- Name",
        "    ,                        !- Controller List Name",
        "    Desiccant Avail List,    !- Availability Manager List Name",
        "    autosize,                !- Design Supply Air Flow Rate {m3/s}",
        "    Desiccant Air Loop Branches,  !- Branch List Name",
        "    ,                        !- Connector List Name",
        "    Desiccant DXSystem Air Loop Inlet Node,  !- Supply Side Inlet Node Name",
        "    Desiccant DXSystem Return Air Mixer Outlet Node,  !- Demand Side Outlet Node Name",
        "    Desiccant DXSystem Zone Equipment Inlet Node,  !- Demand Side Inlet Node Names",
        "    Desiccant DXSystem Air Loop Outlet Node;  !- Supply Side Outlet Node Names",

        "  AirLoopHVAC:ControllerList,",
        "    Desiccant DXSystem OA System Controllers,  !- Name",
        "    Controller:OutdoorAir,   !- Controller 1 Object Type",
        "    Desiccant OA Controller; !- Controller 1 Name",

        "  AirLoopHVAC:OutdoorAirSystem:EquipmentList,",
        "    Desiccant DXSystem OA System Equipment,  !- Name",
        "    OutdoorAir:Mixer,        !- Component 1 Object Type",
        "    Desiccant DXSystem OA Mixing Box;  !- Component 1 Name",

        "  AirLoopHVAC:OutdoorAirSystem,",
        "    Desiccant DXSystem OA System,  !- Name",
        "    Desiccant DXSystem OA System Controllers,  !- Controller List Name",
        "    Desiccant DXSystem OA System Equipment,  !- Outdoor Air Equipment List Name",
        "    Desiccant Outdoor Air Avail List;  !- Availability Manager List Name",

        "  OutdoorAir:NodeList,",
        "    OutsideAirInletNodes;    !- Node or NodeList Name 1",

        "  OutdoorAir:NodeList,",
        "    Desiccant DXSystem Outside Air Inlet Node,  !- Node or NodeList Name 1",
        "    Desiccant Outside Air Inlet Node 2;  !- Node or NodeList Name 2",

        "  OutdoorAir:NodeList,",
        "    Regen Coil Inlet Node;   !- Node or NodeList Name 1",

        "  OutdoorAir:NodeList,",
        "    Condenser Inlet Node;    !- Node or NodeList Name 1",

        "  OutdoorAir:Mixer,",
        "    Desiccant DXSystem OA Mixing Box,  !- Name",
        "    Desiccant DXSystem Mixed Air Node,  !- Mixed Air Node Name",
        "    Desiccant DXSystem Outside Air Inlet Node,  !- Outdoor Air Stream Node Name",
        "    Desiccant DXSystem Relief Air Outlet Node,  !- Relief Air Stream Node Name",
        "    Desiccant DXSystem Air Loop Inlet Node;  !- Return Air Stream Node Name",

        "  AvailabilityManagerAssignmentList,",
        "    Desiccant Avail List,    !- Name",
        "    AvailabilityManager:Scheduled,  !- Availability Manager 1 Object Type",
        "    Desiccant DXSystem Avail;!- Availability Manager 1 Name",

        "  AvailabilityManagerAssignmentList,",
        "    Desiccant Outdoor Air Avail List,  !- Name",
        "    AvailabilityManager:Scheduled,  !- Availability Manager 1 Object Type",
        "    Desiccant Outdoor Air Avail;  !- Availability Manager 1 Name",

        "  AvailabilityManager:Scheduled,",
        "    Desiccant DXSystem Avail,!- Name",
        "    FanAvailSched;           !- Schedule Name",

        "  AvailabilityManager:Scheduled,",
        "    Desiccant Outdoor Air Avail,  !- Name",
        "    FanAvailSched;           !- Schedule Name",

        "  SetpointManager:SingleZone:Heating,",
        "    Heating Supply Air Temp Manager,  !- Name",
        "    Temperature,             !- Control Variable",
        "    -99.,                    !- Minimum Supply Air Temperature {C}",
        "    99.,                     !- Maximum Supply Air Temperature {C}",
        "    Main Zone,                !- Control Zone Name",
        "    Main Zone Node,           !- Zone Node Name",
        "    Main Zone In Node,        !- Zone Inlet Node Name",
        "    Desiccant DXSystem Air Loop Outlet Node;  !- Setpoint Node or NodeList Name",

        "  SetpointManager:SingleZone:Cooling,",
        "    Cooling Supply Air Temp Manager,  !- Name",
        "    Temperature,             !- Control Variable",
        "    -99.,                    !- Minimum Supply Air Temperature {C}",
        "    99.,                     !- Maximum Supply Air Temperature {C}",
        "    Main Zone,                !- Control Zone Name",
        "    Main Zone Node,           !- Zone Node Name",
        "    Main Zone In Node,        !- Zone Inlet Node Name",
        "    Desiccant Supply Fan Outlet Node;  !- Setpoint Node or NodeList Name",

        "  SetpointManager:SingleZone:Humidity:Maximum,",
        "    Zone Max Humidity Manager,  !- Name",
        "    HX Process Outlet Node,  !- Setpoint Node or NodeList Name",
        "    Main Zone Node;           !- Control Zone Air Node Name",

        "  SetpointManager:MixedAir,",
        "    Adjusted Cooling Supply Air Temp Manager,  !- Name",
        "    Temperature,             !- Control Variable",
        "    Desiccant Supply Fan Outlet Node,  !- Reference Setpoint Node Name",
        "    HX Process Outlet Node,  !- Fan Inlet Node Name",
        "    Desiccant Supply Fan Outlet Node,  !- Fan Outlet Node Name",
        "    HX Process Outlet Node;  !- Setpoint Node or NodeList Name",

        "  Controller:OutdoorAir,",
        "    Desiccant OA Controller, !- Name",
        "    Desiccant DXSystem Relief Air Outlet Node,  !- Relief Air Outlet Node Name",
        "    Desiccant DXSystem Air Loop Inlet Node,  !- Return Air Node Name",
        "    Desiccant DXSystem Mixed Air Node,  !- Mixed Air Node Name",
        "    Desiccant DXSystem Outside Air Inlet Node,  !- Actuator Node Name",
        "    autosize,                !- Minimum Outdoor Air Flow Rate {m3/s}",
        "    autosize,                !- Maximum Outdoor Air Flow Rate {m3/s}",
        "    NoEconomizer,            !- Economizer Control Type",
        "    ModulateFlow,            !- Economizer Control Action Type",
        "    100.,                    !- Economizer Maximum Limit Dry-Bulb Temperature {C}",
        "    ,                        !- Economizer Maximum Limit Enthalpy {J/kg}",
        "    ,                        !- Economizer Maximum Limit Dewpoint Temperature {C}",
        "    ,                        !- Electronic Enthalpy Limit Curve Name",
        "    ,                        !- Economizer Minimum Limit Dry-Bulb Temperature {C}",
        "    NoLockout,               !- Lockout Type",
        "    FixedMinimum,            !- Minimum Limit Type",
        "    Min OA Sched,            !- Minimum Outdoor Air Schedule Name",
        "    ,                        !- Minimum Fraction of Outdoor Air Schedule Name",
        "    ,                        !- Maximum Fraction of Outdoor Air Schedule Name",
        "    0;                       !- Mechanical Ventilation Controller Name",

        "  ZoneHVAC:EquipmentConnections,",
        "    Main Zone,                !- Zone Name",
        "    Main Zone Eq,             !- Zone Conditioning Equipment List Name",
        "    Main Zone In Nodes,       !- Zone Air Inlet Node or NodeList Name",
        "    ,                         !- Zone Air Exhaust Node or NodeList Name",
        "    Main Zone Node,           !- Zone Air Node Name",
        "    Main Zone Out Node;       !- Zone Return Air Node Name",

        "  ZoneHVAC:EquipmentList,",
        "    Main Zone Eq,             !- Name",
        "    SequentialLoad,          !- Load Distribution Scheme",
        "    ZoneHVAC:AirDistributionUnit,  !- Zone Equipment 1 Object Type",
        "    Main Zone Direct Air ADU,     !- Zone Equipment 1 Name",
        "    1,                        !- Zone Equipment 1 Cooling Sequence",
        "    1;                        !- Zone Equipment 1 Heating or No-Load Sequence",

        "  ZoneHVAC:AirDistributionUnit,",
        "    Main Zone Direct Air ADU, !- Name",
        "    Main Zone In Node,       !- Air Distribution Unit Outlet Node Name",
        "    AirTerminal:SingleDuct:ConstantVolume:NoReheat,  !- Air Terminal Object Type",
        "    Main Zone Direct Air;    !- Air Terminal Name",

        "  AirTerminal:SingleDuct:ConstantVolume:NoReheat,",
        "    Main Zone Direct Air,    !- Name",
        "    Constant,                !- Availability Schedule Name",
        "    Main Zone In Node 2AT,   !- Air Inlet Node Name",
        "    Main Zone In Node,       !- Air Outlet Node Name",
        "    autosize;                !- Maximum Air Flow Rate {m3/s}",

        "  CoilSystem:Cooling:DX,",
        "    Desiccant DXSystem Cooling Coil Sys,  !- Name",
        "    FanAvailSched,           !- Availability Schedule Name",
        "    Desiccant DXSystem Mixed Air Node,  !- DX Cooling Coil System Inlet Node Name",
        "    HX Process Inlet Node,   !- DX Cooling Coil System Outlet Node Name",
        "    HX Process Outlet Node,  !- DX Cooling Coil System Sensor Node Name",
        "    Coil:Cooling:DX:VariableSpeed,  !- Cooling Coil Object Type",
        "    Desiccant DXSystem Cooling Coil;  !- Cooling Coil Name",

        "  ZoneControl:Thermostat,",
        "    Main Zone Control,        !- Name",
        "    Main Zone,                !- Zone or ZoneList Name",
        "    Desiccant Control Type Sched,  !- Control Type Schedule Name",
        "    ThermostatSetpoint:DualSetpoint,  !- Control 1 Object Type",
        "    DualSetPoint;            !- Control 1 Name",

        "  ZoneControl:Humidistat,",
        "    Main Zone Humidistat,     !- Name",
        "    Main Zone,                !- Zone Name",
        "    Relative Humidity Sched; !- Humidifying Relative Humidity Setpoint Schedule Name",

        "  Schedule:Compact,",
        "    Relative Humidity Sched, !- Name",
        "    Any Number,              !- Schedule Type Limits Name",
        "    Through: 12/31,          !- Field 1",
        "    For: AllDays,            !- Field 2",
        "    Until: 24:00,45.0;       !- Field 3",

        "  ThermostatSetpoint:SingleHeating,",
        "    HeatingSetpoint,         !- Name",
        "    Htg-SetP-Sch;            !- Setpoint Temperature Schedule Name",

        "  ThermostatSetpoint:SingleHeating,",
        "    PlenumHeatingSetpoint,   !- Name",
        "    PlenumHtg-SetP-Sch;      !- Setpoint Temperature Schedule Name",

        "  ThermostatSetpoint:SingleCooling,",
        "    CoolingSetpoint,         !- Name",
        "    Clg-SetP-Sch;            !- Setpoint Temperature Schedule Name",

        "  ThermostatSetpoint:SingleCooling,",
        "    PlenumCoolingSetpoint,   !- Name",
        "    PlenumClg-SetP-Sch;      !- Setpoint Temperature Schedule Name",

        "  ThermostatSetpoint:DualSetpoint,",
        "    DualSetPoint,            !- Name",
        "    Htg-SetP-Sch,            !- Heating Setpoint Temperature Schedule Name",
        "    Clg-SetP-Sch;            !- Cooling Setpoint Temperature Schedule Name",

        "  AirLoopHVAC:SupplyPath,",
        "    Desiccant DXSystem Supply Path,  !- Name",
        "    Desiccant DXSystem Zone Equipment Inlet Node,  !- Supply Air Path Inlet Node Name",
        "    AirLoopHVAC:ZoneSplitter,!- Component 1 Object Type",
        "    Desiccant DXSystem Zone Supply Air Splitter;  !- Component 1 Name",

        "  AirLoopHVAC:ReturnPath,",
        "    Desiccant DXSystem Return Path,  !- Name",
        "    Desiccant DXSystem Return Air Mixer Outlet Node,  !- Return Air Path Outlet Node Name",
        "    AirLoopHVAC:ZoneMixer,   !- Component 1 Object Type",
        "    Desiccant DXSystem Zone Return Air Mixer;  !- Component 1 Name",

        "  AirLoopHVAC:ZoneSplitter,",
        "    Desiccant DXSystem Zone Supply Air Splitter,  !- Name",
        "    Desiccant DXSystem Zone Equipment Inlet Node,  !- Inlet Node Name",
        "    Main Zone In Node 2AT;        !- Outlet 1 Node Name",

        "  AirLoopHVAC:ZoneMixer,",
        "    Desiccant DXSystem Zone Return Air Mixer,  !- Name",
        "    Desiccant DXSystem Return Air Mixer Outlet Node,  !- Outlet Node Name",
        "    Main Zone Out Node;       !- Inlet 1 Node Name",

        "  Coil:Heating:Fuel,",
        "    Desiccant Process Heating Coil,  !- Name",
        "    Constant,                !- Availability Schedule Name",
        "    Gas,                     !- Fuel Type",
        "    0.8,                     !- Gas Burner Efficiency",
        "    autosize,                !- Nominal Capacity {W}",
        "    Desiccant Supply Fan Outlet Node,  !- Air Inlet Node Name",
        "    Desiccant DXSystem Air Loop Outlet Node,  !- Air Outlet Node Name",
        "    Desiccant DXSystem Air Loop Outlet Node;  !- Temperature Setpoint Node Name",

        "  Coil:Heating:Fuel,",
        "    Desiccant Regen Coil,    !- Name",
        "    FanAvailSched,           !- Availability Schedule Name",
        "    Gas,                     !- Fuel Type",
        "    1,                       !- Gas Burner Efficiency",
        "    autosize,                !- Nominal Capacity {W}",
        "    Regen Coil Inlet Node,   !- Air Inlet Node Name",
        "    HX Regen Inlet Node;     !- Air Outlet Node Name",

        "  Coil:Cooling:DX:VariableSpeed,",
        "    Desiccant DXSystem Cooling Coil,    !- Name",
        "     Desiccant DXSystem Mixed Air Node,  !- Indoor Air Inlet Node Name",
        "    HX Process Inlet Node,   !- Indoor Air Outlet Node Name",
        "    1.0,                     !- Number of Speeds {dimensionless}",
        "    1.0,                     !- Nominal Speed Level {dimensionless}",
        "    22000,                   !- Gross Rated Total Cooling Capacity At Selected Nominal Speed Level {w}",
        "    1.05,                    !- Rated Air Flow Rate At Selected Nominal Speed Level {m3/s}",
        "    1000,                    !- Nominal Time for Condensate to Begin Leaving the Coil {s}",
        "    0.4,                     !- Initial Moisture Evaporation Rate Divided by Steady-State AC Latent Capacity {dimensionless}",
        "    HPACCOOLPLFFPLR,         !- Energy Part Load Fraction Curve Name",
        "    Condenser Inlet Node,    !- Condenser Air Inlet Node Name",
        "    AirCooled,               !- Condenser Type",
        "    ,                        !- Evaporative Condenser Pump Rated Power Consumption {W}",
        "    0.0,                     !- Crankcase Heater Capacity {W}",
        "    10.0,                    !- Maximum Outdoor Dry-Bulb Temperature for Crankcase Heater Operation {C}",
        "    ,                        !- Minimum Outdoor Dry-Bulb Temperature for Compressor Operation {C}",
        "    ,                        !- Supply Water Storage Tank Name",
        "    ,                        !- Condensate Collection Water Storage Tank Name",
        "    ,                        !- Basin Heater Capacity {W/K}",
        "    ,                        !- Basin Heater Setpoint Temperature {C}",
        "    ,                        !- Basin Heater Operating Schedule Name",
        "    36991.44197,             !- Speed 1 Reference Unit Gross Rated Total Cooling Capacity {w}",
        "    0.75,                    !- Speed 1 Reference Unit Gross Rated Sensible Heat Ratio {dimensionless}",
        "    3.5,                     !- Speed 1 Reference Unit Gross Rated Cooling COP {dimensionless}",
        "    3.776,                   !- Speed 1 Reference Unit Rated Air Flow Rate {m3/s}",
        "    10.62,                   !- Speed 1 Reference Unit Rated Condenser Air Flow Rate {m3/s}",
        "    ,                        !- Speed 1 Reference Unit Rated Pad Effectiveness of Evap Precooling {dimensionless}",
        "    HPCoolingCAPFTemp4,      !- Speed 1 Total Cooling Capacity Function of Temperature Curve Name",
        "    HPACFFF,                 !- Speed 1 Total Cooling Capacity Function of Air Flow Fraction Curve Name",
        "    HPCoolingEIRFTemp4,      !- Speed 1 Energy Input Ratio Function of Temperature Curve Name",
        "    HPACFFF;                 !- Speed 1 Energy Input Ratio Function of Air Flow Fraction Curve Name",

        "  Curve:Cubic,",
        "    HPACFFF,                 !- Name",
        "    1.0,                     !- Coefficient1 Constant",
        "    0.0,                     !- Coefficient2 x",
        "    0.0,                     !- Coefficient3 x**2",
        "    0.0,                     !- Coefficient4 x**3",
        "    0.5,                     !- Minimum Value of x",
        "    1.5;                     !- Maximum Value of x",

        "  Curve:Biquadratic,",
        "    HPCoolingEIRFTemp4,      !- Name",
        "    0.0001514017,            !- Coefficient1 Constant",
        "    0.0655062896,            !- Coefficient2 x",
        "    -0.0020370821,           !- Coefficient3 x**2",
        "    0.0067823041,            !- Coefficient4 y",
        "    0.0004087196,            !- Coefficient5 y**2",
        "    -0.0003552302,           !- Coefficient6 x*y",
        "    13.89,                   !- Minimum Value of x",
        "    22.22,                   !- Maximum Value of x",
        "    12.78,                   !- Minimum Value of y",
        "    51.67,                   !- Maximum Value of y",
        "    0.5141,                  !- Minimum Curve Output",
        "    1.7044,                  !- Maximum Curve Output",
        "    Temperature,             !- Input Unit Type for X",
        "    Temperature,             !- Input Unit Type for Y",
        "    Dimensionless;           !- Output Unit Type",

        "  Curve:Biquadratic,",
        "    HPCoolingCAPFTemp4,      !- Name",
        "    1.3544202152,            !- Coefficient1 Constant",
        "    -0.0493402773,           !- Coefficient2 x",
        "    0.0022649843,            !- Coefficient3 x**2",
        "    0.0008517727,            !- Coefficient4 y",
        "    -0.0000426316,           !- Coefficient5 y**2",
        "    -0.0003364517,           !- Coefficient6 x*y",
        "    13.89,                   !- Minimum Value of x",
        "    22.22,                   !- Maximum Value of x",
        "    12.78,                   !- Minimum Value of y",
        "    51.67,                   !- Maximum Value of y",
        "    0.7923,                  !- Minimum Curve Output",
        "    1.2736,                  !- Maximum Curve Output",
        "    Temperature,             !- Input Unit Type for X",
        "    Temperature,             !- Input Unit Type for Y",
        "    Dimensionless;           !- Output Unit Type",

        "  Fan:ConstantVolume,",
        "    Desiccant DXSystem Supply Fan,  !- Name",
        "    FanAvailSched,           !- Availability Schedule Name",
        "    0.7,                     !- Fan Total Efficiency",
        "    571.66,                  !- Pressure Rise {Pa}",
        "    autosize,                !- Maximum Flow Rate {m3/s}",
        "    0.9,                     !- Motor Efficiency",
        "    1.0,                     !- Motor In Airstream Fraction",
        "    HX Process Outlet Node,  !- Air Inlet Node Name",
        "    Desiccant Supply Fan Outlet Node;  !- Air Outlet Node Name",

        "  Fan:ConstantVolume,",
        "    Desiccant Regen Fan,     !- Name",
        "    FanAvailSched,           !- Availability Schedule Name",
        "    0.7,                     !- Fan Total Efficiency",
        "    205.5,                   !- Pressure Rise {Pa}",
        "    autosize,                !- Maximum Flow Rate {m3/s}",
        "    0.9,                     !- Motor Efficiency",
        "    1.0,                     !- Motor In Airstream Fraction",
        "    HX Regen Outlet Node,    !- Air Inlet Node Name",
        "    Regen Fan Outlet Node;   !- Air Outlet Node Name",

        "  Dehumidifier:Desiccant:System,",
        "    Desiccant 1,             !- Name",
        "    FanAvailSched,           !- Availability Schedule Name",
        "    HeatExchanger:Desiccant:BalancedFlow,  !- Desiccant Heat Exchanger Object Type",
        "    Desiccant Heat Exchanger 1,  !- Desiccant Heat Exchanger Name",
        "    HX Process Outlet Node,  !- Sensor Node Name",
        "    Fan:ConstantVolume,      !- Regeneration Air Fan Object Type",
        "    Desiccant Regen Fan,     !- Regeneration Air Fan Name",
        "    DrawThrough,             !- Regeneration Air Fan Placement",
        "    Coil:Heating:Fuel,        !- Regeneration Air Heater Object Type",
        "    Desiccant Regen Coil,    !- Regeneration Air Heater Name",
        "    46.111111,               !- Regeneration Inlet Air Setpoint Temperature {C}",
        "    Coil:Cooling:DX:VariableSpeed,  !- Companion Cooling Coil Object Type",
        "    Desiccant DXSystem Cooling Coil,  !- Companion Cooling Coil Name",
        "    Yes,                     !- Companion Cooling Coil Upstream of Dehumidifier Process Inlet",
        "    Yes,                     !- Companion Coil Regeneration Air Heating",
        "    0.87931,                 !- Exhaust Fan Maximum Flow Rate {m3/s}",
        "    50,                      !- Exhaust Fan Maximum Power {W}",
        "    EXHAUSTFANPLF;           !- Exhaust Fan Power Curve Name",

        "  HeatExchanger:Desiccant:BalancedFlow,",
        "    Desiccant Heat Exchanger 1,  !- Name",
        "    FanAvailSched,           !- Availability Schedule Name",
        "    HX Regen Inlet Node,     !- Regeneration Air Inlet Node Name",
        "    HX Regen Outlet Node,    !- Regeneration Air Outlet Node Name",
        "    HX Process Inlet Node,   !- Process Air Inlet Node Name",
        "    HX Process Outlet Node,  !- Process Air Outlet Node Name",
        "    HeatExchanger:Desiccant:BalancedFlow:PerformanceDataType1,  !- Heat Exchanger Performance Object Type",
        "    HXDesPerf1;              !- Heat Exchanger Performance Name",

        "  HeatExchanger:Desiccant:BalancedFlow:PerformanceDataType1,",
        "    HXDesPerf1,              !- Name",
        "    0.87931,                 !- Nominal Air Flow Rate {m3/s}",
        "    3.25,                    !- Nominal Air Face Velocity {m/s}",
        "    50.0,                    !- Nominal Electric Power {W}",
        "    -2.53636E+00,            !- Temperature Equation Coefficient 1",
        "    2.13247E+01,             !- Temperature Equation Coefficient 2",
        "    9.23308E-01,             !- Temperature Equation Coefficient 3",
        "    9.43276E+02,             !- Temperature Equation Coefficient 4",
        "    -5.92367E+01,            !- Temperature Equation Coefficient 5",
        "    -4.27465E-02,            !- Temperature Equation Coefficient 6",
        "    1.12204E+02,             !- Temperature Equation Coefficient 7",
        "    7.78252E-01,             !- Temperature Equation Coefficient 8",
        "    0.007143,                !- Minimum Regeneration Inlet Air Humidity Ratio for Temperature Equation {kgWater/kgDryAir}",
        "    0.024286,                !- Maximum Regeneration Inlet Air Humidity Ratio for Temperature Equation {kgWater/kgDryAir}",
        "    46.111110,               !- Minimum Regeneration Inlet Air Temperature for Temperature Equation {C}",
        "    46.111112,               !- Maximum Regeneration Inlet Air Temperature for Temperature Equation {C}",
        "    0.005000,                !- Minimum Process Inlet Air Humidity Ratio for Temperature Equation {kgWater/kgDryAir}",
        "    0.015714,                !- Maximum Process Inlet Air Humidity Ratio for Temperature Equation {kgWater/kgDryAir}",
        "    4.583333,                !- Minimum Process Inlet Air Temperature for Temperature Equation {C}",
        "    21.83333,                !- Maximum Process Inlet Air Temperature for Temperature Equation {C}",
        "    2.286,                   !- Minimum Regeneration Air Velocity for Temperature Equation {m/s}",
        "    4.826,                   !- Maximum Regeneration Air Velocity for Temperature Equation {m/s}",
        "    35.0,                    !- Minimum Regeneration Outlet Air Temperature for Temperature Equation {C}",
        "    50.0,                    !- Maximum Regeneration Outlet Air Temperature for Temperature Equation {C}",
        "    5.0,                     !- Minimum Regeneration Inlet Air Relative Humidity for Temperature Equation {percent}",
        "    45.0,                    !- Maximum Regeneration Inlet Air Relative Humidity for Temperature Equation {percent}",
        "    80.0,                    !- Minimum Process Inlet Air Relative Humidity for Temperature Equation {percent}",
        "    100.0,                   !- Maximum Process Inlet Air Relative Humidity for Temperature Equation {percent}",
        "    -2.25547E+01,            !- Humidity Ratio Equation Coefficient 1",
        "    9.76839E-01,             !- Humidity Ratio Equation Coefficient 2",
        "    4.89176E-01,             !- Humidity Ratio Equation Coefficient 3",
        "    -6.30019E-02,            !- Humidity Ratio Equation Coefficient 4",
        "    1.20773E-02,             !- Humidity Ratio Equation Coefficient 5",
        "    5.17134E-05,             !- Humidity Ratio Equation Coefficient 6",
        "    4.94917E-02,             !- Humidity Ratio Equation Coefficient 7",
        "    -2.59417E-04,            !- Humidity Ratio Equation Coefficient 8",
        "    0.007143,                !- Minimum Regeneration Inlet Air Humidity Ratio for Humidity Ratio Equation {kgWater/kgDryAir}",
        "    0.024286,                !- Maximum Regeneration Inlet Air Humidity Ratio for Humidity Ratio Equation {kgWater/kgDryAir}",
        "    46.111110,               !- Minimum Regeneration Inlet Air Temperature for Humidity Ratio Equation {C}",
        "    46.111112,               !- Maximum Regeneration Inlet Air Temperature for Humidity Ratio Equation {C}",
        "    0.005000,                !- Minimum Process Inlet Air Humidity Ratio for Humidity Ratio Equation {kgWater/kgDryAir}",
        "    0.015714,                !- Maximum Process Inlet Air Humidity Ratio for Humidity Ratio Equation {kgWater/kgDryAir}",
        "    4.583333,                !- Minimum Process Inlet Air Temperature for Humidity Ratio Equation {C}",
        "    21.83333,                !- Maximum Process Inlet Air Temperature for Humidity Ratio Equation {C}",
        "    2.286,                   !- Minimum Regeneration Air Velocity for Humidity Ratio Equation {m/s}",
        "    4.826,                   !- Maximum Regeneration Air Velocity for Humidity Ratio Equation {m/s}",
        "    0.007914,                !- Minimum Regeneration Outlet Air Humidity Ratio for Humidity Ratio Equation {kgWater/kgDryAir}",
        "    0.026279,                !- Maximum Regeneration Outlet Air Humidity Ratio for Humidity Ratio Equation {kgWater/kgDryAir}",
        "    5.0,                     !- Minimum Regeneration Inlet Air Relative Humidity for Humidity Ratio Equation {percent}",
        "    45.0,                    !- Maximum Regeneration Inlet Air Relative Humidity for Humidity Ratio Equation {percent}",
        "    80.0,                    !- Minimum Process Inlet Air Relative Humidity for Humidity Ratio Equation {percent}",
        "    100.0;                   !- Maximum Process Inlet Air Relative Humidity for Humidity Ratio Equation {percent}",
    });

    ASSERT_TRUE(process_idf(idf_objects));

    OutputProcessor::TimeValue.allocate(2);
    DataGlobals::DDOnlySimulation = true;

    SimulationManager::GetProjectData();
    OutputReportPredefined::SetPredefinedTables();
    createFacilityElectricPowerServiceObject();
    SetPreConstructionInputParameters(); // establish array bounds for constructions early
    BranchInputManager::ManageBranchInput();
    BeginSimFlag = true;
    BeginEnvrnFlag = true;
    ZoneSizingCalc = true;
    SysSizingCalc = true;
    SizingManager::ManageSizing();

    DataSizing::CurSysNum = 1;
    DataSizing::CurOASysNum = 0;

    GetDesiccantDehumidifierInput();
    EXPECT_EQ(1, NumDesicDehums);
    EXPECT_EQ("DESICCANT 1", DesicDehum(DesicDehumNum).Name);
    EXPECT_EQ("DESICCANT REGEN COIL", DesicDehum(DesicDehumNum).RegenCoilName);

    EXPECT_EQ(DesicDehum(DesicDehumNum).coolingCoil_TypeNum, DataHVACGlobals::Coil_CoolingAirToAirVariableSpeed);

    EXPECT_EQ(DesicDehum(DesicDehumNum).CoilUpstreamOfProcessSide, 1);

    CompName = DesicDehum(DesicDehumNum).Name;
    CompIndex = NumGenericDesicDehums;
    SimDesiccantDehumidifier(CompName, FirstHVACIteration, CompIndex);

    RegCoilDesInletTemp = FinalSysSizing(DataSizing::CurSysNum).HeatOutTemp;
    RegCoilDesOutletTemp = DesicDehum(DesicDehumNum).RegenSetPointTemp;
    RegCoilInletAirMassFlowRate = FinalSysSizing(DataSizing::CurSysNum).DesMainVolFlow * DataEnvironment::StdRhoAir;
    RegCoilCapacity = RegCoilInletAirMassFlowRate * PsyCpAirFnWTdb(0.0, 20.0) * (RegCoilDesOutletTemp - RegCoilDesInletTemp);

    for (loop = 1; loop <= NumHeatingCoils; ++loop) {
        if (HeatingCoil(loop).Name == DesicDehum(DesicDehumNum).RegenCoilName) {
            CoilIndex = loop;
        }
    }
    EXPECT_EQ(RegCoilCapacity, HeatingCoil(CoilIndex).NominalCapacity);
}
} // namespace EnergyPlus
